//
// evalfunc.hpp
//
// function prototypes for the evaluation
//
// created: 01.02.97
//
// last change: 03.02.99
//
// RCS version control information:
//     $Revision: 1.6 $$State: Exp $$Date: 1999/02/03 18:38:36 $
//


#ifndef _EVALFUNC_HPP_
#define _EVALFUNC_HPP_

#include "bool.hpp"
#include "socket.hpp"


//**************************************************************************
// provided functions
//**************************************************************************

extern void ExecuteCommand (CStreamInterface *sock, SCommand *command);
// send the command over the socket

extern void SetParameter (CStreamInterface *sock, SParameter *param);
// send the parameter over the socket

extern EBool GetParameter (CStreamInterface *sock, SSingleParameter *param);
// send the parameter over the socket, receive reply, modify param->value

extern EBool ReceiveEvent (CStreamInterface *sock);
// receives an event and prints it to cout; returns True if the event
// was received successfully, else False

extern EBool ReceiveReply (CStreamInterface *sock, SReply **reply);
// receives a reply, allocates the appropriate reply structure and returns
// it in 'reply'; returns True if the reply was received successfully,
// else False
// the function converts boolean/double replies if g_byteOrderReversed is True

extern void CheckBooleanReply (CStreamInterface *sock, const char *command);
// waits for a reply, prints an error message containing 'command' to cerr
// if (a) there was no reply, (b) it was no boolean reply, or (c) the reply
// contained the boolean value False

extern int SelectSocket ();
// waits for the supervisor sockets to become readable (using select);
// returns the index of the first socket that is readable (if more than one
// socket is readable, the smallest index is returned); if there is a timeout
// or some other failure, NoAccount is returned
// it is assumed, that the arrays supsid[] (global) and supsock[] (local
// in main()) correspond (ie. the data of one socket has the same index in
// both arrays)

extern EBool SocketExists (const ACSocketAddress *addr, ESocketType type);
// checks if the socket with the given address already exists and returns
// True if the check succeeds, False if the socket does not exist
// the function performs its check by creating a socket with this address
// and type and by checking if the bind() call fails

extern void PrintConfiguration ();
// prints the system configuration

extern void PrintPrecision ();
// prints the precision data collected during evaluation

extern void OpenPlotFiles();
// creates and opens all files needed for gnuplot output

extern void ClosePlotFiles();
// closes all files needed for gnuplot output

#endif

