//
// clock_hw.cpp
//
// definition of the base class used for accessing the hardware clock
//
// created: 19.02.97
//
// last change: 14.04.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Exp $$Date: 1998/07/03 13:11:34 $
//


#include "ci.h"
#include "nti.h"

#include "clock_hw.hpp"
#include "error.hpp"

#define MODULE_OFFSET 0x00100000;
#define CLOCK_IF_NUM  2;


extern int EnableUTCSUInterrupt (SINT32 if_num, int area, INT32 mask1,
                                 INT32 mask2);
extern int DisableUTCSUInterrupt (SINT32 if_num, int area, INT32 mask1,
                                  INT32 mask2);


//**************************************************************************
// class CClockTemplate: the template class used for obtaining an instance
//      of the currently used clock (hardware)
//**************************************************************************

//--------------------------------------------------------------------------
//--- return a new instance of the class
//--------------------------------------------------------------------------

// returns a new object of the hardware clock
// assert: area < NumberOfAreas
// assert: frequency > 0 (through the ctor of CClock)
CClock *CClockTemplate::NewClock (double frequency, SINT32 nodeID, int area)
{
    Assert (area < NumberOfAreas);
    return new CHardwareClock (frequency,i6360_VME_BASE+area*MODULE_OFFSET);
    // ignore the nodeID, the hardware clock does not need it
}


//**************************************************************************
// class CHardwareClock: interface to the hardware clock
//**************************************************************************

//--------------------------------------------------------------------------
// constructor, destructor
//--------------------------------------------------------------------------

CHardwareClock::CHardwareClock (double frequency, INT32 ntiAddress)
    : CClock (frequency, (SUtcsuRead*)(&(((NTI*)ntiAddress)->utcsu.r32)),
              (SUtcsuWrite*)(&(((NTI*)ntiAddress)->utcsu.w32)))
{
}

CHardwareClock::~CHardwareClock ()
{
}


//--------------------------------------------------------------------------
// enable/disable single interrupt bits
//--------------------------------------------------------------------------

// calls the global function for enabling UTCSU interrupts; adds the 
// if_num of the clock synchronization
EBool CHardwareClock::EnableInterrupt (int area, INT32 mask1, INT32 mask2)
{
    if (EnableUTCSUInterrupt (CLOCK_IF_NUM, area, mask1, mask2)) return True;
    return False;
}


// calls the global function for disabling UTCSU interrupts; adds the 
// if_num of the clock synchronization
EBool CHardwareClock::DisableInterrupt (int area, INT32 mask1, INT32 mask2)
{
    if (DisableUTCSUInterrupt (CLOCK_IF_NUM, area, mask1, mask2)) return True;
    return False;
}


