//
// cont_hw.hpp
//
// declaration of the synchronization controller (hardware)
//
// note: this version is for Test5
//
// StopExecution() sends an event to the ROOT task
// there are no private functions concerning the task list
// there is no member listOfTasks
// StartApplication() gets the supervisor from the listOfSupervisors and
//      starts a task if none exists, else it returns False
// StopApplication() sends the EvKillTask event to the task (if it exists)
//
// created: 30.03.97
//
// last change: 15.03.98
//
// RCS version control information:
//     $Revision: 1.3 $$State: Exp $$Date: 1998/07/03 13:11:52 $
//


#ifndef _CONT_HW_HPP_
#define _CONT_HW_HPP_

#include "contbase.hpp"



//**************************************************************************
// class CHardwareController: this class controls all clock synchronization
//     applications (which are tasks)
//**************************************************************************

class CHardwareController : public ACController
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CHardwareController (char qname[4], ACSocketAddress *addr);
    // call the queue "CTRL" or something like that; *don't* use a string
    // of (hex) numbers!

    virtual ~CHardwareController();  // deletes the simulation objects



protected:

    //----------------------------------------------------------------------
    //--- starting/stopping applications
    //----------------------------------------------------------------------

    virtual EBool StartApplication (SINT32 nodeID, int area);
    // create and start the application task

    virtual EBool StopApplication (SINT32 nodeID, int area);
    // delete the application task


    //----------------------------------------------------------------------
    //--- stop program execution
    //----------------------------------------------------------------------

    virtual void DeleteAllSimulationObjects();
    // delete all tasks, call base class function

    virtual void StopExecution();
    // call exit()
};


#endif


