//
// gps_hw.cpp
//
// this version is for TestGps
// *) EnableGps(): 1PPS interrupt is only enabled if TEST_SERIAL is not defined
//
// note: take care: this class does not really work for more than one
// receiver!! (isr() and isr_1pps() contain code that cannot be used if
// more than one GPU is occupied)
//
// note: sg_gpsTime for storing the GPS time makes the class unusable for
// more than one GPS receiver per target
//
// created: 13.02.97
//
// last change: 15.04.98
//
// RCS version control information:
//     $Revision: 1.1 $$State: Exp $$Date: 1998/07/01 17:28:25 $
//


#include "clockifc.hpp"
#include "constant.hpp"
#include "gps_hw.hpp"
#include "interupt.hpp"


extern double g_gpstime;
extern EGpsUnit g_gpu;
// both are defined and set in gpstask.cpp


static const INT32 GpsNotHealthy = (INT32)1<<31;
// bit 31 in the MSGPS contains the status information of the receiver
/**/// assumption: bit=0 --> healthy


static CNtpTime sg_gpsTime;  // the time information of the GPS receiver
// dirty hack!!!


//**************************************************************************
//**************************************************************************

ACGps *CGpsTemplate::NewGPS (CClockInterface *clock, int area)
{
    return new CHardwareGps (clock, area);
}


//**************************************************************************
// class CHardwareGps: synchronizes to GPS receiver (in cooperation with
//      a task)
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

CHardwareGps::CHardwareGps (CClockInterface *clock, int area)
    : ACGps (clock, area)
{
    q_ident ("QGPS", 0, &gpsQueueID_);
    synchronized_ = False;
}

CHardwareGps::~CHardwareGps ()
{
    for (int i=0; enableMask_ != 0; i++)  // disable all enabled GPS rec.
    {
        if (enableMask_ & (1<<i))
        {
            DisableGps ((EGpsUnit)i);  // disable the GPU
            enableMask_ &= ~(1<<i);  // mask out the bit
        }
    } // for

    INT32 msg[4];
    msg[0] = EGpsUnitCount;  // tell GPS task to stop
    q_send (gpsQueueID_, msg);
}


//--------------------------------------------------------------------------
//--- enabling/disabling interrupts
//--------------------------------------------------------------------------

// installs the 1PPS ISR for the given GPU
void CHardwareGps::EnableGps (EGpsUnit gpu)
{
    if (!GpsEnabled())  // first installation -> install 1PPS ISR
    {
        SetISR (GetClockIntvec (NoAccount, area_) | INTN, this, Call1PPS);
    }
    enableMask_ |= (1<<gpu);  // store the GPU in the mask

#ifndef TEST_SERIAL
    switch (gpu)  // enable the 1PPS interrupt on the UTCSU
    {
        case GPU1:
            clock_->EnableInterrupt (IntGps1PPS1);  
            break;
        case GPU2:
            clock_->EnableInterrupt (IntGps1PPS2);
            break;
        case GPU3:
            clock_->EnableInterrupt (IntGps1PPS3);
            break;
    } // switch
#endif
}

// removes the 1PPS ISR for the given GPU
void CHardwareGps::DisableGps (EGpsUnit gpu)
{
    switch (gpu)  // disable the 1PPS interrupt on the UTCSU
    {
        case GPU1:
            clock_->DisableInterrupt (IntGps1PPS1);
            break;
        case GPU2:
            clock_->DisableInterrupt (IntGps1PPS2);
            break;
        case GPU3:
            clock_->DisableInterrupt (IntGps1PPS3);
            break;
    } // switch

    if (!(enableMask_ & ~(1<<gpu)))  // the last receiver is disabled
    {
        RemoveISR (GetClockIntvec (NoAccount, area_) | INTN, this, Call1PPS);
        // remove the 1PPS ISR
    }

    enableMask_ &= ~(1<<gpu);  // mask out the GPU
    statusMask_ &= ~(1<<gpu);  // clear the status bit
    // note: do not call SetGpsStatus(), because at that time the GPU must
    // still be enabled, so clock->gpsStatus would be wrongly set to False;
    // clearing the bit keeps the gpsStatus, which is exactly what we want
    // note: the two statements were simply taken from gps_sw.cpp
}


//--------------------------------------------------------------------------
//--- handling GPS time
//--------------------------------------------------------------------------

// returns the GPS time (taken from g_gpstime)
void CHardwareGps::GetGpsTime (CNtpTime& time)
{
    Assert (g_gpstime >= 0);  // GPS time must be valid
    SecondsToNtp (g_gpstime, time);
}

// returns the GPU (taken from g_gpu)
EGpsUnit CHardwareGps::GetGpsNumber()
{
    return g_gpu;
}

void CHardwareGps::ClearGpsInterrupt() {}



//--------------------------------------------------------------------------
//--- difference to GPS time
//--------------------------------------------------------------------------

// computes and returns the difference between the gpsTime_ and the
// time that is kept in the MSGPS and TSGPS registers on the specified
// GPU; the difference is specified in seconds
// when reading the GPS registers, the healthy bit is checked and the
// GpsStatus is set accordingly (if it is not healthy, 0 is returned)
// assert: gpu < EGpsUnitCount
double CHardwareGps::ComputeDifference (EGpsUnit gpu)
{
    Assert (gpu < EGpsUnitCount);

    CNtpTime clockTime;  // time value at the time of the 1PPS
    GetClockGps (gpu, clockTime);  // read the clock value (MSGPS, TSGPS)

    // check GPS status, react to it
    if (clockTime.ms & GpsNotHealthy)
    {
        SetGpsStatus (gpu, False);  // notify the clock of unhealthy GPS
        return 0;  // ignore the GPS time information
    }
    else SetGpsStatus (gpu, True);  // GPS receiver is healthy

    clockTime.ms &= ~GpsNotHealthy;  // mask out the healthy bit

    return NtpToSeconds (sg_gpsTime) - NtpToSeconds (clockTime);
    // compute the time difference between the current clock value
    // and its correct value (instead of using the current values,
    // those at the time of the 1PPS pulse are used; the clock drift
    // is ignored)
}


void CHardwareGps::AmortDifference (EGpsUnit gpu)
{
    double diff = ComputeDifference (gpu);

Report (amortizing difference);

    if ((diff > GpsEpsilon) || (-diff > GpsEpsilon))
    {
        SApplGpsEvent *event = new SApplGpsEvent;
        event->diff = diff;
        ReportEvent (event);

        clock_->Amortize (diff, 0, 0);  // amortize time difference
    }
}


//--------------------------------------------------------------------------
//---  1PPS ISR
//--------------------------------------------------------------------------

void CHardwareGps::isr ()
{
    CNtpTime clockTime;  // time value at the time of the 1PPS
    //CNtpTime gpsTime;  // the time information of the GPS receiver
    EGpsUnit gpu = GetGpsNumber();  // the number of the GPS receiver

    GetGpsTime (sg_gpsTime);  // get the GPS value

    double diff = ComputeDifference (gpu);

/*
    GetClockGps (gpu, clockTime);  // read the clock value (MSGPS, TSGPS)

    if (clockTime.ms & GpsNotHealthy)
    {
        SetGpsStatus (gpu, False);  // notify the clock of unhealthy GPS
        return;  // ignore the GPS time information
    }
    else SetGpsStatus (gpu, True);  // GPS receiver is healthy

    clockTime.ms &= ~GpsNotHealthy;  // mask out the healthy bit

    double diff = NtpToSeconds (sg_gpsTime) - NtpToSeconds (clockTime);
    // compute the time difference between the current clock value
    // and its correct value (instead of using the current values,
    // those at the time of the 1PPS pulse are used; the clock drift
    // is ignored)
*/

    if (!synchronized_)  // first call -> set directly
    {
        sg_gpsTime.ts = (sg_gpsTime.ts & 0xFF000000);
        clock_->SetAtOnce (sg_gpsTime);  // set GPS time at once
        synchronized_ = True;
    }
    else if ((diff > GpsEpsilon) || (-diff > GpsEpsilon))
    {
        SApplGpsEvent *event = new SApplGpsEvent;
        event->diff = diff;
        ReportEvent (event);

        clock_->Amortize (diff, 0, 0);  // amortize time difference
    }
}

void CHardwareGps::isr_1pps ()
{
    INT32 msg[4];  // contains the GPU, is sent to the GPS task
    INT8 gpus = 0;  // bits are set for GPUs that have 1PPS pending

    // service GPU1
    if ((clock_->InterruptPending (IntGps1PPS1)) && (enableMask_ & (1<<GPU1)))
    {
        clock_->ClearInterrupt (IntGps1PPS1);  // clear the 1PPS interrupt
        msg[0] = GPU1;
        gpus += 1;
        if (!synchronized_) q_send (gpsQueueID_, msg);
    }

    // service GPU2
    if ((clock_->InterruptPending (IntGps1PPS2)) && (enableMask_ & (1<<GPU2)))
    {
        clock_->ClearInterrupt (IntGps1PPS2);  // clear the 1PPS interrupt
        msg[0] = GPU2;
        gpus += 2;
        if (!synchronized_) q_send (gpsQueueID_, msg);
    }

    // service GPU3
    if ((clock_->InterruptPending (IntGps1PPS3)) && (enableMask_ & (1<<GPU3)))
    {
        clock_->ClearInterrupt (IntGps1PPS3);  // clear the 1PPS interrupt
        msg[0] = GPU3;
        gpus += 4;
        if (!synchronized_) q_send (gpsQueueID_, msg);
    }

    if (synchronized_ && gpus)
    {
        sg_gpsTime += CNtpTime(1);  // add one second
        if (gpus & 1) AmortDifference (GPU1);  // amortize at once
        if (gpus & 2) AmortDifference (GPU2);  // amortize at once
        if (gpus & 4) AmortDifference (GPU3);  // amortize at once
    }
}



//**************************************************************************
// call function for 1PPS ISR
//**************************************************************************

// to invoke the 1PPS ISR
void Call1PPS (CNeedInterrupt *obj)
{
    ((CHardwareGps *)obj)->isr_1pps ();
    // call the 1PPS ISR
}

