//
// gps_hw.hpp
//
// dummy declaration of the GPS class
//
// created: 29.03.97
//
// last change: 13.04.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Exp $$Date: 1998/07/03 13:11:52 $
//


#ifndef _GPS_HW_HPP_
#define _GPS_HW_HPP_

#include "psos.h"

#include "clockcfg.hpp"
#include "gpsbase.hpp"


//**************************************************************************
// class CHardwareGps: synchronizes to GPS receiver (in cooperation with
//	a task)
//**************************************************************************

class CHardwareGps : public ACGps
{
public:
    friend void Call1PPS (CNeedInterrupt *obj);
    // to invoke the 1PPS ISR


    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CHardwareGps (CClockInterface *clock, int area);
    // assert: clock != NULL (through the ctor of ACGps)

    virtual ~CHardwareGps ();


    //----------------------------------------------------------------------
    //--- enabling/disabling interrupts
    //----------------------------------------------------------------------

    virtual void EnableGps (EGpsUnit gpu);
    // installs the 1PPS ISR for the given GPU

    virtual void DisableGps (EGpsUnit gpu);
    // removes the 1PPS ISR for the given GPU


protected:

    //----------------------------------------------------------------------
    //--- handling GPS time
    //----------------------------------------------------------------------

    virtual void GetGpsTime (CNtpTime& time);
    // returns the GPS time (taken from g_gpstime)

    virtual EGpsUnit GetGpsNumber();
    // returns the GPU (taken from g_gpu)

    virtual void ClearGpsInterrupt();


protected:

    void isr();  // is called by the GPS task to synchronize


private:

    INT32 gpsQueueID_;  // ID of the GPS queue
    EBool synchronized_;  // set to True when GPS time is taken over


    //----------------------------------------------------------------------
    //--- difference to GPS time
    //----------------------------------------------------------------------

    double ComputeDifference (EGpsUnit gpu);
    // computes and returns the difference between sg_gpsTime and the
    // time that is kept in the MSGPS and TSGPS registers on the specified
    // GPU; the difference is specified in seconds
    // when reading the GPS registers, the healthy bit is checked and the
    // GpsStatus is set accordingly (if it is not healthy, 0 is returned)
    // assert: gpu < EGpsUnitCount

    void AmortDifference (EGpsUnit gpu);
    // computes the difference of the clock to sg_gpsTime, and if it is
    // larger than a threshold, the UTCSU is amortized


    //----------------------------------------------------------------------
    //---  1PPS ISR
    //----------------------------------------------------------------------

    virtual void isr_1pps ();
    // sends a message to the GPS task to prompt reading the GPS time
};


#endif


