//
// opnew.hpp
//
// provides an operator new which uses pt_getbuf() to allocate a partition
// buffer; operator delete uses pt_retbuf() to return the buffer to the
// pool
//
// contents:
//      InitMemory()            creates partitions, initializes data table
//      DisposeMemory()         deletes partitions (returns memory to region)
//      CheckMemory()           reports statistical data to the error stream
//      operator new()          for allocating memory with new
//      operator delete()       for freeing memory with delete
//
// created: 27.04.98
//
// last change: 28.04.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Exp $$Date: 1998/07/03 13:11:52 $
//


#ifndef _OPNEW_HPP_
#define _OPNEW_HPP_

#include <stdlib.h>


//**************************************************************************
// partition table management
//**************************************************************************

extern "C" {
void InitMemory();
// allocates a 1 MB memory block via malloc(), creates the partitions and
// initializes the data table
// must be called in the context of a task at the beginning of the program
// (before the first dynamic memory is allocated)
// context: task (no ISR!!)

void DisposeMemory ();
// deletes the partitions and frees the memory
// note: the function assumes (but does not check!) that InitMemory() has
// been called
// context: task (no ISR!!)

void CheckMemory ();
// report the statistical data of the partition table
// context: task or ISR
}



//**************************************************************************
// operators new and delete
//**************************************************************************

extern void *operator new (size_t s);
extern void operator delete (void *p);
// context: task or ISR

#endif

