//
// acctemp.hpp
//
// transmission delay lists
//
// created: 05.10.96
//
// last change: 22.05.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _ACCTEMP_HPP_
#define _ACCTEMP_HPP_

#include "error.hpp"
#include "listbase.hpp"


//**************************************************************************
// class CTransmissionDelayList: this class stores network delays with
//      the key <SINT32 nodeID>
//**************************************************************************

class CTransmissionDelayList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CTransmissionDelayList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {}
    // allocate memory

    virtual ~CTransmissionDelayList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    SNetworkDelay* GetFirst ()
        { return (SNetworkDelay*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    SNetworkDelay* GetLast ()
        { return (SNetworkDelay*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    SNetworkDelay* GetNext ()
        { return (SNetworkDelay*) ACObjectList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    SNetworkDelay* GetPrev ()
        { return (SNetworkDelay*) ACObjectList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    SNetworkDelay* Find (SINT32 nodeID) // put the keys here
    {
        return (SNetworkDelay*) ACObjectList::Find (&nodeID);
    }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    SNetworkDelay* GetCurrentObject ()
        { return (SNetworkDelay*) ACObjectList::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    EBool GetCurrentObjectKey (SINT32& nodeID)
    // references to the key variables
    {
        SINT32 *key = (SINT32*) ACObjectList::GetCurrentObjectKey();

        if (key != NULL)
        {
            nodeID = *key;
            // set the argument variables
            return True;
        }
        else return False;
    }
    // returns True if a member is accessed (in that case the netAddress will
    // contain the key), or False if no element is currently accessed


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (SNetworkDelay* object, SINT32 nodeID)
    {
        SINT32 *key = new SINT32 (nodeID);
        EBool inserted;

        inserted = ACObjectList::Insert (object, key);

        if (!inserted)  // the object was not inserted
        {
            delete key;  // the key is not needed
        }

        return inserted;
    }

    SNetworkDelay* Remove (SINT32 nodeID)
    {
        return (SNetworkDelay*) ACObjectList::Remove (&nodeID);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, True is returned; otherwise
    // False is returned
    EBool Delete (SINT32 nodeID)
    {
        SNetworkDelay* object;
        EBool found = False;

        object = Remove (nodeID);

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        SNetworkDelay* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll


protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        Assert (key1 != NULL);
        Assert (key2 != NULL);

        if (*((SINT32*)key1) < *((SINT32*)key2)) return -1;
        if (*(SINT32*)key1 > *(SINT32*)key2) return +1;
        return 0;
    }

    virtual void DeleteKey (void* key) { delete (SINT32*) key; }
    // delete the key
};


//**************************************************************************
// class CTransmissionList: this class is a list of CTransmissionDelayList
//**************************************************************************

class CTransmissionList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CTransmissionList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {}
    // allocate memory

    virtual ~CTransmissionList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    CTransmissionDelayList* GetFirst ()
        { return (CTransmissionDelayList*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    CTransmissionDelayList* GetLast ()
        { return (CTransmissionDelayList*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    CTransmissionDelayList* GetNext ()
        { return (CTransmissionDelayList*) ACObjectList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    CTransmissionDelayList* GetPrev ()
        { return (CTransmissionDelayList*) ACObjectList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    CTransmissionDelayList* Find (SINT32 key) // put the keys here
    {
        return (CTransmissionDelayList*) ACObjectList::Find (&key);
    }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    CTransmissionDelayList* GetCurrentObject ()
        { return (CTransmissionDelayList*) ACObjectList::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    EBool GetCurrentObjectKey (SINT32& ident)
    // references to the key variables
    {
        SINT32 *key = (SINT32*) ACObjectList::GetCurrentObjectKey();

        if (key != NULL)
        {
            ident = *key;
            return True;
        }
        else return False;
    }
    // returns True if a member is accessed (in that case ident will
    // contain the key), or False if no element is currently accessed


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (CTransmissionDelayList* object, SINT32 ident)
    {
        SINT32 *key = new SINT32 (ident);
        EBool inserted;

        inserted = ACObjectList::Insert (object, key);

        if (!inserted)  // the object was not inserted
        {
            delete key;  // the key is not needed
        }

        return inserted;
    }

    CTransmissionDelayList* Remove (SINT32 key)
    {
        return (CTransmissionDelayList*) ACObjectList::Remove (&key);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, True is returned; otherwise
    // False is returned
    EBool Delete (SINT32 key)
    {
        CTransmissionDelayList* object;
        EBool found = False;

        object = Remove (key);

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        CTransmissionDelayList* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll


protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        Assert (key1 != NULL);
        Assert (key2 != NULL);

        if (*((SINT32*)key1) < *((SINT32*)key2)) return -1;
        if (*(SINT32*)key1 > *(SINT32*)key2) return +1;
        return 0;
    }

    virtual void DeleteKey (void* key) { delete (SINT32*) key; }
    // delete the key
};


#endif


