//
// addrtemp.hpp
//
// stores network addresses
//
// created: 02.01.97
//
// last change: 03.01.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _ADDRTEMP_HPP_
#define _ADDRTEMP_HPP_

#include "ci.h"

#include "error.hpp"
#include "listbase.hpp"


//**************************************************************************
// class CAddressList: stores network addresses with a pair
//      <SINT32 networkID, SINT32 nodeID> as key
//**************************************************************************

class CAddressList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CAddressList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {};
    // allocate memory

    virtual ~CAddressList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    CAddr* GetFirst () { return (CAddr*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    CAddr* GetLast () { return (CAddr*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    CAddr* GetNext () { return (CAddr*) ACObjectList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    CAddr* GetPrev () { return (CAddr*) ACObjectList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    CAddr* Find (SINT32 networkID, SINT32 nodeID) // put the keys here
    {
        SKey key (networkID, nodeID);
        return (CAddr*) ACObjectList::Find (&key);
    }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    CAddr* GetCurrentObject ()
        { return (CAddr*) ACObjectList::GetCurrentObject(); }
    // returns a pointer to the member that is currently being accessed or
    // NULL, if no element is being accessed

    EBool GetCurrentObjectKey (SINT32& networkID, SINT32& nodeID)
    // references to the key variables
    {
        SKey *key = (SKey*) ACObjectList::GetCurrentObjectKey();

        if (key != NULL)
        {
            networkID = key->networkID_;
            nodeID = key->nodeID_;
            // set the argument variables
            return True;
        }
        else return False;
    }
    // returns True if a member is accessed (in that case networkID and
    // nodeID will contain the key), or False if no element is currently
    // accessed


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (CAddr* object, SINT32 networkID, SINT32 nodeID)
    {
        SKey *key = new SKey (networkID, nodeID);
        EBool inserted;

        inserted = ACObjectList::Insert (object, key);

        if (!inserted)  // the object was not inserted
        {
            delete key;  // the key is not needed
        }

        return inserted;
    }

    CAddr* Remove (SINT32 networkID, SINT32 nodeID)
    {
        SKey key (networkID, nodeID);
        return (CAddr*) ACObjectList::Remove (&key);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned



    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    // current points to the position where the element was deleted (or
    // should have be inserted), or to the last element if the deleted
    // object was (would have been) the last
    EBool Delete (SINT32 networkID, SINT32 nodeID)
    {
        CAddr *object;
        EBool found = False;

        object = Remove (networkID, nodeID);  // remove the object

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        CAddr *object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll


protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        Assert (key1 != NULL);
        Assert (key2 != NULL);

        SKey *k1 = (SKey *)key1;
        SKey *k2 = (SKey *)key2;

        if (k1->networkID_ < k2->networkID_) return -1;
        else if (k1->networkID_ > k2->networkID_) return +1;

        if (k1->nodeID_ < k2->nodeID_) return -1;
        else if (k1->nodeID_ > k2->nodeID_) return +1;
        else return 0;
    }


    virtual void DeleteKey (void* key) { delete (SKey*) key; }
    // delete the key


private:
    struct SKey  // this structure contains the key(s) for sorting
    {
        // all key elements are put here
        SINT32 networkID_;
        SINT32 nodeID_;

        SKey (SINT32 networkID, SINT32 nodeID)
        {
            networkID_ = networkID;
            nodeID_ = nodeID;
        }
        // initialize the members
    };
};

#endif

