//
// algfetz.hpp
//
// uses FTA (reference point), but handles GPS nodes according to Fetzer's
// algorithm
//
// created: 15.04.98
//
// last change: 15.04.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _ALGFETZ_HPP_
#define _ALGFETZ_HPP_


#include "psos.h"

#include "algfta.hpp"


//**************************************************************************
// class CFtaWithFetzer: uses CFta as base class, but stores P-Node data
//	separatly and only merges the two after FTA has been computed
//**************************************************************************

class CFtaWithFetzer : public CFta
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CFtaWithFetzer (CClockSync *cs);
    // assert: cs != NULL

    virtual ~CFtaWithFetzer ();


    //----------------------------------------------------------------------
    //--- suspend the synchronization
    //----------------------------------------------------------------------

    virtual void Suspend ();
    // discard the collected interval data


    //----------------------------------------------------------------------
    //--- add time intervals
    //----------------------------------------------------------------------

    virtual EBool StoreTime (const STimestamp& time, double resync,
        SINT32 networkID, const CAddr& netAddress, int area, EBool gps);
    // if 'gps' is True, then the time interval of this node is kept
    // in a separate list; if 'gps' is False, the function of the base class
    // is called to handle the time information


    //----------------------------------------------------------------------
    //--- compute and set the new clock interval
    //----------------------------------------------------------------------

    virtual EBool ConvergenceFunction ();
    // returns True if a new interval could be computed (depends on 'nodes'
    // (= number of nodes in the system) and 'faulty' (the maximum number
    // of faulty nodes); in that case the new clock value should be
    // stored in 'newClockValue' for later use in SetNewTimestamp() and
    // 'timeValid' should be set to True; if no interval could be computed,
    // 'timeValid' should be set to False


    //----------------------------------------------------------------------
    //--- creation function of class CFtaWithFetzer
    //----------------------------------------------------------------------

    static ACClockSyncAlgorithm *CreateFtaWithFetzer (CClockSync *cs);
    // creates and returns a new object of type CFtaWithFetzer


protected:

    STimeInterval *gpsdata;  // array for storing the GPS data
    INT32 gpsdataIndex;  // the first free entry in the array


    //----------------------------------------------------------------------
    //--- create/destroy the members that are modified during reset
    //----------------------------------------------------------------------

    virtual void Init ();
    // calls Init() of the base class, then creates the 'gpsdata' array

    virtual void Dispose ();
    // deletes the 'gpsdata' array, calls Dispose() of the base class
    // deletes the dynamic members created by Init(); a second call
    // (without Init() in between), has no effect


private:

    EBool initCalled_;  // set to True by Init(), to False by Dispose()
};


#endif


