//
// algfta.hpp
//
// declaration of the FTA (fault tolerant average) clock synchronization
// algorithm
//
// created: 12.02.97
//
// last change: 24.07.97
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _ALGFTA_HPP_
#define _ALGFTA_HPP_


#include "algbase.hpp"


struct STimeInterval;


//**************************************************************************
// class CFta: provides the FTA algorithm; uses the reference point
//**************************************************************************

class CFta : public ACClockSyncAlgorithm
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CFta (CClockSync *cs);
    // assert: cs != NULL


    //----------------------------------------------------------------------
    //--- compute and set the new clock interval
    //----------------------------------------------------------------------

    virtual EBool ConvergenceFunction ();
    // returns True if a new interval could be computed (depends on the
    // number of nodes in the system and the maximum number of faulty
    // nodes); in that case 'newTime' is set to the new clock value; if
    // no valid interval could be computed, False is returned and 'newTime'
    // is not changed


    //----------------------------------------------------------------------
    //--- creation function of class CFta
    //----------------------------------------------------------------------

    static ACClockSyncAlgorithm *CreateFta (CClockSync *cs);
    // creates and returns a new object of type CFta


protected:

    EBool midpoint_;
    // is True if FTA() should use the midpoint of the intervals, False if
    // it should use the reference point


private:

#ifdef PUBLIC_IN_TEST
public:
#endif

    //----------------------------------------------------------------------
    //--- FTA algorithm
    //----------------------------------------------------------------------

    EBool FTA (STimeInterval& interval, SINT32 networkID);
    // computes the new clock interval for the network; returns True if the
    // computation was successful, False otherwise
};


//**************************************************************************
// class CMidpoint: same as CFta, but uses the midpoint of the time interval
//       instead of the reference point
//**************************************************************************

class CMidpoint : public CFta
{
public:
    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CMidpoint (CClockSync *cs) : CFta (cs) {midpoint_ = True;}
    // assert: cs != NULL


    //----------------------------------------------------------------------
    //--- creation function of class CMidpoint
    //----------------------------------------------------------------------

    static ACClockSyncAlgorithm *CreateMidpoint (CClockSync *cs)
        { return new CMidpoint (cs); }
    // creates and returns a new object of type CMidpoint
};


#endif


