//
// alggen.hpp
//
// declaration of general matters for a "clock synchronization algorithm"
//
// created: 03.07.97, Klaus Schossmaier
//
// last change: 03.07.97, Klaus Schossmaier
//
// last revision: 21.01.99
//
// RCS version control information:
//     $Revision: 1.8 $$State: Exp $$Date: 1999/01/27 12:25:19 $
//


#ifndef _ALGGEN_HPP_
#define _ALGGEN_HPP_

#include "psos.h"

#include "bool.hpp"
#include "time.hpp"


//**************************************************************************
// structures
//**************************************************************************

// asymetrical interval
struct SAsymInterval
{
    double left;   // left edge
    double ref;    // reference point 
    double right;  // right edge
};


//**************************************************************************
// constants
//**************************************************************************

// for accuray conversion
const INT32 Infinity = 0xFFFFFFFF;
// infinite accuracy; all 1 in (-8,-24)

const INT32 HighestAccuracyBit = 16;
// highest bit in timestamp that is covered by the accuracy, as seen from
// the lsb of timestamp (lsb = -24, highest bit = -8)

const INT32 HighestAccuracyValue = (1 << HighestAccuracyBit+1)-1;
// the highest value in timestamp that is covered by the accuracy

const int WidthOfChars = 14;
// used for function ftoa(), sets the number of used characters in %e format


//**************************************************************************
// functions
//**************************************************************************

extern EBool ApplyMarzullo (SAsymInterval* input, SAsymInterval& output,
                            INT32 size, int f);
// Marzullo function is applied on the input array 'input' which has
// size 'size', 'f' denotes the number of faulty nodes
// the result of the Marzullo function is returned in interval 'output'
// assert: input != NULL

extern void SortArray (double* unsortedArray, double* sortedArray, INT32 size);
// sorts the array 'unsortedArray' which has size 'size', returns the
// result in the array 'sortedArray' (which must be large enough to contain
// the sorted array)
// assert: unsortedArray != NULL, sortedArray != NULL, size > 0

extern EBool Intersect (const SAsymInterval& in1, const SAsymInterval& in2,
                        SAsymInterval& out);
// computes the intersection of intervals 'in1' and 'in2' and returns the
// result in 'out'; if the intersection exists, True is returned
// if the intersection is empty, False is returned, and in this case the
// contents of 'out' is not valid

extern SExtendedTimestamp IntervalToClock (CNtpTime point, double alpha_minus,
                                           double alpha_plus);
// converts the time 'point' with the accuracies 'alpha_minus' and
// 'alpha_plus' into a clock timestamp and returns this timestamp; if the
// accuracies exceed the allowed range, SExtendedTimestamp::acc is set
// to the value representing infinity

#endif

