//
// algmarz.hpp
//
// declaration of pure Marzullo's "clock synchronization algorithm"
//
// created: 03.07.97, Klaus Schossmaier
//
// last change: 03.07.97, Klaus Schossmaier
//
// last revision: 22.01.99
//
// RCS version control information:
//     $Revision: 1.4 $$State: Exp $$Date: 1999/01/22 10:52:41 $
//


#ifndef _ALGMARZ_HPP_
#define _ALGMARZ_HPP_


#include "algbase.hpp"
#include "alggen.hpp"

struct STimeInterval;


//**************************************************************************
// class CMarzullo: provides Marzullo's algorithm
//**************************************************************************

class CMarzullo : public ACClockSyncAlgorithm
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CMarzullo (CClockSync *cs);
    // assert: cs != NULL


    //----------------------------------------------------------------------
    //--- compute and set the new clock interval
    //----------------------------------------------------------------------

    virtual EBool ConvergenceFunction ();
    // returns True if a new interval could be computed (depends on the
    // number of nodes in the system and the maximum number of faulty
    // nodes); in that case 'newTime' is set to the new clock value; if
    // no valid interval could be computed, False is returned and 'newTime'
    // is not changed


    //----------------------------------------------------------------------
    //--- manage clock changes
    //----------------------------------------------------------------------

    virtual void ClockChanged();
    // computes initialization data (clock deterioration, accuracy) for
    // the clock


    //----------------------------------------------------------------------
    //--- creation function of class CMarzullo
    //----------------------------------------------------------------------

    static ACClockSyncAlgorithm *CreateMarzullo (CClockSync *cs);
    // creates and returns a new object of type CMarzullo

private:

#ifdef PUBLIC_IN_TEST
public:
#endif

    //----------------------------------------------------------------------
    //--- Marzullo's algorithm
    //----------------------------------------------------------------------

    EBool Marzullo (STimeInterval& interval, SINT32 networkID);
    // computes the new clock interval for the network; returns True if the
    // computation was successful, False otherwise
};

#endif






