//
// algop.hpp
//
// declaration of the OP "clock synchronization algorithm"
//
// created: 03.07.97, Klaus Schossmaier
//
// last change: 03.07.97, Klaus Schossmaier
//
// last revision: 27.01.99
//
// RCS version control information:
//     $Revision: 1.4 $$State: Exp $$Date: 1999/01/27 12:24:51 $
//


#ifndef _ALGOP_HPP_
#define _ALGOP_HPP_


#include "algbase.hpp"
#include "time.hpp"
#include "alggen.hpp"

struct STimeInterval;

//**************************************************************************
// class COP: provides the OP algorithm
//**************************************************************************

class COP : public ACClockSyncAlgorithm
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    COP (CClockSync *cs);
    // assert: cs != NULL


    //----------------------------------------------------------------------
    //--- compute and set the new clock interval
    //----------------------------------------------------------------------

    virtual EBool ConvergenceFunction ();
    // returns True if a new interval could be computed (depends on the
    // number of nodes in the system and the maximum number of faulty
    // nodes); in that case 'newTime' is set to the new clock value; if
    // no valid interval could be computed, False is returned and 'newTime'
    // is not changed


    //----------------------------------------------------------------------
    //--- add time intervals
    //----------------------------------------------------------------------

    virtual EBool StoreTime (const STimestamp& time, double resync,
        SINT32 networkID, const CAddr& netAddress, int area, EBool gps);
    // if CS:resync is True, the time is not stored; so the algorithm
    // only starts working after resync is False; then the time is
    // passed on to StoreTime() of the base class


    //----------------------------------------------------------------------
    //--- add transmission delay intervals
    //----------------------------------------------------------------------

    virtual EBool StoreDelay (double localDelay, double remoteDelay,
        const SRate& remoteRate, SINT32 networkID, const CAddr& netAddress);
    // counts to N before setting CS::resync to False, calls StoreDelay
    // of base class to process delays


    //----------------------------------------------------------------------
    //--- manage clock changes
    //----------------------------------------------------------------------

    virtual void ClockChanged();
    // computes initialization data (clock deterioration, accuracy) for
    // the clock, and some clock dependent parameters needed by the algorithm


    //----------------------------------------------------------------------
    //--- creation function of class COP
    //----------------------------------------------------------------------

    static ACClockSyncAlgorithm *CreateOP (CClockSync *cs);
    // creates and returns a new object of type COP


protected:

    //----------------------------------------------------------------------
    //--- set members that are modified during reset
    //----------------------------------------------------------------------

    virtual void Init ();
    // sets cntDelay=0, resync. status to True, calls Init() of base class

 
private:

    int cntDelay;  
    // number of delay measurements

    double piHMinus;
    double piHPlus;
    // maximal precision interval

    double piOwnMinus; 
    double piOwnPlus;
    // own precision interval

    double Ups;
    // maximal state correction

#ifdef PUBLIC_IN_TEST
public:
#endif

    //----------------------------------------------------------------------
    //--- OP algorithm
    //----------------------------------------------------------------------

    EBool OP (CNtpTime& point, double& alpha_minus, double& alpha_plus,
              SINT32 networkID);
    // computes the new clock interval for the network; returns True if the
    // computation was successful, False otherwise
};

#endif






