/*******************************************************************************
file:    ci.h
interface between clock synchronization and driver

created: 13.06.1995
author:  Gerda Richter  8925254
*******************************************************************************/
/*******************************************************************************
Id: ci.h,v 1.3 1997/10/22 12:46:44 xray Exp xray
*******************************************************************************/
/*******************************************************************************
Log: ci.h,v
Revision 1.3  1997/10/22 12:46:44  xray
instead of all the functions one entry point for CI, =>
 - functions codes for the various functions
 - one union for the parameters

 * Revision 1.2  1997/10/06  10:26:03  xray
 * # of ASICS became variable
 *
 * Revision 1.1  1997/10/02  09:41:49  xray
 * Initial revision
 *
*******************************************************************************/
#ifndef _CI_H_
#define _CI_H_

#include "psos.h"

#define IMMEDIATE_RELEASE 22 
   /* if this value is returned by the clock synchronization to the driver as */
   /* the result of a ci_announce call, the  concerned message is released,   */
   /* envelope and buffer                                                     */

/* #define ASICCNT 1   */
/* maximum number of UTC asics in the system -> no of timestamps */
#define DEFA 0            /* number of the DMA-area that is usually used */
#define MAXASICS 4        /* maximum allowable number of ASICS per LANCE */
#define PRIOR 0x08        /* Message is to be sent with high priority */
#define RET_PKB_IMM 0x04  /* release message immediately after transmission */
#define DEL_RET_PKB 0x02  /* not needed, only there for completeness */
#define TS_PACKET 0x01    /* timestamped packet */
#define ADDRCNT 6         /* size of ahardware address in bytes */
#define BROADCAST_ADDRESS {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF} /* format of BC add */
#define B_SIZE 64         /* size of a buffer block */

#define CI_INIT	0
#define CI_GTB	1
#define CI_GTE	2
#define CI_RTB	3
#define CI_RTE	4
#define CI_RRB	5
#define CI_RRE	6
#define CI_SEND	7
#define CI_MCON	8

typedef struct /* 12 bytes */
{
  INT32 ts;
  INT32 ms;
  INT32 acc;
} ti_acx;      /* timestamp structure in transmit envelope */

typedef struct /* 8 bytes */
{
  INT32 rts;
  INT32 rms;
} timex;       /* timestamp structure in receive envelopes */

typedef INT8 bufx[B_SIZE];            /* buffer block */
typedef unsigned char addr[ADDRCNT];  /* hardware address of a node */

typedef PACKED struct
{
  INT8  reserved[16];
  addr  da;
  INT16 length_field;
} tenvx;         /* this structure maps both types of transmission envelope */

typedef PACKED struct
{
  INT8  reserved[16];
  addr  da;
  INT16 length_field;
  INT16 ts_high;
  INT16 ts_low;
  INT32 info[9];
  INT32 packet_info;
} ts_envx;     /* this structure maps the timestamped transmission envelope */

typedef PACKED struct
{
  INT8  reserved[22];
  addr  sa;
  INT16 length_field;
  timex utcsu[MAXASICS];
} renvx;                                    /* map of the receive einvelope */


typedef int (*ci_fn)(renvx *renv,bufx *buf,INT16 size,int if_num);
typedef void (*ci_efn)(tenvx *env,bufx *buf,INT16 size,int if_num,int area);
   /* the message announce entry point, one per interface, to be handed down */
   /* by the clock synchronization at initialization */

typedef union
{
  struct inip
  {
    int		if_num;
    ci_fn	ci_announce_address;
    ci_efn	ci_announce_err_add;
  } inip;
  struct gtbp
  {
    int		if_num;
    int		area;
    INT16	size;
    bufx	**buf;
  } gtbp;
  struct gtep
  {
    int		if_num;
    int		area;
    int		mode;
    tenvx	**tenv;
  } gtep;
  struct rtbp
  {
    bufx	*buf;
  } rtbp;
  struct rtep
  {
    tenvx	*tenv;
  } rtep;
  struct rrbp
  {
    bufx	*buf;
  } rrbp;
  struct rrep
  {
    renvx	*renv;
  } rrep;
  struct sendp
  {
    int		if_num;
    int		area;
    int		mode;
    INT16	size;
    tenvx	*tenv;
    bufx	*buf;
  } sendp;
  struct conp
  {
    int		if_num;
    bufx	*buf;
    int		size;
    bufx	**nbuf;
    char	*memory;
  } conp;
} ciparam;

int ci_dispatcher(int fc,ciparam *param); /* calls the CI funtions */
#endif


