//
// ci.hpp
//
// declaration of the CI (Clock Interface) functions
//
// note: when porting to the hardware -> keep this file, but remove the
// corresponding ci.cpp file and link again with the appropriate file from
// the NTI driver; don't forget to implement the ci_announce() and
// ci_announce_error() functions
//
// created: 02.10.96
//
// last change: 30.10.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/06/16 17:11:31 $
//


#ifndef _CI_HPP_
#define _CI_HPP_


#include "ci.h"  /* constants and structures */

/*

//--------------------------------------------------------------------------
// CI function dispatcher
//--------------------------------------------------------------------------

extern int ci_dispatcher(int fc, ciparam *param);
// return values:
// 0               ... everything okay
// INVALID_COMMAND ... invalid function code
// INVALID_POINTER ... pointer for arguments (param) was NULL


//--------------------------------------------------------------------------
// IFC initialization
//--------------------------------------------------------------------------

extern int ci_init (int if_num, ci_fn ci_ann_addr, ci_efn ci_ann_err_addr);
// return values:
// 0                 ... everything okay
// INVALID_IF_NUMBER ... the ifc does not exist
// INVALID_POINTER   ... one of the function pointers was NULL


//--------------------------------------------------------------------------
// memory management
//--------------------------------------------------------------------------

extern int ci_get_tbuf (INT16 size, bufx **buf, int if_num, int area);
// return values:
// 0                 ... everything okay
// INVALID_IF_NUMBER ... ifc (or area) does not exist / is not initialized
// INVALID_POINTER   ... pointer for returning pointer (buf) was NULL
// NO_BUF_FREE       ... no buffer is available (probably all in use)

extern int ci_get_tenv (int mode, tenvx **tenv, int if_num, int area);
// return values:
// 0                 ... everything okay
// INVALID_IF_NUMBER ... ifc (or area) does not exist / is not initialized
// INVALID_POINTER   ... pointer for returning pointer (tenv) was NULL
// NO_ENV_FREE       ... no envelope is available (probably all in use)

extern int ci_ret_tbuf (bufx *buf);
// note: you can return an rbuf with ci_ret_tbuf(), but never return a tbuf
// with ci_ret_rbuf()!
// return values:
// 0                   ... everything okay
// INVALID_BUF_ADDRESS ... address doesn't exist

extern int ci_ret_tenv (tenvx *tenv);
// return values:
// 0                   ... everything okay
// INVALID_ENV_ADDRESS ... address doesn't exist



//--------------------------------------------------------------------------
// message transmission
//--------------------------------------------------------------------------

extern int ci_send (int mode, tenvx *tenv, bufx *buf, INT16 size,
                     int if_num, int area);
// sends a message to the network with the appropriate if_num
// return values:
// 0                    ... everything okay
// INVALID_IF_NUMBER    ... ifc (or area) does not exist/is not initialized
// NO_ENVELOPE_ATTACHED ... pointer to the envelope was empty

*/

//--------------------------------------------------------------------------
// hopefully, I won't need the following functions
//--------------------------------------------------------------------------

/*
extern int ci_ret_rbuf (bufx *rbuf);
// return values:
// 0                   ... everything okay
// INVALID_BUF_ADDRESS ... address doesn't exist

extern int ci_ret_renv (renvx *renv);
// return values:
// 0                   ... everything okay
// INVALID_ENV_ADDRESS ... address doesn't exist

extern int ci_mcon (???);
// return values:
// 0                            ... everything okay
// COULD_NOT_BE_MADE_CONTIGUOUS ... there was not enough space left

extern void ci_announce_up (renvx *renv, bufx *buf, INT16 size, int if_num);
extern void ci_announce_error_up(tenvx *env,bufx *buf,INT16 size,int if_num,
                                 int area);
*/


//--------------------------------------------------------------------------
// the following functions are only needed when using the real driver
//--------------------------------------------------------------------------

/*
extern int ci_announce (renvx *renv, bufx *buf, INT16 size, int if_num);
// return values:
// 0                 ... everything okay
// IMMEDIATE_RELEASE ... return envelope and buffer at once

extern void ci_announce_error (tenvx *env, bufx *buf, INT16 size,
                               int if_num, int area);
*/


#endif

