//
// cloctemp.hpp
//
// a list of all clocks in the system, sorted by <nodeID, area>
//
// created: 26.10.96
//
// last change: 18.02.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _CLOCTEMP_HPP_
#define _CLOCTEMP_HPP_

#include "clockifc.hpp"
#include "nodetemp.hpp"  // for ACNodeList


//**************************************************************************
// class CClockList: this class stores all clocks in the system
//**************************************************************************

class CClockList : public ACNodeList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CClockList (INT32 aSize = ArrayAllocationSize) :
        ACNodeList (aSize) {};
    // allocate memory

    virtual ~CClockList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    CClockInterface* GetFirst ()
        { return (CClockInterface*) ACNodeList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    CClockInterface* GetLast ()
        { return (CClockInterface*) ACNodeList::GetLast(); }
    // get the last object or NULL if the array is empty

    CClockInterface* GetNext ()
        { return (CClockInterface*) ACNodeList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    CClockInterface* GetPrev ()
        { return (CClockInterface*) ACNodeList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    CClockInterface* Find (SINT32 nodeID, int area)
        { return (CClockInterface*) ACNodeList::Find (nodeID, area); }
    // returns the object with this index; if no such object exists,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    CClockInterface* GetCurrentObject ()
        { return (CClockInterface*) ACNodeList::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (void* object, SINT32 nodeID, int area)
        { return ACNodeList::Insert (object, nodeID, area); }
    // insert an object into the array; if the object does not exist in
    // the list, True is returned; else False is returned

    CClockInterface* Remove (SINT32 nodeID, int area)
        { return (CClockInterface*) ACNodeList::Remove (nodeID, area); }
    // remove an object from the array

    // remove all objects from the array
    void RemoveAll ()
    {
        while (!Empty())
        {
            GetLast();  // go to the last item in the array
            RemoveAtCurrent();  // remove the array entry
        } // while
    }

    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    EBool Delete (SINT32 nodeID, int area)
    {
        CClockInterface* object;
        EBool found = False;

        object = Remove (nodeID, area);

        if (object != NULL)  // index does exist
        {
            delete object;  // delete all objects with this index
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        CClockInterface* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
};


#endif


