//
// cont_sw.hpp
//
// declaration of the synchronization controller (pure simulation)
//
// created: 25.01.97
//
// last change: 23.06.98
//
// RCS version control information:
//     $Revision: 1.4 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _CONT_SW_HPP_
#define _CONT_SW_HPP_

#include "contbase.hpp"


//**************************************************************************
// class CSoftwareController: this class controls all clock synchronization
//     applications and the network; for the network it is the link to
//     the simulation
//**************************************************************************

class CSoftwareController : public ACController
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CSoftwareController (char qname[4], ACSocketAddress *addr);
    // call the queue "CTRL" or something like that; *don't* use a string
    // of (hex) numbers!

    virtual ~CSoftwareController();  // deletes the simulation objects


    //----------------------------------------------------------------------
    //--- reset
    //----------------------------------------------------------------------

    virtual void Reset();
    // reset the controller (delete all simulation objects)

    virtual EBool ResetTheSimulation();
    // reset the simulation (set all parameters in all applications and
    // networks to their default values); returns True if at least one
    // object exists, False if no simulation objects exist


    //----------------------------------------------------------------------
    //--- configuration of system parameters
    //----------------------------------------------------------------------

    virtual EBool SetParameter (const SParameter& param);
    // sets the given parameter to its new value; if the parameter does
    // not exist or if the value is not valid, False is returned

    virtual EBool GetParameter (SParameter& param) const;
    // returns the value of the given parameter in the appropriate field of
    // SParameter; if the parameter does not exist, False is returned (the
    // parameter fields may nevertheless be changed!)


    //----------------------------------------------------------------------
    //--- execution of system commands
    //----------------------------------------------------------------------

    virtual EBool ExecuteCommand (const SCommand& command);
    // executes the command; returns True if successfull, otherwise False
    // existing commands: all Controller-Commands of ECommandType in
    // params.hpp


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- starting/stopping applications
    //----------------------------------------------------------------------

    virtual EBool StartApplication (SINT32 nodeID, int area);
    // activates the supervisor process
    // assert: the supervisor must exist

    virtual EBool StopApplication (SINT32 nodeID, int area);
    // cancels the supervisor process, terminates it
    // assert: the supervisor must exist


    //----------------------------------------------------------------------
    //--- stop program execution
    //----------------------------------------------------------------------

    virtual void DeleteAllSimulationObjects ();
    // empty clock list, delete network list, call base class function

    virtual void StopExecution();
    // deschedule the process (the function will not return)


private:

    SINT32 networkID_;  // first free ID (for new networks)


    //----------------------------------------------------------------------
    //--- reaction to network commands
    //----------------------------------------------------------------------

    EBool ExecuteNetworkCommand (SNetworkCommand& command);
    // find out the network and call its ExecuteCommand() function

    EBool CreateNewNetwork (SNetworkCreateCommand& command);
    // create a new network, return the networkID to EvalSys
    // if the network type is not installed, False is returned

    EBool DeleteANetwork (SNetworkCommand& command);
    // if the network exists, it is deleted and True is returned; else
    // False is returned
};

#endif

