//
// crashing.hpp
//
// declaration of a class that can crash and recover
//
// created: 11.08.97
//
// last change: 12.08.97
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _CRASHING_HPP_
#define _CRASHING_HPP_

#include "bool.hpp"


struct SCommand;


//**************************************************************************
// class CCrashingModule: can crash and recover
//**************************************************************************

class CCrashingModule
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CCrashingModule ();  // calls Reset() to set members to default values


    //----------------------------------------------------------------------
    //--- reset all members to their initial values
    //----------------------------------------------------------------------

    void Reset ();
    // sets/restores the initial state


    //----------------------------------------------------------------------
    //--- control the system parameters
    //----------------------------------------------------------------------

    EBool SetProbability (EBool probCrash, double prob);
    // if 'probCrash' is True, member 'probCrash_' is changed, else member
    // 'probRecover_'; the function returns True if 'prob' is in [0, 1],
    // and in that case the member is set to 'prob'

    double GetProbability (EBool probCrash) const;
    // if 'probCrash' is True, the value of member 'probCrash_' is returned,
    // else the value of member 'probRecover_' is returned

    EBool Crashed () const;
    // returns the current crash status of the module
    // context: ISR


    //----------------------------------------------------------------------
    //--- execution of system commands
    //----------------------------------------------------------------------

    virtual EBool ExecuteCommand (const SCommand& command);
    // executes the command; returns True if successfull, otherwise False
    // existing commands: CrashModule, RecoverModule


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- control module crashes
    //----------------------------------------------------------------------

    virtual void Crash ();
    // crash the module; if the module was already crashed, this method has
    // no effect

    virtual void Recover ();
    // recover the module; if the module wasn't crashed, this method has no
    // effect


    //----------------------------------------------------------------------
    //--- fault injection
    //----------------------------------------------------------------------

    virtual EBool ModifyCrashStatus ();
    // may change the crash status of the module (according to the
    // probabilities); returns True if the status was changed
    // context: ISR

    virtual void ReportCrashChange();
    // empty; derived modules should send an event with type
    // 'CrashStatusChange' to the evaluation


private:

    EBool crashed_;  // the crash status of the module
    double probCrash_;  // probability of crashing
    double probRecover_;  // probability of recovering
};

#endif


