//
// ctrled.hpp
//
// declaration of the base class used for all modules that are controlled
// by some other module
//
// created: 13.01.97
//
// last change: 18.08.97
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _CTRLED_HPP_
#define _CTRLED_HPP_

#include "structs.hpp"


class CControllingObject;


//**************************************************************************
// class CControlledObject: this class stores the link to the controlling
//      object and provides a function for reporting events; in addition,
//      it declares some functions that all controlled modules need (ie.
//      parameter and command handling functions)
//**************************************************************************

class CControlledObject
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CControlledObject (CControllingObject *ctrl);
    virtual ~CControlledObject ();


    //----------------------------------------------------------------------
    //--- reset
    //----------------------------------------------------------------------

    virtual void Reset ();
    // calls Dispose() and Init()


    //----------------------------------------------------------------------
    //--- get processID of controlling object (= nodeID for CSAs)
    //----------------------------------------------------------------------

    SINT32 GetNodeID();
    // returns the process ID of the controlling object


    //----------------------------------------------------------------------
    //--- configuration of system parameters
    //----------------------------------------------------------------------

    virtual EBool SetParameter (const SParameter& param);
    // always returns False

    virtual EBool GetParameter (SParameter& param) const;
    // always returns False


    //----------------------------------------------------------------------
    //--- execution of system commands
    //----------------------------------------------------------------------

    virtual EBool ExecuteCommand (const SCommand& command);
    // returns True if the command was executed, else False
    // known commands: ResetModule


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- event reports
    //----------------------------------------------------------------------

    virtual void ReportEvent (EModule module, EEvent eventType);
    // report an event to the controlling object

    virtual void ReportEvent (SEvent *event);
    // pass the event to the controlling object


    //----------------------------------------------------------------------
    //--- create/destroy the members that are modified during reset
    //----------------------------------------------------------------------

    virtual void Init ();
    // creates and/or initializes the members that are modified during
    // reset; empty in this base class

    virtual void Dispose ();
    // deletes all dynamic members; empty in this base class


private:

    CControllingObject *ctrl_;  // the controlling object
};


//**************************************************************************
// class CApplControlledObject: like CControlledObject, but uses only
//    SApplicationEvent's; use it as a base for all application specific
//    modules
//**************************************************************************

class CApplControlledObject : public CControlledObject
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CApplControlledObject (CControllingObject *ctrl);


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- event reports
    //----------------------------------------------------------------------

    virtual void ReportEvent (EModule module, EEvent eventType);
    // report an event to the controlling object

    virtual void ReportEvent (SEvent *event);
    // pass the event to the controlling object
};


#endif


