//
// dmsgtemp.hpp
//
// contains a list class for storing send/receive messages in the Lance
//
// created: 26.08.97
//
// last change: 14.04.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _DMSGTEMP_HPP_
#define _DMSGTEMP_HPP_

#include "duplist.hpp"  // for ACSortedArray


struct SDelayMcb
{
    EBool send; // True if sendData is valid, False if receiveData is valid
    double deliveryTime;  // time of delivery
};

struct SSendDelayMcb : public SDelayMcb
{
    int mode;       // mode for ci_send()
    tenvx *tenv;    // local copy of tenv, *not* the driver envelope
    char *msg;      // msg buffer (is copied into the bufx structure)
    INT16 msgsize;  // size of the msg buffer
    int if_num;     // network IFC number
    int area;       // sender area
};

struct SRecvDelayMcb : public SDelayMcb
{
    renvx *renv;  // local copy of renv, *not* the driver envelope
    bufx *buf;    // local copy of buf, again *not* the driver buffer
    INT16 size;   // size of the buf
    int if_num;   // network IFC number
};


//**************************************************************************
// class CDelayList: this class stores the messages that are to be
//    sent or delivered but are delayed
//**************************************************************************

class CDelayList : public ACSortedArray
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CDelayList (INT32 aSize = ArrayAllocationSize) :
        ACSortedArray (True, aSize) {};
    // allocate memory, duplicates are allowed

    virtual ~CDelayList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    SDelayMcb* GetFirst ()
        { return (SDelayMcb*) ACSortedArray::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    SDelayMcb* GetLast ()
        { return (SDelayMcb*) ACSortedArray::GetLast(); }
    // get the last object or NULL if the array is empty

    SDelayMcb* GetNext ()
        { return (SDelayMcb*) ACSortedArray::GetNext(); }
    // get the next object in the array or NULL if the end of the array
    // has been reached

    SDelayMcb* GetPrev ()
        { return (SDelayMcb*) ACSortedArray::GetPrev(); }
    // return the previous object in the array or NULL if the beginning
    // of the array has been reached

    SDelayMcb* Find (double index)
        { return (SDelayMcb*) ACSortedArray::Find (index); }
    // returns the object with this index; if no such object exists,
    // NULL is returned

    SDelayMcb* FindNext ()
        { return (SDelayMcb*) ACSortedArray::FindNext (); }
    // returns the next object with the same index as the last accessed
    // object; if no more objects with this index exist, or if the last
    // access is not stored anymore, NULL is returned and the function has
    // no effect

    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    SDelayMcb* GetCurrentObject ()
        { return (SDelayMcb*) ACSortedArray::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- remove objects
    //----------------------------------------------------------------------

    SDelayMcb* Remove (double index)
        { return (SDelayMcb*) ACSortedArray::Remove (index); }
    // remove an object from the array


    // remove the first object from the array; this function has been added
    // and needs to be tested!
    SDelayMcb* RemoveFirst ()
    {
        SDelayMcb *object;

        object = GetFirst();  // look at the first object
        if (object == NULL) return NULL;  // array is empty
        else
        {
            return (SDelayMcb*) ACSortedArray::Remove (object->deliveryTime);
        }
    }


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, all objects with this index
    // are deleted and True is returned; otherwise False is returned
    EBool Delete (double index)
    {
        SDelayMcb* object;
        EBool found = False;

        object = Remove (index);

        if (object != NULL)  // index does exist
        {
            do {
                delete object;  // delete all objects with this index
                object = Remove (index);
            } while (object != NULL);
                
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        SDelayMcb* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
};


#endif


