//
// duptemp.hpp
//
// this is a "template" for a derived class of ACSortedArray (cf. duplist.*);
// since the Microtek Compiler does not support templates, I have to use
// separate classes for all list objects
// follow the instructions below on how to obtain a working class from
// the skeleton that is provided in this file
//
// step 1:  copy this file (Save As ...)
// step 2:  replace _DUPTEMP_HPP_ with the new file name
// step 3:  change the class name "TCSortedArray" to the new class name
// step 4:  change the template type T* to the type of the objects that are
//          stored in the list
//          example: if the objects are of Type CFoo, replace T* with CFoo*
//
//
// created: 21.05.97
//
// last change: 28.08.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _DUPTEMP_HPP_
#define _DUPTEMP_HPP_


#include "duplist.hpp"


//**************************************************************************
// class TCSortedArray: this class is a template used to access the abstract
//     class ACSortedArray without having to cast - (c) gg
//     it can delete the data objects
//**************************************************************************

// since the Microtek Compiler does not support templates, I have to use
// separate classes for all list objects
// but still this class can be used as a template for those classes

class TCSortedArray : public ACSortedArray
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    TCSortedArray (EBool duplicates = False,
                  INT32 aSize = ArrayAllocationSize) :
        ACSortedArray (duplicates, aSize) {};
    // allocate memory

    TCSortedArray (INT32 aSize) : ACSortedArray (False, aSize) {};
    // allocate memory

    virtual ~TCSortedArray () { DeleteAll(); }
    // the destructor deletes all objects in the array; since the base
    // destructor is called although it is pure virtual, destruction of
    // the _array itself is done there


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    T* GetFirst () { return (T*) ACSortedArray::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    T* GetLast () { return (T*) ACSortedArray::GetLast(); }
    // get the last object or NULL if the array is empty

    T* GetNext () { return (T*) ACSortedArray::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    T* GetPrev () { return (T*) ACSortedArray::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    T* Find (double index) { return (T*) ACSortedArray::Find (index); }
    // returns the object with this index; if no such object exists,
    // NULL is returned and _current is the index where the object should
    // be inserted

    T* FindNext () { return (T*) ACSortedArray::FindNext (); }
    // returns the next object with the same index as the _current index;
    // if no more objects with this index exist, NULL is returned and
    // the function has no effect

    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    T* GetCurrentObject () { return (T*) ACSortedArray::GetCurrentObject();}
    // returns a pointer to the member that is currently being accessed or
    // NULL, if no element is being accessed


    //----------------------------------------------------------------------
    //--- remove objects
    //----------------------------------------------------------------------

    T* Remove (double index) { return (T*) ACSortedArray::Remove (index); }
    // remove an object from the array



    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, all objects with this index
    // are deleted and True is returned; otherwise False is returned
    // current points to the position where the element was deleted (or
    // should have be inserted), or to the last element if the deleted
    // object was (would have been) the last
    EBool Delete (double index)
    {
        T* object;
        EBool found = False;

        object = Remove (index);

        if (object != NULL)  // index does exist
        {
            do {
                delete object;  // delete all objects with this index
                object = Remove (index);
            } while (object != NULL);
                
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        T* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
    // note: _array now points to an empty array of size _allocSize
};


#endif

