//
// event.hpp
//
// for the event reports to the evaluation
//
// created: 07.12.96
//
// last change: 22.01.99
//
// RCS version control information:
//     $Revision: 1.5 $$State: Exp $$Date: 1999/01/26 14:14:23 $
//


#ifndef _EVENT_HPP_
#define _EVENT_HPP_


//**************************************************************************
// for event logging
//**************************************************************************

// possible events; each may be turned on/off
// note: if new events are added, don't forget to set their class in file
// ctrling.cpp, function InitEventTable()
enum EEvent
{
    // Clock
    ClockUpdate = 0,  // clock is set to a new value
    ClockPreload,  // clock is preloaded to a new value
    Snapshot,  // data from a HW snapshot
    GpsEvent,  // GPS clock value
    ClockReadFault,  // faulty read access to clock
    ClockWriteFault,  // faulty write access to clock
    IgnoreSetting,  // CSA tried to set a register, but was ignored (crash)

    // Network
    MessageLost,  // message was thrown away
    DeliverMessage,  // message is delivered to the Lance
    EarlyMessage,  // early message was transmitted
    LateMessage,  // late message was transmitted

    // Network, Lance, Clock
    CrashStatusChange,  // crash status of module has changed

    // Network, ClockSync
    SendMessage,  // CS: message was sent; Ne: msg is transmitted

    // Lance
    DiscardSendMessage,  // outgoing message was discarded
    DiscardRecvMessage,  // incoming message was discarded
    ScrambleSendMessage,  // outgoing message was scrambled
    ScrambleRecvMessage,  // incoming message was scrambled
    DuplicateSendMessage,  // outgoing message was duplicated
    DuplicateRecvMessage,  // incoming message was duplicated

    // ClockSync
    ChangeAssociation,  // associated network was changed
    ConvergenceFailed,  // CSA could not compute a new clock interval
    DelayEstimation,  // estimated transmission delay data
    IntervalEvent,  // an input interval to the CV, or its result
    RateEvent,  // input data to the rate algorithm, or its result
    ReceivedMessage,  // CS::Announce() was called
    TimerEvent,  // duty timer interrupt (called from CS::isr())
    TransmissionDelay,  // real transmission delay (HW-based sim only)

    UserEvent,  // user defined event from the synchronization algorithm

    // internal administration
    EventCount  // this entry has to stay last even if new items are added!
};

// event category; groups different events (p.e. fault injection in the
// Lance or the Network)
// again, each category may be turned on/off
enum EEventClass
{
    NoClassification=0,  // to mark events that have no associated class

    SystemReport,  // report normal system actions (like clock updates etc.)
    InterruptEvent,  // report interrupts
    FaultInjection  // report injected faults
};


#endif

