//
// globals.hpp
//
// declaration of global items that are needed both in pure and
// hardware-based simulation
//
// note: for adding new algorithm types, see the comments to function
//       InstallAlgorithms() in file globals.cpp
//
// created: 26.10.96
//
// last change: 29.06.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/06/29 13:36:58 $
//


#ifndef _GLOBALS_HPP_
#define _GLOBALS_HPP_


#include "classman.hpp"
#include "suptemp.hpp"


class CClock;


//**************************************************************************
// global variables
//**************************************************************************

extern CAlgorithmManager algorithmManager;
// the algorithm manager that returns the creation function of a certain
// clock synchronization algorithm

extern CClock *g_masterClock;
// the master clock; it is initialized to NULL
// the hardware controller should set it in its ctor

extern int g_masterArea;
// the area of the master clock; it is initialized to NoAccount
// the hardware controller should set it in its ctor to a value within
// [0, NumberOfAreas-1]



//**************************************************************************
// global object lists
//**************************************************************************

extern CSupervisorList listOfSupervisors;
// list of all supervisors in the system



//**************************************************************************
// initialization functions
//**************************************************************************

extern void CheckAssumptions();
// checks all system assumptions (like the existance of globals)
// the function must be called in main() before creating the simulation
// controller

extern void InstallAlgorithms();
// installs all clock synchronization algorithms known in the system
// the function must be called in main() before creating the simulation
// controller



//**************************************************************************
// tidy up functions
//**************************************************************************

extern void DeleteTemplates();
// deletes the template objects
// the function is called by ACController


#endif

