//
// gps_sw.hpp
//
// declaration of the GPS class (for pure simulation)
//
// created: 29.03.97
//
// last change: 01.08.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _GPS_SW_HPP_
#define _GPS_SW_HPP_


#include "clockcfg.hpp"
#include "gpsbase.hpp"


//**************************************************************************
// class CSoftwareGps: the GPS time is taken from the global time
//**************************************************************************

class CSoftwareGps : public ACGps
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CSoftwareGps (CClockInterface *clock, int area);
    // assert: clock != NULL (through the ctor of ACGps)

    virtual ~CSoftwareGps ();
    // disable all GPS interrupts that are stored in the 'enableMask'


    //----------------------------------------------------------------------
    //--- enabling/disabling interrupts
    //----------------------------------------------------------------------

    virtual void EnableGps (EGpsUnit gpu);
    // enables the GPS time interrupt of the GPS interrupt process
    // calls SetGpsStatus() to set the 'gpsStatus' of clock to False

    virtual void DisableGps (EGpsUnit gpu);
    // disables the GPS time interrupt of the GPS interrupt process
    // clears the status bit in 'statusMask'


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- handling GPS time
    //----------------------------------------------------------------------

    virtual void GetGpsTime (CNtpTime& time);
    // obtain the NTP time of the GPS receiver that has triggered the
    // interrupt

    virtual EGpsUnit GetGpsNumber();
    // return the identifier of the GPU to which the receiver that has
    // triggered the interrupt is connected

    virtual void ClearGpsInterrupt();
    // clear the interrupt that was triggered by the end of the GPS time
    // information
};


#endif


