//
// int128.hpp
// 
// definition of an INT128 class based on GNU class Integer
//
// created: 06.11.97
//
// last change: 29.07.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/29 16:34:30 $
//


#ifndef _INT128_HPP_
#define _INT128_HPP_

#include <stdio.h>    // for printf()
#include "Integer.h"  // for class Integer and Ialloc()

#include "psos.h"     // for INT32 and INT8


//**************************************************************************
// class INT128
//**************************************************************************

class INT128 : public Integer
{
public:

    //----------------------------------------------------------------------
    // constructor
    //----------------------------------------------------------------------

    INT128() {}
    // initializes the value with zero

    // the parameters are in the UTCSU format (see SetNtp())
    INT128(INT32 ms, INT32 ts, INT32 us, INT8 ns)
    {
        SetNtp (ms, ts, us, ns);
    }


    //----------------------------------------------------------------------
    // set/get the value directly
    //----------------------------------------------------------------------

    // the function is needed in clock_sw.cpp
    void SetBinary (INT32 a3, INT32 a2, INT32 a1, INT32 a0);
    // a0 is the lowest INT32, a3 the highest

    void GetBinary (INT32& a3, INT32& a2, INT32& a1, INT32& a0);
    // a0 is the lowest INT32, a3 the highest


    //----------------------------------------------------------------------
    // set/get the value in UTCSU format
    //----------------------------------------------------------------------

    void SetNtp (INT32 ms, INT32 ts, INT32 us, INT8 ns);
    // the parameters are in the UTCSU format; that is, only the lowest
    // three bits of the 'ns' are considered valid, and the fourth bit
    // of the internal value is the lsb of 'us'
    // the function cuts off the 'ms' at NTP bit +32 (that is, uses only
    // the first three bytes of the 'ms')

    void GetNtp (INT32& ms, INT32& ts, INT32& us, INT8& ns, INT8& overflow);
    // reads the value into the ms, ts, us and ns parameters; cuts off
    // at NTP bit +32, but sets 'overflow' to 1 if the internal number
    // is larger (if it is smaller, 'overflow' is set to 0)


    //----------------------------------------------------------------------
    // necessary operators
    //----------------------------------------------------------------------

    // assign the Integer
    INT128& operator= (const Integer& i)
    {
        if (&i == (Integer*)this) return *this;  // same object anyway
        *(Integer*)this = i;  // use the Integer::operator=
        return *this;
    }

    INT128& operator~ ();
    // take care: the Integer::operator~ is faulty! it inverts and
    // reverts(!) the highest two byte
    // note: this operator~ inverts in 16-bit steps; so if the number
    // stored is 0x1FFFF, then ~0x2FFFF = 0xFFFD0000 (this is a difference
    // to the Integer::operator~, where (if it were not for the error)
    // ~0x2FFFF would be 0x10000 (the error makes the result 0x20000)
};

#endif

