//
// intproc.hpp
//
// declaration of an interrupt process
// note: may become obsolete, since interrupt processes are only used in
//       the software clock, so CClockInterrupt may be merged with this
//       class
//
// created: 18.01.97
//
// last change: 30.03.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _INTPROC_HPP_
#define _INTPROC_HPP_

#include "process.hpp"


//**************************************************************************
// CInterruptProcess: a simple process generating an interrupt (calling
//    InvokeISR()) when it is activated and then either scheduling itself
//    for the next interrupt or cancelling itself
//**************************************************************************

class CInterruptProcess : public ACProcess
// PLEASE: *never* use any of the process scheduling functions that are
// provided by ACProcess; *only* use RaiseAfter() to schedule the process!
{
public:
    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CInterruptProcess ();
    // interrupt is disabled by default


    //----------------------------------------------------------------------
    //--- interrupt management
    //----------------------------------------------------------------------

    void SetVector (int vector);
    // sets the interrupt vector; *must* be called before the first
    // interrupt is generated

    void Enable ();
    // enable the interrupt; if it is not enabled at the time the interrupt
    // should be generated, the interrupt service routine will not be called
    // assert: interrupt vector has been set

    void Disable ();
    // disable the interrupt (till the next call to Enable())

    EBool Enabled ();
    // returns True if the interrupt is enabled, False if not

    EBool Active ();
    // returns True if the process is active, False if not


    //----------------------------------------------------------------------
    //--- create an interrupt
    //----------------------------------------------------------------------

    void RaiseAfter (double delay);
    // generate an interrupt in 'delay' seconds from now


    //----------------------------------------------------------------------
    //--- deschedule an interrupt
    //----------------------------------------------------------------------

    void CancelInterrupt ();
    // deschedules the interrupt (if it is scheduled) and sets waitTime to
    // NoAccount


    //----------------------------------------------------------------------
    //--- for the simulation scheduler only; do not use
    //----------------------------------------------------------------------

    virtual void Body ();
    // call the ISR (if enabled), maybe schedule for the next interrupt


protected:

    //----------------------------------------------------------------------
    //--- interrupt specific work before and after calling the ISR
    //----------------------------------------------------------------------

    virtual void PreExecution ();
    // is executed prior to calling the ISR; a derived class can overload
    // the function to implement some interrupt specific things
    // the function is always called, even if the interrupt is disabled

    virtual void PostExecution ();
    // is executed after calling the ISR
    // the function is always called, even if the interrupt is disabled


private:

    int vector_;  // the vector number of the interrupt
    EBool enabled_;  // interrupts are only generated if enabled is True
    double waitTime_;  // time until next interrupt
};


#endif


