//
// lantemp.hpp
//
// a list of Lances, sorted after the key <nodeID, area>
//
// derived from ACNodeList in nodetemp.hpp
//
// created: 26.10.96
//
// last change: 31.03.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _LANTEMP_HPP_
#define _LANTEMP_HPP_

#include "nodetemp.hpp"  // for ACNodeList
#include "lance.hpp"


//**************************************************************************
// class CLanceList: this class stores all Lances in the system
//**************************************************************************

class CLanceList : public ACNodeList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CLanceList (INT32 aSize = ArrayAllocationSize) :
        ACNodeList (aSize) {};
    // allocate memory

    virtual ~CLanceList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    CLance* GetFirst ()
        { return (CLance*) ACNodeList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    CLance* GetLast ()
        { return (CLance*) ACNodeList::GetLast(); }
    // get the last object or NULL if the array is empty

    CLance* GetNext ()
        { return (CLance*) ACNodeList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    CLance* GetPrev ()
        { return (CLance*) ACNodeList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    CLance* Find (SINT32 nodeID, int area)
        { return (CLance*) ACNodeList::Find (nodeID, area); }
    // returns the object with this index; if no such object exists,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    CLance* GetCurrentObject ()
        { return (CLance*) ACNodeList::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (void* object, SINT32 nodeID, int area)
        { return ACNodeList::Insert (object, nodeID, area); }
    // insert an object into the array; if the object does not exist in
    // the list, True is returned; else False is returned

    CLance* Remove (SINT32 nodeID, int area)
        { return (CLance*) ACNodeList::Remove (nodeID, area); }
    // remove an object from the array

    // remove all objects from the array (copied from cloctemp.hpp)
    void RemoveAll ()
    {
        while (!Empty())
        {
            GetLast();  // go to the last item in the array
            RemoveAtCurrent();  // remove the array entry
        } // while
    }

    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    EBool Delete (SINT32 nodeID, int area)
    {
        CLance* object;
        EBool found = False;

        object = Remove (nodeID, area);

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object with this index
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        CLance* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
};


#endif


