//
// listtemp.hpp
//
// this is a "template" for a derived class of ACObjectList (cf. listbase.*);
// since the Microtek Compiler does not support templates, I have to use
// separate classes for all list objects
// follow the instructions below on how to obtain a working class from
// the skeleton that is provided in this file
//
// note: don't call an argument 'key', since this is already in use
//
// step 1:  copy this file (Save As ...)
// step 2:  replace _LISTTEMP_HPP_ with the new file name
// step 3:  change the class name "CTemplate" to the new class name
// step 4:  change the template type T* to the type of the objects that are
//          stored in the list
//          example: if the objects are of Type CFoo, replace T* with CFoo*
// step 5:  SKey should contain all keys -> replace the label "KEY_MEMBERS"
//          with a definition of all keys
//          example: double key1_; int key2_;
// step 6:  replace the label "FUNCTION_ARGUMENT_LIST" with the appropriate
//          argument list
//          example: double key1, int key2
// step 7:  replace "CONSTRUCTOR_BODY" with assignments; the variables of
//          FUNCTION_ARGUMENT_LIST (step 6) are assigned to the appropriate
//          variables of KEY_MEMBERS (step 5)
//          example: key1_ = key1; key2_ = key2;
// step 8:  replace "FCT_ARGUMENT_LIST_REFERENCES" with the same argument
//          list as in step 6, but use references (don't forget the escape
//          characters before the & when using the Watcom Editor)
//          example: double\& key1, int\& key2
// step 9:  replace "ARGUMENTS" with a list of the variables of step 6
//          example: key1, key2
// step 10: replace "ASSIGN_KEY" with assignments; the variables of step 5
//          (KEY_MEMBERS) are assigned to the appropriate variables of
//          step 6 (FUNCTION_ARGUMENT_LIST); assume that the structure
//          variable has the name "key"
//          example: key1 = key->key1_; key2 = key->key2_;
// step 11: implement the function Compare()
//
//
// created: 28.10.96
//
// last change: 31.03.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _LISTTEMP_HPP_
#define _LISTTEMP_HPP_

#include "listbase.hpp"


//**************************************************************************
// class CTemplate: this class is a template used to access the abstract
//     class ACObjectList without having to cast - (c) gg
//     it can delete the data objects
//**************************************************************************

class CTemplate : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CTemplate (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {}
    // allocate memory

    virtual ~CTemplate () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    T* GetFirst () { return (T*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    T* GetLast () { return (T*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    T* GetNext () { return (T*) ACOjjectList::GetNext(); }
    // get the next object in the array or NULL if the end of the array
    // has been reached

    T* GetPrev () { return (T*) ACObjectList::GetPrev(); }
    // return the previous object in the array or NULL if the beginning
    // of the array has been reached

    T* Find (FUNCTION_ARGUMENT_LIST) // put the keys here
    {
        SKey key (ARGUMENTS);
        return (T*) ACObjectList::Find (&key);
    }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    T* GetCurrentObject ()
        { return (T*) ACObjectList::GetCurrentObject(); }
    // returns a pointer to the member that is currently being accessed or
    // NULL, if no element is being accessed

    EBool GetCurrentObjectKey (FCT_ARGUMENT_LIST_REFERENCES)
    // references to the key variables
    {
        SKey *key = (SKey*) ACObjectList::GetCurrentObjectKey();

        if (key != NULL)
        {
            ASSIGN_KEY
            // set the argument variables
            return True;
        }
        else return False;
    }
    // returns True if a member is accessed (in that case the arguments will
    // contain the key), or False if no element is currently accessed


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (T* object, FUNCTION_ARGUMENT_LIST)
    {
        SKey *key = new SKey (ARGUMENTS);

        if (!ACObjectList::Insert (object, key))  // object not inserted
        {
            delete key;  // the key is not needed
            return False;
        }
        else return True;
    }

    T* Remove (FUNCTION_ARGUMENT_LIST)
    {
        SKey key (ARGUMENTS);
        return (T*) ACObjectList::Remove (&key);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned



    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    // current points to the position where the element was deleted (or
    // should have be inserted), or to the last element if the deleted
    // object was (would have been) the last
    EBool Delete (FUNCTION_ARGUMENT_LIST)
    {
        T* object;
        EBool found = False;

        object = Remove (ARGUMENTS);  // remove the object

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        T* object;
        
        if (Empty()) return False;

        while (!Empty())
        {
            object = GetLast();  // get the last item in the array
            RemoveAtCurrent();  // remove the array entry
            delete object;  // delete the object
        }

        return True;
    } // DeleteAll


protected:

    // this function is used to compare keys; return -1 if key1<key2, 0 if
    // key1==key2, and 1 if key1>key2
    virtual int Compare (void *key1, void *key2) const;


    virtual void DeleteKey (void *key) { delete (SKey*) key; }
    // delete the key


private:
    struct SKey  // this structure contains the key(s) for sorting
    {
        // all key elements are put here
        KEY_MEMBERS

        SKey (FUNCTION_ARGUMENT_LIST) {CONSTRUCTOR_BODY}
        // initialize the members
    };
};

#endif

