//
// msgform.hpp
//
// contains the message structures for the clock synchronization
// messages
//
// created: 25.06.97
//
// last change: 08.10.98
//
// RCS version control information:
//     $Revision: 1.3 $$State: Exp $$Date: 1998/10/08 17:09:37 $
//


#ifndef _MSGFORM_HPP_
#define _MSGFORM_HPP_


#include "ci.h"

#include "gentypes.hpp"  // for TRateTheta

// this structure contains the data that is sent (during FME) in bufx
struct SSendData
{
    INT32 mode;  // message type
    SINT32 nodeID;  // ID of the sender node
};


// this structure contains the data that is sent (by ReplyToMessage) in
// bufx
struct SReplyData : public SSendData
{
    INT32 msReceive;  // time of reception of the original message
    INT32 tsReceive;
    TRateTheta thetaMinus;  // the current rate bounds of the clock
    TRateTheta thetaPlus;
};


// this structure contains the information that is added by the Lance
// note: all entries *must* have the type INT32
// if the structure is changed, change the order of the elements of
// SMsgPosition::EItems as well!
struct STimeData
{
    INT32 ms;
    INT32 ts;
    INT32 acc;
    INT32 area;
};


// this structure is used for obtaining the position and length of the
// items (clock value and area of the sender) that are automatically
// inserted into the message buffer when the message is transmitted
// the order of the entries in 'EItems' must correspond with STimeData
struct SMsgPosition
{
    enum EItems {Macrostamp=0, Timestamp, Accuracy, Area, EItemCount};
    // this enumeration should contain all items that are added to the
    // original message by the network
    // note: the last entry is for accounting purposes and should stay
    // last even if new items are added

    INT32 position[EItemCount];  // the position of the items
    INT32 length[EItemCount];  // the length of the items
};


#endif


