//
// msgtemp.hpp
//
// access classes to the ACSortedArray; necessary for CDelayer and
// CMessenger;
// the classes are obtained by copying the template class TCSortedArray
// of duptemp.hpp and replacing 'T' with the appropriate type that is
// stored in the list; TCSortedArray was tested using Watcom 10.5, so the
// access classes are assumed to be correct as long as they are not changed
//
// created: 12.10.96
//
// last change: 11.08.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _MSGTEMP_HPP_
#define _MSGTEMP_HPP_

#include "duplist.hpp"  // for ACSortedArray
#include "sim_gen.hpp"


struct SMcb
{
    SMsg *msg;
    double deliveryTime;
    ~SMcb() { delete msg; }
};
// this structure is needed for (internal) scheduling of the messages


//**************************************************************************
// class CMessageList: this class stores the messages that are to be
//    sent or delivered
//**************************************************************************

class CMessageList : public ACSortedArray
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CMessageList (INT32 aSize = ArrayAllocationSize) :
        ACSortedArray (True, aSize) {};
    // allocate memory, duplicates are allowed

    virtual ~CMessageList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    SMcb* GetFirst ()
        { return (SMcb*) ACSortedArray::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    SMcb* GetLast ()
        { return (SMcb*) ACSortedArray::GetLast(); }
    // get the last object or NULL if the array is empty

    SMcb* GetNext ()
        { return (SMcb*) ACSortedArray::GetNext(); }
    // get the next object in the array or NULL if the end of the array
    // has been reached

    SMcb* GetPrev ()
        { return (SMcb*) ACSortedArray::GetPrev(); }
    // return the previous object in the array or NULL if the beginning
    // of the array has been reached

    SMcb* Find (double index)
        { return (SMcb*) ACSortedArray::Find (index); }
    // returns the object with this index; if no such object exists,
    // NULL is returned

    SMcb* FindNext ()
        { return (SMcb*) ACSortedArray::FindNext (); }
    // returns the next object with the same index as the last accessed
    // object; if no more objects with this index exist, or if the last
    // access is not stored anymore, NULL is returned and the function has
    // no effect

    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    SMcb* GetCurrentObject ()
        { return (SMcb*) ACSortedArray::GetCurrentObject();}
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- remove objects
    //----------------------------------------------------------------------

    SMcb* Remove (double index)
        { return (SMcb*) ACSortedArray::Remove (index); }
    // remove an object from the array


    // remove the first object from the array; this function has been added
    // and needs to be tested!
    SMcb* RemoveFirst ()
    {
        SMcb *object;

        object = GetFirst();  // look at the first object
        if (object == NULL) return NULL;  // array is empty
        else
        {
            return (SMcb*) ACSortedArray::Remove (object->deliveryTime);
        }
    }


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, all objects with this index
    // are deleted and True is returned; otherwise False is returned
    EBool Delete (double index)
    {
        SMcb* object;
        EBool found = False;

        object = Remove (index);

        if (object != NULL)  // index does exist
        {
            do {
                delete object;  // delete all objects with this index
                object = Remove (index);
            } while (object != NULL);
                
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        SMcb* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
};


#endif


