//
// netproc.hpp
//
// declaration of the base class of network processes that are used to
// delay messages at the sender/receiver 
//
// created: 09.10.96
//
// last change: 25.08.97
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _NETPROC_HPP_
#define _NETPROC_HPP_

#include "bool.hpp"
#include "process.hpp"


class CMessageList;

struct SMcb;
struct SMsg;


//**************************************************************************
// class ACNetworkProcess: this class is a process class and is needed as
//     a base class for CDelayer and CMessenger; it can store a message
//     together with a simulation time and reschedule itself for the
//     earliest of these times
//**************************************************************************


class ACNetworkProcess : public ACProcess
{
public:
    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    ACNetworkProcess();
    virtual ~ACNetworkProcess();


    //----------------------------------------------------------------------
    //--- reset
    //----------------------------------------------------------------------

    void Reset();
    // discards all messages, suspends process if necessary


    //----------------------------------------------------------------------
    //--- message scheduling
    //----------------------------------------------------------------------

    void Insert (SMsg *msg, double afterTime);
    // inserts the message into its internal message queue
    // assert: msg != NULL


    //----------------------------------------------------------------------
    //--- go through the list of messages
    //----------------------------------------------------------------------

    SMsg *GetFirst();
    // returns the first message in the list or NULL if the list is empty

    SMsg *GetNext();
    // returns the next message in the list or NULL if the list is empty
    // or if the last element has already been read


    //----------------------------------------------------------------------
    //--- the main function of the process
    //----------------------------------------------------------------------

    virtual void Body();
    // calls the Deliver() function for the current msg


protected:

    CMessageList *listOfMessages;
    // the list of messages that are waiting

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- init, dispose
    //----------------------------------------------------------------------

    virtual void Init();
    // initializes all class members to default values; a second call
    // (without Dispose() in between) has no effect

    virtual void Dispose();
    // deletes all dynamic members; a second call (without Init() in
    // between) has no effect


    //----------------------------------------------------------------------
    //--- process state
    //----------------------------------------------------------------------

    EBool Active();
    // returns True if the process is active, otherwise False


    //----------------------------------------------------------------------
    //--- message scheduling
    //----------------------------------------------------------------------

    virtual void Deliver (SMsg *msg) = 0;
    // do whatever has to be done with the message when it is active


private:

    SMcb *msg_;  // the next message that has to be delivered

    EBool initCalled_;  // set to True by Init(), and to False by Dispose()
};


#endif

