//
// nettemp.hpp
//
// stores networks of type ACNetwork
//
// created: 12.10.96
//
// last change: 10.06.96
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _NETTEMP_HPP_
#define _NETTEMP_HPP_

#include "listbase.hpp"
#include "netbase.hpp"


//**************************************************************************
// class CNetworkList: this class stores all networks in the system
//**************************************************************************

class CNetworkList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CNetworkList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {};
    // allocate memory

    virtual ~CNetworkList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    ACNetwork* GetFirst ()
        { return (ACNetwork*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    ACNetwork* GetLast ()
        { return (ACNetwork*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    ACNetwork* GetNext ()
        { return (ACNetwork*) ACObjectList::GetNext(); }
    // get the next object in the array or NULL if the end of the array
    // has been reached

    ACNetwork* GetPrev ()
        { return (ACNetwork*) ACObjectList::GetPrev(); }
    // return the previous object in the array or NULL if the beginning
    // of the array has been reached

    ACNetwork* Find (SINT32 networkID)
        { return (ACNetwork*) ACObjectList::Find (&networkID); }
    // returns the object with this networkID; if no such object exists,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    ACNetwork* GetCurrentObject ()
        { return (ACNetwork*) ACObjectList::GetCurrentObject(); }
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (ACNetwork* object, SINT32 networkID)
    {
        SINT32 *key = new SINT32 (networkID);
        EBool inserted;

        inserted = ACObjectList::Insert (object, key);

        if (!inserted)  // the object was not inserted
        {
            delete key;  // the key is not needed
        }

        return inserted;
    }

    ACNetwork* Remove (SINT32 networkID)
        { return (ACNetwork*) ACObjectList::Remove (&networkID); }
    // remove an object from the array



    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the networkID exists, the object with this ID
    // is deleted and True is returned; otherwise False is returned
    // current points to the position where the element was deleted (or
    // should have be inserted), or to the last element if the deleted
    // object was (would have been) the last
    EBool Delete (SINT32 networkID)
    {
        ACNetwork* object;
        EBool found = False;

        object = Remove (networkID);  // remove the object

        if (object != NULL)  // networkID does exist
        {
            delete object;  // delete the object
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        ACNetwork* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll

protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        Assert (key1 != NULL);
        Assert (key2 != NULL);

        SINT32 *k1 = (SINT32 *)key1;
        SINT32 *k2 = (SINT32 *)key2;

        if (*k1 < *k2) return -1;
        else if (*k1 > *k2) return +1;
        else return 0;
    }

    virtual void DeleteKey (void* key) { delete (SINT32*) key; }
    // delete the key
};


#endif


