//
// network.hpp
//
// declaration of the network classes
//
// note: the definition is split into two files: netbase.?pp contains
//       the implementation of ACNetwork, network.?pp contains that of
//       CBroadcastNetwork
//
// created: 02.07.96
//
// last change: 27.07.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _NETWORK_HPP_
#define _NETWORK_HPP_

#include "bool.hpp"
#include "netacc.hpp"
#include "netbase.hpp"


struct SMsg;


//**************************************************************************
// class CBroadcastNetwork: this class implements a broadcast network
//**************************************************************************

class CBroadcastNetwork : public ACNetwork
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CBroadcastNetwork (CControllingObject *ctrl, SINT32 networkID);


    //----------------------------------------------------------------------
    //--- creation function of class CBroadcastNetwork
    //----------------------------------------------------------------------

    static ACNetwork *CreateNetwork (CControllingObject *ctrl,
                                     SINT32 networkID);
    // creates and returns a new object of type CBroadcastNetwork


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- internal control of message transmission
    //----------------------------------------------------------------------

    virtual void SendMsg (SMsg *msg);
    // delay the message (broadcast delay), the delayer calls Transmit()
    // assert: msg != NULL

    virtual void Transmit (SMsg *msg);
    // send or broadcast the message after it was delayed; it is assumed
    // that listOfNodes from the base class contains all receiver nodes
    // assert: msg != NULL
};


#endif


