//
// nodetemp.hpp
//
// stores objects with the sort key <nodeID, area>
//
// created: 07.11.96
//
// last change: 06.01.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _NODETEMP_HPP_
#define _NODETEMP_HPP_

#include "listbase.hpp"
#include "error.hpp"


//**************************************************************************
// class ACNodeList: this class stores objects with the nodeID and area
//      as search key
//**************************************************************************

class ACNodeList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    ACNodeList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {};
    // allocate memory

    virtual ~ACNodeList () {};
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    void* Find (SINT32 nodeID, int area)
    {
        SKey key (nodeID, area);
        return ACObjectList::Find (&key);
    }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    EBool GetCurrentObjectKey (SINT32& nodeID, int& area)
    // references to the key variables
    {
        SKey *key = (SKey*) ACObjectList::GetCurrentObjectKey();

        if (key != NULL)
        {
            nodeID = key->nodeID_;
            area = key->area_;
            return True;
        }
        else return False;
    }
    // returns True if a member is accessed (in that case the netAddress will
    // contain the key), or False if no element is currently accessed


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (void* object, SINT32 nodeID, int area)
    {
        SKey *key = new SKey (nodeID, area);
        EBool inserted;

        inserted = ACObjectList::Insert (object, key);

        if (!inserted)  // the object was not inserted
        {
            delete key;  // the key is not needed
        }

        return inserted;
    }

    void* Remove (SINT32 nodeID, int area)
    {
        SKey key (nodeID, area);
        return ACObjectList::Remove (&key);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned


protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        Assert (key1 != NULL);
        Assert (key2 != NULL);

        return ((SKey*)key1)->CompareTo ((SKey*)key2);
    }


    virtual void DeleteKey (void* key) { delete (SKey*) key; }
    // delete the key

private:
    // the structure of the key
    struct SKey
    {
        SINT32 nodeID_;
        int area_;

        SKey (SINT32 ID, int area)
        {
            nodeID_ = ID;
            area_ = area;
        }

        int CompareTo (const SKey* key1)
        {
            if (nodeID_ < key1->nodeID_) return -1;
            if (nodeID_ > key1->nodeID_) return +1;
            if (area_ < key1->area_) return -1;
            if (area_ > key1->area_) return +1;
            return 0;
        }
    };
};


#endif


