//
// psos.h
//
// declaration of all psos system calls that are used in the hardware based
// simulation and that have to be simulated in the software based sim.
// remove this file and the psos.cpp file and use the real psos.h and
// object file when porting to the hardware
// note: the defines and declarations were copied from the real psos.h
// (the department version, which checks each INTxx macro separately before
// defining it)
//
// created: 14.09.96
//
// last change: 01.04.97
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 10:26:18 $
//


#ifndef _PSOS_H_
#define _PSOS_H_


/*-------------------------------------------------------------------*/ 
/*  DATA Types definitions                                           */
/*-------------------------------------------------------------------*/ 
#ifndef INT32
#define INT32   unsigned long
#endif

#ifndef INT8
#define INT8    unsigned char
#endif

#ifndef INT16
#define INT16   unsigned short
#endif

#ifndef SINT32
#define SINT32  long
#endif

#ifndef SINT8
#define SINT8   char
#endif

#ifndef SINT16
#define SINT16  short
#endif

/***********************************************************************/
/* General Definitions                                                 */
/***********************************************************************/
#define UCHAR  unsigned char
#define USHORT unsigned short
#define UINT   unsigned int
#define ULONG  unsigned long

/***********************************************************************/
/* pSOS+ System Calls (defined in kim.s)                               */
/***********************************************************************/
extern INT32 t_mode(INT32 mask, INT32 new_mode, INT32 *old_mode);

/*
 NOTE: in pure simulation (not pSOS), if you have any static objects
 that use the following queue functions, make sure that the functions
 are never called during the startup phase of the program (ie. before
 execution of main())
 reason: (simulated) queue management is done by a queue manager object,
 which is static in module psos.cpp; but it is not assured which object
 will be created first - the queue manager or the object using the queue
 functions; so only use the queue functions during program execution, never
 in the ctor of a static object!
*/
extern INT32 q_create(char name[4], INT32 count, INT32 flags, INT32 *qid);
extern INT32 q_delete(INT32 qid);
extern INT32 q_ident(char name[4], INT32 node, INT32 *qid);
extern INT32 q_receive(INT32 qid, INT32 flags, INT32 timeout, INT32 msg_buf[4]);
extern INT32 q_send(INT32 qid, INT32 msg_buf[4]);
extern INT32 q_urgent(INT32 qid, INT32 msg_buf[4]);

/***********************************************************************/
/* pSOS+ Symbol Definitions                                            */
/* note: some definitions from psos.h are not included - they can't be */
/*       used                                                          */
/***********************************************************************/

/*---------------------------------------------------------------------*/
/* q_create() Definitions                                              */
/*---------------------------------------------------------------------*/
#define Q_LOCAL         0x00000000  /* 0 = Local */
#define Q_FIFO          0x00000000  /* Queue by FIFO order */
#define Q_NOLIMIT       0x00000000  /* No limit on message queue */
#define Q_SYSBUF        0x00000000  /* Use system buffers */

/*---------------------------------------------------------------------*/
/* q_receive() Definitions                                             */
/*---------------------------------------------------------------------*/
#define Q_NOWAIT        0x00000001  /* Don't wait for a message */

/***********************************************************************/
/* pSOS+ Error Codes                                                   */
/***********************************************************************/

/*---------------------------------------------------------------------*/
/* Common Errors (across several service groups)                       */
/*---------------------------------------------------------------------*/
#define ERR_OBJDEL   0x05     /* Object has been deleted */
#define ERR_OBJID    0x06     /* Illegal or invalid object id */
#define ERR_OBJNF    0x09     /* Object was not found */

/*---------------------------------------------------------------------*/
/* Queue and Message Service Group Errors                              */
/*---------------------------------------------------------------------*/
#define ERR_NOMSG    0x37     /* Cannot receive; no pending message */
                              /* (nowait only) */
//#define ERR_TATQDEL  0x38     /* Informative; tasks were waiting at */
                              /* time of deletion */
//#define ERR_MATQDEL  0x39     /* Informative; messages were pending at */
                              /* time of deletion */


#endif

