//
// psos.hpp
//
// declaration of the classes necessary to implement the pSOS+ system calls
// that are used in the hardware based simulation and that have to be
// simulated in the software based sim.
// remove the psos.cpp and psos.hpp files and the psos.h file and use the
// real psos.h and object file when porting to the hardware
//
// created: 21.09.96
//
// last change: 09.02.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _PSOS_HPP_
#define _PSOS_HPP_


#include "psos.h"
#include "bool.hpp"


#define Q_NoError    0  // this is not defined in the real psos.h; why?


//**************************************************************************
// class CQueue: a FIFO queue
//               it stores a pointer to the data
//**************************************************************************

class CQueue
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CQueue();
    // set the pointers to NULL
    
    ~CQueue();
    // empty the queue


    //----------------------------------------------------------------------
    //--- access functions
    //----------------------------------------------------------------------

    INT32 Send (INT32 data[4], EBool first = False);
    // append the data (first=False) or insert it in front (first=True)
    
    INT32 Receive (INT32 data[4]);
    // remove and return the first element in the queue;


    //----------------------------------------------------------------------
    //--- status functions
    //----------------------------------------------------------------------

    EBool Empty();
    // returns True if the queue is empty, False otherwise


private:

    // an element of the list that CQueue uses to store the data
    struct SLink
    {
        INT32 data[4];
        SLink *next;
    };

    SLink *first_;
    SLink *last_;
};



//**************************************************************************
// class CQueueManager: manages named Queues and allows access to them
//**************************************************************************

class CQueueManager
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CQueueManager();
    // start with an empty list
    
    ~CQueueManager();
    // delete all queues in the list


    //----------------------------------------------------------------------
    //--- access functions
    //----------------------------------------------------------------------

    INT32 Create (char name[4], INT32 *qid);
    // create a queue with that name and set its ID
    // there is no check for duplicates

    INT32 Ident (char name[4], INT32 *qid);
    // return the ID of the queue with this name
    // if the name is not unique, the first queue with this name is returned

    INT32 Delete (INT32 qid);
    // delete the queue with the given index

    INT32 Send (INT32 qid, INT32 msg[4], EBool first = False);
    // store the message in the appropriate queue

    INT32 Receive (INT32 qid, INT32 msg[4]);
    // remove the message from the appropriate queue and return it


protected:

    //----------------------------------------------------------------------
    //--- internal access functions
    //----------------------------------------------------------------------

    CQueue *Find (INT32 qid);
    // search for the queue with the given ID, return a pointer to it
    // if the queue is not found, NULL is returned


private:

    // an element of the list that CQueueManager uses to store the data
    struct SNamedLink
    {
        char name[5];  // the 5th char is always '\0'
        INT32 queueID;
        CQueue *queue;
        SNamedLink *next;
    };

    SNamedLink *firstQueue_;
};


#endif

