//
// sim_gen.hpp
//
// some general declarations of structures, classes and global functions
// that are only needed in the simulation
//
// created: 02.10.96
//
// last change: 18.08.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _SIM_GEN_HPP_
#define _SIM_GEN_HPP_


#include "ci.h"
#include "psos.h"

#include "bool.hpp"
#include "time.hpp"


//**************************************************************************
// global functions
//**************************************************************************

extern SINT32 GetCurrentNodeID ();
// returns the ID of the node that is currently active or NoAccount if
// no node is active
// used by: ACNetwork


extern EBool GetClockvalue (SINT32 nodeID, int area, STimestamp& time);
// returns the timestamp of the specified area on the given node
// if the (node,area) is not in use, False is returned


//**************************************************************************
// declarations concerning the network
// used by: ACNetwork (and derived classes), ACNetworkProcess (and derived
//          classes), ci_send()
//**************************************************************************

// the data structure for a message
struct SMsg
{
    SINT32 sender;  // sending node
    int area;  // DMA area of the sending node
    INT32 msgID;  // ID of the message (<sender, area, msgID> is unique)
    SINT32 receiver;  // receiving node
    EBool broadcast;  // message should be broadcasted
    // (if not, it is only sent to receiver)
    EBool timestamped;  // message should be timestamped
    int if_num;  // network ID
    INT32 rtime[MAXASICS];  // for the test of delay measurement
    bufx *buf;  // data buffer
    INT16 bufsize;  // size of the buffer

    SMsg ();  // use default values for all variables
    SMsg (bufx *b, INT16 size, int ifcNumber, int a);
    // copy the parameter values into the corresponding variables
    SMsg (const SMsg& m);  // copy the data of "m"

    ~SMsg();

    SMsg& operator= (const SMsg& m);  // copy the data of "m"

    void SetTime (STimestamp time);
    // put the area and time into the appropriate positions in buf (the
    // position is obtained from GetPosition()), shifts the rest of the data; 
    // if timestamped is False, the method has no effect
    // 'sender', 'if_num' and 'timestamped' must be set correctly
};


#endif

