//
// sockbase.hpp
//
// declaration of a basic socket interface class for stream/datagram
// sockets
//
// created: 07.02.97
//
// last change: 06.04.97
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _SOCKBASE_HPP_
#define _SOCKBASE_HPP_

#include "psos.h"

#include "bool.hpp"


class ACSocketAddress;


//**************************************************************************
// class CSocket: used as an interface to a (stream or datagram) socket;
//      can send and receive messages
//**************************************************************************

class CSocket
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CSocket (ACSocketAddress* addr, INT32 sid, EBool stream);
    // assume that a socket with the ID 'sid' exists and is open
    // assert: addr != NULL

    virtual ~CSocket ();
    // closes the socket


    //----------------------------------------------------------------------
    //--- reset
    //----------------------------------------------------------------------

    virtual void Reset ();
    // set members to default values (calls Init())


    //----------------------------------------------------------------------
    //--- socket information
    //----------------------------------------------------------------------

    ACSocketAddress *GetSocketAddress();
    // returns the socket address

    INT32 GetSocketID();
    // returns the file descriptor of the socket

    EBool IsStream ();
    // returns True if the socket is a stream socket, else False


    //----------------------------------------------------------------------
    //--- control timeouts
    //----------------------------------------------------------------------

    void SetTimeout (double timeout);
    // sets the timeout (in seconds) for message reception; use a negative
    // value to block indefinitely
    // note: blocking indefinitly does not imply that the call only returns
    //       if a message has been received; an error (pe. socket deletion)
    //       may also result in a return

    double GetTimeout ();
    // returns the currently used timeout (in seconds)


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- internal constructor
    //----------------------------------------------------------------------

    CSocket (ACSocketAddress *addr, EBool stream);
    // store the address and type of the socket
    // assert: addr != NULL


    //----------------------------------------------------------------------
    //--- internal member initialization
    //----------------------------------------------------------------------

    virtual void Init ();
    // set the timeout to its default value


    //----------------------------------------------------------------------
    //--- internal message transmission
    //----------------------------------------------------------------------

    EBool Send (void *data, INT32 size);
    // send the data using the appropriate s_send() function; returns True
    // if the send operation was successful, else False
    // assert: data != NULL


    //----------------------------------------------------------------------
    //--- internal message reception
    //----------------------------------------------------------------------

    EBool Receive (void *data, INT32 *size);
    // reads the data from the socket (assumes that 'data' points to a
    // buffer of size '*size'); blocks according to the internal timeout;
    // modifies '*size' to reflect the number of bytes read; returns True
    // if the receive operation was successful, else False
    // assert: data, size != NULL

    EBool Receive (void *data, INT32 *size, double timeout);
    // same as Receive() above, but timeout is controlled through
    // 'timeout'; please see SetTimeout() for a description of 'timeout'


    //----------------------------------------------------------------------
    //--- internal socket ID handling
    //----------------------------------------------------------------------

    void SetSocketID (INT32 sid);
    // sets the socket ID


private:

    INT32 sid_;  // ID of the socket to the evaluation
    ACSocketAddress *addr_;  // the socket address
    double timeout_;  // timeout for message reception in seconds
    EBool stream_;  // True if it's a stream socket, False if it's datagram
};

#endif


