//
// sockdgrm.hpp
//
// declaration of the basic datagram socket interface class, and access
// classes for client and server datagram sockets
//
// created: 07.02.97
//
// last change: 06.04.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _SOCKDGRM_HPP_
#define _SOCKDGRM_HPP_

#include "psos.h"

#include "bool.hpp"
#include "sockbase.hpp"


//**************************************************************************
// class CDatagramInterface: used as an interface to a datagram socket;
//      can send/receive datagram messages
//**************************************************************************

class CDatagramInterface : public CSocket
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CDatagramInterface (ACSocketAddress* addr, INT32 sid);
    // assume that a datagram socket with the ID 'sid' exists and is open
    // assert: addr != NULL


    //----------------------------------------------------------------------
    //--- message transmission
    //----------------------------------------------------------------------

    EBool SendMessage (void *data, INT32 size);
    // transmits the message over the socket; returns True if the operation
    // was successful, else False
    // assert: data != NULL


    //----------------------------------------------------------------------
    //--- message reception
    //----------------------------------------------------------------------

    EBool ReceiveMessage (void *data, INT32 *size);
    // reads the data from the socket (assumes that 'data' points to a
    // buffer of size '*size'); blocks according to the internal timeout;
    // modifies '*size' to reflect the number of bytes read; returns True
    // if the receive operation was successful, else False
    // assert: data, size != NULL


protected:

    //----------------------------------------------------------------------
    //--- internal constructor
    //----------------------------------------------------------------------

    CDatagramInterface (ACSocketAddress *addr);
    // store the address of the socket
    // assert: addr != NULL
};



//**************************************************************************
// class CDatagramServer: creates a datagram socket
//**************************************************************************

class CDatagramServer : public CDatagramInterface
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CDatagramServer (ACSocketAddress *addr);
    // create the socket
    // assert: addr != NULL

    virtual ~CDatagramServer ();
    // close the socket (unlink it if necessary)
};



//**************************************************************************
// class CDatagramClient: identifies a datagram socket
//**************************************************************************

class CDatagramClient : public CDatagramInterface
{
public:

    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CDatagramClient (ACSocketAddress *addr);
    // identify the socket
    // assert: addr != NULL
};


#endif


