//
// socket.hpp
//
// socket interface in pSOS style
//
// created: 30.01.97
//
// last change: 06.02.97
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 09:57:06 $
//


#ifndef _SOCKET_H_
#define _SOCKET_H_

#include <stdlib.h>  // for NULL

#include "psos.h"


class ACSocketAddress;
// base class for socket addresses


//**************************************************************************
// Socket Parameters
//**************************************************************************

enum ESocketType
{
    SStream,  // stream socket; needs accept/connect
    SDatagram  // datagram socket; just send/receive the data
};


enum ESocketMode
{
    STimeout,  // use timeouts
    SNoTimeout  // block
};


//**************************************************************************
// Socket "System" Calls
//**************************************************************************

extern INT32 s_create (const ACSocketAddress *addr, ESocketType type,
                       INT32 *sid);
// create a socket, bind address to it; listen if it's of type SStream

extern INT32 s_ident (const ACSocketAddress *addr, ESocketType type,
                      INT32 *sid);
// create a socket; connect it to the socket specified by 'addr' if it's
// of type SStream

extern INT32 s_accept (INT32 sid, INT32 *accsid);
// accept a new connection

extern INT32 s_send (INT32 sid, const void *data, INT32 len);
// send 'len' bytes from the buffer pointed to by 'data'; assume a stream
// socket

extern INT32 s_send (INT32 sid, const ACSocketAddress *addr,
                     const void *data, INT32 len);
// send 'len' bytes from the buffer pointed to by 'data'; assume
// a datagram socket with address 'addr'

extern INT32 s_receive (INT32 sid, ESocketMode mode, double timeout,
                        void *data, INT32 *len);
// read at most 'len' bytes into the buffer pointed to by 'data';
// the 'mode' and 'timeout' allow non-blocking calls: if mode==STimeout,
// timeout specifies the timeout in seconds (timeout==0 means polling)
// with a resolution of microseconds;
// len is modified to represent the real size that was read

extern INT32 s_delete (INT32 sid, const ACSocketAddress *addr = NULL);
// close an existing socket; if addr != NULL and it's AF_UNIX, unlink it


//**************************************************************************
// Socket Error Codes
//**************************************************************************

enum ESocketError
{
    SOkay = 0,  // no errors
    SFailed,  // some socket call has gone wrong
    SNoPendingData,  // call to s_receive() had a timeout
    SEOF  // call to s_receive() encountered EOF
};


#endif


