//
// supervis.hpp
//
// declaration of the synchronization supervisor
//
// created: 06.12.96
//
// last change: 11.03.98
//
// RCS version control information:
//     $Revision: 1.3 $$State: Stable $$Date: 1998/07/23 08:55:28 $
//


#ifndef _SUPERVIS_HPP_
#define _SUPERVIS_HPP_

#include "ctrling.hpp"
#include "general.hpp"


class CClockInterface;
class CClockSync;
class CLance;


//**************************************************************************
// class CSupervisor: this class controls the initialization and execution
//      of the clock synchronization; it creates all necessary moduls and
//      is their link to the evaluation
//**************************************************************************

class CSupervisor : public CControllingObject
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CSupervisor (char qname[4], ACSocketAddress *addr, SINT32 nodeID,
                 int area, double clockFrequency);
    // the name of the queue is a string of hex numbers

    virtual ~CSupervisor ();


    //----------------------------------------------------------------------
    //--- reset of the application
    //----------------------------------------------------------------------

    void Reset();
    // set all parameters in all modules to their default values


    //----------------------------------------------------------------------
    //--- get the nodeID/area identifiers
    //----------------------------------------------------------------------

    void GetIdentification (SINT32& nodeID, int& area) const;
    // copy the internal nodeID/area into the parameters

    
    //----------------------------------------------------------------------
    //--- suspend/resume application
    //----------------------------------------------------------------------

    void SuspendOperation ();
    // suspend the whole application

    void ResumeOperation ();
    // resume the whole application


    //----------------------------------------------------------------------
    //--- return associated objects
    //----------------------------------------------------------------------

    CClockInterface *GetClock ();
    CClockSync *GetClockSync ();
    CLance *GetLance ();


    //----------------------------------------------------------------------
    //--- configuration of system parameters
    //----------------------------------------------------------------------

    virtual EBool SetParameter (const SParameter& param);
    // sets the given parameter to its new value (or passes it on to one
    // of the submodules); if the parameter does not exist or if the value
    // is not valid, False is returned

    virtual EBool GetParameter (SParameter& param) const;
    // returns the value of the given parameter in the appropriate field of
    // SParameter; if the parameter does not exist, False is returned (the
    // parameter fields may nevertheless be changed!)


    //----------------------------------------------------------------------
    //--- execution of system commands
    //----------------------------------------------------------------------

    virtual EBool ExecuteCommand (const SCommand& command);
    // executes the command (or passes it on to one of the submodules);
    // returns True if successfull, otherwise False
    // own commands: EnableSnapshot, DisableSnapshot


    //----------------------------------------------------------------------
    //--- event logging
    //----------------------------------------------------------------------

    virtual void NoteEvent (SEvent *event);
    // stores the event until it can be processed


protected:

#ifdef PUBLIC_IN_TEST
public:
#endif
    //----------------------------------------------------------------------
    //--- periodic activities
    //----------------------------------------------------------------------

    virtual void DoPeriodicActivities();
    // is called once every period; initiates a snapshot if the triggering
    // of periodic snapshots is enabled


    //----------------------------------------------------------------------
    //--- error reports
    //----------------------------------------------------------------------

    virtual int ReportError (EReportType type, char *file, int line,
                             char *expression);


private:

    CClockInterface *clock_;  // clock object
    CClockSync *cs_;  // clock synchronization object
    CLance *lance_;  // lance object

    SINT32 nodeID_;  // node ID
    int area_;  // area number

    EBool snapshot_;  // True if periodic snapshots are triggered
};


#endif


