//
// suptemp.hpp
//
// a list of all supervisors in the system, sorted by <nodeID, area>
//
// created: 08.12.96
//
// last change: 11.03.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:37:28 $
//


#ifndef _SUPTEMP_HPP_
#define _SUPTEMP_HPP_

#include "nodetemp.hpp"  // for ACNodeList
#include "supervis.hpp"


//**************************************************************************
// class CSupervisorList: this class stores all supervisors in the system
//**************************************************************************

class CSupervisorList : public ACNodeList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CSupervisorList (INT32 aSize = ArrayAllocationSize) :
        ACNodeList (aSize) {};
    // allocate memory

    virtual ~CSupervisorList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    CSupervisor* GetFirst ()
        { return (CSupervisor*) ACNodeList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    CSupervisor* GetLast ()
        { return (CSupervisor*) ACNodeList::GetLast(); }
    // get the last object or NULL if the array is empty

    CSupervisor* GetNext ()
        { return (CSupervisor*) ACNodeList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    CSupervisor* GetPrev ()
        { return (CSupervisor*) ACNodeList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    CSupervisor* Find (SINT32 nodeID, int area)
        { return (CSupervisor*) ACNodeList::Find (nodeID, area); }
    // returns the object with this index; if no such object exists,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    CSupervisor* GetCurrentObject ()
        { return (CSupervisor*) ACNodeList::GetCurrentObject(); }
    // returns a pointer to the member that has been accessed last, or
    // NULL, if this information is not available anymore


    //----------------------------------------------------------------------
    //--- add/remove objects
    //----------------------------------------------------------------------

    EBool Insert (void* object, SINT32 nodeID, int area)
        { return ACNodeList::Insert (object, nodeID, area); }
    // insert an object into the array; if the object does not exist in
    // the list, True is returned; else False is returned

    CSupervisor* Remove (SINT32 nodeID, int area)
        { return (CSupervisor*) ACNodeList::Remove (nodeID, area); }
    // remove an object from the array


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // note: since the object is a process, it may be unref'd instead of
    // deleted; so a special function is needed to delete processes; it
    // is called by Delete() and DeleteAll()
    void DeleteProcess (ACProcess* object)
    {
        object->Cancel();
        object->terminate();  // terminate the process

#ifndef NO_RESOURCE
        Resource::unref(object);  // either unref the process in C++SIM
#else
        delete object;  // or delete it
#endif
    }


    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    EBool Delete (SINT32 nodeID, int area)
    {
        CSupervisor* object;
        EBool found = False;

        object = Remove (nodeID, area);

        if (object != NULL)  // index does exist
        {
            DeleteProcess (object);  // delete the process
            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        CSupervisor* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                DeleteProcess (object);  // delete the process
            }

            return True;
        }
    } // DeleteAll
};


#endif


