//
// time.hpp
//
// all structures, classes, functions etc. that are necessary for the
// time management
//
// created: 11.02.97
//
// last change: 14.09.98
//
// RCS version control information:
//     $Revision: 1.2 $$State: Exp $$Date: 1998/09/14 09:42:21 $
//


#ifndef _TIME_
#define _TIME_


#include "psos.h"


class ostream;


//**************************************************************************
// class CNtpTime: stores the macrostamp, timestamp and microstamp; allows
//      arithmetic operations on the data (only those that are needed)
//**************************************************************************

class CNtpTime
{
public:
    INT32 ms;  // clock time value
    INT32 ts;
    INT32 us;


    //----------------------------------------------------------------------
    //--- constructor
    //----------------------------------------------------------------------

    CNtpTime ();
    CNtpTime (INT32 macro, INT32 time, INT32 micro=0);
    CNtpTime (double seconds);
    CNtpTime (const CNtpTime& time);


    //----------------------------------------------------------------------
    //--- operators
    //----------------------------------------------------------------------

    CNtpTime operator= (const CNtpTime& time);
    CNtpTime operator+ (const CNtpTime& time) const;
    CNtpTime operator- (const CNtpTime& time) const;
    // result negative: uses (0,0)
    CNtpTime operator+= (const CNtpTime& time);
    CNtpTime operator-= (const CNtpTime& time); // negative: returns (0,0)
};


ostream& operator<< (ostream& o, const CNtpTime& time);


//**************************************************************************
// global structures
//**************************************************************************

// storage of the NTP time and accuracy that is transmitted over the network
struct STimestamp  // use ti_acx (ci.h) for the network class!
{
    INT32 ms;  // macrostamp
    INT32 ts;  // timestamp
    INT32 acc;  // accuracy
};

// the clock synchronization algorithm has also access to the microstamp
struct SExtendedTimestamp : public STimestamp
{
    INT32 us;  // microstamp

    // assign from an STimestamp by setting the 'us' to zero
    SExtendedTimestamp operator= (const STimestamp& t)
    { ms = t.ms; ts = t.ts; acc = t.acc; us = 0; return *this; }

    // assign from a CNtpTime by setting the 'acc' to zero
    SExtendedTimestamp operator= (const CNtpTime& t)
    { ms = t.ms; ts = t.ts; us = t.us; acc = 0; return *this; }
};


ostream& operator<< (ostream& o, const STimestamp& time);
ostream& operator<< (ostream& o, const SExtendedTimestamp& time);


//**************************************************************************
// conversion functions NTP <--> seconds
// context: ISR, Task
//**************************************************************************

extern double NtpToSeconds (INT32 ms, INT32 ts);
// this function converts NTP time into seconds

extern double NtpToSeconds (INT32 ms, INT32 ts, INT32 us);
// this function converts extended NTP time into seconds

inline double NtpToSeconds (const STimestamp& time)
// also converts NTP time into seconds, but uses STimestamp
{
    return NtpToSeconds (time.ms, time.ts);
}

inline double NtpToSeconds (const CNtpTime& time)
// also converts extended NTP time into seconds, but uses CNtpTime
{
    return NtpToSeconds (time.ms, time.ts, time.us);
}

inline double NtpToSeconds (const SExtendedTimestamp& time)
// converts extended NTP time into seconds, uses SExtendedTimestamp
{
    return NtpToSeconds (time.ms, time.ts, time.us);
}


extern void SecondsToNtp (double sec, INT32& ms, INT32& ts);
// this function converts seconds into NTP

extern void SecondsToNtp (double sec, INT32& ms, INT32& ts, INT32& us);
// this function converts seconds into extended NTP

inline void SecondsToNtp (double sec, STimestamp& time)
// also converts seconds into NTP, but uses STimestamp
{
    SecondsToNtp (sec, time.ms, time.ts);
}

inline void SecondsToNtp (double sec, CNtpTime& time)
// also converts seconds into extended NTP, but uses CNtpTime
{
    SecondsToNtp (sec, time.ms, time.ts, time.us);
}

inline void SecondsToNtp (double sec, SExtendedTimestamp& time)
// converts seconds into extended NTP, uses SExtendedTimestamp
{
    SecondsToNtp (sec, time.ms, time.ts, time.us);
}

#endif

