//
// errtask.hpp
//
// declaration of the error report class used for storing and processing
// error reports
// note: this implementation stores the message in a queue and periodically
// prints it to cerr
// note: the class can be used as a task in hardware (for printing to the
// terminal)
//
// created: 31.03.97
//
// last change: 11.03.98
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 09:57:06 $
//


#ifndef _ERRTASK_HPP_
#define _ERRTASK_HPP_


#include "errbase.hpp"
#include "process.hpp"


//**************************************************************************
// class CErrorTask: stores strings (error messages) in a queue and
//      periodically prints them to cerr
//**************************************************************************

class CErrorTask : public CErrorHandler,
                   public ACProcess
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CErrorTask (char qname[4]);
    // create a queue with the qiven name
    // call the queue "QERR" or something like that; *don't* use a string
    // of (hex) numbers!

    virtual ~CErrorTask();
    // emergency flush and delete the queue


    //----------------------------------------------------------------------
    //--- store a message
    //----------------------------------------------------------------------

    virtual void NoteError (char *message);
    // stores a copy of the message in the queue


    //----------------------------------------------------------------------
    //--- message queue handling
    //----------------------------------------------------------------------

    virtual void Flush();
    // reads at most 'MaxErrorMessages' messages from the message queue and
    // processes them


    //----------------------------------------------------------------------
    //--- message queue handling (emergency)
    //----------------------------------------------------------------------

    void EmergencyFlush();
    // empties the queue, processes messages using EmergencyProcessMessage()


    //----------------------------------------------------------------------
    //--- for the simulation scheduler only; do not use
    //----------------------------------------------------------------------

    virtual void Body ();
    // periodically flush the queue


protected:

    //----------------------------------------------------------------------
    //--- internal message output
    //----------------------------------------------------------------------

    virtual void ProcessMessage (char *message);
    // print the message (followed by a newline) to an output screen
    // (in pSOS: the XRAY standard output screen; else: cerr)


    //----------------------------------------------------------------------
    //--- internal message processing (emergency)
    //----------------------------------------------------------------------

    virtual void EmergencyProcessMessage (char *message);
    // same as ProcessMessage()


    //----------------------------------------------------------------------
    //--- internal message buffer handling
    //----------------------------------------------------------------------

    static char *GetMessageString (INT32 len);
    // allocates and returns a string of the given length; use only this
    // function for obtaining memory for message strings

    static void ReturnMessageString (char *string);
    // deallocates the string; it is assumed that the string has been
    // allocated using GetMessageString()


private:

    INT32 qid_;  // the ID of the error queue
};

#endif


