//
// acctest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test struct SNetworkDelay in account.hpp
// test class CAccountant in account.hpp
//
// note: for this test, Get/SetDelay(), DeleteKey1/2(), Init() and Dispose()
//       were made public (instead of protected) [CAccountant]
//       all function of CUserAccountant were made public as well
//
// created: 15.11.96
//
// last changed: 01.08.98
//



#include "..\account.hpp"

INT32 t_mode (INT32, INT32, INT32*) { return 0; }
// needed for general.cpp

int main()
{
    //----------------------------------------------------------------------
    // variables necessary for the test of account.hpp
    //----------------------------------------------------------------------

    CAccountant *accountant;
    EBool result;


    //----------------------------------------------------------------------
    // test the constructor of SNetworkDelay
    //----------------------------------------------------------------------

    SNetworkDelay delay;
    // all members should be 0 (NO_DISTRIB_LIB)
    // members are 3 x {Uniform,0,0,0,0,0}, 0, 0 (YES_DISTRIB_LIB)

    // tested on 15.11.96, 14:44: all tests okay (NO_DISTRIB_LIB)
    // tested on 01.08.98, 10:40: all tests okay (YES_DISTRIB_LIB)


    //----------------------------------------------------------------------
    // test the behaviour of the class CAccountant
    //    correct behaviour of the classes
    //        ACObjectList, CTransmissionDelayList and CTransmissionList
    //    is assumed
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor

    accountant = new CAccountant;
    // data = NULL, listOfDelays = "empty list", initCalled = True

    // tested on 02.01.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test Init() and Dispose()

    accountant->Init();
    // should return at once (trace into the code to verify that!)

    accountant->Dispose();
    // initCalled = False
    accountant->Dispose();
    // should return at once (trace into the code to verify that!)

    accountant->Init();
    // initCalled = True

    // tested on 02.01.97, 20:25: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetDelay()
    // note: these functions call SetupDataEntry(), which calls
    //       GetInformation() and SetInformation(); so these functions
    //       will be tested as well (implicitely)

#ifdef NO_DISTRIB_LIB
    delay.delta = 4;
    delay.epsilonPlus = 5;
    delay.epsilonMinus = 3;
    delay.probEarly = 0.1;
    delay.probLate = 0.2;

    accountant->SetDelay (delay, 1, 2);
    // key1=1, key2=2, data=(4,5,3) [listOfDelays.firstFree=1]
    // listOfDelays = (1,2)

    delay.epsilonMinus = 4;
    accountant->SetDelay (delay, 1, 2);
    // data=(4,5,4)

    delay.epsilonMinus = 2;
    accountant->SetDelay (delay, 1, 3);
    // key2=3, data=(4,5,2)
    // SetInformation() tested completely
    // listOfDelays = (1,2), (1,3)

    delay.epsilonMinus = 3;
    accountant->SetDelay (delay, 1, 2);
    // key2=2, data=(delays: 4,5,3)
    // GetInformation() tested completely
    // SetupDataEntry() tested completely
    // --> SetDelay() tested completely

    delay.delta = .1; delay.epsilonPlus = .2; delay.epsilonMinus = .3;
    delay.probEarly = 0.4; delay.probLate = 0.5;
    result = accountant->GetDelay (&delay, 2, 3);  // return False
    // delay = (0.1,0.2,0.3,0.4,0.5)

    result = accountant->GetDelay (&delay, 1, 3);  // return True
    // key2=3, data=(4,5,2), delay = (4,5,2,0.1,0.2)
    // GetDelay() tested completely

    // tested on 16.08.97, 16:20: all tests okay
#endif


    //----------------------------------------------------------------------
    // test DeleteKey1/2()

    accountant->SetDelay (delay, 1, 4);  // create some new nodes
    accountant->SetDelay (delay, 2, 4);
    accountant->SetDelay (delay, 2, 3);
    accountant->SetDelay (delay, 4, 2);
    // key1=4, key2=2, data(4,2)
    // listOfDelays = (1,2),(1,3),(1,4); (2,3),(2,4); (4,2)

    result = accountant->DeleteKey1 (5);  // should return False
    result = accountant->DeleteKey1 (4);  // should return True
    // data = NULL
    // listOfDelays = (1,2),(1,3),(1,4); (2,3),(2,4)
    // DeleteKey1() tested completely

    result = accountant->DeleteKey2 (5);  // should return False

    accountant->SetDelay (delay, 4, 1);
    // key1=4, key2=1, data = data(4,1)
    // listOfDelays = (1,2),(1,3),(1,4); (2,3),(2,4); (4,1)

    result = accountant->DeleteKey2 (4);  // should return True
    // listOfDelays = (1,2),(1,3); (2,3); (4,1)

    result = accountant->DeleteKey2 (3);  // should return True
    // listOfDelays = (1,2); (4,1)

    accountant->DeleteKey2 (2);
    // listOfDelays = (4,1)

    result = accountant->DeleteKey2 (1);  // should return True
    // data = NULL
    // listOfDelays = "empty list"

    result = accountant->DeleteKey2 (1);  // should return False

    // DeleteKey2() tested completely

    // tested on 22.05.97, 19:20: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    accountant->SetDelay (delay, 1, 4);  // create some new nodes
    accountant->SetDelay (delay, 2, 3);
    // key1=2, key2=3, data(2,3)
    // listOfDelays = (1,4); (2,3)

    accountant->Reset();
    // data = NULL, listOfDelays = "empty list", initCalled = True

    // tested on 22.05.97, 19:20: all tests okay


    delete accountant;
    

    //----------------------------------------------------------------------

    return 0;
}

#include "inlines.hpp"

