//
// algtest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test ACClockSyncAlgorithm (through a derived class) [algbase.?pp]
// note: all protected functions were made public for this test
// test CFta, CMidpoint [algfta.?pp]
// FTA() was made public for this test
// Mean() must be made global (instead of static) if it is to be tested
// test CFreeRun [algfree.?pp]
//
// created: 12.02.97
//
// last change: 04.02.99
//


#include <iostream.h>

#include "..\address.hpp"
#include "..\algbase.hpp"
#include "..\algfetz.hpp"
#include "..\algfree.hpp"
#include "..\algfta.hpp"
#include "..\clockifc.hpp"
#include "..\clocksyn.hpp"
#include "..\constant.hpp"
#include "..\database.hpp"
#include "..\globals.hpp"
#include "..\idlist.hpp"
#include "..\structs.hpp"
#include "..\template.hpp"
#include "..\time.hpp"

#define Need_ACProcess
#define Need_CControlledObject
#define Need_CCrashingModule


extern STimestamp Mean (const CNtpTime& low, const CNtpTime& high);


// for testing ACClockSyncAlgorithm
class CDummy : public ACClockSyncAlgorithm
{
public:
    CDummy (CClockSync *cs) : ACClockSyncAlgorithm (cs) {}

    virtual EBool ConvergenceFunction ()
    { timeValid = True; newClockValue->ms=1; newClockValue->ts=2;
      newClockValue->us=3; newClockValue->acc=4; return True; }

    // check the cardinalities of the database
    INT32 CardinalityDelay (SINT32 networkID)
    {return database->Cardinality (CApplicationDatabase::Delay, networkID);}

    INT32 CardinalityRate (SINT32 networkID)
    {return database->Cardinality (CApplicationDatabase::Rate, networkID);}

    INT32 CardinalityTime (SINT32 networkID)
    {return database->Cardinality (CApplicationDatabase::Time, networkID);}

    // return the data stored in the database
    EBool GetDelay (SDelay& delay, SINT32 networkID, const CAddr& address)
    { return database->GetDelay (delay, networkID, address); }

    SRate *GetRate (SINT32 networkID, const CAddr& address, int area)
    { return database->GetRate (networkID, address, area); }

    EBool GetTime (STimeInterval& time, SINT32 networkID, const CAddr& address, int area)
    { return database->GetTime (time, networkID, address, area); }

    // set the rate of the local clock
    void SetRate (SRate& localRate) { *rate = localRate; }
    
    void ReportEvent ()
    {
        SApplEvent *event = new SApplEvent;
        ACClockSyncAlgorithm::ReportEvent (event);
        delete event;
    }

    void ReportInterval (EBool input, const STimeInterval& interval)
    {
        ACClockSyncAlgorithm::ReportInterval (input, interval);
    }

    void ReportRate (EBool input, const SRate& rate, double refPoint)
    {
        ACClockSyncAlgorithm::ReportRate (input, rate, refPoint);
    }
};


void SetTimestamp (STimestamp& ntptime, double time, double accPlus,
                   double accMinus);
// converts the time to NTP-Time, and sets the 'acc' member of STimestamp
// accordingly (all times are in seconds)


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of the class ACClockSyncAlgorithm
    //    correct behaviour of class CIntervalSet (and CListOfIntervalSets)
    //    is assumed
    //----------------------------------------------------------------------

    SSingleParameter param;
    CClockSync *cs = new CClockSync (NULL, 0);
    CClockInterface *clock = new CClockInterface (NULL, 1, 3);
    CDummy *csa;
    SINT32 networkID = 1;
    int area = 2;
    CAddr address;
    for (int i=0; i<MAXASICS; i++) address[i] = 0;
    address[0] = 1;
    EBool result;
    double value;


    //----------------------------------------------------------------------
    // test the ctor and Init()

    csa = new CDummy (cs);
    // initcalled = True, cs_ = cs, numberOfNodes=1, numberOfFaultyNodes=0,
    // timeValid = False, assocNetwork = NoAccount,
    // rate: thetaPlus=WcRhoPlus (1e-4), thetaMinus=WcRhoMinus (1e-4), u=0

    // tested on 04.02.99, 11:40: all tests okay


    //----------------------------------------------------------------------
    // test Get/SetAssociatedNetwork()

    result = csa->SetAssociatedNetwork (3);  // return False
    result = csa->SetAssociatedNetwork (2);  // return True
    // assocNetwork = 2
    result = csa->SetAssociatedNetwork (-1);  // return True
    // assocNetwork = -1 (NoAccount)
    result = csa->SetAssociatedNetwork (2);  // return True
    // assocNetwork = 2

    i = csa->GetAssociatedNetwork ();  // return 2

    // tested on 04.08.97, 22:00: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetResynchronizationStatus()

    result = csa->GetResynchronizationStatus();  // return False

    csa->SetResynchronizationStatus (True);
    // cs::resynchronizing_ = True

    result = csa->GetResynchronizationStatus();  // return True

    csa->SetResynchronizationStatus (False);
    // cs::resynchronizing_ = False

    // tested on 08.05.97, 12:00: all tests okay


    //----------------------------------------------------------------------
    // test GetClock()

    cs->SetClock (clock);
    clock = NULL;
    clock = csa->GetClock();  // should return cs::clock

    // tested on 01.05.97, 19:40: all tests okay


    //----------------------------------------------------------------------
    // test GetRoundPeriod(), GetRatePeriod(), GetDelayCompensation(),
    // GetComputationDelayCompensation()
    // you may want to set the ClockSync-members to values different from 0

    value = csa->GetRoundPeriod();  // should return cs::roundPeriod

    // tested on 02.07.98, 14:40: all tests okay so far


    value = csa->GetRatePeriod();  // return cs::roundPeriod*cs::rateCycles

    // tested on 18.11.98, 12:40: all tests okay so far


    value = csa->GetDelayCompensation();
    // should return cs::delayCompensation

    // tested on 09.10.98, 13:40: all tests okay so far


    value = csa->GetComputationDelayCompensation();
    // return cs::computationDelayCompensation

    // tested on 09.10.98, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test StoreRate()

    SRate rate;
    rate.thetaPlus = 2e-1;  // 0.2
    rate.thetaMinus = 1e-1;  // 0.1
    rate.u = 0;
    address[0] = 1;
    CNtpTime local, remote;
    result = csa->StoreRate (rate, local, remote, networkID, address, area);
    // should return True

    // tested on 09.10.98, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test GetLocalRate()

    rate.thetaPlus = 2e-1;  // 0.2
    rate.thetaMinus = 1e-1;  // 0.1
    rate.u = 1.23;
    csa->SetRate (rate);

    cs->SetClock (NULL);
    result = csa->GetLocalRate (rate);  // return False

    cs->SetClock (clock);
    result = csa->GetLocalRate (rate);  // return True
    // rate = (0.1, 0.2, 1.23)

    // tested on 01.05.97, 19:50: all tests okay


    //----------------------------------------------------------------------
    // test ClockChanged()

    cs->SetClock (NULL);
    csa->ClockChanged ();
    // csa::rate = (1e-4, 1e-4, 0) [the default wc. drift bounds]

    cs->SetClock (clock);
    csa->ClockChanged ();
    // csa::rate = (1e-3, 1e-3, 0) [the clock wc. drift bounds]

    // tested on 04.02.99, 11:40: all tests okay


    //----------------------------------------------------------------------
    // test SetAssociation(), DeleteAssociation()

    result = csa->SetAssociation (networkID, address, 1);  // True

    result = csa->DeleteAssociation (networkID, 1);  // return True
    result = csa->DeleteAssociation (networkID, 1);  // return False

    // tested on 01.05.97, 19:50: all tests okay


    //----------------------------------------------------------------------
    // test GetOwnID()

    SINT32 ownNodeID = 0xFF;
    int ownArea = 0xFF;

    csa->GetOwnID (ownNodeID, ownArea);
    // should set both parameters to 0 with the current dummy implementations
    // of CControlledObject and CClockSync

    // tested on 21.09.98, 13:00: all tests okay


    //----------------------------------------------------------------------
    // test SetNewClockValue ()

    csa->SetNewClockValue();
    // should return at once (timeValid is False) [trace it!]

    csa->ConvergenceFunction();  // set newClockValue to a valid value

    csa->SetNewClockValue();
    // should call clock->SetTimestamp() [trace it!]
    // timeValid = False

    // tested on 29.07.97, 16:00: all tests okay


    //----------------------------------------------------------------------
    // test PreloadNewClockValue ()

    csa->PreloadNewClockValue(1);
    // should return at once (timeValid is False) [trace it!]

    csa->ConvergenceFunction();  // set newClockValue to a valid value

    csa->PreloadNewClockValue(1);
    // should call clock->PreloadTimestamp() [trace it!]
    // timeValid = False

    // tested on 07.12.97, 16:40: all tests okay


    //----------------------------------------------------------------------
    // test ComputeNewDelay()

    SDelay delay;
    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;

    address[0] = 5;
    result = csa->SetAssociation (networkID, address, 1);  // True
    // the following call needs it to report the delay
    csa->ComputeNewDelay (10, 10, networkID, address);
    // should report the delay with event DelayEstimation [trace it!]
    // result: event is reported
    csa->GetDelay (delay, networkID, address);
    // delta = 10, epsPlus=epsMinus=0

    // tested on 21.09.98, 13:10: all tests okay

    csa->ComputeNewDelay (5, 5, networkID, address);
    csa->GetDelay (delay, networkID, address);
    // delta = 7.5, epsPlus = 2.5, epsMinus = 2.5

    csa->ComputeNewDelay (7, 7, networkID, address);
    csa->GetDelay (delay, networkID, address);
    // delta = 7.25, epsPlus = 2.75, epsMinus = 2.25

    csa->ComputeNewDelay (11.75, 11.75, networkID, address);
    csa->GetDelay (delay, networkID, address);
    // delta = 9.5, epsPlus = 2.25, epsMinus = 4.5

    csa->ComputeNewDelay (4, 6, networkID, address);
    csa->GetDelay (delay, networkID, address);
    // delta = 7.25, epsPlus = 4.5, epsMinus = 3.25

    csa->ComputeNewDelay (11, 14.5, networkID, address);
    csa->GetDelay (delay, networkID, address);
    // delta = 10, epsPlus = 4.5, epsMinus = 6

    // tested on 01.05.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()
    // assumption: addr {5,0...} has delay [delta=10, epsPlus=4.5, epsMin=6]

    param.type = UserParameter;
    result = csa->SetParameter (param);  // return False
    result = csa->GetParameter (param);  // return False

    param.type = NumberOfNodes;
    param.value = 5;
    result = csa->SetParameter (param);  // return True
    // #nodes = 5
    param.value = 0;
    result = csa->GetParameter (param);  // return True
    // value = 5

    param.type = NumberOfFaultyNodes;
    param.value = 2;
    result = csa->SetParameter (param);  // return True
    // #nodes = 2
    param.value = 0;
    result = csa->GetParameter (param);  // return True
    // value = 2

    STransmissionData transParam;
    transParam.networkID = networkID;
    transParam.nodeID = 5;  // no association stored
    transParam.value = 2.3;
    transParam.type = Delta;
    result = csa->SetParameter (transParam);  // return False
    result = csa->GetParameter (transParam);  // return False
    transParam.type = EpsilonPlus;
    result = csa->SetParameter (transParam);  // return False
    result = csa->GetParameter (transParam);  // return False
    transParam.type = EpsilonMinus;
    result = csa->SetParameter (transParam);  // return False
    result = csa->GetParameter (transParam);  // return False

    param.type = ThetaPlus;
    param.value = 0.3;
    result = csa->SetParameter (param);  // return True
    param.type = ThetaMinus;
    param.value = 0.4;
    result = csa->SetParameter (param);  // return True
    param.type = U;
    param.value = 0.5;
    result = csa->SetParameter (param);  // return True

    param.type = ThetaPlus;
    result = csa->GetParameter (param);  // return True
    // value = 0.3
    param.type = ThetaMinus;
    result = csa->GetParameter (param);  // return True
    // value = 0.4
    param.type = U;
    result = csa->GetParameter (param);  // return True
    // value = 0.5

    // tested on 29.07.97, 16:20: all tests okay so far

    param.type = AssociatedNetwork;
    result = csa->GetParameter (param);  // return True
    // value = 2

    param.value = 1;
    result = csa->SetParameter (param);  // return True
    // assocNetwork = 1

    param.value = 2;
    result = csa->SetParameter (param);  // return True
    // assocNetwork = 2

    // tested on 04.08.97, 21:30: all tests okay so far

    result = csa->SetAssociation (networkID, address, 5);  // True
    // make an association for the network address

    transParam.type = Delta;
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 10
    transParam.value = 5;
    result = csa->SetParameter (transParam);  // return True
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 5

    transParam.type = EpsilonPlus;
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 4.5
    transParam.value = 2.3;
    result = csa->SetParameter (transParam);  // return True
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 2.3

    transParam.type = EpsilonMinus;
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 6
    transParam.value = 7.1;
    result = csa->SetParameter (transParam);  // return True
    transParam.value = 0;
    result = csa->GetParameter (transParam);  // return True
    // value = 7.1

    // tested on 05.05.97, 20:20: all tests okay so far


    //----------------------------------------------------------------------
    // test ExecuteCommand()
    // assumption: assocNetwork = 2

    SNetworkCommand netc;

    netc.type = ConnectNetwork;
    cs->ExecuteCommand (netc);  // make another network with ID=3

    netc.type = DisconnectNetwork;
    netc.networkID = 1;
    cs->ExecuteCommand (netc);
    result = csa->ExecuteCommand (netc);  // return True
    // assocNetwork does not change

    netc.networkID = 2;
    cs->ExecuteCommand (netc);
    result = csa->ExecuteCommand (netc);  // return True
    // assocNetwork = 3

    netc.networkID = 3;
    cs->ExecuteCommand (netc);
    result = csa->ExecuteCommand (netc);  // return True
    // assocNetwork = NoAccount (-1)


    netc.type = ConnectNetwork;
    netc.networkID = 3;
    cs->ExecuteCommand (netc);
    cs->ExecuteCommand (netc);
    result = csa->ExecuteCommand (netc);  // return False

    netc.networkID = 1;
    result = csa->ExecuteCommand (netc);  // return True
    // assocNetwork = 1

    netc.networkID = 0;
    result = csa->ExecuteCommand (netc);  // return True
    // assocNetwork does not change


    netc.type = UserCommand;
    result = csa->ExecuteCommand (netc);  // return False

    // tested on 04.08.97, 22:00: all tests okay


    //----------------------------------------------------------------------
    // test operator=()

    CDummy *testcsa = new CDummy (cs);
    *testcsa = *csa;
    // #nodes = 5, #faulty = 2, assocNetwork = 0

    // tested on 04.08.97, 21:30: all tests okay


    //----------------------------------------------------------------------
    // test StoreDelay()
    // assumption: addr {1,0...} has rate [0.9, 1.2],
    //             addr {5,0...} has already a delay stored

    csa->ClockChanged ();  // 0.001 for both thetas

    rate.thetaPlus = -2e-1;  // -0.2
    rate.thetaMinus = 0;  // 0
    rate.u = 0;
    address[0] = 2;
    // addr {2,0...} has rate [1, 0.8] (wrong on purpose!)

    result = csa->StoreDelay (10, 5, rate, networkID, address);  // False
    // dmin > dmax

    address[0] = 1;
    rate.thetaPlus = 0.2;
    rate.thetaMinus = 0.1;
    result = csa->StoreDelay (10, 12, rate, networkID, address);  // False
    // dmax < 0

    // tested on 29.07.97, 16:30: all tests okay so far

    result = csa->StoreDelay (20, 18, rate, networkID, address);  // True
    csa->GetDelay (delay, networkID, address);
    // (dmin = 0 (-0.81, set to zero), dmax = 1.1 (1.91, set to 1.10)
    // => delay=(0.55, 2x 0.55)
    // result: 0.550...07 (small rounding error)

    // tested on 28.09.97, 15:10: all tests okay


    //----------------------------------------------------------------------
    // test StoreTime()

    STimestamp time;  // the time that the clock has sent
    STimeInterval interval;  // resulting time interval in the database
    double resvalue;  // the resulting interval bounds, in seconds

    SetTimestamp (time, 10, 1e-3, 1e-4);
    // so the time is in the interval [9.999, 10.0001]
    // ms=0, ts = 167 772 160, acc = 0x 0347 20C4

    CNtpTime tmp = CNtpTime (time.ms, time.ts) + CNtpTime (0, (time.acc&0x0FFFF)<<1);
    resvalue = NtpToSeconds (tmp);  // real upper bound (multiple of G_A)
    tmp = CNtpTime (time.ms, time.ts) - CNtpTime (0, (time.acc&0xFFFF0000)>>15);
    resvalue = NtpToSeconds (tmp);  // lower bound (multiple of G_A)

    address[0] = 4;  // no delay stored
    result = csa->StoreTime (time, 0, networkID, address, area, False);
    // return False

    address[0] = 1;  // delay is in [0, 1.91]
    result = csa->StoreTime (time, 1, networkID, address, area, False);
    // return True

    csa->GetTime (interval, networkID, address, area);
    // reference point: 11.955 sec
    // low: 10.997 999 840 79 sec, high: 12.911 100 218 81 sec
    // these values are not rounded; compare with the resvalue below!

    resvalue = NtpToSeconds (interval.low);  // 10.997 999 734 456 02
    resvalue = NtpToSeconds (interval.high);  // 12.911 100 175 803 22
    resvalue = NtpToSeconds (interval.refpoint);  // 11.955
    // note: real values are different (because of rounding errors in the
    // accuracies (has to be a multiple of G_A))

    result = csa->StoreTime (time, 1, networkID, address, area, False);
    // return False
    // time is already stored

    // tested on 29.07.97, 22:10: all tests okay


    //----------------------------------------------------------------------
    // test Init(), Dispose()

    csa->Init();
    // should return at once (trace into the code to verify that!)

    csa->Dispose();
    // initCalled = False
    csa->Dispose();
    // should return at once (trace into the code to verify that!)

    csa->Init();
    // initCalled = True

    // tested on 01.05.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test Reset ()

    csa->StoreDelay (10, 0, rate, networkID, address);
    i = csa->CardinalityDelay (networkID);  // return 1

    csa->Reset ();
    i = csa->CardinalityDelay (networkID);  // return 0

    // tested on 01.05.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test Suspend ()

    csa->StoreDelay (10, 0, rate, networkID, address);
    i = csa->CardinalityDelay (networkID);  // return 1

    // DEBUGGER: set timeValid = True
    csa->Suspend ();
    i = csa->CardinalityDelay (networkID);  // return 0
    // timeValid = False

    // tested on 02.07.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test ReportEvent ()

    csa->ReportEvent ();
    // output: event = UserEvent, module = ClockSync

    // tested on 14.02.97, 17:00: all tests okay


    //----------------------------------------------------------------------
    // test ReportInterval ()

    csa->ReportInterval (True, interval);
    // result: create event of type SApplTimeEvent, type = IntervalEvent,
    //  module = ClockSync, interval = interval, input = True
    //  call ReportEvent()
    // [trace it!]

    // tested on 06.11.98, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test ReportRate ()

    csa->ReportRate (False, rate, 12.3);
    // result: create event of type SApplRateEvent, type = RateEvent,
    //  module = ClockSync, rate = rate, input = False, refPoint = 12.3
    //  call ReportEvent()
    // [trace it!]

    // tested on 06.11.98, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test ReportError ()

    csa->ReportError (Reporting, "dummyfile", 0, "test of the error report");
    // output: CClockSync::ReportError() called with: test of the error
    // report

    // tested on 01.05.97, 20:00: all tests okay


    delete csa;


    //----------------------------------------------------------------------
    // test the behaviour of the class CFta and of CCsaTemplate
    //    correct behaviour of class ACClockSyncAlgorithm is assumed
    //----------------------------------------------------------------------

/*
// commented out so that the source code does not have to be edited
// every time something is tested

    //----------------------------------------------------------------------
    // test global function Mean()

    CNtpTime low, up;

    SecondsToNtp (200, low);
    SecondsToNtp (400, up);
    time = Mean (low, up);  // mean should be 300 sec NTP
    // ms = 1, ts = 738 197 504, acc = 0xFFFFFFFF (4 294 967 295)

    SecondsToNtp (399.9998, low);
    time = Mean (low, up);
    // ms = 1, ts = 2 415 917 426, acc = 0x0346 0346

    SecondsToNtp (255.2, low);  // lsb of ts = 1
    SecondsToNtp (255.3, up);  // lsb of ts = 1
    time = Mean (low, up);
    // ms = 0, ts = 4 282 384 384, acc = 4 294 967 295

    SecondsToNtp (2, low);
    SecondsToNtp (256, up);
    time = Mean (low, up);
    // ms = 0, ts = 2 164 260 864, acc = 4 294 967 295

    // tested on 02.05.97, 19:10: all tests okay
*/

    //----------------------------------------------------------------------
    // test the creation function and the ctor of CFta

    CFta *fta;
    ACClockSyncAlgorithm *base;

    base = CFta::CreateFta (cs);
    fta = (CFta *)base;
    // midpoint=False

    // tested on 30.07.97, 21:50: all tests okay


    //----------------------------------------------------------------------
    // test FTA() with midpoint=False

    area = 2;  // make complete new area/address
    for (i=0; i<ADDRCNT; i++) address[i] = (INT8)i;
    // address = {0,1,2,3,4,5}

    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    fta->ComputeNewDelay (0, 2, networkID, address);  // [0,2]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 3; SetTimestamp (time, 5, 0, 0);
    fta->ComputeNewDelay (0, 5, networkID, address);  // [0,5]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 7.5

    param.type = NumberOfNodes; param.value = 4;
    fta->SetParameter (param);
    param.type = NumberOfFaultyNodes; param.value = 0;
    fta->SetParameter (param);
    interval.low.ts = 0; interval.high.ts = 0;
    result = fta->FTA (interval, networkID);  // return True
    // interval (low=high=refpoint) = {ms=0, ts=241 172 480}

    // tested on 02.07.98, 14:40: all tests okay so far


    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 3; SetTimestamp (time, 5, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 7.5

    param.type = NumberOfFaultyNodes; param.value = 1;
    fta->SetParameter (param);
    interval.low.ts = 0; interval.high.ts = 0;
    result = fta->FTA (interval, networkID);  // return True
    // interval = {0, 251 658 240}


    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 3; SetTimestamp (time, 5, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 7.5

    param.type = NumberOfFaultyNodes; param.value = 2;
    fta->SetParameter (param);
    interval.low.ts = 0; interval.high.ts = 0;
    result = fta->FTA (interval, networkID);  // return False
    // interval = {0, 0}


    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15

    param.type = NumberOfFaultyNodes; param.value = 1;
    fta->SetParameter (param);
    interval.low.ts = 0; interval.high.ts = 0;
    result = fta->FTA (interval, networkID);  // return True
    // interval = {0, 251 658 240}


    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20

    interval.low.ts = 0; interval.high.ts = 0;
    result = fta->FTA (interval, networkID);  // return False
    // interval = {0, 0}

    // tested on 05.05.97, 20:10: all tests okay


    //----------------------------------------------------------------------
    // test ConvergenceFunction()

    result = base->ConvergenceFunction ();  // return False (no assoc.nets)

    fta->SetAssociatedNetwork (1);
    // all data is stored for network 1, but the test of Set/GetAssoc.()
    // made network 2 the associated network

    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 15
    address[0] = 3; SetTimestamp (time, 5, 0, 0);
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 7.5
    address[0] = 4; SetTimestamp (time, 15, 0, 0);
    fta->ComputeNewDelay (0, 6, networkID, address);  // [0,6]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 18
    address[0] = 5; SetTimestamp (time, 20, 0, 0);
    fta->ComputeNewDelay (0, 5, networkID, address);  // [0,5]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 22.5
    address[0] = 6; SetTimestamp (time, 2, 0, 0);
    fta->ComputeNewDelay (0, 3, networkID, address);  // [0,3]
    fta->StoreTime (time, 0, networkID, address, area, True);  // mean = 3.5

    param.type = NumberOfNodes; param.value = 7;
    fta->SetParameter (param);
    param.type = NumberOfFaultyNodes; param.value = 2;
    fta->SetParameter (param);
    result = base->ConvergenceFunction ();  // return True
    // newTime: (ms,ts) = (0, 268 435 456), acc=0, timeValid = True

    // tested on 07.05.97, 18:50: all tests okay so far


    //----------------------------------------------------------------------
    // test SetNewClockValue() and the rest of ConvergenceFunction()

    base->SetNewClockValue ();
    // output: (ms,ts) = (0, 268 435 456), acc=0

    result = base->ConvergenceFunction ();  // return False
    // timeValid = False

    base->SetNewClockValue ();
    // no output (clock is not set)

    // tested on 07.05.97, 18:50: all tests okay


    delete base;


    //----------------------------------------------------------------------
    // test the creation function and the ctor of CMidpoint

    CMidpoint *midpoint;

    base = CMidpoint::CreateMidpoint (cs);
    midpoint = (CMidpoint *)base;
    // midpoint=True

    // tested on 30.07.97, 22:10: all tests okay


    //----------------------------------------------------------------------
    // test FTA() with midpoint=True
    // note: same example as in CFta, but the result should be less than
    // that obtained using the reference point, because the intervals are
    // not fully symmetrical: -G is added to the lower bound, whereas 0 is
    // added to the upper; so the result is -G/2 less than that above

    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    midpoint->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    midpoint->StoreTime (time, 0, networkID, address, area, True);  // 15
    address[0] = 1; SetTimestamp (time, 15, 0, 0);
    midpoint->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    midpoint->StoreTime (time, 0, networkID, address, area, True);  // 20
    address[0] = 2; SetTimestamp (time, 14, 0, 0);
    midpoint->ComputeNewDelay (0, 2, networkID, address);  // [0,2]
    midpoint->StoreTime (time, 0, networkID, address, area, True);  // 15
    address[0] = 3; SetTimestamp (time, 5, 0, 0);
    midpoint->ComputeNewDelay (0, 5, networkID, address);  // [0,5]
    midpoint->StoreTime (time, 0, networkID, address, area, True);  // 7.5

    param.type = NumberOfNodes; param.value = 4;
    midpoint->SetParameter (param);
    param.type = NumberOfFaultyNodes; param.value = 0;
    midpoint->SetParameter (param);
    interval.low.ts = 0; interval.high.ts = 0;
    result = midpoint->FTA (interval, networkID);  // return True
    // interval (low=high) = {ms=0, ts=241 172 479, us=2 147 483 648}

    // tested on 30.07.97, 22:20: all tests okay


    delete base;


    //----------------------------------------------------------------------
    // test the creation function and the ctor of CFreeRun

    CFreeRun *free;

    base = CFreeRun::CreateFreeRun (cs);
    free = (CFreeRun *)base;

    // tested on 30.07.97, 22:30: all tests okay


    //----------------------------------------------------------------------
    // test ConvergenceFunction()

    result = base->ConvergenceFunction ();  // return True
    // there should be no effect (timeValid is still False)

    // tested on 30.07.97, 22:30: all tests okay


    delete base;


    //----------------------------------------------------------------------
    // test the behaviour of the class CFtaWithFetzer
    //    correct behaviour of class ACClockSyncAlgorithm is assumed
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the creation function and the ctor of CFtaWithFetzer - calls
    //  Init(), so that is tested as well

    CFtaWithFetzer *fetzer;

    base = CFtaWithFetzer::CreateFtaWithFetzer (cs);
    fetzer = (CFtaWithFetzer *)base;
    // gpsdata is created, gpsdataIndex=0, initCalled = True

    // tested on 02.07.98, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test StoreTime()

    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);//15
    // should return True, gpsdataIndex = 1

    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    result = fetzer->StoreTime (time, 0, networkID, address, area, False);//15
    // should return True, gpsdataIndex is not changed

    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    // should all return True, gpsdataIndex = 10

    result = fetzer->StoreTime (time, 0, networkID, address, area, True);
    // should return False (array size exceeded)

    // tested on 02.07.98, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test Reset() - calls Dispose() and Init(), so they are tested as well

    fetzer->Reset();
    // initCalled = True, gpsdataIndex = 0

    // tested on 02.07.98, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test Suspend()

    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, True);
    // gpsdataIndex = 1

    fetzer->Suspend();
    // initCalled = True, gpsdataIndex = 0

    // tested on 02.07.98, 15:30: all tests okay


    //----------------------------------------------------------------------
    // test ConvergenceFunction()

    // DEBUGGER: set cs::roundPeriod = 100 --> M = 0.1

    fetzer->SetAssociatedNetwork (1);
    result = base->ConvergenceFunction ();  // return False
    // nothing should happen

    // tested on 02.07.98, 15:30: all tests okay so far

    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, False);  //15
    // gpsdataIndex is not changed

    result = base->ConvergenceFunction ();  // return True
    // newClockValue = 15 = {ms=0, ts=251 658 240, us=0}

    // tested on 02.07.98, 15:40: all tests okay so far

    fetzer->Reset(); fetzer->SetAssociatedNetwork (1);
    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, False);  //15
    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 12, networkID, address);  // [0,12]
    fetzer->StoreTime (time, 0, networkID, address, area, True);  // 16
    // should return True, gpsdataIndex = 1

    result = base->ConvergenceFunction ();  // return True
    // newClockValue = 15+0.1 = {ms=0, ts=253 335 961 , us=2 576 980 377}
    // real result: us=...352 (probably due to rounding errors when
    // computing the above values with the WinNT calculator)

    // tested on 24.07.98, 15:45: all tests okay so far

    fetzer->Reset(); fetzer->SetAssociatedNetwork (1);
    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, False);  //15
    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 8, networkID, address);  // [0,8]
    fetzer->StoreTime (time, 0, networkID, address, area, True);  // 14
    // should return True, gpsdataIndex = 1

    result = base->ConvergenceFunction ();  // return True
    // newClockValue = 15-0.1 = {ms=0, ts=249 980 518, us=1 717 986 918}
    // real result: us=...944

    // tested on 24.07.98, 16:10: all tests okay so far

    fetzer->Reset(); fetzer->SetAssociatedNetwork (1);
    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 9.9, networkID, address);  // [0,9.9]
    fetzer->StoreTime (time, 0, networkID, address, area, True);  // 14.95
    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, False);  //15

    result = base->ConvergenceFunction ();  // return True
    // newClockValue = 14.95 = {ms=0, ts=250 819 379, us=858 993 459}
    // real result: us=...408

    // tested on 24.07.98, 16:15: all tests okay so far

    fetzer->Reset(); fetzer->SetAssociatedNetwork (1);
    address[0] = 0; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 9.9, networkID, address);  // [0,9.9]
    fetzer->StoreTime (time, 0, networkID, address, area, True);  // 14.95
    address[0] = 1; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 10, networkID, address);  // [0,10]
    fetzer->StoreTime (time, 0, networkID, address, area, False);  //15
    address[0] = 2; SetTimestamp (time, 10, 0, 0);
    fetzer->ComputeNewDelay (0, 9.95, networkID, address);  // [0,9.95]
    fetzer->StoreTime (time, 0, networkID, address, area, True);  // 14.975

    result = base->ConvergenceFunction ();  // return True
    // newClockValue = 14.9625 = {ms=0, ts=251 029 094, us=1 717 986 918}
    // real result: us=...816

    // tested on 24.07.98, 16:15: all tests okay


    delete base;


    //----------------------------------------------------------------------

    return 0;
}


//--------------------------------------------------------------------------
// module global functions
//--------------------------------------------------------------------------

// converts the time to NTP-Time, and sets the 'acc' member of STimestamp
// accordingly (all times are in seconds)
void SetTimestamp (STimestamp& ntptime, double time, double accMinus,
                   double accPlus)
{
    SecondsToNtp (time, ntptime);  // convert the timestamp

    CNtpTime acc;

    SecondsToNtp (accPlus, acc);  // convert the positive accuracy
    // the ts (bits -8, -23) is the accuracy, which should be stored in
    // the lower 16 Bits of time.acc
    Assert (acc.ms == 0);
    Assert (acc.ts < 0x020000);  // at most bit -8 should be in use
    if (acc.ts & 1) acc.ts++;  // round up
    ntptime.acc = (acc.ts & 0x01FFFE) >> 1;

    SecondsToNtp (accMinus, acc);  // convert the negative accuracy
    // the ts (bits -8, -23) is the accuracy, which should be stored in
    // the upper 16 Bits of time.acc
    Assert (acc.ms == 0);
    Assert (acc.ts < 0x020000);  // at most bit -8 should be in use
    if (acc.ts & 1) acc.ts++;  // round up
    ntptime.acc += (acc.ts & 0x01FFFE) << 15;
}



//--------------------------------------------------------------------------
// implementation of the necessary functions of the other classes
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// CApplControlledObject

CApplControlledObject::CApplControlledObject (CControllingObject *ctrl) :
    CControlledObject (ctrl) {}
void CApplControlledObject::ReportEvent (EModule, EEvent) {}
void CApplControlledObject::ReportEvent (SEvent *event)
{
    Assert (event != NULL);
    cout << "event report: type = ";
    if (event->type == UserEvent) cout << "UserEvent";
    else cout << event->type;
    cout << ", module = ";
    if (event->module == ClockSync) cout << "ClockSync";
    else cout << event->module;
    cout << endl;
}

//--------------------------------------------------------------------------
// CClockInterface

CClockInterface::CClockInterface (CControllingObject *ctrl,
    double, int) : CApplControlledObject (ctrl) {}
CClockInterface::~CClockInterface () {}
EBool CClockInterface::SetParameter (const SParameter&) { return False; }
EBool CClockInterface::GetParameter (SParameter&) const { return False; }
EBool CClockInterface::ExecuteCommand (const SCommand&) { return False; }
double CClockInterface::GetRhoPlus() const {return 1e-3;}
double CClockInterface::GetRhoMinus() const {return 1e-3;}
INT32 CClockInterface::GetAccuracy() const {return 0;}
double CClockInterface::GetGranularity() const {return DefaultClockGranularity;}
double CClockInterface::GetGranularityAcc() const {return DefaultAccGranularity;}
double CClockInterface::GetGranularityOsc() const {return 1/25e6;}
EBool CClockInterface::GetTimestamp (STimestamp&) { return False; }
void CClockInterface::SetTimestamp (const STimestamp& time)
{
    cout << "clock: setting the timestamp to (ms,ts) = (";
    cout << time.ms << "," << time.ts << ")" << endl;
    cout << "with the accuracy ";
    cout << "acc = " << time.acc << endl;
}
void CClockInterface::SetTimestamp (const SExtendedTimestamp& time)
{
    cout << "clock: setting the timestamp to (ms,ts,us) = (";
    cout << time.ms << "," << time.ts << "," << time.us << ")" << endl;
    cout << "with the accuracy ";
    cout << "acc = " << time.acc << endl;
}
void CClockInterface::PreloadTimestamp (double at, const STimestamp& time)
{
    cout << "clock: preloading the timestamp to (ms,ts) = (";
    cout << time.ms << "," << time.ts << ")" << endl;
    cout << "with the accuracy ";
    cout << "acc = " << time.acc;
    cout << " to load at time " << at << endl;
}
void CClockInterface::PreloadTimestamp (double at,
  const SExtendedTimestamp& time)
{
    cout << "clock: preloading the timestamp to (ms,ts,us) = (";
    cout << time.ms << "," << time.ts << "," << time.us << ")" << endl;
    cout << "with the accuracy ";
    cout << "acc = " << time.acc;
    cout << " to load at time " << at << endl;
}
void CClockInterface::EnableInterrupt (EInterruptSource) {}
void CClockInterface::DisableInterrupt (EInterruptSource) {}
void CClockInterface::DisableDutyTimer (EInterruptSource) {}
EBool CClockInterface::InterruptPending (EInterruptSource) { return False; }
EBool CClockInterface::InterruptPending (EClockInterrupt) { return False;  }
void CClockInterface::ClearInterrupt (EInterruptSource) {}
void CClockInterface::ClearInterrupt (EClockInterrupt) {}
void CClockInterface::InterruptAfter (EInterruptSource, double) {}
double CClockInterface::InterruptAtMult (EInterruptSource, double)
  { return 0; }
void CClockInterface::InterruptAt (EInterruptSource, double) {}
void CClockInterface::Init () {}
void CClockInterface::Dispose() {}
void CClockInterface::isr() {}
void CClockInterface::wobble_isr() {}
void CClockInterface::Amortize (double, INT32, INT32) {}
void CClockInterface::PreloadAmort (double, INT32, INT32) {}
void CClockInterface::Crash () {}
void CClockInterface::Recover () {}
void CClockInterface::Reset () {}
int CClockInterface::ReportError (EReportType, char*, int, char*) {return 0;}
void CClockInterface::ReportCrashChange () {}


//--------------------------------------------------------------------------
// CClockSync

CClockSync::CClockSync (CControllingObject *ctrl, int)
    : CApplControlledObject (ctrl)
{
    listOfConnectedNetworks_ = new CNodeIdList;
    listOfConnectedNetworks_->Insert (1);
    listOfConnectedNetworks_->Insert (2);
}

CClockSync::~CClockSync () { delete listOfConnectedNetworks_; }

EBool CClockSync::SetParameter (const SParameter&) { return False; }
EBool CClockSync::GetParameter (SParameter&) const { return False; }
EBool CClockSync::ExecuteCommand (const SCommand& command)
{
    switch (command.type)
    {
        case ConnectNetwork:  // add a new ID
            listOfConnectedNetworks_->Insert (listOfConnectedNetworks_->GetLast()+1);
            break;
        case DisconnectNetwork:  // remove first ID
            listOfConnectedNetworks_->Remove (listOfConnectedNetworks_->GetFirst());
            break;
        default: return False;
    } // switch

    return True;
}
void CClockSync::SetClock (CClockInterface *clock) {clock_ = clock;}
void CClockSync::isr () {}
void CClockSync::Init () {}
void CClockSync::Dispose () {}
int CClockSync::ReportError (EReportType, char *, int, char *expression)
{
    cout << "CClockSync::ReportError() was called with expression: ";
    cout << expression << endl;
    return 0;
}


#include "inlines.hpp"


