//
// alistest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the array management classes in alist.hpp
//
// created: 25.7.96
//
// last changed: 29.7.96
//


#include "..\alist.hpp"


// this is a test class to check if the array handles its objects (which
// are defined as templates in the array management class) correctly
class TEST
{
    INT32 val1;
    double *val2;

public:

    TEST (INT32 a, double b) { val1 = a; val2 = new double(b); }
    ~TEST ();
};

TEST::~TEST ()
{
    delete val2;
}

 
int main()
{

    //----------------------------------------------------------------------
    // variables necessary for the test of alist.hpp
    //----------------------------------------------------------------------

    // necessary for the test of CArrayElement
    CArrayElement *el1;

    TEST *obj1;
    TEST *testobj;
    
    double index;


    // additionally necessary for the test of TCSortedArray
    TCSortedArray<TEST> *list;
    TCSortedArray<TEST> *noduplist;  // no duplicates
    TEST *obj2, *obj3, *obj4, *obj5;
    EBool test;


    //----------------------------------------------------------------------
    // test the behaviour of the class CArrayElement
    //----------------------------------------------------------------------

    // test the constructor
    el1 = new CArrayElement;  // make a new array element
    // objectID = noAccount, object = NULL

    obj1 = new TEST (11, 1.1);  // make a new object

    // test SetObject
    el1->SetObject (obj1, 1.3);  // set the object with index 1.3
    // object = obj1, objectID = 1.3
    
    testobj = (TEST *)el1->GetObject();  // should return obj1
    index = el1->GetObjectID ();  // should return 1.3



    delete el1;  // delete el1, but not obj1
    delete obj1;

    // tested on 27.7.96, 12:52: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of the class ACSortedArray
    // (correct behaviour of class CArrayElement is assumed)
    //----------------------------------------------------------------------

    // create five objects to store in the array
    obj1 = new TEST(11, 1.1);
    obj2 = new TEST(22, 2.2);
    obj3 = new TEST(33, 3.3);
    obj4 = new TEST(44, 4.4);
    obj5 = new TEST(55, 5.5);


    //----------------------------------------------------------------------
    // test the constructor and destructor

    // use default params
    list = new TCSortedArray<TEST>;
    // _firstFree = 0, _current = 0, _duplicates = False,
    // _size=_allocSize=ArrayAllocationSize. _array = *CArrayElement[_size]

    delete list;
    // this should call DeleteAll() and then the base constructor that
    // is responsible for deleting _array; trace into the functions to
    // verify that!

    // use duplicates
    list = new TCSortedArray<TEST>(True);
    // same as above, but _duplicates = True

    delete list;

    // use another alloc size
    list = new TCSortedArray<TEST>(2);
    // _firstFree = 0, _current = 0, _duplicates = False,
    // _size=_allocSize=2. _array = *CArrayElement[_size]

    // test FindNext when no duplicates are allowed
    testobj = list->FindNext();  // should return NULL

    delete list;

    // use another alloc size and allow duplicates
    list = new TCSortedArray<TEST>(True, 2);
    // _firstFree = 0, _current = 0, _duplicates = True,
    // _size=_allocSize=2. _array = *CArrayElement[_size]


    //----------------------------------------------------------------------
    // test the status functions in case of an empty array

    // test Empty on empty array
    test = list->Empty();  // should return True
    // test BeginOfArray on empty list
    test = list->BeginOfArray();  // should return False
    // test EndOfArray on empty list
    test = list->EndOfArray();  // should return False


    //----------------------------------------------------------------------
    // test the access functions in case of an empty array

    // test GetFirst on empty array
    testobj = list->GetFirst();  // should return NULL
    // test GetLast on empty array
    testobj = list->GetLast();  // should return NULL
    // test GetNext on empty array
    testobj = list->GetNext();  // should return NULL
    // test GetPrev on empty array
    testobj = list->GetPrev();  // should return NULL

    // test Find on empty array
    testobj = list->Find(1);  // should return NULL
    // test FindNext on empty array
    testobj = list->FindNext();  // should return NULL

    // test GetCurrentObject on empty array
    testobj = list->GetCurrentObject();  // should return NULL
    // test GetCurrentObjectID on empty array
    index = list->GetCurrentObjectID();  // should return NoAccount


    // tested on 27.7.96, 22:52: all tests okay so far


    //----------------------------------------------------------------------
    // test Insert() - Find() and InsertAtCurrent() are called
    
    // test Insert with NULL
    test = list->Insert (NULL, 3);  // should return False
    
    // test Insert with new object ID
    test = list->Insert (obj3, 3);  // should return True
    // current = 0, firstFree = 1

    // test Insert with new object ID
    test = list->Insert (obj1, 1);  // should return True
    // current = 0, firstFree = 2, size=4, array points to a new location
    // InsertAtCurrent, AllocateMemory, UpdateArray tested completely

    //----------------------------------------------------------------------
    // test the status functions in case of a non-empty array

    // test Empty
    test = list->Empty();  // should return False
    // Empty tested completely
    
    // test BeginOfList
    test = list->BeginOfArray();  // should return True
    // test EndOfList
    test = list->EndOfArray();  // should return False


    //----------------------------------------------------------------------
    // test the access functions in case of a non-empty array

    // test GetFirst
    testobj = list->GetFirst();  // should return obj1
    // current=0
    // GetFirst tested completely
    
    // test GetLast
    testobj = list->GetLast();  // should return obj3
    // current=1
    // GetLast tested completely

    // test BeginOfArray
    test = list->BeginOfArray();  // should return False
    // BeginOfArray tested completely

    // test EndOfArray
    test = list->EndOfArray();  // should return True
    // EndOfArray tested completely

    // test GetNext
    testobj = list->GetNext();  // should return obj3
    // test GetPrev
    testobj = list->GetPrev();  // should return obj1
    // current=0
    testobj = list->GetPrev();  // should return obj1
    // GetPrev tested completely

    // test GetNext
    testobj = list->GetNext();  // should return obj3
    // current=1
    // GetNext tested completely


    // test Find
    testobj = list->Find(3);  // should return obj3
    // test FindNext
    testobj = list->FindNext();  // should return NULL
    // test Find
    testobj = list->Find(1);  // should return obj1
    // current=0
    
    // test FindNext
    testobj = list->FindNext();  // should return NULL
    // test Find
    testobj = list->Find(3);  // should return obj3
    // current=1

    // test GetCurrentObject
    testobj = list->GetCurrentObject();  // should return obj3
    // GetCurrentObject, CurrentIsValid, GetCurrent tested completely
    
    // test GetCurrentObjectID
    index = list->GetCurrentObjectID();  // should return 3
    // GetCurrentObjectID, GetCurrentID tested completely


    // tested on 28.7.96, 09:02: all tests okay so far


    //----------------------------------------------------------------------
    // test Insert() - continued
    
    // test Insert with duplicate
    test = list->Insert (obj2, 1);  // should return True
    // current=0, firstFree=3

    // test FindNext
    testobj = list->FindNext();  // should return obj1
    // current=1
    // FindNext tested completely

    // test Find
    testobj = list->Find(1);  // should return obj2
    // current=0

    // test Find
    testobj = list->Find(3);  // should return obj3
    // current=2

    testobj = list->GetFirst();
    // current=0
    
    test = list->Insert (obj5, 5);  // should return True
    // current=3, firstFree=4, size=6

    testobj = list->GetFirst();
    // current=0

    test = list->Insert (obj4, 4);  // should return True
    // current=3, firstFree=5

    testobj = list->GetFirst();
    // current=0

    // test Find
    testobj = list->Find(3);  // should return obj3
    // current=2

    // test Find
    testobj = list->Find(3);  // should return obj3
    // current=2

    testobj = list->GetLast();
    // current=4

    // test Find
    testobj = list->Find(1);  // should return obj2
    // current=0

    // test Find
    testobj = list->Find(6);  // should return NULL
    // current=5

    // test Find
    testobj = list->Find(5);  // should return obj5
    // current=4

    // Find completely tested if duplicates are allowed


    noduplist = new TCSortedArray<TEST>(2);
    // _firstFree = 0, _current = 0, _duplicates = False,
    // _size=_allocSize=2. _array = *CArrayElement[_size]

    test = noduplist->Insert (obj1, 1);  // should return True
    // firstFree = 1
    test = noduplist->Insert (obj2, 1);  // should return False
    
    test = noduplist->Insert (obj3, 3);
    test = noduplist->Insert (obj4, 4);
    test = noduplist->Insert (obj5, 5);
    // current=3, firstFree=4, size=6

    testobj = noduplist->Find(1);  // should return obj1
    // current = 0

    // Find tested completely --> Insert tested completely

    // tested on 28.7.96, 11:22: all tests okay so far


    //----------------------------------------------------------------------
    // test Remove() - Find() and RemoveAtCurrent() are called
    // correct behaviour of Find() is assumed

    testobj = noduplist->Remove (6);  // should return NULL
    // current = 4

    testobj = noduplist->Remove (5);  // should return obj5
    // current = 2, firstFree = 3

    testobj = noduplist->Remove (3);  // should return obj3
    // current = 1, firstFree = 2, size = 4, array points to new location

    testobj = noduplist->Remove (4);

    testobj = noduplist->Remove (1);  // should return obj1
    // current=firstFree = 0, size = 2, array points to new location

    // Remove(), RemoveAtCurrent() and FreeMemory() tested completely

    delete noduplist;
    // note: I have removed all elements from the list before calling the
    // destructor, because the delete destroys the objects as well


    //----------------------------------------------------------------------
    // test Delete() - Remove() is called
    // correct behaviour of Remove() is assumed

    test = list->Delete (6);  // should return False
    // current = 4

    test = list->Delete (1);  // should return True
    // current=0, firstFree=3

    testobj = list->Find(1);  // should return NULL

    // Delete() tested completely


    //----------------------------------------------------------------------
    // test DeleteAll() - Remove(), Empty(), GetLast() are called
    // correct behaviour of the called fuctions is assumed

    test = list->DeleteAll();  // should return True
    // current=firstFree = 0, size = 2, array points to a new location
    
    test = list->DeleteAll();  // should return False

    // DeleteAll() tested completely
    
    // tested on 29.7.96, 11:14: all tests okay


    delete list;
    
    //----------------------------------------------------------------------

    
    return 0;
}
    


