//
// citest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the ci functions in ci.hpp
//
// created: 09.11.96
//
// last change: 07.12.97
//


#include "..\ci.hpp"
#include "..\ctrling.hpp"
#include "..\network.hpp"
#include "..\sim_glob.hpp"

#define Need_ACClockSyncAlgorithm;
#define Need_ACNetwork;
#define Need_CApplControlledObject;
#define Need_CBroadcastNetwork;
#define Need_CBitmask;
#define Need_CClassManager;
#define Need_CClockInterface;
#define Need_CControlledObject;
#define Need_CCrashingModule;
#define Need_CLance
#define Need_CNtpTime


extern INT32 ci_buf_size (INT16 size);
// only defined in ci.cpp


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of the ci functions
    // correct behaviour of CNetworkList, the ctor of ACNetwork, SMsg and
    // the global function GetCurrentNodeID() is assumed
    //----------------------------------------------------------------------

    CBroadcastNetwork network (NULL, 0);  // make a dummy network
    bufx *buf;
    INT16 size;
    int mode;
    tenvx *tenv;
    int result;
    ciparam param;  // needed for all CI functions


    //----------------------------------------------------------------------
    // test ci_dispatcher()
    // note: only the errors; if the functions really calls the CI
    //       functions is tested below (implicitly)

    result = ci_dispatcher (CI_INIT, NULL);  // should return -10

    result = ci_dispatcher (150, &param);  // should return -12

    // tested on 07.12.96, 11:50: all tests okay


    //----------------------------------------------------------------------
    // test ci_init()

    listOfNetworks.Remove (0);  // remove the network from the list

    param.inip.if_num = 0;
    param.inip.ci_announce_address = (ci_fn)1;
    param.inip.ci_announce_err_add = (ci_efn)1;

    result = ci_dispatcher (CI_INIT, &param);  // should return -9

    listOfNetworks.Insert (&network, 0);  // now network with ID 0 exists

    param.inip.ci_announce_address = NULL;
    result = ci_dispatcher (CI_INIT, &param);  // should return -10

    param.inip.ci_announce_address = (ci_fn)1;
    param.inip.ci_announce_err_add = NULL;
    result = ci_dispatcher (CI_INIT, &param);  // should return -10

    param.inip.ci_announce_err_add = (ci_efn)1;
    result = ci_dispatcher (CI_INIT, &param);  // should return 0

    // tested on 07.12.97, 12:10: all tests okay


    //----------------------------------------------------------------------
    // test ci_buf_size()

    int bsize = B_SIZE;  // needed for the computation of the sizes

    result = ci_buf_size (0);  // should return 0
    result = ci_buf_size (bsize);  // should return bsize
    result = ci_buf_size (bsize+1);  // should return 2*bsize

    // tested on 07.01.97, 16:05: all tests okay


    //----------------------------------------------------------------------
    // test ci_get_... ()

    size = bsize+1;
    mode = TS_PACKET | RET_PKB_IMM;

    param.gtbp.size = size;
    param.gtbp.buf = NULL;
    param.gtbp.if_num = 0;
    param.gtbp.area = 0;

    result = ci_dispatcher (CI_GTB, &param); // should return -10

    param.gtbp.buf = &buf;
    param.gtbp.if_num = 1;
    result = ci_dispatcher (CI_GTB, &param); // should return -9

    param.gtbp.if_num = 0;
    param.gtbp.area = MAXASICS;
    result = ci_dispatcher (CI_GTB, &param); // should return -9

    param.gtbp.size = 0;
    param.gtbp.area = 0;
    result = ci_dispatcher (CI_GTB, &param); // should return 0
    // buf = NULL

    param.gtbp.size = size;
    result = ci_dispatcher (CI_GTB, &param); // should return 0


    param.gtep.mode = mode;
    param.gtep.tenv = NULL;
    param.gtep.if_num = 0;
    param.gtep.area = 0;

    result = ci_dispatcher (CI_GTE, &param); // should return -10

    param.gtep.tenv = &tenv;
    param.gtep.if_num = 1;
    result = ci_dispatcher (CI_GTE, &param); // should return -9

    param.gtep.if_num = 0;
    param.gtep.area = MAXASICS;
    result = ci_dispatcher (CI_GTE, &param); // should return -9

    param.gtep.area = 0;
    result = ci_dispatcher (CI_GTE, &param); // should return 0
    // tenv->lengthfield = mode

    // tested on 18.01.97, 13:35: all tests okay


    //----------------------------------------------------------------------
    // test ci_send ()

    listOfNetworks.Remove (0);  // remove the network from the list

    param.sendp.mode = mode;
    param.sendp.tenv = tenv;
    param.sendp.buf = buf;
    param.sendp.size = size;
    param.sendp.if_num = 0;
    param.sendp.area = 0;

    result = ci_dispatcher (CI_SEND, &param); // should return -9

    param.sendp.if_num = 2;
    result = ci_dispatcher (CI_SEND, &param); // should return -9

    listOfNetworks.Insert (&network, 0);  // again insert network

    param.sendp.if_num = 0;
    param.sendp.area = MAXASICS;
    result = ci_dispatcher (CI_SEND, &param); // should return -9

    param.sendp.tenv = NULL;
    param.sendp.area = 0;
    result = ci_dispatcher (CI_SEND, &param); // should return -4

    param.sendp.tenv = tenv;
    result = ci_dispatcher (CI_SEND, &param); // should return 0
    // buf and tenv should not be returned (trace into the code!)

    param.sendp.mode = mode|RET_PKB_IMM;
    result = ci_dispatcher (CI_SEND, &param); // should return 0
    // should return 0, buf and tenv should be returned (trace it!)

    // tested on 18.01.97, 13:35: all tests okay


    //----------------------------------------------------------------------
    // test ci_ret_... ()

    param.gtbp.size = size;
    param.gtbp.buf = &buf;
    param.gtbp.if_num = 0;
    param.gtbp.area = 0;

    result = ci_dispatcher (CI_GTB, &param);  // request the buffers anew

    param.gtep.mode = mode;
    param.gtep.tenv = &tenv;
    param.gtep.if_num = 0;
    param.gtep.area = 0;

    result = ci_dispatcher (CI_GTE, &param);


    param.rtbp.buf = NULL;
    result = ci_dispatcher (CI_RTB, &param);  // should return -7

    param.rtbp.buf = buf;
    result = ci_dispatcher (CI_RTB, &param);  // should return 0

    param.rtep.tenv = NULL;
    result = ci_dispatcher (CI_RTE, &param);  // should return -8

    param.rtep.tenv = tenv;
    result = ci_dispatcher (CI_RTE, &param);  // should return 0

    // tested on 07.01.97, 21:30: all tests okay


    listOfNetworks.Remove (0);
    // remove the network from the list (it is static and autom. destroyed)


    //----------------------------------------------------------------------

    return 0;
}

#include "inlines.hpp"

