//
// clk2test.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test CClockInterface
// module internal function SourceToMask() was made global for the test
//
// created: 27.03.97
//
// last change: 04.02.99
//

#include <iostream.h>
#include <math.h>
#include <stdlib.h>

#include "..\clock_sw.hpp"
#include "..\clockcfg.hpp"
#include "..\clockifc.hpp"
#include "..\clockirq.hpp"
#include "..\constant.hpp"
#include "..\ctrling.hpp"
#include "..\sim_glob.hpp"
#include "..\time.hpp"
#include "settime.hpp"


#define Need_CControllingObject

//#define TEST_STATIC_FUNCS
// uncomment it to test the static functions


CClockList listOfClocks;
CClock *g_masterClock = NULL;  // not needed
unsigned long g_streamOffset = 0;  // for distrib.cpp

extern INT32 SourceToMask (EInterruptSource source);


// copied from clock_sw.cpp; if the struct is changed there it should be
// changed here as well!
// note: the struct is needed here because otherwise the WATCOM debugger
// does not know the type of clock->internals_

//**************************************************************************
// struct SInternalRegisters: internal clock registers that have to be
//      simulated in the software clock
//**************************************************************************

struct SInternalRegisters
{
    // NTPTIME
    INT32 ms;  // CS and MS [+31,+8]
    INT32 ts;  // [+7,-24]
    INT32 us;  // [-25,-56]
    INT32 ns;  // [-57,-59]
    EBool overflow;  // set to True by UpdateNtp() if an overflow occured

    // STEP value during pure phase
    INT32 STEPPUREACT;  // [-20,-51]

    // deterioration values for the accuracies during pure phase
    INT16 LAMBDAPPUREACT;  // [-37,-51] (signed)
    INT16 LAMBDANPUREACT;

    INT32 ALPHAP;  // accuracy envelopes [-8,-38] (signed)
    INT32 ALPHAN;
    INT16 NALPHAP;  // fractional part of the accuracies [-39,-51]
    INT16 NALPHAN;

    // amortization timer
    INT32 AMORTTIMER;
};


const double TwoPow51 = 2.251799813685e15;


void SetISR (int vector, CNeedInterrupt *obj, CallFunc caller)
{
    cout << "SetISR() called with vector = " << vector;
    cout << ", @obj = " << (long)obj;
    cout << ", @callerfct = " << (long)caller << endl;
}

void RemoveISR (int vector, CNeedInterrupt *obj, CallFunc caller)
{
    cout << "RemoveISR() called with vector = " << vector;
    cout << ", @obj = " << (long)obj;
    cout << ", @callerfct = " << (long)caller << endl;
}

void InvokeISR (int vector)
{
    cout << "InvokeISR() called with vector = " << vector << endl;
}

void CallISR (CNeedInterrupt *obj)
{
    obj->isr();  // call the isr of the object
}


/*
void TestFunction (const int *i)
// read: *i is a constant int
{
    i++;   // that's allowed
//    *i=3;  // not allowed
}

void TestFunction (const int *const i)
// read: *i is a constant int, i is a constant int*
{
//    i++;   // not allowed
//    *i=3;  // not allowed
}
*/


int main()
{
    //----------------------------------------------------------------------
    // test the computation of step and steplow using the clock frequency
    //----------------------------------------------------------------------

    double f = 25e6;
    double dummy;
    INT8 steplow = (INT8)(modf (TwoPow51/f, &dummy) * 256);
    INT32 step = (INT32)dummy;
    // step = 90 071 992, steplow = 140

    // tested on 16.03.97, 17:50: all tests okay


#ifdef TEST_STATIC_FUNCS

    //----------------------------------------------------------------------
    // test the global functions in clockifc.cpp
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test SourceToMask()

    INT32 retval;

    retval = SourceToMask (IntSnapshot);  // return 0x10000
    retval = SourceToMask (IntDutyB1);  // return 0x04000000
    retval = SourceToMask (IntDutyB4);  // return 0x20000000
    retval = SourceToMask (IntDutyB5);  // return 0x40000000

    // tested on 07.12.97, 13:40: all tests okay

    retval = SourceToMask (IntGps1PPS1);  // return 0x00000001
    retval = SourceToMask (IntGps1PPS2);  // return 0x00000002
    retval = SourceToMask (IntGps1PPS3);  // return 0x00000004
    retval = SourceToMask (IntDutyA1);  // return 0x00100000
    retval = SourceToMask (IntDutyA2);  // return 0x00200000
    retval = SourceToMask (IntDutyA3);  // return 0x00400000
    retval = SourceToMask (IntDutyA4);  // return 0x00800000
    retval = SourceToMask (IntDutyA5);  // return 0x01000000
    retval = SourceToMask (IntDutyA6);  // return 0x02000000
    retval = SourceToMask (IntDutyB2);  // return 0x08000000
    retval = SourceToMask (IntDutyB3);  // return 0x10000000
    retval = SourceToMask (IntDutyB6);  // return 0x80000000

    // tested on 28.07.98, 16:20: all tests okay

#endif

    //----------------------------------------------------------------------
    // test CClockInterface
    //    correct behaviour of CSoftwareClock is assumed
    //    correctness of EInterruptBits in clockcfg.hpp is assumed
    //----------------------------------------------------------------------

    CClockInterface *clock;
    CControllingObject *ctrl = new CControllingObject ("QNAM", NULL, 2);
    EBool result;
    SSingleParameter param;


    //----------------------------------------------------------------------
    // test the ctor, Init() and InitializeClock()

    cout << endl << "testing the ctor" << endl;

    clock = new CClockInterface (ctrl, 25e6, 3);
    // rhoPlus=rhoMinus = 1e-4, steplow = 0x8C000000,
    // steppure = 90 071 992, gps_ = NULL, gpsStatus = False, area = 3,
    // amorttime = 1 (DefaultAmortizationDuration),
    // deltaSteppure=deltaSteplow = 0, deltaRho = 0, lambdaP=lambdaN = 0,
    // wobbleData = {{Uniform,0,0,0,0,0}, period=0, rhoMin=rhoMax=0,
    //               addDrift=True},
    // pFast=pSlow=pDriftRate=pReadFault=pWriteFault = 0,
    // distFast=distRegular=distSlow = {Uniform,RhoPlus/0/RhoMinus,0,0}
    // clock: frequency = 25e6, MSSET = 0x8C000000 (steplow),
    // STEPPURE = 90 071 992, UTCINTEN1 = 0x10000 (snapshot enabled),
    // UTCINTEN2 = 0x80000000 (DTB6 enabled), STARTAMORT = 0
    // output: SetISR() called with ... [snapshot ISR]
    // output: SetISR() called with ... [wobble ISR]
    listOfClocks.Insert (clock, 2, 3);

    // tested on 23.09.98, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test GetClock()

    CSoftwareClock *swclock = (CSoftwareClock *)clock->GetClock();
    // internals_->AMORTTIMER = 25e6, internals_->STEPPUREACT = 90 071 992,
    // internals_->ALPHAP=ALPHAN=0x7FFFFFFF
    // nodeID = 2, area = 3

    // tested on 10.02.98, 12:00: all tests okay


    //----------------------------------------------------------------------
    // test AddGps() and RemoveGps()

    cout << endl << "testing AddGps()" << endl;

    clock->AddGps (GPU1);
    // gps != NULL, amortTime = 0.99
    // output: SetISR() called with ...

    clock->AddGps (GPU3);
    // output: SetISR() called with ... (same isr() as above)


    cout << endl << "testing RemoveGps()" << endl;

    clock->RemoveGps (GPU1);
    // output: RemoveISR() called with ... (see AddISR() of GPU1)
    clock->RemoveGps (GPU2);
    clock->RemoveGps (GPU3);
    // output: RemoveISR() called with ... (see AddISR() of GPU3)
    // gps == NULL

    // tested on 16.01.98, 17:30: all tests okay


    //----------------------------------------------------------------------
    // test SetParameter()

    STimeParameter timeparam;
    timeparam.module = Clock;

    param.module = Clock;
    param.type = RhoMinus;
    param.value = 0.00009;
    result = clock->SetParameter (param);  // return False (read-only)

    param.type = RhoPlus;
    param.value = 0.000013;
    result = clock->SetParameter (param);  // return False (read-only)

    // tested on 30.07.98, 16:40: all tests okay so far

    param.type = Accuracy;
    param.value = 0x76543210;
    result = clock->SetParameter (param);  // return True
    // swclock->internals->ALPHAP,ALPHAN = 0x76543210

    timeparam.type = ClockValue;
    timeparam.macrostamp = 0x11114711;
    timeparam.timestamp = 0x22223634;
    result = clock->SetParameter (timeparam);  // return True
    // swclock->internals->MS = 0x00114711, TS = 0x87654321, US = 0

    // tested on 08.05.97, 09:20: all tests okay so far

    param.type = AmortizationDuration;
    param.value = 7.6;
    result = clock->SetParameter (param);  // return True
    // amortTime = 7.6

    // tested on 09.08.97, 11:20: all tests okay so far

    param.type = ClockDrift;
    param.value = 1e-6;
    result = clock->SetParameter (param);  // return True
    // swclock->fOsc = 25000025

    // tested on 08.05.97, 09:20: all tests okay so far

    param.type = PFast;
    param.value = -0.2;
    result = clock->SetParameter (param);  // return False
    param.value = 2.0;
    result = clock->SetParameter (param);  // return False
    param.value = 0.2;
    result = clock->SetParameter (param);  // return True
    // pFast = 0.2

    param.type = PSlow;
    param.value = -0.3;
    result = clock->SetParameter (param);  // return False
    param.value = 3.0;
    result = clock->SetParameter (param);  // return False
    param.value = 0.9;
    result = clock->SetParameter (param);  // return False (0.9+pFast > 1)
    param.value = 0.3;
    result = clock->SetParameter (param);  // return True
    // pSlow = 0.3

    param.type = PFast;
    param.value = 0.8;
    result = clock->SetParameter (param);  // return False (0.8+pFast > 1)

    // tested on 04.02.98, 20:40: all tests okay so far

    param.type = PDriftChange;
    param.value = -0.4;
    result = clock->SetParameter (param);  // return False
    param.value = 1.1;
    result = clock->SetParameter (param);  // return False
    param.value = 1;
    result = clock->SetParameter (param);  // return True
    // pDriftChange = 1

    param.type = PReadFault;
    param.value = -0.4;
    result = clock->SetParameter (param);  // return False
    param.value = 1.1;
    result = clock->SetParameter (param);  // return False
    param.value = 0.7;
    result = clock->SetParameter (param);  // return True
    // pReadFault = 0.7

    param.type = PWriteFault;
    param.value = -0.4;
    result = clock->SetParameter (param);  // return False
    param.value = 1.1;
    result = clock->SetParameter (param);  // return False
    param.value = 0.1;
    result = clock->SetParameter (param);  // return True
    // pWriteFault = 0.1

    // tested on 30.07.98, 16:50: all tests okay so far

    SApplDistribution dist;
    dist.module = Clock;
    dist.type = DistributionFast;
    dist.data.dist = Exponential;
    dist.data.low = 1e-5;
    dist.data.high = 2e-5;
    dist.data.constant = 3e-5;
    dist.data.mean = 1.5e-5;  // int = (4,5)e-5 with m=4.5e-5
    dist.data.freeparam = 2;

    dist.data.constant = 4;  // constant too high
    result = clock->SetParameter (dist);  // return False
    // constant+high > WcRhoPlus

    dist.data.constant = 3e-5;
    result = clock->SetParameter (dist);  // return True
    // distFast = ..., rhoPlus = 4e-5,
    // distReg.high = rhoPlus

    // tested on 30.07.98, 16:50: all tests okay so far

    dist.type = DistributionSlow;
    dist.data.constant = 4;  // constant too high
    result = clock->SetParameter (dist);  // return False
    // constant+high > WcRhoMinus

    dist.data.constant = 5e-5;  // int = (6,7)e-5 with m=6.5e-5
    result = clock->SetParameter (dist);  // return True
    // distSlow = ..., rhoMinus = 6e-5
    // distReg: low = 0, high = 1e-4, const = -6e-5, mean=6e-5

    // tested on 30.07.98, 16:50: all tests okay so far

    dist.type = DistributionRegular;
    dist.data.constant = 4;
    result = clock->SetParameter (dist);  // return False
    // constant+mean not in (-rhoMinus, rhoPlus)

    dist.data.constant = -5e-5;
    dist.data.mean = 1e-4;
    result = clock->SetParameter (dist);  // return False
    // constant+mean not in (-rhoMinus, rhoPlus)

    dist.data.mean = 1e-5;
    result = clock->SetParameter (dist);  // return True
    // distReg: low=0, high=1e-4, const=-6e-5, mean=2e-5
    //          freeparam and dist are taken from the data

    // tested on 30.07.98, 16:50: all tests okay so far

    param.type = PRecover;
    param.value = 0.4;
    result = clock->SetParameter (param);  // return True
    // pRecover = 0.4 (inherited member)

    // tested on 04.02.98, 19:40: all tests okay so far

    SApplWobbleDrift paramWobble;
    paramWobble.module = Clock;
    paramWobble.type = WobbleDrift;
    paramWobble.period = 10;
    paramWobble.rhoMin = 1e-5;
    paramWobble.rhoMax = 2e-5;
    paramWobble.data.dist = Exponential;
    paramWobble.data.low = 1e-6;
    paramWobble.data.high = 2e-6;
    paramWobble.data.mean = 1e-6;
    paramWobble.data.constant = 1e-8;
    paramWobble.data.freeparam = 1;
    result = clock->SetParameter (paramWobble);  // return True
    // wobbleData = ... (above values)
    // clock_->utcsuWrite->DUTYB6L is enabled (bit 0x10000 is set),
    // ...->DUTYBH is set to 0x11471128 (HIGH-value has granularity
    // of 1 sec, and current time in seconds is 0x11471122; the
    // 0x22 is 34 seconds, so the next multiple of 10 is 40 seconds
    // = 0x28)

    // now check setting invalid data
    paramWobble.period = -10;  // negative period
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.period = 20;  // period is okay
    paramWobble.rhoMin = 2e-3;  // lower drift above rhoMinus
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.rhoMin = -1;  // lower drift is negative
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.rhoMin = 1e-5;  // lower drift is okay
    paramWobble.rhoMax = 1e-2;  // upper drift above rhoPlus
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.rhoMax = -1;  // upper drift negative
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.rhoMax = 2e-5;  // upper drift is okay
    paramWobble.data.high = 2e-7;  // distribution data not valid
    result = clock->SetParameter (paramWobble);  // return False
    // no changes to wobbleData and DUTYB6L

    paramWobble.data.high = 2e-6;  // distribution data is okay
    paramWobble.period = 0;  // period is zero
    result = clock->SetParameter (paramWobble);  // return True
    // wobbleData.period = 0
    // clock_->utcsuWrite->DUTYB6L is disabled (bit 0x10000 is not set)

    // tested on 22.09.98, 11:30: all tests okay so far

    // check the adaption of wobble bounds if rhoPlus/rhoMinus is changed

    dist.type = DistributionFast;
    dist.data.low = 1.5e-5;
    dist.data.mean = 1.5e-5;  // was changed in test of DistributionRegular
    dist.data.constant = 3e-5;  // was also changed in this test
    result = clock->SetParameter (dist);  // return True
    // rhoPlus = 4.5e-5 (higher), wobbleData does not change

    dist.data.constant = 0;
    result = clock->SetParameter (dist);  // return True
    // rhoPlus = 1.5e-5, wobbleData.rhoMax = 1.5e-5

    dist.data.low = 1e-5;
    dist.data.high = 2e-5;
    dist.data.constant = 3e-5;
    result = clock->SetParameter (dist);  // return True
    // set fast drift to their old values (they are checked in the
    // test of GetParameter())

    dist.type = DistributionSlow;
    dist.data.constant = 5e-5;
    dist.data.low = 1.5e-5;
    result = clock->SetParameter (dist);  // return True
    // rhoMinus = 6.5e-5 (higher), wobbleData does not change

    dist.data.constant = 0;
    dist.data.low = 0.5e-5;
    result = clock->SetParameter (dist);  // return True
    // rhoMinus = 0.5e-5, wobbleData.rhoMin = 0.5e-5

    dist.data.low = 1e-5;
    dist.data.high = 2e-5;
    dist.data.constant = 5e-5;
    result = clock->SetParameter (dist);  // return True
    // set slow drift to their old values (they are checked in the
    // test of GetParameter())

    // tested on 22.09.98, 11:50: all tests okay


    //----------------------------------------------------------------------
    // test GetParameter()
    //  note: 'Accuracy' calls GetAccuracy(), so that is tested as well

    param.type = RhoMinus;
    result = clock->GetParameter (param);  // return True
    // value = 6e-5

    param.type = RhoPlus;
    result = clock->GetParameter (param);  // return True
    // value = 4e-5

    // tested on 22.09.98, 11:50: all tests okay so far

    param.type = Accuracy;
    result = clock->GetParameter (param);  // return True
    // value = 0xECA8ECA8 (= 3 970 493 608)

    timeparam.type = ClockValue;
    timeparam.macrostamp = 0;
    timeparam.timestamp = 0;
    result = clock->GetParameter (timeparam);  // return True
    // macrostamp = 0x00114711, timestamp = 0x22223634

    // tested on 08.05.97, 10:20: all tests okay

    param.type = AmortizationDuration;
    result = clock->GetParameter (param);  // return True
    // value = 7.6

    // tested on 09.08.97, 11:20: all tests okay so far

    param.type = ClockDrift;
    result = clock->GetParameter (param);  // return True
    // value = 1e-6

    // tested on 08.05.97, 10:20: all tests okay

    param.type = PFast;
    result = clock->GetParameter (param);  // return True
    // value = 0.2

    param.type = PSlow;
    result = clock->GetParameter (param);  // return True
    // value = 0.3

    param.type = PDriftChange;
    result = clock->GetParameter (param);  // return True
    // value = 1

    param.type = PReadFault;
    result = clock->GetParameter (param);  // return True
    // value = 0.7

    param.type = PWriteFault;
    result = clock->GetParameter (param);  // return True
    // value = 0.1

    dist.data.dist = Uniform;
    dist.data.low=dist.data.high=dist.data.constant=dist.data.mean=dist.data.freeparam=0;
    dist.type = DistributionFast;
    result = clock->GetParameter (dist);  // return True
    // data: Exponential, low = 1e-5, high = 2e-5, const = 3e-5, mean=1.5e-5, freeparam=2

    dist.data.dist = Uniform;
    dist.data.low=dist.data.high=dist.data.constant=dist.data.mean=dist.data.freeparam=0;
    dist.type = DistributionSlow;
    result = clock->GetParameter (dist);  // return True
    // data: Exponential, low = 1e-5, high = 2e-5, const = 5e-5, mean=1.5e-5, freeparam=2

    dist.data.dist = Uniform;
    dist.data.low=dist.data.high=dist.data.constant=dist.data.mean=dist.data.freeparam=0;
    dist.type = DistributionRegular;
    result = clock->GetParameter (dist);  // return True
    // data: Exponential, low = 0, high = 1e-4, const = -6e-5, mean=2e-5, freeparam=2

    // tested on 30.07.98, 17:00: all tests okay so far

    param.type = PRecover;
    result = clock->GetParameter (param);  // return True
    // value = 0.4

    // tested on 04.02.98, 19:40: all tests okay

    paramWobble.period = -1;
    paramWobble.rhoMin = 0;
    paramWobble.rhoMax = 0;
    paramWobble.data.dist = Uniform;
    paramWobble.data.low = 0;
    paramWobble.data.high = 0;
    paramWobble.data.mean = 0;
    paramWobble.data.constant = 0;
    paramWobble.data.freeparam = 0;
    result = clock->GetParameter (paramWobble);  // return True
    // period=10, rhoMin=0.5e-5, rhoMax = 1.5e-5,
    // dist={Exp,1e-6,2e-6,1e-6,1,1e-8}

    // tested on 22.09.98, 12:00: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand()

    SGpsConnectCommand gpscon;
    gpscon.type = ConnectGps;

    gpscon.gpu = EGpsUnitCount;
    result = clock->ExecuteCommand (gpscon);  // return False

    gpscon.gpu = GPU1;
    result = clock->ExecuteCommand (gpscon);  // return True
    // gps != NULL, amortTime = 0.99

    gpscon.gpu = GPU2;
    result = clock->ExecuteCommand (gpscon);  // return True


    gpscon.type = DisconnectGps;

    gpscon.gpu = EGpsUnitCount;
    result = clock->ExecuteCommand (gpscon);  // return False

    gpscon.gpu = GPU1;
    result = clock->ExecuteCommand (gpscon);  // return True

    gpscon.gpu = GPU2;
    result = clock->ExecuteCommand (gpscon);  // return True
    // gps == NULL

    // tested on 09.08.97, 11:20: all tests okay

    SCommand com;
    com.module = Clock;
    com.type = CrashModule;
    result = clock->ExecuteCommand (com);  // return True
    // crashed = True

    com.type = RecoverModule;
    result = clock->ExecuteCommand (com);  // return True
    // crashed = False

    // tested on 04.02.98, 19:50: all tests okay


    //----------------------------------------------------------------------
    // test GpsConnected()

    result = clock->GpsConnected();  // return False

    gpscon.type = ConnectGps;
    gpscon.gpu = GPU2;
    clock->ExecuteCommand (gpscon);

    result = clock->GpsConnected();  // return True

    // tested on 09.08.97, 11:20: all tests okay


    //----------------------------------------------------------------------
    // test SetAmortizationDuration()
    //   assumption: GPU2 is in use

    result = clock->SetAmortizationDuration (-0.9);  // return False
    result = clock->SetAmortizationDuration (0);  // return False
    result = clock->SetAmortizationDuration (1);  // return False
    result = clock->SetAmortizationDuration (0.9);  // return True
    // amortTime = 0.9

    gpscon.type = DisconnectGps;
    gpscon.gpu = GPU2;
    clock->ExecuteCommand (gpscon);

    result = clock->SetAmortizationDuration (-0.9);  // return False
    result = clock->SetAmortizationDuration (0);  // return True
    // amortTime = 0
    result = clock->SetAmortizationDuration (1);  // return True
    // amortTime = 1

    // tested on 09.08.97, 11:20: all tests okay


    //----------------------------------------------------------------------
    // test GetRhoPlus(), GetRhoMinus()

    double rho;

    rho = clock->GetRhoPlus();  // return 4e-5
    rho = clock->GetRhoMinus();  // return 6e-5

    // tested on 04.02.99, 12:30: all tests okay


    //----------------------------------------------------------------------
    // test GetFrequency()

    rho = clock->GetFrequency();  // should return 25e6

    // tested on 04.02.99, 12:20: all tests okay


    //----------------------------------------------------------------------
    // test GetGranularity() functions

    rho = clock->GetGranularity();  // should return 5.960 464 477 539e-8
    rho = clock->GetGranularityAcc();  // should return 1.192 092 895 508e-7
    rho = clock->GetGranularityDT();  // should return 1.525 878 906 25e-5
    rho = clock->GetGranularityOsc();  // should return 4e-8
    rho = clock->GetGranularityS();  // should return 1.387 778 780 781e-17

    // tested on 09.08.97, 11:30: all tests okay


    //----------------------------------------------------------------------
    // test SetStepRegisters()

    clock->SetStepRegisters (0x12345678, 0x9a123456);
    // clock: STEPPUREACT = 0x12345678, MSSET = 0x9A000000

    //** DEBUGGER **: set deltaSteppure = 0x98765432, deltaSteplow = 0x0051
    clock->SetStepRegisters (0x12345678, 0x9a000000);
    // clock: STEPPUREACT = 0xAAAAAAAA, MSSET = 0xEB000000

    clock->SetStepRegisters (0, 0);
    // clock: STEPPUREACT = 0, MSSET = 0

    //** DEBUGGER **: set deltaSteppure = 0xF0000000, deltaSteplow = 0x0070
    clock->SetStepRegisters (0x12345678, 0x9a000000);
    // clock: STEPPUREACT = 0xFFFFFFFF, MSSET = 0xFF000000

    // ** DEBUGGER **: set deltaSteppure = 0x98765432, deltaSteplow = 0x03A1
    clock->SetStepRegisters (0x12345678, 0x9a000000);
    // clock: STEPPUREACT = 0, MSSET = 0

    // ** DEBUGGER **: set deltaSteppure = 0x11234567, deltaSteplow = 0x0351
    clock->SetStepRegisters (0x12345678, 0x9a000000);
    // clock: STEPPUREACT = 0x01111111, MSSET = 0x49000000

    // ** DEBUGGER **: set deltaSteppure = 0, deltaSteplow = 0

    // tested on 10.02.98, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test Crash/Recover()

    clock->Crash();
    // crashed = True
    // clock: STEPPUREACT = 0, MSSET = 0

    clock->Recover();
    // crashed = False
    // clock: STEPPUREACT = 0x55E63B8, MSSET = 0x8C000000

    // tested on 04.02.98, 19:40: all tests okay


    //----------------------------------------------------------------------
    // test SetSteppure(), SetSteplow(), SetLambdapure(), SetBounds()

    clock->Crash();  // functions do not take over new values
    // swclock: STEPPUREACT = 0, MSSET = 0

    clock->SetSteppure (0x12345678);
    // nothing happens, swclock: internals_->STEPPUREACT = 0

    clock->SetSteplow (0x12);
    // nothing happens, clock: MSSET = 0

    clock->SetLambdapure (0x1234, 0x5678);
    // nothing happens, clock: LAMBDAPPURE=LAMBDANPURE=0

    clock->SetBounds (0x12345678);
    // nothing happens, clock: BOUNDP = 0xFFFFFFFF


    clock->Recover();  // functions do take over new values

    clock->SetSteppure (0x12345678);
    // swclock: internals_->STEPPUREACT = 0x12345678

    clock->SetSteplow (0x12);
    // clock: MSSET = 0x12000000

    // tested on 10.02.98, 11:00: all tests okay so far

    clock->SetLambdapure (0x1234, 0x5678);
    // clock: LAMBDAPPURE=0x1234, LAMBDANPURE=0x5678,
    // lambdaP = 0x1234, lambdaN = 0x5678

    // tested on 23.09.98, 15:00: all tests okay so far

    clock->SetBounds (0x12345678);
    // clock: BOUNDP = 0x12345678

    // tested on 10.02.98, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test GetSteppure()

    step = clock->GetSteppure();  // should return 0x12345678

    // tested on 12.10.98, 11:50: all tests okay


    //----------------------------------------------------------------------
    // test SetFaultyDrift(), GetFaultyDrift()
    //  also test SetSteppure() and SetSteplow() with deltaRho != 0

    clock->SetSteppure (0x55E63B8);
    // clock: steppure = 0x55E63B8
    clock->SetSteplow (0x8C);
    // clock: MSSET = 0x8C000000

    double drift;  // the faulty drift of the clock

    drift = clock->GetFaultyDrift();  // return 0

    clock->SetFaultyDrift (1e-4);  // drift into positive direction
    // deltaSteppure = 0x232F, deltaSteplow = 0
    // deltaSteplow: steplow=140, 140*1.0001 = 140.014 -> no change in
    //  steplow
    // deltaSteppure: steppure*1.0001 = 900800999, and the difference
    //  to the current steppure value is 0x232F

    drift = clock->GetFaultyDrift();  // return 1e-4

    clock->SetFaultyDrift (-1e-4);  // drift into negative direction
    // deltaSteppure = 0x2330, deltaSteplow = 0x301

    drift = clock->GetFaultyDrift();  // return -1e-4

    // tested on 06.11.98, 13:00: all tests okay so far


    clock->SetSteppure (0x12345678);  // also influences deltaStep values
    // clock: steppure = 0x12345678
    //  deltaSteppure = 0x774E, deltaSteplow = 0x301

    clock->SetSteplow (0x12);
    // clock: MSSET = 0x12000000
    //  deltaSteppure = 0x774E, deltaSteplow = 0x301

    // tested on 06.11.98, 13:20: all tests okay so far


    clock->SetFaultyDrift (1e-4);  // drift into positive direction
    // deltaSteppure = 0x774D, deltaSteplow = 0

    // tested on 06.11.98, 13:20: all tests okay


    //----------------------------------------------------------------------
    // test Reset() - calls Dispose()

    // connect a GPS receiver
    SetGlobalTime (1);
    gpscon.type = ConnectGps;
    gpscon.gpu = GPU2;
    clock->ExecuteCommand (gpscon);

    // set the wobbling data to enable wobbling (period > 0)
    paramWobble.period = 10;
    paramWobble.rhoMin = 5e-6;
    paramWobble.rhoMax = 1.5e-5;
    paramWobble.data.dist = Exponential;
    paramWobble.data.low = 1e-6;
    paramWobble.data.high = 2e-6;
    paramWobble.data.mean = 1e-6;
    paramWobble.data.constant = 1e-8;
    paramWobble.data.freeparam = 1;
    result = clock->SetParameter (paramWobble);  // return True

    com.type = ResetModule;
    result = clock->ExecuteCommand (com);  // return True
    // rhoPlus=rhoMinus = 1e-3, steplow = 0x8C000000, steppure = 90 071 992,
    // gps=NULL, amortTime = 1
    // deltaSteppure=deltaSteplow = 0, rhoDrift = 0,
    // wobbleData = {{Uniform,0,0,0,0,0}, period=0, rhoMin=rhoMax=0,
    //               addDrift=True},
    // pFast=pSlow=pDriftRate=pReadFault=pWriteFault = 0,
    // distFast=distRegular=distSlow = Uniform
    // clock: MSSET = 0x8C000000 (steplow), TSSET=16 777 216, STEPPURE=
    // =STEPAMORT = 90 071 992, AMORTTIMESET = 25e6, STARTAMORT = 1,
    // UTCINTEN1 = 0x10000 (snapshot enabled), UTCINTEN2 = 0x80000000 (DTB6),
    // UTCCONF2 = 0x1000 (GPS pol.), ssu[5].DUTYBL = 0 (DTB6 disabled)

    // tested on 22.09.98, 12:10: all tests okay


    //----------------------------------------------------------------------
    // test InjectFaults()

    clock->InjectFaults();
    // nothing should happen (all fault probabilities are zero)

    // tested on 10.02.98, 12:30: all tests okay so far

    // set crash/recover probabilities to one
    param.type = PCrash;
    param.value = 1;
    clock->SetParameter (param);
    param.type = PRecover;
    clock->SetParameter (param);

    clock->InjectFaults();
    // crashed = True

    clock->InjectFaults();
    // crashed = False

    // tested on 10.02.98, 12:30: all tests okay so far


    param.value = 0;
    param.type = PCrash;  // crash prob is 0
    clock->SetParameter (param);
    param.type = PRecover;
    clock->SetParameter (param);

    dist.data.dist = Uniform;  // set rhoPlus and rhoMinus to 0
    dist.data.low=dist.data.high=dist.data.constant=dist.data.mean=dist.data.freeparam=0;
    dist.type = DistributionFast;
    result = clock->SetParameter (dist);  // return True
    dist.type = DistributionSlow;
    result = clock->SetParameter (dist);  // return True

    param.value = 1;
    param.type = PDriftChange;   // prob. of drift change is 1
    clock->SetParameter (param);

    clock->InjectFaults();
    // should compute new regular drift, which is zero -->
    // deltaSteppure = 0, deltaSteplow = 0, deltaRho = 0

    // tested on 22.09.98, 13:40: all tests okay so far

    dist.type = DistributionFast;  // set rhoPlus = 1e-4
    dist.data.low=dist.data.mean = 1e-4;
    dist.data.high = 1e-3;  // worst case faulty bound is 1e-3
    result = clock->SetParameter (dist);  // return True

    clock->InjectFaults();
    // should compute regular drift, which is not zero -->
    // deltaSteppure is in [0, 0x232F], deltaSteplow = [0, 0x1xx]
    // note: 0x232F, because if no drift: step register is 0x55E63B8,
    // which means 1 sec/sec; if we need additional 10^-4 sec/sec:
    // that must be 0x55E63B8/10^4 = 0x232F

    // tested on 22.09.98, 13:40: all tests okay so far


    param.value = 1;
    param.type = PFast;   // prob. of fast drift is 1
    clock->SetParameter (param);
    clock->InjectFaults();
    // should compute fast drift
    // deltaSteppure = [0x232F, 0x15FD7], deltaSteplow = [xxx, 0xFF],
    // deltaRho = [1e-4, 1e-3]

    // tested on 22.09.98, 13:40: all tests okay so far


    param.value = 0;
    param.type = PFast;
    clock->SetParameter (param);
    param.value = 1;
    param.type = PSlow;   // prob. of slow drift is 1

    dist.type = DistributionSlow;  // set rhoMinus = 0
    dist.data.low=dist.data.mean = 0;
    dist.data.high = 1e-3;  // worst case faulty bound is 1e-3
    result = clock->SetParameter (dist);  // return True

    clock->SetParameter (param);
    clock->InjectFaults();
    // should compute slow drift
    // deltaSteppure = (0, 0x15FD7], deltaSteplow = [0x200, 0x3FF],
    // deltaRho = (0, -1e-3]

    // tested on 22.09.98, 14:00: all tests okay


    //----------------------------------------------------------------------
    // test GetTimestamp()

    STimestamp time;
    time.ms = 1; time.ts = 0; time.acc = 4;
    clock->Reset();  // set back the clock value
    param.type = ClockDrift;
    param.value = 0;
    clock->SetParameter (param);  // need to explicitly set back the drift
    SetGlobalTime (1);

    result = clock->GetTimestamp (time);  // return True
    // ms = 0, ts = 16 777 216, acc = 0x7FFF7FFF
    // clock: TSGET = 16 777 216;   swclock: internals->AMORTTIMER = 0
    // result: ts = 16 777 215 (us = 0xFFF9 7860)

    clock->AddGps (GPU1);
    result = clock->GetTimestamp (time);  // return False

    // tested on 10.02.98, 13:10: all tests okay so far

    param.value = 1;
    param.type = PReadFault;
    clock->SetParameter (param);
    param.type = PCrash;
    clock->SetParameter (param);

    clock->GetTimestamp (time);
    // crashed=True, ms = xxx, ts = xxx, acc = xxx

    param.value = 0;
    param.type = PReadFault;
    clock->SetParameter (param);
    param.type = PCrash;
    clock->SetParameter (param);
    clock->Recover();

    // tested on 22.09.98, 16:10: all tests okay


    //----------------------------------------------------------------------
    // test GetCurrentClockTime()

    CNtpTime ntptime;

    ntptime = clock->GetCurrentClockTime();
    // ms = 0, ts = 16 777 215

    // tested on 09.08.97, 21:20: all tests okay


    //----------------------------------------------------------------------
    // test SetDutyTimer()

    CClockInterrupt *irq;

    ntptime.ms = 1;
    ntptime.ts = 1<<24;  // that's 257 secs NTP

    clock->SetDutyTimer (IntDutyB4, ntptime);

    // get the DutyTimerB4 interrupt process
    irq = swclock->CreateInterrupt (61);
    // irq->scheduleTime = 257, ipBit=29, enabled=False, scheduled

    ntptime.ts = 1<<25;  // that's 258 secs NTP
    clock->SetDutyTimer (IntDutyB4, ntptime);
    // irq->scheduleTime = 258, rest as above

    // tested on 09.08.97, 21:50: all tests okay so far

    ntptime.ts = 2<<24;  // 258 secs NTP
    clock->SetDutyTimer (IntDutyB1, ntptime);
    irq = swclock->CreateInterrupt (58);
    // irq->scheduleTime = 258, ipBit=26, enabled=False, scheduled

    ntptime.ts = 3<<24;  // 259 secs NTP
    clock->SetDutyTimer (IntDutyB5, ntptime);
    irq = swclock->CreateInterrupt (62);
    // irq->scheduleTime = 259, ipBit=30, enabled=False, scheduled

    // tested on 07.12.97, 14:00: all tests okay

    ntptime.ts = 4<<24;  // 260 secs NTP
    clock->SetDutyTimer (IntDutyA1, ntptime);
    irq = swclock->CreateInterrupt (IntDutyA1);
    // irq->scheduleTime = 260, ipBit=20, enabled=False, scheduled

    // tested on 28.07.98, 16:50: all tests okay


    //----------------------------------------------------------------------
    // test EnableInterrupt()

    timeparam.type = ClockValue;
    timeparam.macrostamp = 0;
    timeparam.timestamp = 33554432;  // clock time = 2 seconds
    result = clock->SetParameter (timeparam);  // return True

//    clock->EnableInterrupt (IntSnapshot);
    // clock: UTCINTEN1 = 0x10000
    // has already been done in InitializeClock()

    irq = swclock->CreateInterrupt (IntDutyB4);  // irq points to DT B4
    clock->EnableInterrupt (IntDutyB4);
    // clock: UTCINTEN2 = 0x20000000
    // irq->enabled = True, scheduled with assocSimTime = 257 (approx.)

    // tested on 09.08.97, 21:50: all tests okay so far


    clock->EnableInterrupt (IntGps1PPS1);
    clock->EnableInterrupt (IntGps1PPS2);
    clock->EnableInterrupt (IntGps1PPS3);

    clock->EnableInterrupt (IntDutyA1);
    clock->EnableInterrupt (IntDutyA2);
    clock->EnableInterrupt (IntDutyA3);
    clock->EnableInterrupt (IntDutyA4);
    clock->EnableInterrupt (IntDutyA5);
    clock->EnableInterrupt (IntDutyA6);

    clock->EnableInterrupt (IntDutyB1);
    clock->EnableInterrupt (IntDutyB2);
    clock->EnableInterrupt (IntDutyB3);
    clock->EnableInterrupt (IntDutyB5);
    clock->EnableInterrupt (IntDutyB6);
    // clock: UTCINTEN2 = 0xFFF00007

    // tested on 28.07.98, 16:20: all tests okay


    //----------------------------------------------------------------------
    // test DisableInterrupt()

    clock->DisableInterrupt (IntSnapshot);
    // clock: UTCINTEN1 = 0x0

    clock->DisableInterrupt (IntDutyB4);
    // clock: UTCINTEN2 = 0xDFF00007
    // irq->enabled = False, process is not scheduled

    // tested on 28.07.98, 16:20: all tests okay so far

    clock->DisableInterrupt (IntGps1PPS1);
    clock->DisableInterrupt (IntGps1PPS2);
    clock->DisableInterrupt (IntGps1PPS3);

    clock->DisableInterrupt (IntDutyA1);
    clock->DisableInterrupt (IntDutyA2);
    clock->DisableInterrupt (IntDutyA3);
    clock->DisableInterrupt (IntDutyA4);
    clock->DisableInterrupt (IntDutyA5);
    clock->DisableInterrupt (IntDutyA6);

    clock->DisableInterrupt (IntDutyB1);
    clock->DisableInterrupt (IntDutyB2);
    clock->DisableInterrupt (IntDutyB3);
    clock->DisableInterrupt (IntDutyB5);
    clock->DisableInterrupt (IntDutyB6);
    // clock: UTCINTEN2 = 0x0

    // tested on 28.07.98, 16:20: all tests okay


    //----------------------------------------------------------------------
    // test ClearInterrupt()

    // DEBUGGER: set clock_->utcsuRead->UTCINTSTAT1/2 = 0xFFFFFFFF
    clock->ClearInterrupt (IntSnapshot);
    // utcsuRead->UTCINTSTAT1 = 0xFFFEFFFF

    clock->ClearInterrupt (IntDutyB4);
    // utcsuRead->UTCINTSTAT2 = 0xDFFFFFFF

    // tested on 28.03.97, 13:40: all tests okay so far

    clock->ClearInterrupt (IntGps1PPS1);
    clock->ClearInterrupt (IntGps1PPS2);
    clock->ClearInterrupt (IntGps1PPS3);
    clock->ClearInterrupt (IntDutyA1);
    clock->ClearInterrupt (IntDutyA2);
    clock->ClearInterrupt (IntDutyA3);
    clock->ClearInterrupt (IntDutyA4);
    clock->ClearInterrupt (IntDutyA5);
    clock->ClearInterrupt (IntDutyA6);
    clock->ClearInterrupt (IntDutyB1);
    clock->ClearInterrupt (IntDutyB2);
    clock->ClearInterrupt (IntDutyB3);
    clock->ClearInterrupt (IntDutyB5);
    clock->ClearInterrupt (IntDutyB6);
    // utcsuRead->UTCINTSTAT2 = 0x000FFFF8

    // tested on 28.07.98, 16:25: all tests okay so far

    // DEBUGGER: set clock_->utcsuRead->UTCINTSTAT1/2 = 0xFFFFFFFF
    clock->ClearInterrupt (INTA);
    // utcsuRead->UTCINTSTAT1 = 0xC008EFFF

    // DEBUGGER: set clock_->utcsuRead->UTCINTSTAT1/2 = 0xFFFFFFFF
    clock->ClearInterrupt (INTN);
    // utcsuRead->UTCINTSTAT2 = 0xFFF000C0

    // DEBUGGER: set clock_->utcsuRead->UTCINTSTAT1/2 = 0xFFFFFFFF
    clock->ClearInterrupt (INTT);
    // utcsuRead->UTCINTSTAT1 = 0xFFFFF808
    // utcsuRead->UTCINTSTAT2 = 0x000FFFFF

    // tested on 28.03.97, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test InterruptPending()

    // DEBUGGER: set clock_->utcsuRead->UTCINTSTAT1/2 = 0xFFFFFFFF
    result = clock->InterruptPending (IntSnapshot);  // return True
    result = clock->InterruptPending (IntDutyB4);  // return True

    clock->ClearInterrupt (IntSnapshot);
    result = clock->InterruptPending (IntSnapshot);  // return False

    clock->ClearInterrupt (IntDutyB4);
    result = clock->InterruptPending (IntDutyB4);  // return False

    // tested on 28.03.97, 13:40: all tests okay so far

    result = clock->InterruptPending (IntGps1PPS1);
    result = clock->InterruptPending (IntGps1PPS2);
    result = clock->InterruptPending (IntGps1PPS3);
    result = clock->InterruptPending (IntDutyA1);
    result = clock->InterruptPending (IntDutyA2);
    result = clock->InterruptPending (IntDutyA3);
    result = clock->InterruptPending (IntDutyA4);
    result = clock->InterruptPending (IntDutyA5);
    result = clock->InterruptPending (IntDutyA6);
    result = clock->InterruptPending (IntDutyB1);
    result = clock->InterruptPending (IntDutyB2);
    result = clock->InterruptPending (IntDutyB3);
    result = clock->InterruptPending (IntDutyB5);
    result = clock->InterruptPending (IntDutyB6);
    // should all return True

    // tested on 28.07.98, 16:25: all tests okay so far

    result = clock->InterruptPending (INTA);  // return True
    result = clock->InterruptPending (INTN);  // return True
    result = clock->InterruptPending (INTT);  // return True

    clock->ClearInterrupt (INTA);
    result = clock->InterruptPending (INTA);  // return False

    clock->ClearInterrupt (INTN);
    result = clock->InterruptPending (INTN);  // return False

    clock->ClearInterrupt (INTT);
    result = clock->InterruptPending (INTT);  // return False
    // utcsuRead->UTCINTSTAT1 = 0xC008E808 (unused bits)
    // utcsuRead->UTCINTSTAT2 = 0xC0

    // tested on 28.03.97, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test InterruptAfter()
    //    assumption: clock value = 2 secs, global time = 1 sec

    clock->InterruptAfter (IntDutyB4, 6);
    irq = swclock->CreateInterrupt (61);
    // irq: scheduleTime=8, assocSimTime=7 (approx.), Scheduled
    // (enabled=False)
    // error for assocSimTime:  6e-11
    // note: both for the computation of scheduleTime and of assocSimTime,
    // there's a rounding error involved; since the last 8 bits of the
    // timestamp are cut off for the duty timer, the rounding error is
    // magnified
    // the error depends on the rounding strategy in SetDutyTimer():
    // round-down leads to higher errors than round-to-nearest, so the
    // latter was implemented

    clock->EnableInterrupt (IntDutyB4);
    // irq: enabled=True

    // tested on 07.12.97, 14:10: all tests okay


    //----------------------------------------------------------------------
    // test InterruptAtMult()

    SetGlobalTime (6);
    clock->EnableInterrupt (IntDutyB4);  // just to init. a clock update
    // swclock->internals->ts=117 440 512 (approx.) [7 secs.]
    // result: ts=117 440 511, us=0xFFDF 59E0

    dummy = clock->InterruptAtMult (IntDutyB4, 5);  // return 10
    // irq: scheduleTime=10 (enabled=True, Scheduled, assocSimTime=9)
    // error for assocSimTime: 5e-11

    // tested on 09.08.97, 22:00: all tests okay so far

    dummy = clock->InterruptAtMult (IntDutyB4, 4300000000.);// return 43e8
    // report: overflow in InterruptAtMult [trace it!]
    // irq: scheduleTime=5032704 (enabled=True, Scheduled,
    //                            assocSimTime=5032704-1)
    // (note: 5032704 = ((43e8 & 0x0FFFFFF) << 8))

    // tested on 28.07.98, 17:35: all tests okay


    //----------------------------------------------------------------------
    // test InterruptAt()

    clock->InterruptAt (IntDutyB1, 16.5);
    // SSU1, DUTYBH = 16, DUTYBL = 98304 (enable bit + msb)

    // tested on 07.12.97, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test DisableDutyTimer()

    clock->InterruptAt (IntDutyB1, 16.5);
    // SSU1, DUTYBH = 16, DUTYBL = 98304 (enable bit + msb)
    // (see last test)
    clock->DisableDutyTimer (IntDutyB1);
    // SSU1, DUTYBL = 0

    clock->InterruptAt (IntDutyB4, 16.5);
    clock->DisableDutyTimer (IntDutyB4);
    // SSU4, DUTYBL = 0

    clock->InterruptAt (IntDutyB5, 16.5);
    clock->DisableDutyTimer (IntDutyB5);
    // SSU5, DUTYBL = 0

    // tested on 07.12.97, 14:10: all tests okay so far

    // test all other duty timers
    clock->InterruptAt (IntDutyA1, 16.5);
    clock->DisableDutyTimer (IntDutyA1);

    clock->InterruptAt (IntDutyA2, 16.5);
    clock->DisableDutyTimer (IntDutyA2);
    clock->InterruptAt (IntDutyB2, 16.5);
    clock->DisableDutyTimer (IntDutyB2);

    clock->InterruptAt (IntDutyA3, 16.5);
    clock->DisableDutyTimer (IntDutyA3);
    clock->InterruptAt (IntDutyB3, 16.5);
    clock->DisableDutyTimer (IntDutyB3);

    clock->InterruptAt (IntDutyA4, 16.5);
    clock->DisableDutyTimer (IntDutyA4);
    clock->InterruptAt (IntDutyA5, 16.5);
    clock->DisableDutyTimer (IntDutyA5);

    clock->InterruptAt (IntDutyA6, 16.5);
    clock->DisableDutyTimer (IntDutyA6);
    clock->InterruptAt (IntDutyB6, 16.5);
    clock->DisableDutyTimer (IntDutyB6);

    // tested on 28.07.98, 16:35: all tests okay


    //----------------------------------------------------------------------
    // test isr()

    ((CSoftwareClock*)clock->GetClock())->HwSnapshot();  // make a snapshot

    clock->isr();
    // send a Snapshot event (containing the snapshot clock data) to ctrl
    // (trace it!)

    // tested on 28.03.97, 19:30: all tests okay


    //----------------------------------------------------------------------
    // test wobble_isr()
    // note: all except the last test were done with code that simply
    //  called InterruptAtMult() (the code that is now executed if PSOS_SIM
    //  is defined), so you can consider the PSOS_SIM part to be tested;
    //  the last test just checks if the behaviour of the new code for
    //  the other environments is the same

    // set the wobble distribution data to a point (rhoAdd = 1e-6)
    paramWobble.module = Clock;
    paramWobble.type = WobbleDrift;
    paramWobble.period = 1.5;
    paramWobble.rhoMin = 1.5e-6;  // wobble drift is in +/- 1.5e-6
    paramWobble.rhoMax = 1.5e-6;
    paramWobble.data.dist = Uniform;
    paramWobble.data.low = 1e-6;
    paramWobble.data.high = 1e-6;
    paramWobble.data.mean = 1e-6;
    paramWobble.data.constant = 0;
    paramWobble.data.freeparam = 0;
    result = clock->SetParameter (paramWobble);  // return True


    timeparam.type = ClockValue;  // set current time to multiple of period
    timeparam.macrostamp = 0;
    timeparam.timestamp = 0x07800000;  // current time is 7.5
    result = clock->SetParameter (timeparam);  // return True


//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000 (interrupt pending)

    clock->wobble_isr();
    // deltaRho = 1e-6, DTB6 is scheduled for t=9 (DUTYBH=9)
    // should send event TimerEvent [trace it!]

    // tested on 23.09.98, 17:20: all tests okay so far


//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = False,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = -1e-6, wobbleData.addDrift = False,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

    // tested on 23.09.98, 17:20: all tests okay so far


//---- DEBUGGER: set deltaRho = 3e-6
//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 2e-6, wobbleData.addDrift = False,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 1e-6, wobbleData.addDrift = False,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = False,
    // DTB6 is scheduled for t=9 (DUTYBH=9)
    // result: deltaRho = 4.235e-22, apparently a rounding error

    // tested on 23.09.98, 17:20: all tests okay so far


//---- DEBUGGER: set deltaRho = -3e-6
//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = -2e-6, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = -1e-6, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

    // tested on 23.09.98, 17:20: all tests okay so far


    // set the wobble bounds very tight
    paramWobble.rhoMin = 0.5e-6;
    paramWobble.rhoMax = 0.5e-6;
    result = clock->SetParameter (paramWobble);  // return True

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)
    // result: deltaRho = -4.235e-22, again a rounding error

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = True,
    // DTB6 is scheduled for t=9 (DUTYBH=9)

    // tested on 23.09.98, 17:20: all tests okay so far


//---- DEBUGGER: set wobbleData.rhoMax=rhoMin=0
//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = True,
    // DTB6 is descheduled (DUTYBL=0)

    // tested on 23.09.98, 17:50: all tests okay so far


//---- DEBUGGER: set wobbleData.rhoMax=rhoMin=1e-6
//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 1e-6, wobbleData.addDrift = True,
    // DTB6 is scheduled (DUTYBH=9)

//---- DEBUGGER: set wobbleData.period = 0
//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = False,
    // DTB6 is descheduled (DUTYBL=0)

    // tested on 23.09.98, 17:50: all tests okay


//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000
//---- NOTE: you should have defined WOBBLE_USE_VERSION_2 in wobble_isr()
//---- NOTE: it is assumed that the tests above have not been executed

    paramWobble.period = 1.5;  // set to known values
    paramWobble.rhoMin = 1e-6;
    paramWobble.rhoMax = 1e-6;
    result = clock->SetParameter (paramWobble);  // return True

    clock->wobble_isr();
    // deltaRho = 1e-6, wobbleData.addDrift = True,
    // DTB6 is scheduled (DUTYBH=9)

    // tested on 28.09.98, 14:20: all tests okay


//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000
//---- NOTE: you should have defined WOBBLE_USE_VERSION_1 in wobble_isr()
//---- NOTE: it is assumed that the tests above have not been executed

    clock->wobble_isr();
    // deltaRho = 1e-6, wobbleData.addDrift = True,
    // DTB6 is scheduled (DUTYBH=9)

    timeparam.type = ClockValue;  // set current time
    timeparam.macrostamp = 0;
    timeparam.timestamp = 0x077FFFFF;  // current time is a bit before 7.5
    result = clock->SetParameter (timeparam);  // return True

//---- DEBUGGER: set utcsuRead->UTCINTSTAT2 = 0x80000000

    clock->wobble_isr();
    // deltaRho = 0, wobbleData.addDrift = False,
    // DTB6 is scheduled (DUTYBH=9)

    // tested on 28.09.98, 14:30: all tests okay


    //----------------------------------------------------------------------
    // test Amortize()
    // note: the function calls PreloadAmort(), so that is tested as well

    SetGlobalTime (0);
    clock->Reset();

    clock->Amortize (1, 0x12345678, 0x23456789);
    // clock: AMORTTIMESET=25 000 000, STEPAMORT = 180 143 985 (rounded up),
    // STATEPSET=0x12345678, STATENSET=0x23456789

    clock->Amortize (-0.5, 0, 0);
    // clock: STEPAMORT = 45 035 996, STATEPSET=STATENSET = 0

    clock->Amortize (116, 0, 0);
    // clock: STEPAMORT = 0xffffffff

    clock->Amortize (-2, 0, 0);
    // clock: STEPAMORT = 0

    // tested on 07.12.97, 13:50: all tests okay so far

    param.type = AmortizationDuration;
    param.value = 2.1;
    clock->SetParameter (param);

    clock->Amortize (1, 0, 0);
    // clock: AMORTTIMESET=52 500 000, STEPAMORT = 132 963 417 (rounded up)

    // tested on 22.09.98, 14:20: all tests okay


    //----------------------------------------------------------------------
    // test DragAccuracy()
    // assumptions: steppure is 0x055E63B8, steplow = 0x8C000000

    SINT32 newAccuracy;  // the accuracy in the given future time

    newAccuracy = clock->DragAccuracy (0x12345678, 0, 24.3);
    // should return 0x12345678 (lambda = 0)

    // tested on 23.09.98, 10:50: all tests okay so far

//---- DEBUGGER: set steppure_ and stepplow_ to zero (but remember their old
    // values, you have to restore them afterwards!

    newAccuracy = clock->DragAccuracy (0x12345678, 0, 24.3);
    // should return 0x12345678 (lambda = 0)
    newAccuracy = clock->DragAccuracy (0x12345678, 0xA004, 24.3);
    // should return 0x80000000 (lambda < 0)
    newAccuracy = clock->DragAccuracy (0x12345678, 0x2004, 24.3);
    // should return 0x7FFFFFFF (lambda > 0)

    // tested on 23.09.98, 12:40: all tests okay so far

//---- DEBUGGER: restore the values of steppure_ and stepplow_

    // test cases:
    // *) positive alpha gets higher, but is below positive bound
    // *) positive alpha gets higher and is above positive bound
    // *) positive alpha gets lower, but is above negative bound
    // *) positive alpha gets lower and is below negative bound
    // *) negative alpha gets higher, but is below positive bound
    // *) negative alpha gets lower, but is above negative bound

    newAccuracy = clock->DragAccuracy (0x12345678, 0x1234, 24.3);
    // should return 0x26CD649F
    // note: the return value is computed like this: the steppure/steplow
    // values are that of the normal clock without drift; so 1 second logical
    // (clock) time corresponds to 1 second of real time or 25e6 cycles of
    // the clock (25e6 is the clock frequency); so we have 25e6*24.3 clock
    // cycles in each of which the accuracy deteriorates by lambda=0x1234;
    // thus, the value 0x1234*25e6*24.3 (in format [-37,-51]) is added to
    // the accuracy; but the accuracy is in format S:[-8,-38], so we have
    // to divide the added value by 2^13=8192 to get the value that is
    // added to alpha; in our case: 0x1234*25e6*24.3/8192 = 0x14990E27,
    // 0x14990E27 + 0x12345678 = 0x26CD649F

    newAccuracy = clock->DragAccuracy (0x12345678, 0x7FFF, 24.3);
    // should return 0x7FFFFFFF

    // tested on 23.09.98, 12:40: all tests okay so far

    newAccuracy = clock->DragAccuracy (0x12345678, 0xEDCC, 24.3);
    // should return 0xFD9B4851
    // note: 0xEDCC is -0x1234, so we have to subtract 0x14990E27 from the
    // current accuracy

    newAccuracy = clock->DragAccuracy (0x02345678, 0x8000, 24.3);
    // should return 0x80000000

    newAccuracy = clock->DragAccuracy (0xEDCBA988, 0x1234, 24.3);
    // should return 0x0264B7AF
    // 0xEDCBA988 is -0x12345678, so the result should be
    // 0x14990E27 - 0x12345678 = 0x0264B7AF

    newAccuracy = clock->DragAccuracy (0xEDCBA988, 0xEDCC, 24.3);
    // should return 0xD9329B61
    // the result is -0x12345678-0x14990E27 = 0xD9329B61

    // tested on 23.09.98, 12:50: all tests okay


    //----------------------------------------------------------------------
    // test ComputeAccuracyCorrection()
    //     correct behaviour of function DragAccuracy() is assumed
    // assumptions: steppure = 0x055E63B8, steplow = 0x8C000000,
    //              alphaP = 0x7FFFFFFF, alphaN = 0x7FFFFFFF,
    //              lambdaP = 0, lambdaN = 0

    INT32 diffPlus;   // difference between the current accuracy and the
    INT32 diffMinus;  // desired one (value of STATEP(N)SET in amortization)

    // accuracy is at positive bounds, we have to subtract to get new values
    clock->ComputeAccuracyCorrection (0x2468468A, 0, diffPlus, diffMinus);
    // diffPlus = 0xA345 0001, diffMinus = 0x9234 0001
    // note: diffPlus = 0x23450000 (0x468A unsigned is 0x2345 signed)
    //                  - 0x7FFFFFFF = -0x5CBAFFFF = 0xA345 0001

    param.type = Accuracy;
    param.value = 0x80000000;  // negative bound
    result = clock->SetParameter (param);  // return True
    // swclock->internals->ALPHAP,ALPHAN = 0x80000000

    // accuracy is at positive bounds, but we have to add more than 0x7FFFFFFF
    // to get to their values, so the correction value is again negative
    clock->ComputeAccuracyCorrection (0x2468468A, 0, diffPlus, diffMinus);
    // diffPlus = 0xA345 0000, diffMinus = 0x9234 0000

    param.type = Accuracy;
    param.value = 0xF0000000;  // small negative value
    result = clock->SetParameter (param);  // return True
    // swclock->internals->ALPHAP,ALPHAN = 0xF0000000

    // accuracy is negative, but we do not have very much to add, so the
    // correction value is positive
    clock->ComputeAccuracyCorrection (0x2468468A, 0, diffPlus, diffMinus);
    // diffPlus = 0x3345 0000, diffMinus = 0x2234 0000

    param.type = Accuracy;
    param.value = 0x10000000;  // small positive value
    result = clock->SetParameter (param);  // return True
    // swclock->internals->ALPHAP,ALPHAN = 0x10000000

    // accuracy is positive and less than the desired value, so the
    // correction value is again positive
    clock->ComputeAccuracyCorrection (0x2468468A, 0, diffPlus, diffMinus);
    // diffPlus = 0x1345 0000, diffMinus = 0x0234 0000

    // tested on 23.09.98, 15:20: all tests okay so far

    // test with preload data

    clock->SetLambdapure (0x1234, 0xEDCC);  // set symmetrical lambda values

    clock->ComputeAccuracyCorrection (0x2468468A, 24.3, diffPlus, diffMinus);
    // diffPlus = 0xFEAB F1D9, diffMinus = 0x26CD 0E27
    // note: alphaP after 24.3 seconds is 0x1499 0E27 (the deterioration
    // due to lambdaP=0x1234) + 0x1000 0000 (current alphaP) = 0x2499 0E27;
    // alphaN is 0xEB66 F1D9 (= 9xDB66 F1D9+0x1000 0000)
    // we want an alphaP = 0x2345 0000 and an alphaN = 0x1234 0000, so we
    // have to add 0xFEAB F1D9 to current alphaP and 0x26CD 0E27 to alphaN

    // tested on 23.09.98, 16:10: all tests okay


    //----------------------------------------------------------------------
    // test SetTimestamp()
    // note: only the version using SExtendedTimestamp is tested, since
    //       the other function simply calls it with us=0

    SExtendedTimestamp xtime;

    param.type = Accuracy;
    param.value = 0x7FFFFFFF;  // set alphas back to their initial values
    result = clock->SetParameter (param);


    SecondsToNtp (2.5, xtime);
    xtime.acc = 0;

    param.type = PCrash;
    param.value = 1;
    clock->SetParameter (param);
    clock->SetTimestamp (xtime);
    // crashed=True, nothing else happens
 
    param.type = PCrash;
    param.value = 0;
    clock->SetParameter (param);
    clock->Recover();
    param.type = PWriteFault;
    param.value = 1;
    clock->SetParameter (param);
    clock->SetTimestamp (xtime);
    // clock: STEPAMORT = xxx, STATEP(N)SET = xxx

    param.type = PWriteFault;
    param.value = 0;
    clock->SetParameter (param);
    clock->SetTimestamp (xtime);
    // clock: STEPAMORT = 197 300 555 (rounded up),
    // STATEP(N)SET = 0x800000001 (current acc is 0x7FFFFFFF, and
    // we want to have acc = 0x00000000)

    // tested on 23.09.98, 16:20: all tests okay so far


    SecondsToNtp (0, xtime);
    xtime.acc = 0x2468468A;
    clock->SetTimestamp (xtime);
    // clock: STEPAMORT = 90 071 992 (=steppure), STATEPSET = 0xA345 0001,
    // STATENSET = 0x9234 0001
    // note: STATEPSET = 0x23450000 (0x468A unsigned is 0x2345 signed)
    //                   - 0x7FFFFFFF = -0x5CBAFFFF = 0xA345 0001

    // tested on 23.09.98, 16:20: all tests okay so far

    param.type = Accuracy;
    param.value = 0x32100000;
    result = clock->SetParameter (param);  // return True
    // swclock->internals->ALPHAP,ALPHAN = 0x32100000

    xtime.acc = 0x246868ad;
    clock->SetTimestamp (xtime);
    // clock: STEPAMORT = 90 071 992 (=steppure), STATEPSET = 0x0246 8000,
    // STATENSET = 0xE024 0000

    // tested on 23.09.98, 16:20: all tests okay so far

    param.type = AmortizationDuration;
    param.value = 0;
    clock->SetParameter (param);

    SecondsToNtp (1, xtime);
    xtime.acc = 0x2468468A;
    clock->SetTimestamp (xtime);
    // clock: ms=0, ts=16 777 216, ALPHAP=ALPHAN = 0x23450000
    // (the greater value of the two accuracies)

    // tested on 07.12.97, 15:10: all tests okay so far


    gpscon.type = ConnectGps;
    gpscon.gpu = GPU1;
    clock->ExecuteCommand (gpscon);

    clock->SetTimestamp (xtime);
    // should return at once [trace it!]

    // tested on 07.12.97, 15:10: all tests okay


    //----------------------------------------------------------------------
    // test PreloadTimestamp()
    // note: only the version using SExtendedTimestamp is tested, since
    //       the other function simply calls it with us=0

    clock->SetLambdapure (0, 0);
    // set the lambdas back to their initial values (this makes the
    // computation of the STATEP(N)SET values easier)

    clock->PreloadTimestamp (20, xtime);
    // should return at once [trace it!]

    gpscon.type = DisconnectGps;
    gpscon.gpu = GPU1;
    clock->ExecuteCommand (gpscon);

    // tested on 07.12.97, 15:20: all tests okay so far

    param.type = PCrash;
    param.value = 1;
    clock->SetParameter (param);
    clock->PreloadTimestamp (20, xtime);
    // crashed=True, nothing else happens
 
    // tested on 22.09.98, 15:50: all tests okay so far

    param.type = PCrash;
    param.value = 0;
    clock->SetParameter (param);
    clock->Recover();
    param.type = PWriteFault;
    param.value = 1;
    clock->SetParameter (param);
    clock->PreloadTimestamp (20, xtime);
    // clock: MSSET = 0x8Cxxxxxx, TSSET=USSET=ALPHAP(N) = xxxxxxxx

    param.type = PWriteFault;
    param.value = 0;
    clock->SetParameter (param);

    // tested on 22.09.98, 15:50: all tests okay so far

    SecondsToNtp (1, xtime);
    xtime.acc = 0x2468468A; xtime.us = 6;
    clock->PreloadTimestamp (20, xtime);
    // note: amorttime = 0 -> SetTimestamp() is called [trace it!]
    // clock: ms=0, ts=16 777 216, us = 6; ALPHAP=ALPHAN = 0x23450000

    // tested on 22.09.98, 15:50: all tests okay so far

    param.type = AmortizationDuration;
    param.value = 1;
    clock->SetParameter (param);

    SecondsToNtp (4.5, xtime);
    xtime.acc = 0x2468468A;
    clock->PreloadTimestamp (2, xtime);
    // clock: STEPAMORT = 315 251 973 (rounded up), STATEPSET=0,
    // STATENSET = 0xEEEF0000
    // SSU1: DUTYBH = 2, DUTYBL = 65536 (enable bit)
/**/// note: this "rounding up" - is this maybe the result of not
    // considering STEPLOW? but no: STEP and STEPLOW are what is needed
    // to pass one clock cycle in pure phase; STEPLOW cannot be changed,
    // so the question is, what do we have to add to STEP to amortize
    // the difference; and that value should be as close as possible

    // tested on 23.09.98, 16:20: all tests okay


    //----------------------------------------------------------------------

    clock->DisableInterrupt (IntDutyB4);
    clock->DisableInterrupt (IntDutyB5);
    // disable all interrupts that may still be enabled due to the tests
    // to pass the assert of UTCINTEN == 0 in the dtor

    listOfClocks.Remove (2, 3);


    //----------------------------------------------------------------------
    // test the dtor

    cout << endl << "testing the dtor" << endl;

    delete clock;
    // output: RemoveISR() called with ... [snapshot ISR] (see ctor)
    // output: RemoveISR() called with ... [wobble ISR] (see ctor)

    // tested on 22.09.98, 12:00: all tests okay


    //----------------------------------------------------------------------

    return 0;
}


#include "inlines.hpp"



