//
// cloctest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test CClockInterrupt
// note: PreExecution() was made public for this test
// test CDutyB1Interrupt
// note: PreExecution() was made public for this test
// test CGpsInterrupt
// note: PreExecution() and PostExecution() were made public for this test
// test CClock
// test the global variable clockTemplate
// test CSoftwareClock (and EInterruptBits in clockcfg.hpp)
// note: all private functions were made public for this test
//       the static global functions were made non-static for this test
//
// created: 09.11.96
//
// last change: 29.07.98
//

#include <stdlib.h>

#include "..\clockbas.hpp"
#include "..\clock_sw.hpp"
#include "..\clockcfg.hpp"
#include "..\clockirq.hpp"
#include "..\ctrling.hpp"
#include "..\general.hpp"
#include "..\process.hpp"
#include "..\sim_glob.hpp"
#include "..\structs.hpp"
#include "..\template.hpp"
#include "..\time.hpp"
#include "..\utcsu.hpp"
#include "settime.hpp"

#define Need_CControllingObject

//#define TEST_STATIC_FUNCS
// uncomment it to test the static functions

CClockList listOfClocks;
CClock *g_masterClock = NULL;  // not needed


extern EClockInterrupt GetInterruptType (INT8 interruptBit);
extern double DutyToSeconds (INT32 dutyh, INT32 dutyl);
extern double GetTime (const CNtpTime& cycles, double frequency);


// copied from clock_sw.cpp; if the struct is changed there it should be
// changed here as well!
// note: the struct is needed here because otherwise the WATCOM debugger
// does not know the type of clock->internals_

//**************************************************************************
// struct SInternalRegisters: internal clock registers that have to be
//      simulated in the software clock
//**************************************************************************

struct SInternalRegisters
{
    // NTPTIME
    INT32 ms;  // CS and MS [+31,+8]
    INT32 ts;  // [+7,-24]
    INT32 us;  // [-25,-56]
    INT32 ns;  // [-57,-59]
    EBool overflow;  // set to True by UpdateNtp() if an overflow occured

    // STEP value during pure phase
    INT32 STEPPUREACT;  // [-20,-51]

    // deterioration values for the accuracies during pure phase
    INT16 LAMBDAPPUREACT;  // [-37,-51] (signed)
    INT16 LAMBDANPUREACT;

    INT32 ALPHAP;  // accuracy envelopes [-8,-38] (signed)
    INT32 ALPHAN;
    INT16 NALPHAP;  // fractional part of the accuracies [-39,-51]
    INT16 NALPHAN;

    // amortization timer
    INT32 AMORTTIMER;
};


int main()
{
/* obsolete; cf. CClock::EnableInterrupt()

    //----------------------------------------------------------------------
    // global function for setting INTEN registers; for tmandl?
    //----------------------------------------------------------------------

    EBool ieRes;

    // inten1=0,0

    ieRes = SetINTEN1 (0, 29, True);  // True
    // inten1=0x20000000,0

    ieRes = SetINTEN1 (0, 10, True);  // True
    // inten1=0x20000400,0

    ieRes = SetINTEN1 (0, 2, False);  // True
    // inten1=0x20000400,0

    ieRes = SetINTEN1 (0, 29, False);  // True
    // inten1=0x400,0

    ieRes = SetINTEN1 (1, 3, True);  // True
    // inten1=0x400,8

    // tested on 26.03.97, 15:00: all tests okay
*/


    //----------------------------------------------------------------------
    // test CClockInterrupt
    //----------------------------------------------------------------------

    CClockInterrupt *irq;
    SUtcsuRead *utcsuRead = new SUtcsuRead;
    double retirq;  // return value of irq


    //----------------------------------------------------------------------
    // test the functions

    irq = new CClockInterrupt (1);
    // nodeID = 1, scheduleTime = -1, ipBit = 32, intstat = NULL

    irq->SetScheduleTime (7.3);
    // scheduleTime = 7.3

    retirq = irq->GetScheduleTime ();  // return 7.3

    irq->PreExecution ();
    // scheduleTime = -1, nothing else should happen

    utcsuRead->UTCINTSTAT1 = 0;
    irq->SetStatusRegister (utcsuRead->UTCINTSTAT1);
    // intstat = &utcsuWrite->UTCINTSTAT1

    irq->PreExecution ();
    // *intstat = 0

    irq->SetPendingBit (3);
    // ipBit = 3

    irq->PreExecution ();
    // *intstat=utcsuWrite->UTCINTSTAT1 = 8

    // tested on 07.12.97, 16:50: all tests okay


    delete irq;


    //----------------------------------------------------------------------
    // test CDutyB1Interrupt - correct behaviour of CClockInterrupt is
    //    assumed
    //----------------------------------------------------------------------

    CDutyB1Interrupt *dutyirq;
    CSoftwareClock *dummyclock = new CSoftwareClock(25e6, 2, 3);
    // just to have something to play around with; the class is tested
    // later on


    //----------------------------------------------------------------------
    // test the functions

    dutyirq = new CDutyB1Interrupt (3, dummyclock);
    // nodeID = 3, scheduleTime = -1, clock = dummyclock

    dutyirq->SetStatusRegister (utcsuRead->UTCINTSTAT2);
    dutyirq->SetPendingBit (26);
    dummyclock->SetAMORTTIMESET (2);  // allow amort. phase

    dutyirq->PreExecution ();
    // *intstat = 0x04000000, dummyclock->utcsuWrite->STARTAMORT=1

    // tested on 26.08.97, 16:40: all tests okay


    //----------------------------------------------------------------------
    // test CGpsInterrupt - correct behaviour of CClockInterrupt is assumed
    //----------------------------------------------------------------------

    CGpsInterrupt *gpsirq;


    //----------------------------------------------------------------------
    // test the functions

    SetGlobalTime (4.0/3);
    gpsirq = new CGpsInterrupt (2, dummyclock, GPU3);
    // nodeID = 2, scheduleTime = -1, clock = dummyclock, gpu = GPU3,
    // Scheduled, assocTime = 2
    // gpstime: normal clock-intproc with nodeID=2, enabled=False

    // tested on 31.07.97, 08:50: all tests okay so far

    gpsirq->SetTimeVector(1);
    // gpstime->vector = 0x4000 02B0
    gpsirq->EnableTime();
    // gpstime->enabled=True
    gpsirq->DisableTime();
    // gpstime->enabled=False

    // tested on 31.07.97, 08:50: all tests okay so far

    utcsuRead->UTCINTSTAT2 = 0;
    gpsirq->SetStatusRegister (utcsuRead->UTCINTSTAT2);
    gpsirq->SetPendingBit (0);
    ACProcess::SetActiveProcess (gpsirq);  // make the process active
    // (because processes are always active when calling Pre/PostExecution)

    SetGlobalTime (2);
    gpsirq->PreExecution ();
    // calls clock->CaptureTime (GPU3) [trace it!]

    // tested on 31.07.97, 08:50: all tests okay so far

    gpsirq->PostExecution ();
    // gpstime: Scheduled, scheduleTime=2.01 (current+GpsDelay)
    // waitTime=1 (time until next full second)

    // tested on 31.07.97, 09:00: all tests okay


    delete gpsirq;


    SetGlobalTime (0);  // for the following tests


    //----------------------------------------------------------------------
    // test CClock
    //----------------------------------------------------------------------

    CClock *clockreg;
    SUtcsuWrite *utcsuWrite = new SUtcsuWrite;
    SSingleParameter param;
    EBool result;
    INT32 res;
    double fosc;
    int i;


    //----------------------------------------------------------------------
    // test the ctor

    clockreg = new CClock (25e6, utcsuRead, utcsuWrite);
    // utcsuRead/Write_ set to utcsuRead/Write, oscFrequ = 25e6,
    // granularity_ = 5.960464477539e-8, granularityDT_ = 1.52587890625e-5,
    // granularityS_= 1.38777878078e-17, granularityAcc_=1.192092895508e-7,
    // granularityOsc_ = 4e-8, utcinten1=utcinten2 = 0

    // tested on 31.07.97, 09:10: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    param.type = RhoMinus;
    param.value = 0.1;

    result = clockreg->SetParameter (param);  // return False
    result = clockreg->GetParameter (param);  // return False

    param.type = ClockGranularity;
    param.value = 0.1;
    result = clockreg->SetParameter (param);  // return True
    // granularity = 0.1

    param.type = TimerGranularity;
    result = clockreg->SetParameter (param);  // return False

    param.type = AccuracyTransmissionLoss;
    param.value = 0.05;
    result = clockreg->SetParameter (param);  // return True
    // granularityAcc = 0.05

    param.type = ClockGranularity;
    result = clockreg->GetParameter (param);  // return True
    // value = 0.1

    param.type = TimerGranularity;
    result = clockreg->GetParameter (param);  // return True
    // value = 1.52587890625e-5

    param.type = AccuracyTransmissionLoss;
    result = clockreg->GetParameter (param);  // return True
    // value = 0.05

    // tested on 12.05.97, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test GetFrequency()

    fosc = clockreg->GetFrequency ();  // should return 25e6

    // tested on 20.02.97, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test GetGranularity() etc. functions

    fosc = clockreg->GetGranularity();  // should return 0.1
    fosc = clockreg->GetGranularityAcc();  // should return 0.05
    fosc = clockreg->GetGranularityDT();  // should return 1.52587890625e-5
    fosc = clockreg->GetGranularityOsc();  // should return 4e-8
    fosc = clockreg->GetGranularityS();  // should return 1.38777878078e-17

    // tested on 31.07.97, 09:10: all tests okay


    //----------------------------------------------------------------------
    // test the write access functions

    // first initialize the write structure
    for (i=0; i<sizeof(SUtcsuWrite)/sizeof(INT32); i++)
    {
        ((INT32*)utcsuWrite)[i] = 1;
    }
    // all members of utcsuWrite set to 1

    // LTU (Local Time Unit)
    clockreg->SetMSSET(0);
    clockreg->SetTSSET(0);
    clockreg->SetUSSET(0);
    clockreg->SetNTPSET(0);
    clockreg->SetSTEPAMORT(0);
    clockreg->SetSTEPPURE(0);
    clockreg->SetAMORTTIMESET(0);
    clockreg->SetSTARTAMORT(0);

    // ACU (Accuracy Unit)
    clockreg->SetLAMBDAPPURE(0);
    clockreg->SetLAMBDANPURE(0);
    clockreg->SetSTATEPSET(0);
    clockreg->SetSTATENSET(0);
    clockreg->SetBOUNDP(0);
    clockreg->SetALPHASET(0);
    clockreg->SetALPHAPNSET(0);

    // CLU (Clock Unit)
    clockreg->SetSWRESET(0);

    // ITU (Interrupt Unit)
    clockreg->SetUTCCONF1(0);
    clockreg->SetUTCCONF2(0);
    clockreg->SetUTCINTEN1(0);
    clockreg->SetUTCINTEN2(0);
    clockreg->SetUTCINTCLEAR1(0);
    clockreg->SetUTCINTCLEAR2(0);

    // SNU (Snapshot Unit)
    clockreg->SetSWSNAP(0);
    clockreg->SetDUTYH(0);
    clockreg->SetDUTYL(0);

    // APU (Application Unit)
    clockreg->SetAPPDUTYH(0);
    clockreg->SetAPPDUTYL(0);
    clockreg->SetAPPSEL(0);
    clockreg->SetAPPCLEAR(0);

    // SSU1-6 (Synchronization Subnet Unit 1-6)
    for (i=0; i<6; i++)
    {
        clockreg->SetDUTYAH ((ESSUnit)i, 0);
        clockreg->SetDUTYAL ((ESSUnit)i, 0);
        clockreg->SetDUTYBH ((ESSUnit)i, 0);
        clockreg->SetDUTYBL ((ESSUnit)i, 0);
    }
    // all members of utcsuWrite (except unused) should be set to zero;
    // step through and compare the register names with the function names!

    // tested on 17.05.97, 12:50: all tests okay


    //----------------------------------------------------------------------
    // test the read access functions

    // first initialize the read structure
    for (i=0; i<sizeof(SUtcsuRead)/sizeof(INT32); i++)
    {
        ((INT32*)utcsuRead)[i] = i;
    }

    // LTU (Local Time Unit)
    res = clockreg->GetMSGET();
    res = clockreg->GetTSGET();
    res = clockreg->GetUSGET();
    res = clockreg->GetTSGETL();
    res = clockreg->GetNSGET();
    res = clockreg->GetAMORTTIMEGETL();
    res = clockreg->GetAMORTTIMEGET();

    // ACU (Accuracy Unit)
    res = clockreg->GetSTATEPGETL();
    res = clockreg->GetSTATEPGET();
    res = clockreg->GetALPHAPGETL();
    res = clockreg->GetALPHANGETL();
    res = clockreg->GetALPHAPGET();
    res = clockreg->GetALPHANGET();
    res = clockreg->GetNALPHAPGET();

    // ITU (Interrupt Unit)
    res = clockreg->GetIDENTIFICATION();
    res = clockreg->GetUTCSTAT1();
    res = clockreg->GetUTCSTAT2();
    res = clockreg->GetUTCCONF1();
    res = clockreg->GetUTCCONF2();
    res = clockreg->GetUTCINTSTAT1();
    res = clockreg->GetUTCINTSTAT2();

    // SNU (Snapshot Unit)
    res = clockreg->GetMSSNU();
    res = clockreg->GetTSSNU();
    res = clockreg->GetACCPSNU();

    // APU (Application Unit)
    res = clockreg->GetMSAPP();
    res = clockreg->GetTSAPP();
    res = clockreg->GetACCPAPP();
    res = clockreg->GetMSAPPX();
    res = clockreg->GetTSAPPX();
    res = clockreg->GetACCPAPPX();

    // GPU1-3 (GPS Unit 1-3)
    for (i=0; i<3; i++)
    {
        res = clockreg->GetMSGPS((EGpsUnit)i);
        res = clockreg->GetTSGPS((EGpsUnit)i);
    }

    // SSU1-6 (Synchronization Subnet Unit 1-6)
    for (i=0; i<6; i++)
    {
        res = clockreg->GetMSRCV((ESSUnit)i);
        res = clockreg->GetTSRCV((ESSUnit)i);
        res = clockreg->GetMSXMT((ESSUnit)i);
        res = clockreg->GetTSXMT((ESSUnit)i);
        res = clockreg->GetACCPXMT((ESSUnit)i);
        res = clockreg->GetACCNPLXMT((ESSUnit)i);
        res = clockreg->GetACCNPHXMT((ESSUnit)i);
    }
    // should return the value of the read registers (except unused)
    // step through and compare the register names with the function names!

    // tested on 17.05.97, 12:50: all tests okay


    //----------------------------------------------------------------------
    // test Dis-/EnableInterrupt()

    clockreg->DisableInterrupt (0, 0xFFFFFFFF, 0xFFFFFFFF);
    // utscuWrite: UTCINTEN1=UTCINTEN2 = 0

    clockreg->EnableInterrupt (0, 0x10101010, 0x21001009);
    // UTCINTEN1 = 0x10101010, UTCINTEN2 = 0x21001009

    clockreg->EnableInterrupt (0, 0x21001009, 0x12345678);
    // UTCINTEN1 = 0x31101019, UTCINTEN2 = 0x33345679

    clockreg->DisableInterrupt (0, 0x22334455, 0x12345678);
    // UTCINTEN1 = 0x11001008, UTCINTEN2 = 0x21000001

    // tested on 16.08.97, 18:00: all tests okay


    //----------------------------------------------------------------------
    // test SetSWRESET() - continued

    clockreg->SetSWRESET(1);
    // utcint1=utcinten2 = 0

    // tested on 16.08.97, 18:00: all tests okay


    delete clockreg;


#ifdef TEST_STATIC_FUNCS
// commented out so the functions don't have to be made global every time

    //----------------------------------------------------------------------
    // test the global functions in clock_sw.cpp
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test GetInterruptType() - EInterruptBits in clockcfg.hpp is tested
    // as well

    EClockInterrupt itype;
    int dummy;

    for (i=0; i<64; i++)
    {
        if ((i!=3) && (i!=11) && (i!=13) && (i!=14) && (i!=15) && (i!=19)
            && (i!=30) && (i!=31) && (i!=38) && (i!=39))
            itype = GetInterruptType ((INT8)i);

        dummy=i;  // dummy in order to have a line for the breakpoint
    }
    // 0..11: INTT, 12..31: INTA, 32..51: INTN, 52..63: INTT

    // tested on 28.03.97, 14:20: all tests okay


    //----------------------------------------------------------------------
    // test DutyToSeconds()

    INT32 dutyh, dutyl;
    SecondsToNtp (2, dutyh, dutyl);
    dutyh = (dutyh<<8) + (dutyl>>24 & 0x0FF);
    dutyl = (dutyl>>8 & 0x0FFFF);

    double dutysec = DutyToSeconds (dutyh, dutyl);  // return (approx.) 2

    // tested on 06.12.97, 20:40: all tests okay


    //----------------------------------------------------------------------
    // test GetTime()

    CNtpTime testcycles;

    testcycles.ts = 25e6;
    fosc = GetTime (testcycles, 25e6);  // should return 1

    testcycles.ts = 25e6-1;
    fosc = GetTime (testcycles, 25e6);  // should return 0.999 999 96

    testcycles.ts = 3;
    fosc = GetTime (testcycles, 25e6);  // should return 0.000 000 12

    testcycles.ms = 1;
    fosc = GetTime (testcycles, 25e6);  // should return 171.798 691 96

    // tested on 08.08.97, 21:50: all tests okay

#endif


    //----------------------------------------------------------------------
    // test CSoftwareClock and the clock template
    // correct behaviour of CClock is assumed
    //----------------------------------------------------------------------

    CSoftwareClock *clock;


    //----------------------------------------------------------------------
    // test the ctor (which calls Init()) and the clock template

    SetGlobalTime (2);

    clockreg = clockTemplate->NewClock (25e6, 3, 0);
    clock = (CSoftwareClock *)clockreg;
    // clock: nodeID = 3, area =0, assocSimTime = 2, fOsc = 25e6,
    // gpu=EGpsUnitCount, utcsuRead->IDENTIFICATION = 0x49435410

    // tested on 21.09.97, 18:20: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    param.type = ClockDrift;
    param.value = 1e-6;

    result = clockreg->SetParameter (param);  // should return True
    // fOsc = 25 000 025

    param.value = -1e-12;
    result = clockreg->SetParameter (param);  // should return True
    // fOsc = 24 999 999. 999 975

    param.value = 1;
    result = clockreg->SetParameter (param);  // should return False
    // fOsc unchanged

    param.value = 0;
    result = clockreg->GetParameter (param);  // should return True
    // value = -1e-12 (yes, with a rounding error of +1.67e-17)

    // tested on 21.02.97, 17:20: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    SetGlobalTime (3.7);

    clockreg->SetSWRESET (1);
    // clock: all regs of utcsuRead/utcsuWrite/internals = 0, only
    // utcsuWrite->BOUNDP = 0xFF..F, utcsuRead->IDENTIFICATION = 0x49435410,
    // assocSimTime = 3.7

    SetGlobalTime (2);

    clockreg->SetSWRESET (1);
    // assocSimTime = 2

    // tested on 21.09.98, 18:20: all tests okay


    //----------------------------------------------------------------------
    // test GetRate()

    double rate;

    rate = clock->GetRate (1);  // return 1.25e-8
    // well, the result is 1.11e-8; but since the normal rate should be
    // approx. 1, rounding errors in faulty clocks are fine by me
    // (if it poses a problem, one would have to implement a different
    // algorithm for small values of STEP and STEPLOW)

    rate = clock->GetRate (90071993);  // return 1 (approx.)
    // it had a rounding error of +5ns, ie. it returned 1+5e-9
    // note: that's fine for bad clocks with rate drifts in 1e-6 range,
    // but won't do for GPS (drift range 1e-11, 1e-12)! So STEPLOW has
    // to be set as well to allow for the non binary oscillator frequency

    clockreg->SetMSSET (140*(1<<24));
    // set STEPLOW (that should make the result more accurate)

    rate = clock->GetRate (90071992);  // return 1 (approx.)
    // it had a rounding error of -6e-12, ie. it returned 1-6e-12
    // note: here I didn't round to the nearest number for step, but I
    // rounded down (round-to-nearest makes the result worse than that
    // without STEPLOW); since STEPLOW always adds to the result, step
    // should always be rounded down!

    // tested on 21.02.97, 19:00: all tests okay


    //----------------------------------------------------------------------
    // test GetClockCycles()

    CNtpTime cycles (0,0);

    clock->GetClockCycles (1, cycles);
    // cycles.ms = 0, cycles.ts = 25e6

    clock->GetClockCycles (116.79, cycles);
    // cycles.ms = 0, cycles.ts = 2 919 750 000

    clock->GetClockCycles (400, cycles);
    // cycles.ms = 2, cycles.ts = 1 410 065 408

    clock->GetClockCycles (1e-12, cycles);
    // cycles.ms = 0, cycles.ts = 0 (too small, not a whole cycle)

    clock->GetClockCycles (1e-7, cycles);
    // cycles.ms = 0, cycles.ts = 2

    clock->GetClockCycles (-5, cycles);
    // cycles.ms = 0, cycles.ts = 0

    // tested on 21.02.97, 19:10: all tests okay


    //----------------------------------------------------------------------
    // test GetRealDelay()

    double delta;

    // first have to set STEPPUREACT (through STEPPURE)
    clockreg->SetSTEPPURE (90071992);  // step reg. of a 25MHz clock
    clockreg->SetSTARTAMORT (1);  // that should take over into STEPPUREACT

    delta = clock->GetRealDelay (1);  // should return 1 (approx.)
    // rounding error: 5.94e-12
    delta = clock->GetRealDelay (100);  // should return 100 (approx.)
    // rounding error: 5.94e-10
    delta = clock->GetRealDelay (0.01);  // should return 0.01 (approx.)
    // rounding error: 5.94e-14

    param.value = -1e-3;  // change the drift rate
    clockreg->SetParameter (param);
    // fOsc = 24 975 000

    delta = clock->GetRealDelay (1);  // should return ca. 1.0010010...
    // rounding error: 5.95e-12
    delta = clock->GetRealDelay (100);  // should return 10.01001001...
    // rounding error: 5.95e-10
    delta = clock->GetRealDelay (0.01);  // should return 0.01001001...
    // rounding error: 5.95e-14

    // now try to compensate the drift through amortization
    double TwoPow51 = 2.251799813685e15;
    double freal = 24.975*1e5;  // using the real frequ. to compensate
    clockreg->SetAMORTTIMESET (freal);  // 0.1 sec amort time for delta=1
    clockreg->SetSTEPAMORT (0.1009/freal*TwoPow51);
    clockreg->SetSTARTAMORT (1);  // that should take over into AMORTTIMER
    delta = clock->GetRealDelay (1);  // should return 1 (approx.)
    // rounding error: -5e-10

    freal = 24.975*1e7;
    clockreg->SetAMORTTIMESET (freal);  // 10 sec amort time
    clockreg->SetSTEPAMORT (10.09/freal*TwoPow51);
    clockreg->SetSTARTAMORT (1);  // that should take over into AMORTTIMER
    delta = clock->GetRealDelay (100);  // should return 100 (approx.)
    // rounding error: -5e-8

    freal = 24.975*1e3;
    clockreg->SetAMORTTIMESET (freal);  // 0.001 sec amort time
    clockreg->SetSTEPAMORT (0.001009/freal*TwoPow51);
    clockreg->SetSTARTAMORT (1);  // that should take over into AMORTTIMER
    delta = clock->GetRealDelay (0.01);  // should return 0.01 (approx.)
    // rounding error: -5e-12

    // tested on 27.03.97, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test GetAmortCycles()

    clockreg->SetAMORTTIMESET (2);  // preload AMORTTIMER
    clockreg->SetSTARTAMORT (1);  // should load the AMORTTIMER
    cycles.ts = 5;
    INT32 amortCycles = clock->GetAmortCycles (cycles);  // return 2

    clockreg->SetAMORTTIMESET (freal);  // 0.001 sec amort time
    clockreg->SetSTARTAMORT (1);  // should load the AMORTTIMER
    amortCycles = clock->GetAmortCycles (cycles);  // return 5

    // tested on 16.03.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test UpdateNtp()

    CNtpTime updateCycles;
    param.value = 0;  // change the drift rate
    clockreg->SetParameter (param);
    // fOsc = 25 000 000

    updateCycles.ts = 25e6;
    clock->UpdateNtp (90071992, updateCycles);  // should be approx. 1 sec.
    // internals.ms=0, ts=16 777 216
    // result: ts = 16 777 215, us = 0xFFFx xxxx

/*
// commented out because the following test depends on the previous value
// of timestamp; but this test was needed to find a bug in UpdateNtp()

    updateCycles.ts = 25e8-25e6;
    clock->UpdateNtp (90071992, updateCycles);  // should be appr. 100 sec.
    // internal.ts = 1 677 721 600
    // result:  ts = 1 677 721 599, us = 0xFDxx xxxx
*/

    // tested on 10.02.98, 17:00: all tests okay


    //----------------------------------------------------------------------
    // test SetUpperLimit(), SetLowerLimit()

    clock->SetUpperLimit (True);
    // ALPHAP = 0x7FFFFFFF, NALPHAP = 0x1FFF

    clock->SetUpperLimit (False);
    // ALPHAN = 0x7FFFFFFF, NALPHAN = 0x1FFF

    // tested on 10.02.98, 19:00: all tests okay so far

    clock->SetLowerLimit (True);
    // ALPHAP = 0x80000000, NALPHAP = 0

    clock->SetLowerLimit (False);
    // ALPHAN = 0x80000000, NALPHAN = 0

    // tested on 25.05.97, 09:00: all tests okay


    //----------------------------------------------------------------------
    // test SetEnvelope(), GetEnvelope()

    clock->SetEnvelope (True, 0x12345678, 0x9abc);
    // ALPHAP = 0x12345678, NALPHAP = 0x1abc

    clock->SetEnvelope (False, 0xcba98765, 0x4321);
    // ALPHAN = 0xcba98765, NALPHAN = 0x0321

    INT32 alpha = 0;
    INT16 nalpha = 0;

    clock->GetEnvelope (True, alpha, nalpha);
    // alpha = 0x12345678, nalpha = 0x1abc

    clock->GetEnvelope (False, alpha, nalpha);
    // alpha = 0xcba98765, nalpha = 0x0321

    // tested on 10.02.98, 19:00: all tests okay


    //----------------------------------------------------------------------
    // test UpdateEnvelopes()

    clock->SetLowerLimit (True);  // set both envelopes to the lower limit
    clock->SetLowerLimit (False);

    updateCycles.ts = 5;
    clockreg->SetLAMBDAPPURE (0x7FFF0000);  // add some values
    clockreg->SetLAMBDANPURE (0x00010000);
    clockreg->SetSTATEPSET (0x1000FFFF);
    clockreg->SetSTATENSET (0x2000FFFF);
    clockreg->SetAMORTTIMESET (25e2);
    clockreg->SetSTARTAMORT (1);

    clock->UpdateEnvelopes (False, updateCycles);
    // ALPHAP=0x9001000E, NALPHAP=(0xFFE0>>3=)0x1FFC
    // ALPHAN=0xA000FFFF, NALPHAN=(0x0020>>3=)0x0004

    updateCycles.ts = 5;
    clockreg->SetLAMBDAPPURE (0x80010000);  // subtract previously...
    clockreg->SetLAMBDANPURE (0xFFFF0000);  // ... added values
    clockreg->SetSTATEPSET (0xEFFF0001);
    clockreg->SetSTATENSET (0xDFFF0001);

    clock->UpdateEnvelopes (False, updateCycles);
    // ALPHAP=0x80000000, NALPHAP=0
    // ALPHAN=0x80000000, NALPHAN=0

    // tested on 11.02.98, 15:30: all tests okay so far


    clockreg->SetLAMBDAPPURE (0x7FFF0000);  // add again
    clockreg->SetLAMBDANPURE (0x00010000);
    clockreg->SetSTATEPSET (0x1000FFFF);
    clockreg->SetSTATENSET (0x2000FFFF);

    clock->UpdateEnvelopes (False, updateCycles);
    // ALPHAP=0x9001000E, NALPHAP=(0xFFE0>>3=)0x1FFC
    // ALPHAN=0xA000FFFF, NALPHAN=(0x0020>>3=)0x0004

    updateCycles.ts = 6;
    clockreg->SetLAMBDAPPURE (0x80010000);  // now subtract more
    clockreg->SetLAMBDANPURE (0xFFFF0000);
    clockreg->SetSTATEPSET (0xEFFF0001);
    clockreg->SetSTATENSET (0xDFFF0001);

    clock->UpdateEnvelopes (False, updateCycles);
    // ALPHAP=0x80000000, NALPHAP=0 (lower limit)
    // ALPHAN=0x80000000, NALPHAN=0 (lower limit)

    // tested on 11.02.98, 15:30: all tests okay so far


    clockreg->SetLAMBDAPPURE (0x7FFF);  // add much in pure phase
    clockreg->SetLAMBDANPURE (0x6FFF);
    clockreg->SetSTARTAMORT (1);  // take over the pure values
    updateCycles.ts = 0xC0000000; // large enough for positive overflow

    clock->UpdateEnvelopes (True, updateCycles);
    // ALPHAP=0x7FFFFFFF, NALPHAP=1FFF (upper limit)
    // ALPHAN=0x7FFFFFFF, NALPHAN=1FFF (upper limit)


    clockreg->SetLAMBDAPPURE (0x8001);  // subtract it again
    clockreg->SetLAMBDANPURE (0x9001);
    clockreg->SetSTARTAMORT (1);

    clock->UpdateEnvelopes (True, updateCycles);
    // ALPHAP=0x80000000, NALPHAP=0 (lower limit)
    // ALPHAN=0x80000000, NALPHAN=0 (lower limit)

    // tested on 11.02.98, 15:30: all tests okay so far


    clockreg->SetLAMBDAPPURE (0x7FFF);  // add much
    clockreg->SetLAMBDANPURE (0x4001);
    clockreg->SetSTARTAMORT (1);  // take over the pure values
    updateCycles.ts = 0x40000000; // large enough so that result is positive

    clock->UpdateEnvelopes (True, updateCycles);
    // ALPHAP=0x7FFE0000, NALPHAP=0
    // ALPHAN=0x00020000, NALPHAN=0


    clockreg->SetLAMBDAPPURE (0x8001);  // subtract it again
    clockreg->SetLAMBDANPURE (0xBFFF);
    clockreg->SetSTARTAMORT (1);

    clock->UpdateEnvelopes (True, updateCycles);
    // ALPHAP=0x80000000, NALPHAP=0
    // ALPHAN=0x80000000, NALPHAN=0

    // tested on 11.02.98, 15:30: all tests okay so far


    updateCycles.ts = 5;
    clockreg->SetLAMBDAPPURE (0x00020000);  // add some values
    clockreg->SetLAMBDANPURE (0x00010000);
    clockreg->SetSTATEPSET (0x1000FFFF);
    clockreg->SetSTATENSET (0x2000FFFF);
    clockreg->CClock::SetSTARTAMORT (2);  // set to middle of amort. phase
    // --> all five cycles are used for step value

    clock->UpdateEnvelopes (False, updateCycles);
    // ALPHAP=0x80000000, NALPHAP=0x000A
    // ALPHAN=0x80000000, NALPHAN=0x0005

    // tested on 11.02.98, 15:30: all tests okay


    //----------------------------------------------------------------------
    // test Update()

    clock->Update();  // should return at once (clock up to date)

    param.value = -1e-3;  // change the drift rate again
    clockreg->SetParameter (param);
    freal = 24.975*1e5;  // copied from the test of GetRealDelay()
    clockreg->SetAMORTTIMESET (freal);
    clockreg->SetSTEPAMORT (0.1009/freal*TwoPow51);
    clockreg->SetSTARTAMORT (1);
    SetGlobalTime (3+25e-9);  // 25e-9 is no full cycle --> is not updated

    clock->Update();  // NTP Time should be approx. 2 seconds
    // assocSimTime = 3
    // internals: ms=0, ts=33 554 432
    // ts = 33 554 432, us = 0x21B58C4 (I think the rounding error is the
    // result of the rounded step registers; is this really the case?)
    // ALPHAP=0x90010260, NALPHAP=0x17C0
    // ALPHAN=0xA001012F, NALPHAN=0x1BE0

    // tested on 11.02.98, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test GetCurrentClockValue()

    delta = clock->GetCurrentClockValue();  // return 2 (ideal)
    // result: 2.000 000 000 490 533

    // tested on 08.08.97, 22:00: all tests okay


    //----------------------------------------------------------------------
    // test SetOverflow()

    clock->SetOverflow (1);
    // internals->oberflow = True

    clock->SetOverflow (0);
    // internals->oberflow = False

    // tested on 30.03.97, 10:10: all tests okay


    //----------------------------------------------------------------------
    // test CreateInterrupt()
    //     remember: nodeID = 3, area = 0

    clockreg->SetUTCINTEN1 (1<<5);
    irq = clock->CreateInterrupt (5);
    // vector = 0x30A2, ipBit = 5, intstat = &clock->utcsuRead->UTCINTSTAT1,
    // nodeID = 3, enabled = True

    irq = clock->CreateInterrupt (44);
    // vector = 0x30A4, ipBit = 12, intstat = &clock->utcsuRead->UTCINTSTAT2,
    // nodeID = 3, enabled = False

    // tested on 26.03.97, 20:00: all tests okay so far


    //----------------------------------------------------------------------
    // test Enable/DisableInterrupt()

    clock->EnableInterrupt (44);
    // enabled = True

    clock->DisableInterrupt (44);
    // enabled = False

    // tested on 19.03.97, 22:00: all tests okay


    //----------------------------------------------------------------------
    // test CreateInterrupt() - continued

    clock->DisableInterrupt (5);

    irq = clock->CreateInterrupt (5);
    // vector = 0x30A2, ipBit = 5, intstat = &clock->utcsuRead->UTCINTSTAT1,
    // nodeID = 3, enabled = False
    
    // tested on 16.03.97, 22:10: all tests okay so far


    gpsirq = (CGpsInterrupt *)clock->CreateInterrupt (32);
    // vector = 0x30A4, ipBit = 0, gpu = GPU1,
    // intstat = &clock->utcsuRead->UTCINTSTAT2,
    // nodeID = 3, enabled = True, Scheduled, assocSimTime = 4

    // tested on 08.08.97, 22:40: all tests okay


    dutyirq = (CDutyB1Interrupt *)clock->CreateInterrupt (58);
    // vector = 0x30A2, ipBit = 26, intstat = &clock->utcsuRead->UTCINTSTAT2,
    // nodeID = 3, enabled = False, clock_ = clock

    // tested on 26.08.97, 16:30: all tests okay


    //----------------------------------------------------------------------
    // test ScheduleInterrupt()
    //     assumption: irq points to irq(5), clock value =^= 2 sec.,
    //                 global time = 3 sec.

    irq->SetScheduleTime (-1);
    clock->ScheduleInterrupt (irq);
    // nothing should happen (irq has no scheduleTime)

    irq->SetScheduleTime (1);  // that's a time in the past
    clock->ScheduleInterrupt (irq);
    // irq should be scheduled for t=3 (at once)

    irq->SetScheduleTime (4);
    clock->ScheduleInterrupt (5);  // calls ScheduleInterrupt (irq)
    // irq should be scheduled for global time t=5.002002..
    // rounding error 5.5e-10

    // tested on 26.08.97, 16:30: all tests okay


    //----------------------------------------------------------------------
    // test DescheduleInterrupt()

    clock->DescheduleInterrupt (5);
    // irq is descheduled

    // tested on 26.03.97, 09:40: all tests okay


    //----------------------------------------------------------------------
    // test ModifiedDutyTimer()

    CNtpTime duty;
    duty.ms = 1<<8;  // lsb = bit 0, ie. = 1 second; ms starts at bit 8
    duty.ts = 11<<8;

    clock->SetUTCINTEN2 (1<<(CSoftwareClock::DutyTimerBSSU4-32));
    clock->ModifiedDutyTimer (CSoftwareClock::DutyTimerBSSU4, duty);
    // should do nothing (trace it!)

    duty.ts = (11<<8) + (1<<16);
    clock->ModifiedDutyTimer (CSoftwareClock::DutyTimerBSSU4, duty);
    irq = clock->CreateInterrupt (CSoftwareClock::DutyTimerBSSU4);
    // irq should be scheduled at some time in the future (assocTime > 3)

    // DEBUGGER: set clock->utcsuRead->UTCSTAT2 = 0xFFFFFFFF
    duty.ts = 11<<8;
    clock->ModifiedDutyTimer (CSoftwareClock::DutyTimerBSSU4, duty);
    // irq should be descheduled (Passive state)
    // clock->utcsuRead->UTCSTAT2 = 0xFBFFFFFF

    duty.ts = (11<<8) + (1<<16);
    clock->ModifiedDutyTimer (CSoftwareClock::DutyTimerBSSU4, duty);
    // irq is scheduled again

    // tested on 26.08.97, 16:30: all tests okay


    //----------------------------------------------------------------------
    // test SetSignal()

    // assumption: DutyTimerBSSU4 int. process is scheduled (see test of
    // ModifiedDutyTimer())
    clock->SetSignal (CSoftwareClock::DutyTimerBSSU4);
    // nothing should happen

    // assumption: int. process with ipBit=5 is not scheduled (was only
    // used in test of CreateInterruptProcess())
    clock->SetSignal (5);
    // utcsuRead->UTCSTAT1 = 0x20

    clock->SetSignal (33);
    // utcsuRead->UTCSTAT2 = 0x02

    // tested on 26.03.97, 21:00: all tests okay


    //----------------------------------------------------------------------
    // test GetSignal()

    result = clock->GetSignal (1);  // return False
    result = clock->GetSignal (5);  // return True
    result = clock->GetSignal (32);  // return False
    result = clock->GetSignal (33);  // return True

    // tested on 26.03.97, 21:00: all tests okay


    //----------------------------------------------------------------------
    // test ClearSignal()

    clock->ClearSignal (3);
    // nothing should happen
    clock->ClearSignal (5);
    // utcsuRead->UTCSTAT1 = 0x0
    clock->ClearSignal (33);
    // utcsuRead->UTCSTAT2 = 0x0

    // tested on 26.03.97, 21:00: all tests okay


    //----------------------------------------------------------------------
    // test GetPolarity()

    clockreg->SetUTCCONF1 (1<<6);
    clockreg->SetUTCCONF2 (1<<4);

    result = clock->GetPolarity (1);  // return False
    result = clock->GetPolarity (6);  // return True
    result = clock->GetPolarity (33);  // return False
    result = clock->GetPolarity (36);  // return True

    // tested on 26.03.97, 21:00: all tests okay


    //----------------------------------------------------------------------
    // test the write access functions

    // LTU (Local Time Unit)
    clockreg->SetMSSET ((140<<24) + 1);
    // internals.steplow = 140
    clockreg->SetTSSET (10);
    clockreg->SetUSSET (11);
    clockreg->SetNTPSET (0);
    // internals.ms = 1, ts=10, us=11, ns=0
    clockreg->SetSTEPAMORT (15);
    clockreg->SetSTEPPURE (20);
    clockreg->SetAMORTTIMESET (11);
    clockreg->SetSTARTAMORT (0);
    // internals.steppureact = 20, amorttimer = 11
    clockreg->SetSTEPAMORT (15);
    // output: report (set during amort. phase, module Clock, id=3, area=4)

    // tested on 19.03.97, 19:10: all tests okay so far


    // ACU (Accuracy Unit)
    clockreg->SetLAMBDAPPURE ((1<<16) + 3);
    clockreg->SetLAMBDANPURE ((2<<16) + 4);
    clockreg->SetSTARTAMORT (0);
    // internals.lambdappureact = 3, lambdanpureact = 4
    clockreg->SetALPHASET (23);
    clockreg->SetALPHAPNSET (0);
    // internals.alphap=alphan = 23, nalphap(n)=0x1FFF

    // tested on 10.02.98, 19:10: all tests okay so far


    // ITU (Interrupt Unit)
    irq = clock->CreateInterrupt (5);
    irq->SetScheduleTime (5);
    clockreg->SetUTCINTEN2 (1<<12);
    clockreg->SetUTCINTEN1 (1<<5);  // should enable irq(5), schedule it
    clockreg->SetUTCINTEN1 (0);  // disable and deschedule irq(5)
    // DEBUGGER: set clock->utcsuRead->UTCINTSTAT1 = 33, UTCINTSTAT2 = 4097
    clockreg->SetUTCINTCLEAR1 (1<<5);
    // UTCINTSTAT1 = 1
    clockreg->SetUTCINTCLEAR2 (1<<12);
    // UTCINTSTAT2 = 1

    // tested on 26.03.97, 20:15: all tests okay so far


    // SNU (Snapshot Unit)
    clockreg->SetSWSNAP (0);
    // calls SwSnapshot() [which calls GetTSGETL()], so that is tested
    // as well
    // has same effect as TSGETL: MSGET, TSGET, USGET, NSGET <- internals,
    // ALPHAP(N)GET, NALPHAP(N)GET, STATEP(N)GET are latched as well; the
    // functions are tested below, just check for MSGET, TSGET <- internals

    // tested on 26.03.97, 20:15: all tests okay so far


    // APU (Application Unit)
    irq = clock->CreateInterrupt (CSoftwareClock::DutyTimerAPP);
    clockreg->SetUTCINTEN1 (1<<(CSoftwareClock::DutyTimerAPP));
    // create and enable the interrupt (ignore the wrong schedule time, it
    // is not needed for this test and therefore not set)
    clockreg->SetAPPDUTYL (0);
    // irq is descheduled, scheduleTime = -1
    clockreg->SetAPPDUTYH (1<<8);
    // irq does not change
    clockreg->SetAPPDUTYL ((1<<28) + (1<<16));
    // irq is scheduled again

    // tested on 26.08.97, 16:30: all tests okay so far


    // SSU4 (Synchronization Subnet Unit 4)
    irq = clock->CreateInterrupt (CSoftwareClock::DutyTimerBSSU4);
    clockreg->SetUTCINTEN2 (1<<(CSoftwareClock::DutyTimerBSSU4-32));
    // create and enable the interrupt (ignore the wrong schedule time, it
    // is not needed for this test and therefore not set)
    clockreg->SetDUTYBL (SSU4, 0);
    // irq is descheduled, scheduleTime = -1
    clockreg->SetDUTYBH (SSU4, 1<<8);
    // irq does not change
    clockreg->SetDUTYBL (SSU4, (1<<28) + (1<<16));
    // irq is scheduled again

    // tested on 17.05.97, 12:50: all tests okay so far

    irq = clock->CreateInterrupt (CSoftwareClock::DutyTimerASSU2);
    clockreg->SetUTCINTEN2 (1<<(CSoftwareClock::DutyTimerASSU2-32));
    // create and enable the interrupt (ignore the wrong schedule time, it
    // is not needed for this test and therefore not set)
    clockreg->SetDUTYAL (SSU2, 0);
    // irq is descheduled, scheduleTime = -1
    clockreg->SetDUTYAH (SSU2, 1<<8);
    // irq does not change
    clockreg->SetDUTYAL (SSU2, (1<<28) + (1<<16));
    // irq is scheduled again

    // tested on 29.07.98, 15:30: all tests okay


    //----------------------------------------------------------------------
    // test the read access functions

    clockreg->SetMSSET ((140<<24) + 2);
    clockreg->SetTSSET (20);
    clockreg->SetUSSET (30);
    clockreg->SetNTPSET (0);
    clockreg->SetALPHASET (0x130);
    clockreg->SetALPHAPNSET (0);
    // internals.ms = 2, ts=20, us=30, ns=0, alphap(n)=0x130
    // DEBUGGER: set internals.NALPHAP(N) = 0x1234

    // LTU (Local Time Unit)
    res = clockreg->GetTSGETL();  // return 20 (internals.ts)
    // should have copied internals.ms/ts/us/ns into utcsuRead.ms/ts/us/ns,
    // utcsuRead.ALPHAP(N)GET = 0x130, NALPHAPGET = 0x12341234

    clockreg->SetSTARTAMORT (1);
    // utcsuRead.AMORTTIMEGET should be zero
    res = clockreg->GetAMORTTIMEGETL();  // return internals.AMORTTIMER
    // utcsuRead.AMORTTIMEGET should be set to internals.AMORTTIMER

    // tested on 27.03.97, 15:40: all tests okay so far


    // ACU (Accuracy Unit)
    clockreg->SetALPHASET (0x12345678);
    clockreg->SetALPHAPNSET (0);
    // alphap(n)=0x12345678, nalphap(n)=0x1FFF
    // DEBUGGER: set internals.NALPHAP(N) = 0x3567

    // note: GetSTATEPGETL() calls GetAccuracy(), so that is tested as well
    res = clockreg->GetSTATEPGETL();  // return 0x24682468
    // utcsuRead.STATEPGET = 0x24682468
    // DEBUGGER: set internals.ALPHAP = 0x81110000  // negative!
    res = clockreg->GetSTATEPGETL();  // return 0x24680000
    res = clockreg->GetALPHAPGETL();
    // utcsuRead.ALPHAPGET = 0x81110000, utcsuRead.NALPHAPGET = 0xmmmm1567
    res = clockreg->GetALPHANGETL();
    // utcsuRead.ALPHANGET = 0x12345678, utcsuRead.NALPHAPGET = 0x15671567

    // tested on 10.02.98, 19:00: all tests okay so far


    // ITU (Interrupt Unit)
    res = clockreg->GetUTCSTAT1();  // return SAPPDUTY | SCAMA (0x20000001)
    clockreg->SetAMORTTIMESET (0);
    clockreg->SetSTARTAMORT (0);  // turn off amortization
    res = clockreg->GetUTCSTAT1();  // should return SAPPDUTY (0x20000000)
    res = clockreg->GetUTCCONF1();  // should return CPOLHWSNAP (0x10000000)
    // utcsuRead.UTCCONF1 = 0x10000000
    clockreg->SetUTCCONF2 (32);
    res = clockreg->GetUTCCONF2();  // should return 32
    // utcsuRead.UTCCONF2 = 32

    // tested on 27.03.97, 16:00: all tests okay


    //----------------------------------------------------------------------
    // test HwSnapshot()
    //    assumption: global time = 3 secs

    clockreg->SetUTCINTEN1 (SUtcsuWrite::IHWSNAP);  // enable HW snapshot

    clock->HwSnapshot();
    // utcsuRead.MSSNU = 2, utcsuRead.TSSNU = 20,
    // utcsuRead.ACCPSNU = 0x00010001 (or 0x00010000, depending on the
    // value of internals->ALPHAP)

    irq = clock->CreateInterrupt (16);  // get the snapshot interrupt
    // irq: enabled=True, Scheduled, assocSimTime=3

    // tested on 28.03.97, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test CaptureTime()

    clockreg->SetMSSET (0xfadeabbe);
    clockreg->SetTSSET (0xaffeaffe);
    clockreg->SetUSSET (0xFFFFFFFF);
    clockreg->SetNTPSET (1);
    // internals.ms = 0x0ffffff, ts=us = 0xffffffff

    clock->CaptureTime (GPU2);
    // utcsuRead->MSGPS (GPU2) = 0x00deabbe, utcsuRead->TSGPS = 0xaffeaffe,
    // gpu=GPU2

    // tested on 29.07.98, 13:50: all tests okay


    //----------------------------------------------------------------------
    // test GetGpsUnit()

    EGpsUnit captured = clock->GetGpsUnit();  // return GPU2

    // tested on 08.08.97, 22:30: all tests okay


    //----------------------------------------------------------------------
    // test GetGpsInterrupt()

    gpsirq = NULL;  // remember the address of the interrupt!
    gpsirq = clock->GetGpsInterrupt (GPU1);  // return the same address

    // tested on 08.08.97, 22:30: all tests okay


    //----------------------------------------------------------------------
/**/// note: NOT tested were the following functions
    // *) InitiateSnapshot() - see clk2test.cpp, test of InitSnapshot()
    // *) ReportError() - okay (see the DOS box, some errors are reported)
    // *) RescheduleInterrupts()
    // *) dtor and Dispose()


    delete clockreg;
    // here, an assert will fail, because there are still interrupts
    // enabled; the dtor of the software clock checks if all interrupts
    // that were enabled through the clock are disabled again before the
    // clock is deleted


    //----------------------------------------------------------------------

    return 0;
}

/* obsolete

EBool SetINTEN1 (int area, INT8 bitnumber, EBool on);
// test for a global function to modify single bits in INTEN


static INT32 inten1[2] = {0,0};

#define UTCINTEN1ADDR_MOD0 (&inten1[0])
#define UTCINTEN1ADDR_MOD1 (&inten1[1])

// enable/disable the interrupt bit in the UTCINTEN1 register of the
// UTCSU corresponding to 'area'
// params: area: 0..ASICCNT-1 (assumption: ASICCNT = 6, found in all.h of gr)
//         bitnumber: 0..31
//         on: True (bit is set to 1), False (bit is set to zero)
// assert: bitnumber < 32, area < MAXASICS
EBool SetINTEN1 (int area, INT8 bitnumber, EBool on)
{
    static INT32 bitmask[MAXASICS] = {0,0};
    static INT32 *utcinten1[MAXASICS] =
        {(INT32 *)UTCINTEN1ADDR_MOD0, (INT32 *)UTCINTEN1ADDR_MOD1};

    Assert (bitnumber < 32);
    Assert (area < MAXASICS);

/ ** /// the following section should be made atomic; use a non-blocking
    // semaphore, otherwise ISRs cannot disable the interrupts
    // hm, alternative: don't make it atomic, leave it to the tasks (ISRs)
    // to ensure atomicity
    // seconds version sounds easier and less error prone, would prefer it
    // if (sm_p (...)) return False;  // semaphore couldn't be acquired
    {
        if (on) bitmask[area] |= (INT32)1<<bitnumber; // add int. bit to mask
        else bitmask[area] &= ~(1<<bitnumber);  // remove int. bit from mask

        *utcinten1[area] = bitmask[area];  // set the interrupt bits anew
    }
    // sm_v (...);

    return True;  // bit was set successfully
}
*/


#include "inlines.hpp"


