//
// conttest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test ACController
// note: ReportError() was made public for this test
// test CSoftwareController
// note: StartApplication()/StopApplication() were made public for this test
//
// created: 25.01.97
//
// last change: 29.07.98
//


#include <iostream.h>

#include "..\sockstre.hpp"
#include "..\cont_sw.hpp"
#include "..\general.hpp"
#include "..\globals.hpp"
#include "..\sim_glob.hpp"
#include "..\sockunix.hpp"

extern int g_masterArea;
// for checking if CreateNewApplication() refuses to use this area

EBool g_finished = False;
// for checking if CSoftwareController::StopExecution() sets it to True


class CTest : public ACController
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CTest (char name[4], ACSocketAddress *addr): ACController (name,addr) {}


    //----------------------------------------------------------------------
    //--- starting/stopping applications
    //----------------------------------------------------------------------

    virtual EBool StartApplication (SINT32 nodeID, int area) {
        cout << "StartApplication() called with (" << nodeID << ", ";
        cout << area << ")" << endl; return True; }

    virtual EBool StopApplication (SINT32 nodeID, int area) {
        cout << "StopApplication() called with (" << nodeID << ", ";
        cout << area << ")" << endl; return True; }


    //----------------------------------------------------------------------
    //--- stop program execution
    //----------------------------------------------------------------------

    virtual void StopExecution() {
        cout << "StopExecution() called" << endl; }
};


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of class ACController
    // correct behaviour of classes CSupervisor (ctor, GetIdentification()),
    // CControllingObject and CBroadcastNetwork is assumed
    //----------------------------------------------------------------------

    CTest *basecont;  // basic simulation controller
    CUnixAddress addr ("dummy");
    EBool result;


    //----------------------------------------------------------------------
    // test ctor()

    basecont = new CTest ("CTRL", &addr);
    // qid = ...
    // should call SendModule (Controller) and SendEnumSizes() (trace it!)

    // tested on 29.07.98, 14:30: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    SEventReport report;
    report.module = Controller;
    report.Set (ReceivedMessage, True);
    result = basecont->SetParameter (report);  // return True
    // eventmask = {64,0,0}

    report.Set (ReceivedMessage, False);
    result = basecont->GetParameter (report);  // return True
    // report.set = True

    report.module = Lance;
    result = basecont->SetParameter (report);  // return False
    result = basecont->GetParameter (report);  // return False

    // tested on 11.02.97, 19:30: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand()
    // note: DeleteApplication results in a call to DeleteSupervisor(),
    // Suspend/ResumeSimulation --> ExecuteCommandForAll(),
    // CreateApplication --> CreateNewApplication()
    // so these functions are tested as well

    //--- unknown command

    SCommand command;
    command.module = Controller;
    command.type = CreateNetwork;
    result = basecont->ExecuteCommand (command);  // return False

    // tested on 25.01.97, 21:15: all tests okay so far


    //--- unknown module

    command.module = Lance;
    command.type = ResetModule;
    result = basecont->ExecuteCommand (command);  // return False

    // tested on 11.02.97, 19:30: all tests okay so far


    //--- application commands

    result = listOfSupervisors.Empty();  // return True
    // no supervisors exist

    SApplCommand applc;
    applc.module = Controller;
    applc.type = CreateApplication;
    applc.nodeID = 1;
    applc.area = 2;

    g_masterArea = applc.area;
    result = basecont->ExecuteCommand (applc);  // return False (masterArea)
    g_masterArea = -1;
    result = basecont->ExecuteCommand (applc);  // return True

    result = listOfSupervisors.Empty();  // return False
    // supervisor was created
    // output: StartApplication() called with (1,2)

    CSupervisor *sup = listOfSupervisors.GetFirst();
    // sup: nodeID = 1, area = 2, qid=...

    // tested on 10.02.98, 11:30: all tests okay so far

    result = basecont->ExecuteCommand (applc);  // False (already exists)
    applc.area = -1;
    result = basecont->ExecuteCommand (applc);  // return False
    applc.area = NumberOfAreas;
    result = basecont->ExecuteCommand (applc);  // return False

    // tested on 29.07.98, 14:30: all tests okay so far

    SApplCommand appl;
    appl.module = Controller;
    appl.type = ResumeApplication;
    appl.nodeID = 2;
    appl.area = 2;
    result = basecont->ExecuteCommand (appl);  // return False

    appl.nodeID = 1;
    result = basecont->ExecuteCommand (appl);  // return True
    // sup->ResumeOperation() is called (trace it!)

    appl.type = SuspendApplication;
    result = basecont->ExecuteCommand (appl);  // return True
    // sup->SuspendOperation() is called (trace it!)

    appl.type = DeleteApplication;
    appl.nodeID = 2;
    result = basecont->ExecuteCommand (appl);  // return False

    appl.nodeID = 1;
    result = basecont->ExecuteCommand (appl);  // return True
    // output: StopApplication() called with (1,2)

    result = listOfSupervisors.Empty();  // return True
    // list empty again

    // tested on 22.04.97, 12:00: all tests okay so far


    //--- simulation commands

    command.module = Controller;
    command.type = ResumeSimulation;
    result = basecont->ExecuteCommand (command);  // return False

    command.type = SuspendSimulation;
    result = basecont->ExecuteCommand (command);  // return False

    applc.area = 2;
    basecont->ExecuteCommand (applc);  // make two supervisors
    applc.nodeID = 2;
    basecont->ExecuteCommand (applc);

    command.type = ResumeSimulation;
    result = basecont->ExecuteCommand (command);  // return True

    command.type = SuspendSimulation;
    result = basecont->ExecuteCommand (command);  // return True

    command.type = ResetSimulation;
    result = basecont->ExecuteCommand (command);  // return True

/*
    // DEBUGGER: since ExecuteCommand() is normally called in reaction to a
    // command from the socket, CControllingObject::ReplyToMessage (double)
    // is called (return value of command); so set socket->receiveMsgID
    // to a value >= 0, otherwise an Assert() will fail!

    command.type = KillSimulation;
    result = basecont->ExecuteCommand (command);  // return True
    // output: StopExecution() called
*/

    // tested on 05.05.97, 21:10: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    basecont->Reset ();
    // eventmask = {0,0,0}

    result = listOfSupervisors.Empty();  // return True

    // tested on 31.07.97, 10:50: all tests okay


    //----------------------------------------------------------------------
    // test ResetTheSimulation()

    result = basecont->ResetTheSimulation ();  // return False

    applc.type = CreateApplication;
    applc.nodeID = 1;
    applc.area = 2;
    result = basecont->ExecuteCommand (applc);  // return True

    result = basecont->ResetTheSimulation ();  // return True
    // should call sup->Reset() (trace it!)

    // tested on 12.05.97, 16:30: all tests okay


    //----------------------------------------------------------------------
    // test ReportError()

    basecont->ReportError (Reporting, __FILE__, __LINE__, "test");
    // should add "module = Controller" to the output

    // tested on 11.02.97, 19:30: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of class CSoftwareController
    // correct behaviour of classes ACController and CBroadcastNetwork is
    // assumed
    //----------------------------------------------------------------------

    CSoftwareController *controller;
    InstallNetworks();  // in order to be able to create new networks

    //----------------------------------------------------------------------
    // test ctor()

    controller = new CSoftwareController ("CTR1", &addr);
    // networkID = 1

    // tested on 25.01.97, 21:15: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand()
    // note: CreateNetwork results in a call to CreateNewNetwork(),
    // so that is tested as well

    //--- network commands

    result = listOfNetworks.Empty();  // should return True
    // no networks exist

/**/// DEBUGGER: since ExecuteCommand() is normally called in reaction to a
    // command from the socket, CControllingObject::ReplyToMessage (double)
    // is called (networkID of the new network); so set socket->receiveMsgID
    // to a value >= 0, otherwise an Assert() will fail!

    SNetworkCreateCommand netc;
    netc.module = Controller;
    netc.type = CreateNetwork;
    netc.networkType = (ENetworkType)2;
    result = controller->ExecuteCommand (netc);  // return False

    netc.networkType = Broadcast;
    result = controller->ExecuteCommand (netc);  // return True
    // networkID = 2

    result = listOfNetworks.Empty();  // should return False
    // network should exist

    ACNetwork *network = listOfNetworks.GetFirst();
    // network->networkID = 1

    SNetworkConnectCommand conn;
    conn.module = Network;
    conn.type = ConnectNode;
    conn.networkID = 1;
    conn.nodeID = 2;
    result = controller->ExecuteCommand (conn);  // return True
    // network->listOfNodes = {2}

    conn.networkID = 2;
    result = controller->ExecuteCommand (conn);  // return False

    conn.type = DisconnectNode;
    result = controller->ExecuteCommand (conn);  // return False

    conn.networkID = 1;
    result = controller->ExecuteCommand (conn);  // return True
    // network->listOfNodes = {}

    // tested on 11.02.97, 19:30: all tests okay so far


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    SSingleNetworkParameter netp;
    netp.module = Network;
    netp.type = Lambda;
    netp.networkID = 2;
    netp.value = 5;
    result = controller->SetParameter (netp);  // return False

    netp.networkID = 1;
    result = controller->SetParameter (netp);  // return True

    netp.value = 0;
    result = controller->GetParameter (netp);  // return True
    // netp.value = 5

    report.module = Controller;
    report.Set (ReceivedMessage, True);
    result = controller->SetParameter (report);  // return True
    // eventmask = {64,0,0}

    report.Set (ReceivedMessage, False);
    result = controller->GetParameter (report);  // return True
    // report.set = True

    // tested on 11.02.97, 19:30: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand() - continued
    // note: DeleteNetwork calls DeleteANetwork(),
    //    KillSimulation --> DeleteAllSimulationObjects(), StopExecution()
    // so these are tested as well

    //--- network commands

    SNetworkCommand netd;
    netd.module = Controller;
    netd.type = DeleteNetwork;
    netd.networkID = 2;
    result = controller->ExecuteCommand (netd);  // return False

    netd.networkID = 1;
    result = controller->ExecuteCommand (netd);  // return True

    result = listOfNetworks.Empty();  // should return True
    // again no networks

    // tested on 31.07.97, 10:50: all tests okay so far


/*
// commented out after the test, because StopExecution() returns,
// which causes an Assert()

    //--- simulation command (call to the base class)

    result = controller->ExecuteCommand (netc);  // return True
    result = listOfNetworks.Empty();  // should return False

    // tested on 30.03.97, 19:10: all tests okay so far

    controller->Activate ();  // state = Scheduled
    command.type = KillSimulation;
    result = controller->ExecuteCommand (command);  // return True
    // controller->state = Passive, g_finished = True
    // this causes an Assert(), so you should trace it!

    result = listOfNetworks.Empty();  // should return True

    // tested on 29.07.98, 14:30: all tests okay
*/


    //----------------------------------------------------------------------
    // test Start/StopApplication()

    applc.module = Controller;
    applc.type = CreateApplication;
    applc.nodeID = 1;
    applc.area = 2;

    result = listOfLances.Empty();  // return True
    result = listOfClocks.Empty();  // return True

    result = controller->ExecuteCommand (applc);  // return True
    // calls StartApplication()
    sup = listOfSupervisors.GetFirst();
    // sup: nodeID = 1, area = 2, state = Scheduled

    result = listOfLances.Empty();  // return False
    result = listOfClocks.Empty();  // return False


    result = controller->StopApplication (1, 2);  // return True
    // sup: state = Terminated

    result = listOfLances.Empty();  // return True
    result = listOfClocks.Empty();  // return True

    // tested on 31.03.97, 12:30: all tests okay


    //----------------------------------------------------------------------
    // test ResetTheSimulation()

    result = controller->ResetTheSimulation ();  // return True
    // should call sup->Reset() (trace it!)

    netc.module = Controller;
    netc.type = CreateNetwork;
    netc.networkType = Broadcast;
    result = controller->ExecuteCommand (netc);  // return True

    result = controller->ResetTheSimulation ();  // return True
    // should call sup->Reset() (trace it!)
    // should call network->Reset() (trace it!)

    // tested on 31.07.97, 11:30: all tests okay so far


    //----------------------------------------------------------------------
    // test Reset()

    result = listOfNetworks.Empty();  // False
    result = listOfSupervisors.Empty();  // False

    controller->Reset();
    // networkID = 1

    result = listOfNetworks.Empty();  // True
    result = listOfSupervisors.Empty();  // True

    // tested on 06.05.97, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test ResetTheSimulation() - continued

    result = controller->ResetTheSimulation ();  // return False

    netc.module = Controller;
    netc.type = CreateNetwork;
    netc.networkType = Broadcast;
    result = controller->ExecuteCommand (netc);  // return True

    result = controller->ResetTheSimulation ();  // return True
    // should call network->Reset() (trace it!)

    // tested on 31.07.97, 11:30: all tests okay


    //----------------------------------------------------------------------
    // test dtor()

    applc.module = Controller;
    applc.type = CreateApplication;
    applc.nodeID = 1;
    applc.area = 2;
    result = controller->ExecuteCommand (applc);  // return True

    result = listOfSupervisors.Empty();  // return False

    delete controller;
    // clockTemplate=gpsTemplate = NULL (in file template.cpp)

    result = listOfSupervisors.Empty();  // return True

    // tested on 31.07.97, 11:30: all tests okay


    delete basecont;
    // can't delete it after its test, because that would delete the
    // templates and the software controller could not be created

    //----------------------------------------------------------------------

    return 0;
}


//--------------------------------------------------------------------------
// necessary classes
//--------------------------------------------------------------------------

CSupervisor::CSupervisor (char name[4], ACSocketAddress *addr, SINT32 nodeID,
  int area, double clockFrequency) : CControllingObject (name, addr, nodeID)
  { nodeID_ = nodeID; area_ = area;
    listOfSupervisors.Insert (this, nodeID_, area_);
    cout << "New Supervisor created with nodeID=" << nodeID << ", area=";
    cout << area << ", clockFrequency=" << clockFrequency << endl; }

CSupervisor::~CSupervisor () {}
void CSupervisor::Reset() {}
void CSupervisor::GetIdentification (SINT32& nodeID, int& area) const
{ nodeID = nodeID_; area = area_; }
void CSupervisor::SuspendOperation () {}
void CSupervisor::ResumeOperation () {}
CClockInterface *CSupervisor::GetClock () { return NULL; }
CClockSync *CSupervisor::GetClockSync () { return NULL; }
CLance *CSupervisor::GetLance () { return NULL; }
EBool CSupervisor::SetParameter (const SParameter&) { return False; }
EBool CSupervisor::GetParameter (SParameter&) const { return False; }
EBool CSupervisor::ExecuteCommand (const SCommand&) { return False; }
void CSupervisor::NoteEvent (SEvent *) {}
void CSupervisor::DoPeriodicActivities() {}
int CSupervisor::ReportError (EReportType, char *, int, char *) { return 0; }

