//
// ctrltest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the control classes in ctrling.hpp and ctrled.hpp
// note: module function SecondsToTicks() in ctrling.cpp was made global
// for this test
// note: CControllingObject::GetEvent(), ReportError(), GetSocketAddress(),
// ReplyToMessage(), Resynchronize(), ProcessData() and TransmitEvent()
// were made public for this test
// note: CControlledObject::ReportEvent() was made public for this test
//
// note: CControllingObject::ReplyToMessage (double) was not tested; can
// only be tested through a derived class, since replies require that a
// message has been received
//
// created: 15.01.97
//
// last change: 04.02.99
//


#include "..\cloctemp.hpp"
#include "..\ctrled.hpp"
#include "..\ctrling.hpp"
#include "..\error.hpp"
#include "..\general.hpp"
#include "..\psos.h"
#include "..\process.hpp"
#include "..\socket.hpp"
#include "..\sockstre.hpp"
#include "..\sockunix.hpp"
#include "settime.hpp"


#define Need_CClock
#define Need_CClockInterface
#define Need_CCrashingModule
#define Need_CSoftwareClock


// #define TEST_STATIC_FUNCS
// uncomment it to test the static functions

extern INT32 SecondsToTicks (double period);


SEvent *CreateEvent (EModule module, EEvent event);
// create an event

CClockList listOfClocks;

unsigned long g_streamOffset = 0;  // for distrib.cpp

int main()
{
#ifdef TEST_STATIC_FUNCS

    //----------------------------------------------------------------------
    // test the module global functions in ctrling.cpp
    //----------------------------------------------------------------------

    INT32 ticks;

    ticks = SecondsToTicks (3.5);  // return 350
    ticks = SecondsToTicks (3.005);  // return 300
    ticks = SecondsToTicks (-3.5);  // return 1
    ticks = SecondsToTicks (0.01);  // return 1
    ticks = SecondsToTicks (0.02);  // return 2
    ticks = SecondsToTicks (0.009);  // return 1

    // tested on 29.03.97, 10:00: all tests okay
#endif


    //----------------------------------------------------------------------
    // test the behaviour of class CControllingObject
    // correct behaviour of class CBitMask, class CClientSocket and of the
    // pSOS queue management functions is assumed
    //----------------------------------------------------------------------

    CControllingObject *ctrl;  // a control object
    SEventReport report;  // for setting event report parameters
    SSingleParameter param;  // for the interface period
    SEvent *event;  // for testing the event report
    EBool result;
    INT8 mask[4] = {4,3,2,1};
    CUnixAddress *addr = new CUnixAddress ("dummy");


    //----------------------------------------------------------------------
    // test ctor() - calls InitEventTable()

    ctrl = new CControllingObject ("TEST", addr);
    // eventQueueID = ..., eventMask=classMask: size=3, bitmap={0,0,0},
    // socket->addr_ = addr, initialized = True, waitPeriod = 1
    // note: in the test, eventQueueID was 2
    // table has no "NoClassification" entries

    // tested on 04.02.99, 12:20: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter(); calls UpdateMask() and GetBit(), so these
    //    functions are tested as well

    report.Set (DiscardSendMessage, True);
    result = ctrl->SetParameter (report);  // return True
    // eventMask = {0, 32, 0}

    report.Set (ReceivedMessage, False);
    result = ctrl->SetParameter (report);  // return True
    // eventMask = {0, 32, 0}

    report.Set (DiscardSendMessage, False);
    result = ctrl->SetParameter (report);  // return True
    // eventMask = {0, 0, 0}

    mask[0] = 51;
    // turn on ClockUpdate, MessageLost, ReplayFailed, ReceivedMessage
    report.Set (EventReport, mask, 1);
    result = ctrl->SetParameter (report);  // return True
    // eventMask = 1; {51}
    // UpdateMask() tested completely

    // tested on 29.07.98, 14:10: all tests okay so far


    report.Set (SystemReport, True);
    result = ctrl->SetParameter (report);  // return True
    // classMask = {2, 0, 0}

    mask[0] = 6;  // allow SystemReports and InterruptEvents
    report.Set (EventClassReport, mask, 1);
    result = ctrl->SetParameter (report);  // return True
    // classMask = 1; {6}

    // tested on 24.01.97, 19:50: all tests okay so far

    report.type = ClockDrift;
    result = ctrl->SetParameter (report);  // return False

    // tested on 21.09.98, 18:10: all tests okay so far


    param.type = InterfacePeriod;
    param.value = 3.6;
    result = ctrl->SetParameter (param);  // return True
    // waitPeriod = 3.6

    param.value = 0;
    result = ctrl->SetParameter (param);  // return False

    // tested on 29.03.97, 10:10: all tests okay so far


    report.Set (SystemReport, False);  // /**/ 2nd param is not needed!
    result = ctrl->GetParameter (report);  // return True
    // report.set = True

    report.Set (FaultInjection, True);  // /**/ 2nd param is not needed!
    result = ctrl->GetParameter (report);  // return True
    // report.set = False

    report.Set (MessageLost, False);  // /**/ 2nd param is not needed!
    result = ctrl->GetParameter (report);  // return True
    // report.set = True

    // tested on 24.01.97, 19:50: all tests okay so far

    report.type = ClockDrift;
    result = ctrl->GetParameter (report);  // return False

    // tested on 21.09.98, 18:10: all tests okay so far


    param.type = InterfacePeriod;
    result = ctrl->GetParameter (param);  // return True
    // value = 3.6

    // tested on 29.03.97, 10:10: all tests okay


    //----------------------------------------------------------------------
    // test NoteEvent(), GetEvent()
    //    NoteEvent() calls EventReportAllowed(), so that is tested as well

    event = CreateEvent (ClockSync, DiscardSendMessage);
    SetGlobalTime (2);  // set the global time to t=2
    ctrl->NoteEvent (event);
    // event->time = 2

    SApplClockEvent *clockEvent = new SApplClockEvent;
    clockEvent->module = Clock;
    clockEvent->type = ClockUpdate;
    clockEvent->clockTime.ms = 1;
    clockEvent->clockTime.ts = 1;
    clockEvent->newTime.ms = 1;
    clockEvent->newTime.ts = 3;
    SetGlobalTime (3);  // set the global time to t=3
    ctrl->NoteEvent (clockEvent);
    // clockEvent->time = 3

    event = CreateEvent (ClockSync, TimerEvent);
    ctrl->NoteEvent (event);
    // should be discarded (event TimerEvent not set)

    event = CreateEvent (Network, MessageLost);
    ctrl->NoteEvent (event);
    // should be discarded (class FaultInjection not set)

    event = CreateEvent (ClockSync, ReceivedMessage);
    SetGlobalTime (4);  // set the global time to t=4
    ctrl->NoteEvent (event);
    // event->time = 4


    // now three messages should be stored; let's test this by retrieving
    // these messages

    event = ctrl->GetEvent();
    // event->module = ClockSync, event->type = DiscardSendMessage, time = 2
    delete event;

    event = ctrl->GetEvent();
    // event->module = Clock, event->type = ClockUpdate, time = 3
    clockEvent = (SApplClockEvent *)event;  // cast to the correct type
    // clockEvent->clockTime = (1,1), clockEvent->newTime = (1,3)
    delete clockEvent;

    event = ctrl->GetEvent();
    // event->module = ClockSync, event->type = ReceivedMessage, time = 4
    delete event;

    event = ctrl->GetEvent();  // should return NULL

    // tested on 24.01.97, 19:50: all tests okay so far


    report.Set (Snapshot, True);  // allow Snapshot events
    result = ctrl->SetParameter (report);  // return True
    event = CreateEvent (ClockSync, Snapshot);
    SetGlobalTime (5);  // set the global time to t=5
    ctrl->NoteEvent (event);
    // event->time = 5 (calls GetSnapshotTime() - trace it!)

    event = ctrl->GetEvent();
    // read the event

    // tested on 28.07.98, 15:00: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand(); calls Reset(), which calls EmptyQueue(), so
    // these functions are tested as well

    event = CreateEvent (ClockSync, ReceivedMessage);
    ctrl->NoteEvent (event);  // put something into the queue

    SCommand command;
    command.type = ResetModule;
    result = ctrl->ExecuteCommand (command);  // return True
    // eventMask = classMask = 3; {0,0,0}, queue is empty, waitPeriod = 1

    event = ctrl->GetEvent();  // should return NULL

    command.type = CreateApplication;
    result = ctrl->ExecuteCommand (command);  // return False

    // tested on 29.03.97, 10:10: all tests okay


    //----------------------------------------------------------------------
    // test ReportError()

    ctrl->ReportError (Reporting, __FILE__, __LINE__, "test");
    // should simply put out the data, do nothing else (ReportError() just
    // calls the global function)

    // tested on 11.02.97, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test GetSocketAddress()

    ACSocketAddress *address = ctrl->GetSocketAddress();
    // address = addr

    // tested on 11.02.97, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test TransmitEvent ()

// ** //
    INT32 sid = 2;
    // find out the socket ID of ctrl and set that to the correct value
    // (using the debugger)! (note: 2 should be correct if the code above
    // is not changed, 15.04.97)

    CStreamInterface socket (addr, sid);
    // make a stream socket
    EMessageType type;

    event = CreateEvent (Lance, MessageLost);
    event->time = 3;
    ctrl->TransmitEvent (event);
    delete event;

    socket.ReceiveMessageHeader (&type);
    // type = MsgEvent

    socket.ReceiveMessage ((SBase**)(&event));
    // module = Lance, event = MessageLost, time = 3

    // tested on 11.02.97, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test SendByteOrder ()

    ctrl->SendByteOrder();

    result = socket.ReceiveByteOrder ();  // True
    result = socket.ByteOrderReversed ();  // False

    // tested on 07.05.97, 21:00: all tests okay


    //----------------------------------------------------------------------
    // test SendModule ()

    EModule module = Clock;

    ctrl->SendModule (Supervisor);
    ctrl->SendModule (Controller);

    result = socket.ReceiveModule (module);  // True
    // module = Supervisor
    result = socket.ReceiveModule (module);  // True
    // module = Controller
    result = socket.ReceiveModule (module);  // False
    // module is not changed

    // tested on 12.05.97, 16:10: all tests okay


    //----------------------------------------------------------------------
    // test SendEnumSizes ()

    SEnumSizes *msg;

    ctrl->SendEnumSizes ();

    socket.ReceiveMessageHeader (&type);
    // type = MsgEnumSizes

    socket.ReceiveMessage ((SBase**)(&msg));
    // msg should contain the sizes of the enums

    // tested on 07.05.97, 21:10: all tests okay


    //----------------------------------------------------------------------
    // test Resynchronize ()

    ctrl->Resynchronize();
    // resync_ = True

    // tested on 11.02.97, 15:50: all tests okay


/*
// note: Body() is an endless loop, so it is commented out after testing

    //----------------------------------------------------------------------
    // test Body () - calls ProcessData() which calls ReplyToMessage(),
    //     so these are tested as well

    // note: the contents of the socket are important; right now, we
    // have a MsgResynchronize, which will be ignored since the object
    // is in resync mode

    socket.SendSynchronizationByte();
    // now MsgSynchronizationByte, that will be accepted, resync will be
    // set to False

    command.type = CreateNetwork;
    socket.SendMessage (MsgCommand, &command);
    // MsgCommand (data) --> ProcessData() will be called, which will
    // call ReplyToMessage (EBool)

    report.Set (MessageLost, True);
    socket.SendMessage (MsgSetParameter, &report);
    // MsgSetParameter (report) --> eventmask = {2,0,0}; again
    // call ReplyToMessage (EBool)

    report.Set (FaultInjection, True);
    socket.SendMessage (MsgSetParameter, &report);
    // MsgSetParameter (report) --> classmask = {8,0,0};
    // calls ReplyToMessage (EBool)

    report.Set (MessageLost, False);
    socket.SendMessage (MsgGetParameter, &report);
    // MsgGetParameter (report) --> calls ReplyToMessage (data)

    report.Set (DiscardSendMessage, True);
    ctrl->SetParameter (report);
    report.Set (SystemReport, True);
    ctrl->SetParameter (report);

    event = CreateEvent (ClockSync, DiscardSendMessage);
    SetGlobalTime (2);
    ctrl->NoteEvent (event);
    // event->time = 2

    report.Set (DiscardSendMessage, False);  // to reset the event and class
    ctrl->SetParameter (report);         // mask
    report.Set (SystemReport, False);
    ctrl->SetParameter (report);

    // okay, the following command does not return, so trace into the code
    // and check that the following things happen:
    // *) receive message type = MsgSynchronize, discard it
    // *) receive message type = MsgSynchronizationByte, set resync = False
    // *) receive MsgCommand, call ProcessData, call ExecuteCommand, return
    //    False, call ReplyToMessage (False)
    // *) receive MsgSetParameter, call SetParameter, ReplyToMessage (True)
    // *) receive MsgSetParameter, call SetParameter, ReplyToMessage (True)
    // *) receive MsgGetParameter, call GetParameter, ReplyToMessage (param)
    // *) receive MsgReply, call Resynchronize()
    // *) read out the rest of the socket contents, discard them

    // *) read out the event queue, call TransmitEvent(), delete the event

    // *) call DoPeriodicActivities()

    // *) call tm_wkafter(), and start at the top of the loop

    // *) that's it; can stop execution after that

    ctrl->Body();

    // tested on 15.04.97, 20:00: all tests okay
*/

/*
    //----------------------------------------------------------------------
    // test the behaviour of class CControlledObject
    // correct behaviour of class CControllingObject is assumed
    //----------------------------------------------------------------------

    CControlledObject *obj;

    // ctrl: set the event/class report masks again
    // turn on ClockUpdate, MessageLost, ReplayFailed, ReceivedMessage
    mask[0] = 51;
    report.Set (EventReport, mask, 1);
    result = ctrl->SetParameter (report);  // return True
    // eventMask = 1; {51}

    mask[0] = 6;  // allow SystemReports and InterruptEvents
    report.Set (EventClassReport, mask, 1);
    result = ctrl->SetParameter (report);  // return True
    // classMask = 1; {6}


    //----------------------------------------------------------------------
    // test ctor()

    obj = new CControlledObject (ctrl);
    // ctrl_ = ctrl

    // tested on 15.01.97, 20:15: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter(), ExecuteCommand()

    SParameter p;
    SCommand c;
    
    result = obj->SetParameter (p);  // return False
    result = obj->GetParameter (p);  // return False

    c.type = CrashModule;
    result = obj->ExecuteCommand (c);  // return False
    c.type = ResetModule;
    result = obj->ExecuteCommand (c);  // return True

    // tested on 14.02.97, 17:55: all tests okay


    //----------------------------------------------------------------------
    // test ReportEvent ()

    SetGlobalTime (5);  // set the global time to t=5
    obj->ReportEvent (ClockSync, ReceivedMessage);

    event = CreateEvent (ClockSync, DiscardSendMessage);
    SetGlobalTime (6);  // set the global time to t=6
    obj->ReportEvent (event);

    event = ctrl->GetEvent ();
    // module = ClockSync, event = ReceivedMessage, time = 5
    delete event;
    event = ctrl->GetEvent ();
    // module = ClockSync, event = DiscardSendMessage, time = 6
    delete event;
    event = ctrl->GetEvent ();  // should return NULL

    // tested on 18.01.97, 17:50: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of class CApplControlledObject
    // correct behaviour of classes
    //     CControlledObject and CControllingObject is assumed
    //----------------------------------------------------------------------

    CApplControlledObject applobj (ctrl);


    //----------------------------------------------------------------------
    // test ReportEvent ()

    SetGlobalTime (5);  // set the global time to t=5
    applobj.ReportEvent (ClockSync, ReceivedMessage);

    event = CreateEvent (ClockSync, DiscardSendMessage);
    SetGlobalTime (6);  // set the global time to t=6
    applobj.ReportEvent (event);

    event = ctrl->GetEvent ();
    // module = ClockSync, event = ReceivedMessage, time = 5
    delete event;
    event = ctrl->GetEvent ();
    // module = ClockSync, event = DiscardSendMessage, time = 6
    delete event;
    event = ctrl->GetEvent ();  // should return NULL

    // tested on 06.02.97, 21:30: all tests okay

    delete obj;
*/
    delete ctrl;

    //----------------------------------------------------------------------

    return 0;
}


// make a new event
SEvent *CreateEvent (EModule module, EEvent event)
{
    SEvent *ev = new SEvent;
    ev->module = module;
    ev->type = event;
    return ev;
}


#include "inlines.hpp"


