//
// datatest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test CApplicationDatabase in database.hpp
// note: Init() and Dispose() were made public for this test
//
// created: 30.04.97
//
// last change: 09.10.98
//


#include "..\address.hpp"
#include "..\database.hpp"
#include "..\errbase.hpp"
#include "..\error.hpp"
#include "..\structs.hpp"


int main()
{
    CErrorHandler *handler = new CErrorHandler;
    InstallErrorHandler (handler);


    //----------------------------------------------------------------------
    // variables necessary for the test of CApplicationDatabase
    //----------------------------------------------------------------------

    CApplicationDatabase *dbase;
    EBool result;
    int i;
    addr netAddress;


    //----------------------------------------------------------------------
    // test the behaviour of the class CApplicationDatabase
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor

    dbase = new CApplicationDatabase;

    // tested on 01.05.97, 11:40: all tests okay


    //----------------------------------------------------------------------
    // test Init() and Dispose()

    dbase->Init();
    // should return at once (trace into the code to verify that!)

    dbase->Dispose();
    // initCalled = False
    dbase->Dispose();
    // should return at once (trace into the code to verify that!)

    dbase->Init();
    // initCalled = True

    // tested on 01.05.97, 11:40: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetAssociation(), DeleteAssociation()

    for (i=0; i<MAXASICS; i++) netAddress[i] = 0;

    netAddress[0] = 1;  // address of node 1
    result = dbase->SetAssociation (2, &netAddress, 1);  // return True
    result = dbase->SetAssociation (2, &netAddress, 1);  // return True
    result = dbase->SetAssociation (2, &netAddress, 2);  // return False
    netAddress[0] = 2;  // address of node 2
    result = dbase->SetAssociation (2, &netAddress, 1);  // return False
    result = dbase->SetAssociation (1, &netAddress, 1);  // return True
    result = dbase->SetAssociation (2, &netAddress, 2);  // return True

    // tested on 01.05.97, 11:40: all tests okay so far


    netAddress[0] = 0;
    CAddr getNetAddress;  // for GetAssociation()
    result = dbase->GetAssociation (2, 1, getNetAddress);  // return True
    // netAddress[0] = 1

    // tested on 09.10.98, 15:40: all tests okay so far


    SINT32 nodeID = 0;
    result = dbase->GetAssociation (2, &netAddress, &nodeID);  // True
    // nodeID = 1

    // tested on 01.05.97, 15:40: all tests okay so far


    result = dbase->DeleteAssociation (2, 3);  // return False
    result = dbase->DeleteAssociation (2, 2);  // return True
    result = dbase->DeleteAssociation (2, 2);  // return False

    // tested on 01.05.97, 11:40: all tests okay


    //----------------------------------------------------------------------
    // test the access functions to transmission data

    SDelay delay;

    // test SetDelay()
    delay.delta = 1; delay.epsilonPlus = 2; delay.epsilonMinus = 3;
    netAddress[0] = 1;
    dbase->SetDelay (delay, 2, &netAddress);

    delay.delta = 4; delay.epsilonPlus = 5; delay.epsilonMinus = 6;
    netAddress[0] = 2;
    dbase->SetDelay (delay, 1, &netAddress);

    delay.delta = 7; delay.epsilonPlus = 8; delay.epsilonMinus = 9;
    dbase->SetDelay (delay, 2, &netAddress);


    // test GetDelay()
    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetDelay (delay, 3, &netAddress);  // return False
    // delay = {0,0,0}

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    netAddress[0] = 1;
    result = dbase->GetDelay (delay, 2, &netAddress);  // return True
    // delay = {1,2,3}


    // test GetFirstDelay()
    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetFirstDelay (delay, 3);  // return False

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetFirstDelay (delay, 2);  // return True
    // delay = {1,2,3}


    // test GetNextDelay()
    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetNextDelay (delay, 3);  // return False

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetNextDelay (delay, 2);  // return True
    // delay = {7,8,9}

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->GetNextDelay (delay, 2);  // return False


    // test RemoveDelay()
    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    netAddress[0] = 1;
    result = dbase->RemoveDelay (delay, 1, &netAddress);  // return False
    // delay = {0,0,0}

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    netAddress[0] = 2;
    result = dbase->RemoveDelay (delay, 1, &netAddress);  // return True
    // delay = {4,5,6}

    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = dbase->RemoveDelay (delay, 1, &netAddress);  // return False
    // delay = {0,0,0}
    result = dbase->GetFirstDelay (delay, 1);  // return False

    // tested on 01.05.97, 12:00: all tests okay


    //----------------------------------------------------------------------
    // test the access functions to rate data

    SRate *rate1 = new SRate;
    SRate *rate2 = new SRate;
    SRate *rate3 = new SRate;
    int area = 0;

    // test SetRate()
    rate1->thetaPlus = 1; rate1->thetaMinus = 2; rate1->u = 3;
    netAddress[0] = 1;
    dbase->SetRate (rate1, 2, &netAddress, area);

    rate2->thetaPlus = 4; rate2->thetaMinus = 5; rate2->u = 6;
    netAddress[0] = 2;
    dbase->SetRate (rate2, 1, &netAddress, area);

    rate3->thetaPlus = 7; rate3->thetaMinus = 8; rate3->u = 9;
    dbase->SetRate (rate3, 2, &netAddress, area);


    // test GetRate()
    SRate *rate;
    rate = dbase->GetRate (3, &netAddress, area);  // return NULL

    netAddress[0] = 1;
    rate = dbase->GetRate (2, &netAddress, area);  // return rate1
    // rate = rate1 = {1,2,3}


    // test GetFirstRate()
    rate = dbase->GetFirstRate (3);  // return NULL

    rate = dbase->GetFirstRate (2);  // return rate1
    // rate = rate1 = {1,2,3}


    // test GetNextRate()
    rate = dbase->GetNextRate (3);  // return NULL

    rate = dbase->GetNextRate (2);  // return rate3
    // rate = rate3 = {7,8,9}

    rate = dbase->GetNextRate (2);  // return NULL


    // test RemoveRate()
    netAddress[0] = 1;
    rate = dbase->RemoveRate (1, &netAddress, area);  // NULL

    netAddress[0] = 2;
    rate = dbase->RemoveRate (1, &netAddress, area);  // rate2
    // rate = rate2 = {4,5,6}

    rate = dbase->RemoveRate (1, &netAddress, area);  // NULL
    rate = dbase->GetFirstRate (1);  // return NULL

    // tested on 09.10.98, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test the access functions to time data
    // note: copied from the test of access functions to rate data

    STimeInterval time;

    // test SetTime()
    time.low.ms = 1; time.low.ts = 2; time.high.ms = 3; time.high.ts = 3;
    netAddress[0] = 1;
    dbase->SetTime (time, 2, &netAddress, area);

    time.low.ms = 4; time.low.ts = 5; time.high.ms = 6; time.high.ts = 6;
    netAddress[0] = 2;
    dbase->SetTime (time, 1, &netAddress, area);

    time.low.ms = 7; time.low.ts = 8; time.high.ms = 9; time.high.ts = 9;
    dbase->SetTime (time, 2, &netAddress, area);


    // test GetTime()
    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetTime (time, 3, &netAddress, area);  // return False
    // time = {0,0,0,0}

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    netAddress[0] = 1;
    result = dbase->GetTime (time, 2, &netAddress, area);  // return True
    // time = {1,2,3,3}


    // test GetFirstTime()
    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetFirstTime (time, 3);  // return False

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetFirstTime (time, 2);  // return True
    // time = {1,2,3,3}


    // test GetNextTime()
    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetNextTime (time, 3);  // return False

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetNextTime (time, 2);  // return True
    // time = {7,8,9,9}

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->GetNextTime (time, 2);  // return False


    // test RemoveTime()
    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    netAddress[0] = 1;
    result = dbase->RemoveTime (time, 1, &netAddress, area);  // False
    // time = {0,0,0,0}

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    netAddress[0] = 2;
    result = dbase->RemoveTime (time, 1, &netAddress, area);  // True
    // time = {4,5,6,6}

    time.low.ms = 0; time.low.ts = 0; time.high.ms = 0; time.high.ts = 0;
    result = dbase->RemoveTime (time, 1, &netAddress, area);  // False
    // time = {0,0,0,0}
    result = dbase->GetFirstTime (time, 1);  // return False

    // tested on 01.05.97, 12:30: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()
    // note: two assocs are stored: network 1, node 1, addr {2,0..} and
    //       network 2, node 1, addr {1,0..}
    //       data (1,2,3) is stored for netID 2, addr {1,0...} (area 0) and
    //       data (7,8,9) for network 2, addr {2,0...} (area 0)

    STransmissionData delayparam;
    SRateData rateparam;

    delayparam.type = Delta;
    delayparam.networkID = 2;
    delayparam.nodeID = 2;
    delayparam.value = 2;
    result = dbase->SetParameter (delayparam);  // return False

    delayparam.networkID = 1;
    delayparam.nodeID = 1;
    result = dbase->SetParameter (delayparam);  // return False

    delayparam.networkID = 2;
    result = dbase->SetParameter (delayparam);  // return True

    delayparam.type = EpsilonPlus;
    delayparam.value = 3;
    result = dbase->SetParameter (delayparam);  // return True

    delayparam.type = EpsilonMinus;
    delayparam.value = 4;
    result = dbase->SetParameter (delayparam);  // return True

    // tested on 01.05.97, 13:40: all tests okay so far


    rateparam.type = ThetaPlus;
    rateparam.networkID = 2;
    rateparam.nodeID = 2;
    rateparam.area = 0;
    rateparam.value = 2;
    result = dbase->SetParameter (rateparam);  // return False

    rateparam.networkID = 1;
    rateparam.nodeID = 1;
    result = dbase->SetParameter (rateparam);  // return False

    rateparam.networkID = 2;
    result = dbase->SetParameter (rateparam);  // return True
    // rate1.thetaPlus = 2

    rateparam.type = ThetaMinus;
    rateparam.value = 3;
    result = dbase->SetParameter (rateparam);  // return True
    // rate1.thetaPlus = 3

    rateparam.type = U;
    rateparam.value = 4;
    result = dbase->SetParameter (rateparam);  // return True
    // rate1.U = 4

    // tested on 09.10.98, 15:40: all tests okay so far


    rateparam.type = ClockDrift;
    result = dbase->SetParameter (rateparam);  // return False

    // tested on 01.05.97, 13:40: all tests okay so far


    delayparam.type = Delta;
    delayparam.networkID = 2;
    delayparam.nodeID = 2;
    delayparam.value = 0;
    result = dbase->GetParameter (delayparam);  // return False

    delayparam.networkID = 1;
    delayparam.nodeID = 1;
    result = dbase->GetParameter (delayparam);  // return False

    delayparam.networkID = 2;
    result = dbase->GetParameter (delayparam);  // return True
    // value = 2

    delayparam.type = EpsilonPlus;
    delayparam.value = 0;
    result = dbase->GetParameter (delayparam);  // return True
    // value = 3

    delayparam.type = EpsilonMinus;
    delayparam.value = 0;
    result = dbase->GetParameter (delayparam);  // return True
    // value = 4

    // tested on 01.05.97, 13:40: all tests okay so far


    rateparam.type = ThetaPlus;
    rateparam.networkID = 2;
    rateparam.nodeID = 2;
    rateparam.value = 0;
    result = dbase->GetParameter (rateparam);  // return False

    rateparam.networkID = 1;
    rateparam.nodeID = 1;
    result = dbase->GetParameter (rateparam);  // return False

    rateparam.networkID = 2;
    result = dbase->GetParameter (rateparam);  // return True
    // value = 2

    rateparam.type = ThetaMinus;
    rateparam.value = 0;
    result = dbase->GetParameter (rateparam);  // return True
    // value = 3

    rateparam.type = U;
    rateparam.value = 0;
    result = dbase->GetParameter (rateparam);  // return True
    // value = 4

    // tested on 09.10.98, 15:40: all tests okay so far


    rateparam.type = ClockDrift;
    rateparam.value = 0;
    result = dbase->GetParameter (rateparam);  // return False

    // tested on 01.05.97, 13:40: all tests okay


    //----------------------------------------------------------------------
    // test Cardinality()

    INT32 card = 0;

    card = dbase->Cardinality (CApplicationDatabase::Delay, 2);  // 2
    card = dbase->Cardinality (CApplicationDatabase::Delay, 1);  // 0

    card = dbase->Cardinality (CApplicationDatabase::Rate, 2);  // 2
    card = dbase->Cardinality (CApplicationDatabase::Rate, 1);  // 0

    card = dbase->Cardinality (CApplicationDatabase::Time, 2);  // 2
    card = dbase->Cardinality (CApplicationDatabase::Time, 1);  // 0

    // tested on 01.05.97, 16:00: all tests okay


    //----------------------------------------------------------------------
    // test Delete()

    netAddress[0] = 2;

    result = dbase->GetDelay (delay, 2, &netAddress);  // return True
    dbase->Delete (CApplicationDatabase::Delay, 2, &netAddress, 25);
    result = dbase->GetDelay (delay, 2, &netAddress);  // return False

    dbase->Delete (CApplicationDatabase::Rate, 2, &netAddress, 25);
    rate = dbase->GetRate (2, &netAddress, area);  // return != NULL
    dbase->Delete (CApplicationDatabase::Rate, 2, &netAddress, area);
    rate = dbase->GetRate (2, &netAddress, area);  // return NULL

    dbase->Delete (CApplicationDatabase::Time, 2, &netAddress, 25);
    result = dbase->GetTime (time, 2, &netAddress, area);  // return True
    dbase->Delete (CApplicationDatabase::Time, 2, &netAddress, area);
    result = dbase->GetTime (time, 2, &netAddress, area);  // return False

    // tested on 09.10.97, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test DeleteAll (type, networkID), DeleteAll (type)

    const int EntryCnt = 4;
    SRate *dynrate[EntryCnt*2];
    for (i=0; i<EntryCnt*2; i++) dynrate[i] = new SRate;

    for (i=1; i<EntryCnt; i++)  // make some entries
    {
        netAddress[0] = i;
        dbase->SetDelay (delay, 3, &netAddress);
        dbase->SetRate (dynrate[i], 3, &netAddress, area);
        dbase->SetTime (time, 3, &netAddress, area);
        dbase->SetDelay (delay, 4, &netAddress);
        dbase->SetRate (dynrate[i+EntryCnt], 4, &netAddress, area);
        dbase->SetTime (time, 4, &netAddress, area);
    } // for

    result = dbase->GetDelay (delay, 3, &netAddress);  // return True
    dbase->DeleteAll (CApplicationDatabase::Delay, 3);
    result = dbase->GetDelay (delay, 3, &netAddress);  // return False

    rate = dbase->GetRate (3, &netAddress, area);  // return != NULL
    dbase->DeleteAll (CApplicationDatabase::Rate, 3);
    rate = dbase->GetRate (3, &netAddress, area);  // return NULL

    result = dbase->GetTime (time, 3, &netAddress, area);  // return True
    dbase->DeleteAll (CApplicationDatabase::Time, 3);
    result = dbase->GetTime (time, 3, &netAddress, area);  // return False

    // tested on 09.10.98, 15:50: all tests okay so far


    result = dbase->GetDelay (delay, 4, &netAddress);  // return True
    dbase->DeleteAll (CApplicationDatabase::Delay);
    result = dbase->GetDelay (delay, 4, &netAddress);  // return False

    rate = dbase->GetRate (4, &netAddress, area);  // return != NULL
    dbase->DeleteAll (CApplicationDatabase::Rate);
    rate = dbase->GetRate (4, &netAddress, area);  // return NULL

    result = dbase->GetTime (time, 4, &netAddress, area);  // return True
    dbase->DeleteAll (CApplicationDatabase::Time);
    result = dbase->GetTime (time, 4, &netAddress, area);  // return False

    // tested on 09.10.98, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test Reset()
    // note: two assocs are stored: network 1, node 1, addr {2,0..} and
    //       network 2, node 1, addr {1,0..}

    rate = new SRate;  // make a new rate
    netAddress[0] = 1;
    dbase->SetDelay (delay, 3, &netAddress);
    dbase->SetRate (rate, 3, &netAddress, area);
    dbase->SetTime (time, 3, &netAddress, area);

    dbase->Reset();  // discard all data, discard the associations

    result = dbase->GetDelay (delay, 3, &netAddress);  // return False
    rate = dbase->GetRate (3, &netAddress, area);  // return NULL
    result = dbase->GetTime (time, 3, &netAddress, area);  // return False
    result = dbase->DeleteAssociation (1, 1);  // return False
    result = dbase->DeleteAssociation (2, 1);  // return False

    // tested on 09.10.98, 15:50: all tests okay


    delete dbase;

    DeinstallErrorHandler ();
    delete handler;

    //----------------------------------------------------------------------

    return 0;
}


