//
// datest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the address manager class in addrman.hpp (03.03.97)
// note: the functions Init() and Dispose() were made public for this test
// test the delay accountant class in delacc.hpp (15.11.96)
// note: the functions SetNodeDelay(), GetNodeDelay(), Init() and Dispose()
//       were made public for this test
//
// created: 14.11.96
//
// last changed: 15.04.97
//


#include "..\addrman.hpp"
#include "..\delacc.hpp"
#include "..\structs.hpp"


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of the class CAddressmanager
    //----------------------------------------------------------------------

    CAddressManager *addressmanager;
    EResult nodeResult;
    EBool result;
    SINT32 nodeID;
    addr address;
    int i;


    //----------------------------------------------------------------------
    // test the ctor

    addressmanager = new CAddressManager;
    // listOfAddresses=listOfNodes = "empty list", initCalled = True

    // tested on 03.01.97, 20:15: all tests okay


    //----------------------------------------------------------------------
    // test Init(), Dispose()

    addressmanager->Init();
    // should return at once (trace into the code to verify that!)

    addressmanager->Dispose();
    // initCalled = False
    addressmanager->Dispose();
    // should return at once (trace into the code to verify that!)

    addressmanager->Init();
    // initCalled = True

    // tested on 03.01.97, 20:15: all tests okay


    //----------------------------------------------------------------------
    // test Insert() - CheckAddress(), AddressesAreEqual() and
    //     CopyAddress() are tested as well

    for (i=0; i<ADDRCNT; i++) address[i] = 0;

    address[0] = 1;
    nodeResult = addressmanager->Insert (1, 1, &address);
    // should return NodeIDStored  --> <1,1,1>

    address[0] = 2;
    nodeResult = addressmanager->Insert (1, 2, &address);
    // should return NodeIDStored  --> <1,2,2>

    nodeResult = addressmanager->Insert (1, 2, &address);
    // should return NodeIDStored

    nodeResult = addressmanager->Insert (1, 1, &address);
    // should return BothInUse
    // CopyAddress(), AddressesAreEqual() tested completely

    nodeResult = addressmanager->Insert (1, 3, &address);
    // should return NodeAddressInUse

    address[0] = 3;
    nodeResult = addressmanager->Insert (1, 1, &address);
    // should return NodeIDInUse
    // CheckAddress() tested comletely

    // tested on 03.01.97, 20:20: all tests okay


    //----------------------------------------------------------------------
    // test FindAddress() and FindNodeID()

    address[0] = 0;
    result = addressmanager->FindAddress (1, 1, &address);  // True
    // address[0] = 1

    address[0] = 0;
    result = addressmanager->FindAddress (1, -1, &address);  // False
    // address is not changed


    address[0] = 2;
    result = addressmanager->FindNodeID (1, &address, &nodeID);  // True
    // nodeID = 2

    address[0] = 0; nodeID = 0;
    result = addressmanager->FindNodeID (1, &address, &nodeID);  // False
    // nodeID is not changed

    // tested on 03.01.97, 20:20: all tests okay


    //----------------------------------------------------------------------
    // test DeleteAddress() and DeleteNodeID()

    address[0] = 0;
    result = addressmanager->DeleteAddress (1, &address);  // False

    address[0] = 1;
    result = addressmanager->DeleteAddress (1, &address);  // True

    result = addressmanager->DeleteNodeID (1, 1);  // False

    result = addressmanager->DeleteNodeID (1, 2);  // True

    // tested on 03.01.97, 20:35: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    // create a node
    address[0] = 1;
    addressmanager->Insert (1, 1, &address);  // --> <1,1,1>

    addressmanager->Reset();
    // listOfAddresses=listOfNodes = "empty list", initCalled = True

    // tested on 03.01.97, 20:40: all tests okay


    delete addressmanager;


    //----------------------------------------------------------------------
    // test the behaviour of the class CDelayAccountant
    //     correct behaviour of class CAccountant and CAddressManager is
    //     assumed
    //----------------------------------------------------------------------

    CDelayAccountant *delAcc;
    SDelay delay;
    STransmissionData transParam;


    //----------------------------------------------------------------------
    // test the ctor

    delAcc = new CDelayAccountant;
    // initCalled = True

    // tested on 03.01.97, 21:35: all tests okay


    //----------------------------------------------------------------------
    // test Init() and Dispose()

    delAcc->Init();
    // should return at once (trace into the code to verify that!)

    delAcc->Dispose();
    // initCalled = False
    delAcc->Dispose();
    // should return at once (trace into the code to verify that!)

    delAcc->Init();
    // initCalled = True

    // tested on 02.01.97, 22:05: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetDelay()

    for (i=0; i<ADDRCNT; i++) address[i] = 0;

    delay.delta = 2.5;       // note the difference to CNetworkAccountant:
    delay.epsilonPlus = -1;  // here *any* values are allowed, there is no
    delay.epsilonMinus = 5;  // plausibility check

    address[ADDRCNT-1] = 2;
    nodeResult = delAcc->SetDelay (delay, 1, &address);  // NodeIDStored
    // key1 = 1, key2 = 2, data = (2.5,-1,5)

    address[ADDRCNT-1] = 3;
    nodeResult = delAcc->SetDelay (delay, 2, &address, 2);  // NodeIDStored
    // key1 = 2, key2 = 3, data = (2.5,-1,5)
    // addressManager <-- <2,2,3>


    delay.delta = 0; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = delAcc->GetDelay (&delay, 1, &address);  // return True
    // delay = (2.5,-1,5)

    // tested on 03.01.97, 21:35: all tests okay


    //----------------------------------------------------------------------
    // test DeleteNode(), DeleteNetwork()

    address[ADDRCNT-1] = 5;
    result = delAcc->DeleteNode (&address);  // should return False
    result = delAcc->DeleteNetwork (5);  // should return False

    result = delAcc->DeleteNetwork (1);  // should return True
    address[ADDRCNT-1] = 3;
    result = delAcc->DeleteNode (&address);  // should return True

    // tested on 03.01.97, 21:35: all tests okay


    //----------------------------------------------------------------------
    // test SetNodeDelay(), GetNodeDelay(), DeleteNodeID()

    delAcc->Reset();
    address[ADDRCNT-1] = 1;
    delAcc->SetDelay (delay, 1, &address, 1);  // <1,1,1>
    delAcc->SetDelay (delay, 2, &address, 1);  // <2,1,1>
    delAcc->DeleteNetwork (1); // deletes the data, but not the association

    delay.delta = 5; delay.epsilonPlus = 0; delay.epsilonMinus = 0;
    result = delAcc->SetNodeDelay (delay, 3, 1);  // return False
    result = delAcc->SetNodeDelay (delay, 1, 1);  // return False
    result = delAcc->SetNodeDelay (delay, 2, 1);  // return True
    // key1 = 2, key2 = 1, data = (5,0,0)


    delay.delta = 0;
    result = delAcc->GetNodeDelay (&delay, 3, 1);  // return False
    result = delAcc->GetNodeDelay (&delay, 1, 1);  // return False
    result = delAcc->GetNodeDelay (&delay, 2, 1);  // return True
    // delay = (5,0,0)


    result = delAcc->DeleteNodeID (3, 1);  // return False
    result = delAcc->DeleteNodeID (1, 1);  // return True

    // tested on 04.01.97, 12:10: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()
    // note: correct behaviour of the internal Set/GetNodeDelay() functions
    //       is assumed

    address[ADDRCNT-1] = 1; delay.delta = 10;
    delAcc->SetDelay (delay, 1, &address, 1);  // <1,1,1>
    // key1=1, key2=1, data=(delays: 10,0,0)

    transParam.type = Delta;
    transParam.networkID = 3; transParam.node = 1; transParam.value = 4.5;
    result = delAcc->SetParameter(transParam);  // return False

    transParam.networkID = 1;
    transParam.type = Lambda;
    result = delAcc->SetParameter(transParam);  // return False

    transParam.type = Delta;
    result = delAcc->SetParameter(transParam);  // return True
    // key1=1, key2=1, data=(delays: 4.5,0,0)

    transParam.type = EpsilonPlus; transParam.value = 1.2;
    result = delAcc->SetParameter(transParam);  // return True
    // key1=1, key2=1, data=(delays: 4.5,1.2,0)

    transParam.type = EpsilonMinus; transParam.value = -3.9;
    result = delAcc->SetParameter(transParam);  // return True
    // key1=1, key2=1, data=(delays: 4.5,1.2,-3.9)
    // SetParameter() tested completely


    transParam.type = Delta;
    transParam.networkID = 3; transParam.node = 1; transParam.value = 0.1;
    result = delAcc->GetParameter(transParam);  // return False

    transParam.networkID = 1;
    transParam.type = Lambda;
    result = delAcc->GetParameter(transParam);  // return False

    transParam.type = Delta;
    result = delAcc->GetParameter(transParam);  // return True
    // transParam.value = 4.5

    transParam.type = EpsilonPlus;
    result = delAcc->GetParameter(transParam);  // return True
    // transParam.value = 1.2

    transParam.type = EpsilonMinus;
    result = delAcc->GetParameter(transParam);  // return True
    // transParam.value = -3.9
    // GetParameter() tested completely

    // tested on 01.02.97, 18:45: all tests okay


    delete delAcc;
    

    //----------------------------------------------------------------------

    return 0;
}

