//
// errtest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test CErrorHandler (errbase.?pp)
// test the error report functions and macros in error.hpp
// note: have to compile and debug the test twice, to check behaviour
// if NDEBUG is defined and if it's not defined
// test CErrorTask (errtask.?pp)
// test CErrorEval (err2eval.?pp)
//
// created: 11.01.97
//
// last change: 29.07.98
//

//#define NDEBUG
// uncomment this and try again

//#define TEST_ASSERT
// uncomment it to test the Assert() macro


//#define TEST_BODY_ERRTASK
//#define TEST_BODY_ERR2EVAL
// uncomment it to test the body of the tasks


#include "..\psos.h"

#include "..\constant.hpp"
#include "..\err2eval.hpp"
#include "..\errbase.hpp"
#include "..\error.hpp"
#include "..\errtask.hpp"
#include "..\sockunix.hpp"


int main()
{

#ifdef NDEBUG

    int i=0;


    //----------------------------------------------------------------------
    // test the macros with NDEBUG
    // note: nothing should happen, the debugger goes to the last statement
    // (return 0), the macros are ignored (because they are not there
    // if NDEBUG is defined)

    Assert (i!=0);
    Check (i!=0);
    Report (i==0);

    // tested on 11.01.97, 16:47: all tests okay

#else

    //----------------------------------------------------------------------
    // test class CErrorHandler
    //----------------------------------------------------------------------

    CErrorHandler *reporter = new CErrorHandler;


    //----------------------------------------------------------------------
    // test NoteError() - calls ProcessMessage(), so that is tested
    //    as well

    reporter->NoteError ("CErrorHandler: test 1");
    reporter->NoteError ("CErrorHandler: test 2");
    // output: the two lines above (with trailing newline)

    // tested on 01.04.97, 13:50: all tests okay


    //----------------------------------------------------------------------
    // test the functions and macros in error.hpp
    //----------------------------------------------------------------------

    int i=0;  // just to have an expression
    int result;  // result of Check()
    int line; // the line in the file, in order to compare with the output


    //----------------------------------------------------------------------
    // test the ReportError() functions without an error reporter

    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2");  // 0
    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network);
    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network, 2);
    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network, 2, 3);
    // all functions should return 0, nothing should happen (no output)

    // tested on 01.04.97, 14:10: all tests okay


    //----------------------------------------------------------------------
    // test InstallErrorHandler(), DeinstallErrorHandler() in error.hpp

    InstallErrorHandler (reporter);
    result = ReportError (Reporting, "testfile.cpp", 1, "a != 1");  // 0
    // report: a != 1 in file testfile.cpp at line 1

    DeinstallErrorHandler();
    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2");  // 0
    // (no output)

    // tested on 01.04.97, 14:10: all tests okay


    //----------------------------------------------------------------------
    // test ReportMessage() with and without an error handler

    ReportMessage ("a != 2");
    // nothing should happen (no output)

    InstallErrorHandler (reporter);
    ReportMessage ("a != 2");
    // a != 2

    DeinstallErrorHandler();

    // tested on 28.06.98, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test the ReportError() functions with an error reporter
    //     calls PrintFileSpecs() and GetReturnValue(), so these are tested
    //     as well (except Asserting)

    InstallErrorHandler (reporter);

    result = ReportError (Reporting, "testfile.cpp", 1, "a != 2");  // 0
    // report: a != 2 in file testfile.cpp at line 1

    result = ReportError (Checking, "testfile.cpp", 1, "a != 2");  // 0
    // check failed: a != 2 in file testfile.cpp at line 1

    ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network);
    // report: a != 2 in file testfile.cpp at line 1
    // module = Network

    ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network, 2);
    // report: a != 2 in file testfile.cpp at line 1
    // module = Network, moduleID = 2

    ReportError (Reporting, "testfile.cpp", 1, "a != 2", Network, 2, 3);
    // check failed: a != 2 in file testfile.cpp at line 1
    // module = Network, nodeID = 2, area = 3

    // ReportError() tested completely, but PrintFileSpecs() and
    // GetReturnValue() not tested with type=Asserting

    // tested on 01.04.97, 14:10: all tests okay so far


    //----------------------------------------------------------------------
    // test the Check() and Report() macros

    result = Check (i==0);  // should evaluate to 1
    line = __LINE__ + 1;  // find out the number of the next line
    result = Check (i!=0);  // should evaluate to 0
    // check failed: i!=0 in file errtest.cpp at line <line>

    line = __LINE__ + 1;  // find out the number of the next line
    Report (i==0);
    // report: i==0 in file errtest.cpp at line <line>

    // tested on 11.01.97, 16:45: all tests okay


#ifdef TEST_ASSERT

    //----------------------------------------------------------------------
    // test the Assert() macro
    //    calls ReportError() with type=Asserting, so PrintFileSpecs() and
    //    GetReturnValue() are also called with type=Asserting

    Assert (i==0);  // nothing should happen
    line = __LINE__ + 1;  // find out the number of the next line
    Assert (i!=0);
    // assert failed: i!=0 in file errtest.cpp at line <line>

    // debugger should stop with an "Abnormal Termination", 'reporter'
    // should have been deleted [trace it!]

    // tested on 28.06.98, 16:20: all tests okay

#else

    DeinstallErrorHandler();
    delete reporter;

#endif


    //----------------------------------------------------------------------
    // test class CErrorTask
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test ctor

    reporter = new CErrorTask ("QERR");
    // qid = 1

    // tested on 28.06.98, 16:00: all tests okay


    //----------------------------------------------------------------------
    // test NoteError() - calls GetMessageString(), so that is tested
    //    as well

    reporter->NoteError ("error report test 1");
    reporter->NoteError ("error report test 2");
    reporter->NoteError ("error report test 3");
    // nothing special should happen; the real test is calling Body()

    // tested on 06.12.97, 20:50: all tests okay


#ifdef TEST_BODY_ERRTASK
    //----------------------------------------------------------------------
    // test Body() - calls ReturnMessageString() and ProcessMessage(), so
    //     these are tested as well

    // note: the following function does not return; so step into it and
    // check the following behaviour:

    // *) read queue
    // *) call ProcessMessage() --> output: error report test 1 (plus \n)
    // *) call ReturnMessageString()
    // *) repeat the three steps with test 2,3
    // *) call tm_wkafter()
    // *) next loop: read queue (empty)
    // *) call tm_wkafter()
    // ...

    ((CErrorTask*)reporter)->Body();
    // note: if the messages are in the queue, then consider NoteEvent()
    // tested as well

    // tested on 29.07.98, 15:20: all tests okay
#endif


    //----------------------------------------------------------------------
    // test dtor - calls EmergencyFlush() and EmergencyProcessMessage(),
    //    so these are tested as well

    reporter->NoteError ("error report test 4");
    delete reporter;
    // should call q_delete() [trace it!]
    // output: error report test 1-4 (1-3 are printed now, because Body()
    // is commented out)

    // tested on 06.04.97, 18:30: all tests okay


    //----------------------------------------------------------------------
    // test class CErrorEval
    //----------------------------------------------------------------------

    CUnixAddress *addr = new CUnixAddress ("dummy");


    //----------------------------------------------------------------------
    // test ctor

    reporter = new CErrorEval ("QERR", addr);
    // qid = 1, socket is created [trace it!]

    // tested on 28.06.98, 16:20: all tests okay


#ifdef TEST_BODY_ERR2EVAL

    //----------------------------------------------------------------------
    // test ProcessMessage()

    // the following call to Body() does not return; trace into the code
    // and check that socket::Send() is being called with the appropriate
    // string!

    reporter->NoteError ("error report to eval test 1");
    ((CErrorEval*)reporter)->Body();  // does not return
    
    // tested on 06.12.97, 20:50: all tests okay
#endif


    //----------------------------------------------------------------------
    // test dtor - calls EmergencyProcessMessage(), so that is tested as
    //    well

    for (i=0; i<MaxErrorMessages; i++)
        reporter->NoteError ("error report to eval test 2");

    reporter->NoteError ("error report to eval test 3");
    delete reporter;
    // output: error report to eval test 3 [printed by EmergencyFlush()]

    // tested on 06.04.97, 18:30: all tests okay


#endif

    //----------------------------------------------------------------------

    return 0;
}


