//
// finjtest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test CCrashingModule (crashing.?pp)
//
// created: 12.10.97
//
// last change: 12.10.97
//


#include "../crashing.hpp"
#include "../params.hpp"
#include "../structs.hpp"


const int TestTries = 1000;
// number of calls to ModifyCrashStatus()


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of the class CCrashingModule
    //   correct behaviour of global function CastDice() is assumed
    //----------------------------------------------------------------------

    CCrashingModule *module;
    EBool result;
    double prob = 0;


    //----------------------------------------------------------------------
    // test the constructor and Reset()

    module = new CCrashingModule;
    // crashed = False, probCrash=probRecover = 0

    // tested on 12.08.97, 09:50: all tests okay


    //----------------------------------------------------------------------
    // test Crash(), Crashed() and Recover()

    result = module->Crashed();  // return False

    module->Crash();
    // crashed = True

    result = module->Crashed();  // return True

    module->Recover();
    // crashed = False

    result = module->Crashed();  // return False

    // tested on 12.08.97, 09:50: all tests okay


    //----------------------------------------------------------------------
    // test SetProbability() and GetProbability()

    result = module->SetProbability (True, -1);  // False
    result = module->SetProbability (True, 1.1);  // False
    result = module->SetProbability (True, 1);  // True
    // probCrash = 1
    result = module->SetProbability (True, 0);  // True
    // probCrash = 0
    result = module->SetProbability (True, 0.4);  // True
    // probCrash = 0.4

    result = module->SetProbability (False, -0.1);  // False
    result = module->SetProbability (False, 1.1);  // False
    result = module->SetProbability (False, 1);  // True
    // probRecover = 1
    result = module->SetProbability (False, 0);  // True
    // probRecover = 0
    result = module->SetProbability (False, 0.5);  // True
    // probRecover = 0.5

    // tested on 12.08.97, 09:50: all tests okay so far

    prob = module->GetProbability (True);
    // prob = 0.4
    prob = module->GetProbability (False);
    // prob = 0.5

    // tested on 12.08.97, 09:50: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand()

    SCommand command;

    command.type = CrashModule;
    result = module->ExecuteCommand (command);  // return True
    // crashed = True

    command.type = RecoverModule;
    result = module->ExecuteCommand (command);  // return True
    // crashed = False

    command.type = ResetModule;  // unknown
    result = module->ExecuteCommand (command);  // return False

    // tested on 12.08.97, 09:50: all tests okay


    //----------------------------------------------------------------------
    // test ModifyCrashStatus()
    // note: P(crashed) = probCrash/(probRecover+probCrash)
    //       so with probCrash=0.4, probRecover=0.5, P(crashed)=5/21=0.444

    int i;

    prob=0;
    for (i=0; i<TestTries; i++)
    {
        module->ModifyCrashStatus();
        if (module->Crashed()) prob = prob+1;
    } // for

    prob = prob/TestTries;
    // prob = 0.444 (approx)
    // result: 0.457

    // tested on 12.08.97, 09:50: all tests okay so far

    module->SetProbability (True, 0.8);
    // probCrash = 0.8; now P(crashed) = 0.615

    prob=0;
    for (i=0; i<TestTries; i++)
    {
        module->ModifyCrashStatus();
        if (module->Crashed()) prob = prob+1;
    } // for

    prob = prob/TestTries;
    // prob = 0.615 (approx)
    // result: 0.609

    // tested on 12.08.97, 10:00: all tests okay


    delete module;

    //----------------------------------------------------------------------

    return 0;
}

