//
// gentest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// bool.hpp:
// test the EBool operator|= (EBool&, EBool) (05.05.97)
// test the EBool operator&= (EBool&, EBool) (05.05.97)
//
// general.hpp:
// test the delay data structure SDelay (8.11.96)
// test the global function CastDice() (11.07.96)
// test the global function EnableInterrupts() (09.01.98)
// test the global function DisableInterrupts() (09.01.98)
//
// sim_gen.hpp:
// test the message structure SMsg (16.11.96)
// test the global function GetClockvalue() (8.11.96)
// test the global function GetCurrentNodeID() [DOS part] (06.04.97)
//
// netgen.hpp:
// test the global function NodeToNetwork() (18.11.96)
// test the global function NetworkToNode() (18.11.96)
//
// cloctemp.hpp:
// test the function RemoveAll() (11.02.97)
//
// time.hpp:
// test the global function NtpToSeconds() (28.03.97)
// test the global function SecondsToNtp() (28.03.97)
// test class CNtpTime (12.02.97)
// test operator<< (15.11.97)
// test SExtendedTimestamp::operator= (07.12.97)
//
// impl_sw.hpp:
// test the global function GetClockIntvec() (31.07.97)
// test the global function GetGpsIntvec() (31.07.97)
// test the global function GetGlobalTime() (31.07.97)
// test the global function GetSnapshotTime() (29.07.98)
// test the global function SetPosition() (31.07.97)
// test the global function GetPosition() (31.07.97)
// test the global function SetClockFrequency() (31.07.97)
// test the global function GetClockFrequency() (31.07.97)
// test the global function GetTransmitTimestamp() (25.08.97)
// test the global function MakeSystemSnapshot() (25.08.97)
//
// address.hpp:
// test class CAddr (04.08.97)
//
// int128.hpp:
// test class INT128 (10.02.98)
//
// globals.hpp:
// test InstallAlgorithms() (29.07.98)
//
// sim_glob.hpp:
// test InstallNetworks() (29.07.98)
//
// created: 09.07.96
//
// last change: 01.08.98
//


#include <iostream.h>

#include "..\address.hpp"
#include "..\ci.hpp"
#include "..\classman.hpp"
#include "..\clockbas.hpp"
#include "..\clockcfg.hpp"
#include "..\ctrling.hpp"
#include "..\general.hpp"
#include "..\globals.hpp"
#include "..\int128.hpp"
#include "..\msgform.hpp"
#include "..\netgen.hpp"
#include "..\network.hpp"
#include "..\sim_gen.hpp"
#include "..\sim_glob.hpp"
#include "..\sockunix.hpp"
#include "settime.hpp"


#define Need_CControllingObject
#define Need_CSoftwareClock
#define Need_CSupervisor
#define Need_templates


const TestCount = 1000;  // number of tests for the random generator


extern ci_init (int, ci_fn, ci_efn);
// need prototype for direct calls


int main()
{
    //----------------------------------------------------------------------
    // variables necessary for the test of bool.hpp
    //----------------------------------------------------------------------

    EBool result = False;


    //----------------------------------------------------------------------
    // test the behaviour of the EBool operator|= (EBool&, EBool)
    //   should behave like a boolean OR combined with an assignment

    result |= False;  // should return False
    result |= True;   // should return True
    result |= False;  // should return True
    result |= True;   // should return True

    // tested on 02.07.98, 14:10: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of the EBool operator&= (EBool&, EBool)
    //   should behave like a boolean AND combined with an assignment

    result = True;
    result &= True;   // should return True
    result &= False;  // should return False
    result &= True;   // should return False
    result &= False;  // should return False

    // tested on 02.07.98, 14:10: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of general.hpp
    //----------------------------------------------------------------------

    // necessary for the test of CastDice()

    INT32 true, false;  // for counting the results of the function
    int i;


    //----------------------------------------------------------------------
    // test the module global functions
    //----------------------------------------------------------------------

    INT32 mode = DisableInterrupts();  // return 0xFFFFF1FF
    // should call t_mode() with params mask=0x0700, new_mode=0x0700
    // [trace it!]

    EnableInterrupts (mode);
    // should call t_mode() with params mask=0x0700, new_mode=0xFFFFF1FF
    // [trace it!]

    // tested on 09.01.98, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of the global function CastDice()
    // note: this is a random generator, so the results will not be fully
    //       predictable; but general behaviour should be as described
    //----------------------------------------------------------------------

    false=true = 0;
    for (i=0; i<TestCount; i++)
    {
        if (CastDice (0.5)) true++;
        else false++;
    }
    // true and false should be approx. the same

    false=true = 0;
    for (i=0; i<TestCount; i++)
    {
        if (CastDice (0)) true++;
        else false++;
    }
    // true should be zero, false = TestCount

    false=true = 0;
    for (i=0; i<TestCount; i++)
    {
        if (CastDice (1)) true++;
        else false++;
    }
    // false should be zero, true = TestCount

    false=true = 0;
    for (i=0; i<TestCount; i++)
    {
        if (CastDice (0.2)) true++;
        else false++;
    }
    // false should be greater than true (approx. 1:4)
    
    false=true = 0;
    for (i=0; i<TestCount; i++)
    {
        if (CastDice (0.8)) true++;
        else false++;
    }
    // true should be greater than false (approx. 1:4)
    
    // tested on 21.09.98, 12:48: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of sim_gen.hpp
    //----------------------------------------------------------------------

    // necessary for the test of SMsg
    CUnixAddress address ("dummy");
    CControllingObject ctrl ("NONE", &address);
    SMsg *msg1, *msg2, *msg3;
    bufx *buf = (bufx*) new bufx;
    INT16 size;
    STimestamp time;
    // necessary for the test of GetTimestamp

    CBroadcastNetwork network (&ctrl, 0);
    listOfNetworks.Insert (&network, 2);  // insert second time!
    ci_init (0, (ci_fn)1, (ci_efn)1);
    ci_init (2, (ci_fn)1, (ci_efn)1);
    // necessary for SMsg

    CClockInterface *clock;


    //----------------------------------------------------------------------
    // test the behaviour of SMsg
    //----------------------------------------------------------------------

    SMsgPosition pos;


    //----------------------------------------------------------------------
    // test the default ctor

    msg1 = new SMsg;
    // bufsize = 0, buf = NULL, broadcast = True, timestamped = False

    // tested on 16.11.96, 13:00: all tests okay


    //----------------------------------------------------------------------
    // test SetTime()
    // correct behaviour of Set/GetPosition() is assumed (tested below)

    msg1->sender = 1;
    msg1->area = 3;
    time.ts = 0x12345678;
    time.ms = 0xaffeaffe;
    time.acc = 0xabcddcba;
    msg1->SetTime (time);
    // nothing should happen (timestamped = False)
    msg1->timestamped = True;
    msg1->SetTime (time);
    // SMsg::buf = {3,0,0,0,120,86,52,18,254,175,254,175,186,220,205,171,...},
    // bufsize = 16

    pos.position[SMsgPosition::Area]=2;
    pos.length[SMsgPosition::Area]=3;
    pos.position[SMsgPosition::Macrostamp]=8;
    pos.length[SMsgPosition::Macrostamp]=2;
    pos.position[SMsgPosition::Timestamp]=12;
    pos.length[SMsgPosition::Timestamp]=4;
    pos.position[SMsgPosition::Accuracy]=17;
    pos.length[SMsgPosition::Accuracy]=2;
    // so the new buffer should look like this:
    // xxAAAxxxMMxxTTTTxCCxx...

    msg1->sender = 2;
    msg1->if_num = 0;
    SetPosition (pos, msg1->sender, msg1->if_num);
    msg1->SetTime (time);
    // buf = xxAAAxxxMMxxTTTTxCCxx... with x from the buffer shown above,
    // bufsize=27

    msg1->SetTime (time);
    msg1->SetTime (time);
    msg1->SetTime (time);
    msg1->SetTime (time);
    // bufsize = 71, SMsg::buf[64..71] = <see above>

    // tested on 31.07.97, 22:30: all tests okay


    //----------------------------------------------------------------------
    // test the ctor with parameters

    (*buf)[0] = 1; (*buf)[1] = 2;
    size = 2;
    msg2 = new SMsg (NULL, size, 2, 3);
    // if_num = 2, area = 3, bufsize = 0, broadcast = True,
    // timestamped = False, SMsg::buf unchanged

    delete msg2;

    msg2 = new SMsg (buf, size, 2, 3);
    // if_num = 2, area = 3, bufsize = 2, broadcast = True,
    // timestamped = False, SMsg::buf = {1,2,...}

    // tested on 16.11.96, 13:00: all tests okay


    //----------------------------------------------------------------------
    // test the copy ctor

    msg2->sender = 1;
    msg2->receiver = 2;
    msg2->broadcast = False;
    msg2->timestamped = True;
    msg2->msgID = 15;
    for (i=0; i<MAXASICS;i++) msg2->rtime[i] = i;
    msg3 = new SMsg (*msg2);
    // sender=1, area=3, msgID=15, receiver=2, if_num=2, broadcast=False,
    // timestamped=True, stime={ts=3,ms=5,acc=9}, rtime={0,1,2,3},
    // buf={1,2,...}, bufsize=2

    // tested on 25.08.97, 22:40: all tests okay


    //----------------------------------------------------------------------
    // test the assignment operator

    *msg3 = *msg3;
    // nothing should happen

    *msg3 = *msg1;
    // msg3 should have the same values as msg1

    delete msg3;
    msg3 = new SMsg;
    // buf == NULL

    *msg3 = *msg2;  // msg3 should equal msg2

    *msg3 = *msg1;  // msg3 <- msg1

    // SMsg tested completely

    // tested on 25.08.97, 22:40: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of GetClockvalue()
    // correct behaviour of CClockInterface and CClockList is assumed
    //----------------------------------------------------------------------

    clock = new CClockInterface (&ctrl, 25e6, 0);
    listOfClocks.Insert (clock, 1, 0);
    // clock with ID <1,0> inserted
    // set the timestamp of this clock to the value 100


    result = GetClockvalue (2, 0, time);  // should return False

    result = GetClockvalue (1, 0, time);  // should return True
    // time = {ms=0, ts=100, acc=0}

    // tested on 8.11.96, 16:15: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of GetCurrentNodeID()
    //----------------------------------------------------------------------

    SINT32 nodeID = GetCurrentNodeID();  // should return NoAccount

    // tested on 06.04.97, 15:20: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of netgen.hpp
    //----------------------------------------------------------------------

    nodeID = (2<<24) + (15<<16) + (29<<8) + 196;
    addr nodeAddress;

    for (i=0; i<ADDRCNT; i++)
    {
        nodeAddress[i] = 1;
    }

    NodeToNetwork (nodeID, &nodeAddress);
    // nodeAddress = {0,0,2,15,29,196}

    for (i=0; i<ADDRCNT; i++)
    {
        nodeAddress[i] = 1;
    }
    nodeID = 0;

    NetworkToNode (&nodeAddress, nodeID);
    // nodeID = 1<<24 + 1<<16 + 1<<8 + 1

    // tested on 25.12.96, 20:58: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of cloctemp.hpp
    //----------------------------------------------------------------------

    CClockInterface clock1 (&ctrl, 25e6, 0), clock2 (&ctrl, 25e6, 0);

    listOfClocks.Insert (&clock1, 1, 0);
    listOfClocks.Insert (&clock2, 2, 0);

    result = listOfClocks.Empty();  // should return False
    listOfClocks.RemoveAll();
    result = listOfClocks.Empty();  // should return True
    listOfClocks.RemoveAll();
    result = listOfClocks.Empty();  // should return True

    // tested on 11.02.97, 19:55: all tests okay


    //----------------------------------------------------------------------
    // test time.hpp
    //----------------------------------------------------------------------

    CNtpTime ntptime;
    SExtendedTimestamp xtime;


    //----------------------------------------------------------------------
    // test NtpToSeconds(), SecondsToNtp()

    double value;
    value = NtpToSeconds (1, 10000);
    // value = 256.000596... (ms: 1 bit = 256 seconds, ts: 1 bit = 59.6 ns)

    ntptime.ms = 1; ntptime.ts = 10000; value = 0;
    value = NtpToSeconds (ntptime);
    // value = 256.000596...

    time.ms = 1; time.ts = 10000; value = 0;
    value = NtpToSeconds (ntptime);
    // value = 256.000596...

    SecondsToNtp (600, time.ms, time.ts);
    // time.ms = 2, time.ts = 1 476 395 008 (using the Win95 calculator)

    SecondsToNtp (600, ntptime);
    // ntptime.ms = 2, ntptime.ts = 1 476 395 008

    time.ms = 0; time.ts = 0;
    SecondsToNtp (600, time);
    // time.ms = 2, time.ts = 1 476 395 008

    // tested on 28.03.97, 17:00: all tests okay so far


    SecondsToNtp (10, ntptime);  // ts = 167 772 160
    value = NtpToSeconds (ntptime);  // value = 10

    // tested on 28.03.97, 17:00: all tests okay so far


    SecondsToNtp (285.28, ntptime);
    // ms=1, ts=491 236 884, us=2 061 584 302
    // note: the value of us may be different (the last digits are below
    // the resolution of the Win95 calculator)
    // TI-92 computes 2 061 589 383
    // real (computer) value: 2 061 582 336
    value = NtpToSeconds (ntptime);  // value = 285.28

    SecondsToNtp (285.28, xtime);  // like ntptime
    value = NtpToSeconds (xtime);  // value = 285.28

    // tested on 29.07.97, 20:20: all tests okay


    //----------------------------------------------------------------------
    // test CNtpTime - correct behaviour of SecondsToNtp() is assumed

    double seconds = 200;
    INT32 ms, ts, us;

    SecondsToNtp (seconds, ms, ts, us);
    // ms=0, ts = 3 355 443 200, us=0

    CNtpTime time1;
    // time1: ms=ts=us = 0
    CNtpTime time2 (2,3,4);
    // time2: ms=2, ts=3, us=4
    CNtpTime time3 (time2);
    // time3: ms=2, ts=3, us=4
    CNtpTime time4 (seconds);
    // time4.ms=ms, time4.ts=ts, time4.us=us

    // tested on 29.07.97, 18:20: all tests okay so far


    time1 = time2;
    // time1: ms=2, ts=3, us=4

    time1 = time1 + time2;
    // time1: ms = 4, ts = 6, us=8

    time3 = time4 + time4;
    // time3: ms = 1, ts = 2 415 919 104, us=0

    time3 = time3 - time4;
    // time3: ms = 0, ts = 3 355 443 200, us=0

    time3 = time3 - time4;
    // time3: ms=ts=us=0

    time3 = time3 - time4;
    // time3: ms=ts=us=0

    time1 += time2;
    // time1: ms = 6, ts = 9, us=12

    // tested on 29.07.97, 18:20: all tests okay so far


    time1 = 285.28;
    value = NtpToSeconds (time1);  // value = 285.28

    time2 = time1 + time1 + time1;
    value = NtpToSeconds (time2);  // value = 855.84

    time2 = time2 - time1;
    value = NtpToSeconds (time2);  // value = 570.56

    // tested on 29.07.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test operator<<

    cout << "CNtpTime: " << time4 << endl;  // operator<< (CNtpTime)
    // ms=0, ts = 3 355 443 200, us=0

    xtime.ms = 1; xtime.ts = 2; xtime.us = 3; xtime.acc = 4;
    cout << "SExtendedTimestamp: " << xtime << endl;
    cout << "STimestamp: " << (STimestamp)xtime << endl;
    // acc=4, ms=1, ts=2, us=3

    // tested on 15.11.97, 22:10: all tests okay


    //----------------------------------------------------------------------
    // test SExtendedTimestamp::operator= ()

    // test assignment from STimestamp
    time.ms = 12; time.ts = 16; time.acc = 83;
    xtime = time;
    // ms = 12, ts = 16, acc = 83, us = 0

    // tested on 07.12.97, 14:50: all tests okay

    // test assignment from CNtpTime
    time4.us = 25;
    xtime = time4;
    // ms = 0, ts = 3 355 443 200, us = 25, acc = 0

    // tested on 21.09.98, 13:10: all tests okay


    //----------------------------------------------------------------------
    // test impl_sw.hpp
    //----------------------------------------------------------------------

    int intvec;
    double globTime;

    intvec = GetClockIntvec (2, 3);  // return 0x20D0
    intvec = GetClockIntvec (2, 2);  // return 0x20C0

    // tested on 31.07.97, 21:10: all tests okay so far


    intvec = GetGpsIntvec (2, 3, GPU1);  // return 0x400020D0
    intvec = GetGpsIntvec (2, 2, GPU3);  // return 0x400020C0

    // tested on 31.07.97, 21:10: all tests okay so far


    SetGlobalTime (2.3);  // set the time to some value != 0

    globTime = GetGlobalTime();  // return 2.3

    // tested on 29.07.98, 14:45: all tests okay so far


    globTime = GetSnapshotTime();  // return 2.3

    // tested on 29.07.98, 14:45: all tests okay so far


    SetGlobalTime (0);  // set back the time

    for (i=0; i<SMsgPosition::EItemCount; i++)
    {
        pos.position[i] = 4-i;
        pos.length[i] = i+10;
    }

    SetPosition (pos, 3, 1);

    for (i=0; i<SMsgPosition::EItemCount; i++)
    {
        pos.position[i] = 0;
        pos.length[i] = 0;
    }

    GetPosition (pos, 3, 1);
    // pos = (position={4,3,2,1}, length={10,11,12,13})

    GetPosition (pos, 2, 1);
    // pos = (position={0,8,4,12}, length={4,4,4,4})

    // tested on 31.07.97, 21:10: all tests okay so far


    SetClockFrequency (3, 1, 1000);
    globTime = GetClockFrequency (3, 1);  // return 1000
    globTime = GetClockFrequency (2, 1);  // return 25000000

    // tested on 31.07.97, 21:10: all tests okay

    CAddr netaddr;
    time.ms = 3; time.ts = 4; time.acc = 6;
    GetTransmitTimestamp (netaddr, 0, NULL, time);
    // ms=ts = 0; acc unchanged

    // tested on 15.11.97, 22:10: all tests okay


    listOfClocks.Insert (&clock1, 1, 0);
    listOfClocks.Insert (&clock2, 2, 0);

    MakeSystemSnapshot();
    // 2x output: CClockInterface()::GetClock() called

    // tested on 25.08.97, 22:40: all tests okay

    listOfClocks.Remove (1, 0);
    listOfClocks.Remove (2, 0);

    //----------------------------------------------------------------------
    // test CAddr (address.hpp)

    CAddr addr1;
    for (i=0; i<ADDRCNT; i++) addr1[i]=i;
    // addr1.addr = {0,1,2,3,4,5}

    CAddr addr2 (addr1);
    // addr2.addr = {0,1,2,3,4,5}

    CAddr addr3 (&nodeAddress);
    // addr3.addr = {1,1,1,1,1,1}

    CAddr addr4 (&nodeAddress);
    // addr4.addr = {1,1,1,1,1,1}


    if (addr1 < addr2) cout << "addr1 < addr2" << endl;
    if (addr1 == addr2) cout << "addr1 == addr2" << endl;
    if (addr1 > addr2) cout << "addr1 > addr2" << endl;
    if (addr1 != addr2) cout << "addr1 != addr2" << endl;

    if (addr2 < addr3) cout << "addr2 < addr3" << endl;
    if (addr2 == addr3) cout << "addr2 == addr3" << endl;
    if (addr2 > addr3) cout << "addr2 > addr3" << endl;
    if (addr2 != addr3) cout << "addr2 != addr3" << endl;

    if (addr3 < addr4) cout << "addr3 < addr4" << endl;
    if (addr3 == addr4) cout << "addr3 == addr4" << endl;
    if (addr3 > addr4) cout << "addr3 > addr4" << endl;
    if (addr3 != addr4) cout << "addr3 != addr4" << endl;

    if (addr4 < addr1) cout << "addr4 < addr1" << endl;
    if (addr4 == addr1) cout << "addr4 == addr1" << endl;
    if (addr4 > addr1) cout << "addr4 > addr1" << endl;
    if (addr4 != addr1) cout << "addr4 != addr1" << endl;
    // output:
    //   addr1 == addr2
    //   addr2 < addr3
    //   addr2 != addr3
    //   addr3 == addr4
    //   addr4 > addr1
    //   addr4 != addr1

    // tested on 05.08.97, 09:30: all tests okay


    //----------------------------------------------------------------------
    // test INT128 (int128.hpp)

    INT128 alpha, beta (1,2,3,4);
    // second ctor calls SetNtp()

    INT32 a,b,c,d;

    beta.SetNtp (0,0,1,2);
    // call SetNtp() again (should cause the first rep to be deleted or
    // reused) [trace it!]

    // tested on 02.07.98, 14:10: all tests okay so far

    alpha.SetBinary (0,0,1,3);
    beta.SetBinary (0,0,0,1);
    beta.SetBinary (0,0,0,2);
    // rep is set accordingly (the second call to SetBinary() should cause
    // the first rep to be deleted or reused) [trace it!]

    // tested on 02.07.98, 14:10: all tests okay so far

    alpha *= beta;
    alpha.GetBinary (a,b,c,d);
    // a,b,c,d = 0,0,2,6

    alpha += beta;
    alpha.GetBinary (a,b,c,d);
    // a,b,c,d = 0,0,2,8

    alpha = -alpha;
    if (alpha < 0) cout << "alpha < 0" << endl;
    if (beta < 0) cout << "beta < 0" << endl;
    if (alpha < beta) cout << "alpha < beta" << endl;
    // output:
    //   alpha < 0
    //   alpha < beta

    // tested on 10.02.98, 17:30: all tests okay so far

    alpha = ~alpha;
    alpha.GetBinary (a,b,c,d);
    // a,b,c,d = 0,0,0xFFFD,0xFFFFFFF7

    // tested on 11.02.98, 16:00: all tests okay


    listOfNetworks.Remove (0);
    listOfNetworks.Remove (2);
    // remove the two networks, since they are static and the same object,
    // and the list would try to delete them if they were left in it


    //----------------------------------------------------------------------
    // test InstallAlgorithms() (globals.hpp)

    AlgFunction algf;  // the creation function of an algorithm class
    InstallAlgorithms();  // that installs all algorithms

    algf = algorithmManager.GetFunction (FreeRun);  // return fct address
    algf = algorithmManager.GetFunction (FaultTolerantAverage);  // --"--
    algf = algorithmManager.GetFunction (Midpoint);  // --"--
    algf = algorithmManager.GetFunction (Fetzer);  // --"--
    algf = algorithmManager.GetFunction (EAlgorithmTypeCount);// return NULL

    // tested on 29.07.98, 18:05: all tests okay


    //----------------------------------------------------------------------
    // test InstallNetworks() (sim_glob.hpp)

    NetworkFunction netf;  // the creation function of a network class
    InstallNetworks();  // that installs all networks

    netf = networkManager.GetFunction (Broadcast);  // return fct address
    netf = networkManager.GetFunction (ENetworkTypeCount);  // return NULL

    // tested on 29.07.98, 18:10: all tests okay


    //----------------------------------------------------------------------

    return 0;
}
    

//--------------------------------------------------------------------------
// CClockInterface

CClockInterface::CClockInterface (CControllingObject *ctrl, double, int)
    : CApplControlledObject (ctrl) {}
CClockInterface::~CClockInterface () {}
void CClockInterface::Reset() {}
void CClockInterface::ReportCrashChange() {}
int CClockInterface::ReportError(EReportType, char*, int, char*) { return 1;}
void CClockInterface::Crash() {}
void CClockInterface::Recover() {}
EBool CClockInterface::SetParameter (const SParameter&) { return False; }
EBool CClockInterface::GetParameter (SParameter&) const { return False; }
EBool CClockInterface::ExecuteCommand (const SCommand&) { return False; }
EBool CClockInterface::SetAmortizationDuration (double) { return False; }
EBool CClockInterface::GpsConnected () { return False; }
void CClockInterface::AddGps (EGpsUnit) {}
CClock *CClockInterface::GetClock ()
    { cout << "CClockInterface::GetClock() called" << endl; return NULL; }
void CClockInterface::RemoveGps (EGpsUnit) {}
double CClockInterface::GetRhoPlus() const {return 0;}
double CClockInterface::GetRhoMinus() const {return 0;}
double CClockInterface::GetGranularity() const {return 0;}
double CClockInterface::GetGranularityAcc() const {return 0;}
double CClockInterface::GetGranularityOsc() const {return 0;}
double CClockInterface::GetGranularityDT() const {return 0;}
INT32 CClockInterface::GetAccuracy() const {return 0;}
EBool CClockInterface::GetTimestamp (STimestamp&) { return False; }
void CClockInterface::SetTimestamp (const STimestamp&) {}
void CClockInterface::SetTimestamp (const SExtendedTimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const STimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const SExtendedTimestamp&) {}
void CClockInterface::EnableInterrupt (EInterruptSource) {}
void CClockInterface::DisableInterrupt (EInterruptSource) {}
EBool CClockInterface::InterruptPending (EInterruptSource) { return False; }
EBool CClockInterface::InterruptPending (EClockInterrupt) { return False;  }
void CClockInterface::ClearInterrupt (EInterruptSource) {}
void CClockInterface::ClearInterrupt (EClockInterrupt) {}
void CClockInterface::InterruptAt (EInterruptSource, double) {}
void CClockInterface::InterruptAfter (EInterruptSource, double) {}
double CClockInterface::InterruptAtMult (EInterruptSource, double) { return 0; }
void CClockInterface::Init () {}
void CClockInterface::Dispose() {}
void CClockInterface::isr() {}
void CClockInterface::wobble_isr() {}
void CClockInterface::Amortize (double, INT32, INT32) {}
void CClockInterface::PreloadAmort (double, INT32, INT32) {}
void CClockInterface::DisableDutyTimer (EInterruptSource) {}


CClock::CClock (double, SUtcsuRead *r, SUtcsuWrite *w)
  { utcsuRead_ = r; utcsuWrite_ = w; }
EBool CClock::SetParameter (const SParameter&) { return False; }
EBool CClock::GetParameter (SParameter&) const { return False; }
EBool CClock::EnableInterrupt (int, INT32, INT32) { return False; }
EBool CClock::DisableInterrupt (int, INT32, INT32) { return False; }


#include "inlines.hpp"


