//
// gpstest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test ACGps
// note: ReportEvent() and isr() were made public for this test
// test CGpsTemplate
// test CSoftwareGps
// note: GetGpsTime() was made public for this test
//
// created: 29.03.97
//
// last change: 29.07.98
//


#include <iostream.h>
#include <math.h>
#include <stdlib.h>

#include "..\clock_sw.hpp"
#include "..\clockbas.hpp"
#include "..\clockcfg.hpp"
#include "..\clockifc.hpp"
#include "..\cloctemp.hpp"
#include "..\ctrling.hpp"
#include "..\general.hpp"
#include "..\gps_sw.hpp"
#include "..\template.hpp"
#include "..\time.hpp"
#include "settime.hpp"


#define Need_ACObjectList
#define Need_CApplControlledObject
#define Need_CBitmask
#define Need_CControlledObject
#define Need_CControllingObject
#define Need_CCrashingModule
//#define Need_CGpsInterrupt
#define Need_CInterruptProcess


CClockList listOfClocks;


int main()
{
    //----------------------------------------------------------------------
    // test ACGps (can only be tested through a derived class)
    // test CGpsTemplate
    // test CSoftwareGps
    //    correct behaviour of CSoftwareClock and CGpsInterrupt is assumed
    //----------------------------------------------------------------------

    ACGps *gpsbase;
    CSoftwareGps *gps;
    CControllingObject *ctrl = new CControllingObject ("QNAM", NULL, 2);
    CClockInterface *clock = new CClockInterface (ctrl, 25e6, 0);
    CNtpTime time;
    EGpsUnit gpu;
    EBool result;


    //----------------------------------------------------------------------
    // test the ctors (both CSoftwareGps and ACGps) and the template

    gpsbase = gpsTemplate->NewGPS (clock, 1);
    gps = (CSoftwareGps *)gpsbase;
    // clock_ = clock, area_ = 1, enableMask_=statusMask_ = 0

    // tested on 01.08.97, 10:40: all tests okay


    //----------------------------------------------------------------------
    // test EnableGps(), DisableGps()

    gpsbase->EnableGps (GPU2);
    // enableMask = 2

    gpsbase->EnableGps (GPU1);
    // enableMask = 3

    gpsbase->EnableGps (GPU3);
    // enableMask = 7

    // tested on 01.08.97, 10:40: all tests okay so far


    gpsbase->DisableGps (GPU1);
    // enableMask = 6

    gpsbase->DisableGps (GPU2);
    // enableMask = 4

    // tested on 01.08.97, 10:40: all tests okay


    //----------------------------------------------------------------------
    // test GpsEnabled()

    result = gpsbase->GpsEnabled();  // return True

    gpsbase->DisableGps (GPU3);
    result = gpsbase->GpsEnabled();  // return False

    // tested on 01.08.97, 10:40: all tests okay


    //----------------------------------------------------------------------
    // test SetGpsStatus()

    gpsbase->EnableGps (GPU2);

    gpsbase->SetGpsStatus (GPU2, True);
    // statusMask_ = 2, clock->gpsStatus = True

    gpsbase->SetGpsStatus (GPU2, False);
    // statusMask_ = 0, clock->gpsStatus = False

    gpsbase->SetGpsStatus (GPU2, True);
    // statusMask_ = 2, clock->gpsStatus = True
    gpsbase->EnableGps (GPU3);
    // enableMask = 6, clock->gpsStatus = False

    gpsbase->SetGpsStatus (GPU3, True);
    // statusMask_ = 6, clock->gpsStatus = True

    gpsbase->DisableGps (GPU2);
    // enableMask = 4, clock->gpsStatus = True

    // tested on 01.08.97, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test GetGpsTime()

    time.ms = 0xffff; time.ts = 0xffff; time.us = 0xffff;
    SetGlobalTime (257.5);
    gpsbase->GetGpsTime (time);  // time is rounded to 257
    // time.ms = 1, time.ts = 0x01000000, time.us = 0

    // tested on 01.08.97, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test GetGpsNumber()

    gpu = gpsbase->GetGpsNumber ();  // EGpsUnitCount

    ((CSoftwareClock*)clock->GetClock())->CaptureTime (GPU2);
    gpu = gpsbase->GetGpsNumber ();  // GPU2

    // tested on 01.08.97, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test ClearGpsInterrupt()

    gpsbase->ClearGpsInterrupt();
    // nothing should happen (empty in the software implementation)

    // tested on 01.08.97, 11:00: all tests okay


    //----------------------------------------------------------------------
    // test GetClockGps()

    gpsbase->GetClockGps (GPU1, time);
    // time.ms = 0, time.ts = 0, time.us = 0
    
    gpsbase->GetClockGps (GPU2, time);
    // time.ms = 0x01, time.ts = 0x01800000, time.us = 0
    // real value: time.ts = 0x017FFFFF (internal 'us' must have been high)
    
    // tested on 29.07.98, 13:30: all tests okay


    //----------------------------------------------------------------------
    // test isr()

    gpsbase->EnableGps (GPU2);
    // clock->gpsStatus = False

    gpsbase->isr();
    // statusMask=6, clock->gpsStatus=True
    // should call clock->ClearGpsInterrupt (), call clock->Amortize (-0.5)
    // [see the output!], call ReportEvent() (with event data: module,
    // type, diff) [trace!]

    SetGlobalTime (258);
    ((CSoftwareClock*)clock->GetClock())->CaptureTime (GPU2);
    gpsbase->SetGpsStatus (GPU2, False);
    // statusMask=4, clock->gpsStatus=False

    SetGlobalTime (258.3);
    gpsbase->isr();
    // statusMask=6, clock->gpsStatus=True
    // should call clock->ClearGpsInterrupt (), not call Amortize()

    // tested on 29.07.98, 13:40: all tests okay


    delete gpsbase;

    //----------------------------------------------------------------------

    return 0;
}


CClockInterface::CClockInterface (CControllingObject *ctrl, double,
    int area) : CApplControlledObject (ctrl)
  { clock_ = new CSoftwareClock (25e6, 2, area);
    Init(); }
CClockInterface::~CClockInterface () {}
EBool CClockInterface::SetParameter (const SParameter&) { return False; }
EBool CClockInterface::GetParameter (SParameter&) const { return False; }
EBool CClockInterface::ExecuteCommand (const SCommand&) { return False; }
double CClockInterface::GetRhoPlus() const {return 0;}
double CClockInterface::GetRhoMinus() const {return 0;}
INT32 CClockInterface::GetAccuracy() const {return 0;}
EBool CClockInterface::GetTimestamp (STimestamp& time)
  { time.ts = 0; time.ms = 0; time.acc = 0; return True; }
void CClockInterface::SetTimestamp (const STimestamp&) {}
void CClockInterface::SetTimestamp (const SExtendedTimestamp&) {}
void CClockInterface::EnableInterrupt (EInterruptSource) {}
void CClockInterface::DisableInterrupt (EInterruptSource) {}
EBool CClockInterface::InterruptPending (EInterruptSource) { return False; }
EBool CClockInterface::InterruptPending (EClockInterrupt inttype) {
    cout << "InterruptPending() called";
    if (inttype==INTN) cout << " with INTN"; cout << endl; return False; }
void CClockInterface::ClearInterrupt (EInterruptSource) {
    cout << "ClearInterrupt() called";
    cout << endl; }
void CClockInterface::ClearInterrupt (EClockInterrupt) {}
void CClockInterface::InterruptAfter (EInterruptSource, double) {}
double CClockInterface::InterruptAtMult (EInterruptSource, double) {return 0;}
void CClockInterface::Init ()
  { clock_->SetSWRESET (1);  // reset the clock
static const double TwoPow51 = 4294967296.*524288.;
    double f = clock_->GetFrequency();
    double dummy;
    INT8 steplow = (INT8)(modf (TwoPow51/f, &dummy) * 256);
    steplow_  = (INT32)steplow<<24 & 0xFF000000;
    steppure_ = (INT32)dummy;
    clock_->SetSTEPPURE (steppure_);  // preload step (pure phase)
    clock_->SetMSSET (steplow_);  // set steplow
    clock_->SetSTEPAMORT (steppure_);  // amort. phase is like pure phase
    clock_->SetAMORTTIMESET ((INT32)(clock_->GetFrequency()*amorttime_));
    // amortization cycles
    clock_->SetSTARTAMORT (1);  // to take over the step pure value

INT32 ms, ts;
SecondsToNtp (GetGlobalTime(), ms, ts);

    clock_->SetMSSET (steplow_ | ms & 0x00FFFFFF);
    clock_->SetTSSET (ts);
    clock_->SetUSSET (0);
    clock_->SetNTPSET (0);  // take over the new time value
  }
void CClockInterface::Dispose() {}
void CClockInterface::isr() {}
void CClockInterface::Amortize (double diff, INT32 diffPlus, INT32 diffMinus)
  { cout << "Amortize() called with " << diff << ", " << diffPlus;
    cout << ", " << diffMinus << endl; }
CClock *CClockInterface::GetClock() { return clock_; }
void CClockInterface::Crash() {}
void CClockInterface::Recover() {}
void CClockInterface::Reset() {}
void CClockInterface::ReportCrashChange() {}
void CClockInterface::PreloadTimestamp (double, const STimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const SExtendedTimestamp&) {}
void CClockInterface::DisableDutyTimer (EInterruptSource) {}
void CClockInterface::InterruptAt (EInterruptSource, double) {}
void CClockInterface::PreloadAmort (double, INT32, INT32) {}
int CClockInterface::ReportError (EReportType, char*, int, char*) {return 0;}


#include "inlines.hpp"


