//
// isrtest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the interrupt functions in impl_sw.cpp
// test the interrupt handling in interupt.hpp/interupt.cpp
// note: the underlying class CInterruptHandler is tested through the
//       global functions available in interupt.hpp
// test CInterruptProcess in intproc.cpp/hpp
//
// note: to test ClearISRs(), you have to undefine NO_RESOURCE (it is defined
//      in process.hpp); just try it once to test ClearISRs(), but then keep
//      it defined for all other tests
//
// created: 19.01.97
//
// last change: 24.07.98
//


#include <stdlib.h>  // for NULL

#include "..\interupt.hpp"
#include "..\intproc.hpp"
#include "..\sim_glob.hpp"


#define Need_CApplControlledObject
#define Need_CClock
#define Need_CClockInterface
#define Need_CControlledObject
#define Need_CCrashingModule
#define Need_CSoftwareClock

CNtpTime::CNtpTime() {}
// dummy ctor for CNtpTime

CClockList listOfClocks;

#ifndef NO_RESOURCE
  extern void ClearISRs ();
  extern void ClearISRs (SINT32 nodeID, int area);
#endif

class A1 : public CNeedInterrupt
{
public:
    A1 () { i=0; }
private:
    int i;
    void isr () { i+=1; }
};

class A2 : public CNeedInterrupt
{
public:
    A2 () { j=0; k=0; }
    friend void CallISR3 (CNeedInterrupt *obj);
private:
    int j, k;
    void isr () { j+=2; }
    void isr3 () { k+=3; }
};

void CallISR3 (CNeedInterrupt *obj)
{
    ((A2 *)obj)->isr3();
    // have to cast in order to call A2::isr3()
}


int main()
{
    //----------------------------------------------------------------------
    // variables for the test of the interrupt functions in impl_sw.cpp
    //----------------------------------------------------------------------

    int vector;  // from GetClockIntvec()


    //----------------------------------------------------------------------
    // test GetClockIntvec()

    vector = GetClockIntvec (0, 0);  // return 0x0A0
    vector = GetClockIntvec (0, 1);  // return 0x0B0
    vector = GetClockIntvec (1, 0);  // return 0x10A0
    vector = GetClockIntvec (1, NumberOfAreas-1);  // return 0x10F0

    // tested on 24.07.98, 17:10: all tests okay


    //----------------------------------------------------------------------
    // test GetGpsIntvec()

    vector = GetGpsIntvec (0, 0, GPU1);  // return 0x 4000 00A0
    vector = GetGpsIntvec (0, 1, GPU1);  // return 0x 4000 00B0
    vector = GetGpsIntvec (1, 0, GPU1);  // return 0x 4000 10A0

    // tested on 24.07.98, 17:10: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of interupt.cpp
    //----------------------------------------------------------------------

    A1 *obj1 = new A1;  // the objects whose ISRs are called
    A2 *obj2 = new A2;
    A2 *obj3 = new A2;


    //----------------------------------------------------------------------
    // test SetISR (vector, obj)

    vector = 1;
    SetISR (vector++, obj1);
    SetISR (vector++, obj2);
    // 1 --> obj1.isr, 2 --> obj2.isr

    SetISR (vector++, obj2, CallISR3);
    // 1 --> obj1.isr, 2 --> obj2.isr, 3 --> obj2.isr3

    SetISR (1, obj3, CallISR3);
    // 1 --> obj3.isr3 + obj1.isr, 2 --> obj2.isr, 3 --> obj2.isr3

    SetISR (vector++, obj1);
    // 1 --> obj3.isr3 + obj1.isr, 2 --> obj2.isr, 3 --> obj2.isr3,
    // 4 --> obj1.isr

    // tested on 06.12.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test InvokeISR ()
    // 1 --> obj3.isr3, 2 --> obj2.isr, 3 --> obj2.isr3, 4 --> obj1.isr

    InvokeISR (1);  // obj3->k = 3, obj1->i = 1
    InvokeISR (2);  // obj2->j = 2
    InvokeISR (3);  // obj2->k = 3
    InvokeISR (4);  // obj1->i = 2

/*
    InvokeISR (30);
    // Assert() should fail
    // try this once, then comment it out
*/

    // tested on 06.12.97, 20:00: all tests okay


    //----------------------------------------------------------------------
    // test RemoveISR ()
    // 1 --> obj3.isr3, 2 --> obj2.isr, 3 --> obj2.isr3, 4 --> obj1.isr

    RemoveISR (1, obj2);

    InvokeISR (1);  // obj3->k = 6, obj1->i = 3
    InvokeISR (2);  // obj2->j = 4
    InvokeISR (3);  // obj2->k = 6
    InvokeISR (4);  // obj1->i = 4

    RemoveISR (1, obj1);

    InvokeISR (1);  // obj3->k = 9
    InvokeISR (2);  // obj2->j = 6
    InvokeISR (3);  // obj2->k = 9
    InvokeISR (4);  // obj1->i = 5

    RemoveISR (3, obj2);

    InvokeISR (1);  // obj3->k = 12
    InvokeISR (2);  // obj2->j = 8
    InvokeISR (3);  // obj2->k = 12
    InvokeISR (4);  // obj1->i = 6

    RemoveISR (3, obj2, CallISR3);

    InvokeISR (1);  // obj3->k = 15
    InvokeISR (2);  // obj2->j = 10
/*
    InvokeISR (3);
    // here, an assert should fail; try this once, then comment it out
*/

    // tested on 07.12.97, 17:50: all tests okay


#ifndef NO_RESOURCE
    //----------------------------------------------------------------------
    // test ClearISRs ()
    // obj1->i=6; obj2->j=10,k=12; obj3->j=0,k=15

    int vec1, vec2, vec3;  // interrupt vectors
    int vec3gpu1, vec3gpu2, vec3gpu3;  // GPU interrupt vectors

    vec1 = GetClockIntvec (0, 0);
    vec2 = GetClockIntvec (0, 1);
    vec3 = GetClockIntvec (1, 0);
    vec3gpu1 = GetGpsIntvec (1, 0, GPU1);
    vec3gpu2 = GetGpsIntvec (1, 0, GPU2);
    vec3gpu3 = GetGpsIntvec (1, 0, GPU3);
    // note: in pure simulation, the vectors are the same!

    SetISR (vec1, obj1);
    SetISR (vec2, obj2);

    SetISR (vec3, obj3, CallISR3);  // vec3 base -> obj3->isr3
    SetISR (vec3|INTA, obj3);  // vec3 INTx -> obj3->isr
    SetISR (vec3|INTN, obj3);
    SetISR (vec3|INTT, obj3);
    SetISR (vec3gpu1, obj2, CallISR3);  // vec3 GPU -> obj2->isr
    SetISR (vec3gpu2, obj2, CallISR3);
    SetISR (vec3gpu3, obj2, CallISR3);

    InvokeISR (vec3);  // obj3->k = 18
    InvokeISR (vec3|INTA);  // obj3->j = 2
    InvokeISR (vec3|INTN);  // obj3->j = 4
    InvokeISR (vec3|INTT);  // obj3->j = 6
    InvokeISR (vec3gpu1);  // obj2->k = 21
    InvokeISR (vec3gpu2);  // obj2->k = 30
    InvokeISR (vec3gpu3);  // obj2->k = 39

    ClearISRs (1, 0);  // should remove all ISRs for vec3

    SetISR (vec3|INTA, obj1);  // set another vec3 ISR
    SetISR (vec3|INTN, obj1);
    SetISR (vec3|INTT, obj1);
    SetISR (vec3gpu1, obj3, CallISR3);
    SetISR (vec3gpu2, obj3, CallISR3);
    SetISR (vec3gpu3, obj3, CallISR3);

    InvokeISR (vec3);  // obj3->k = 21
    InvokeISR (vec3|INTA);  // obj1->i = 7
    InvokeISR (vec3|INTN);  // obj1->i = 8
    InvokeISR (vec3|INTT);  // obj1->i = 9
    InvokeISR (vec3gpu1);  // obj3->k = 30
    InvokeISR (vec3gpu2);  // obj3->k = 39
    InvokeISR (vec3gpu3);  // obj3->k = 48

    InvokeISR (vec1);  // obj1->i = 10
    InvokeISR (vec2);  // obj2->j = 12

    ClearISRs ();  // should remove all ISRs

    SetISR (vec1, obj2);
    SetISR (vec2, obj1);
    SetISR (vec3, obj1);
    InvokeISR (vec1);  // obj2->j = 14
    InvokeISR (vec2);  // obj1->i = 11
    InvokeISR (vec3);  // obj1->i = 12
    // no other changes -> the old ISRs have been removed

    // tested on 24.07.98, 17:35: all tests okay
#endif


    //----------------------------------------------------------------------
    // variables necessary for the test of CInterruptProcess
    //----------------------------------------------------------------------

    CInterruptProcess *irq;


    //----------------------------------------------------------------------
    // test ctor

    irq = new CInterruptProcess;
    // vector = NoAccount (-1), enabled = False, waitTime = NoAccount (-1)

    // tested on 25.01.97, 14:45: all tests okay


    //----------------------------------------------------------------------
    // test SetVector()

    irq->SetVector (1);
    // vector = 1

    // tested on 25.01.97, 14:45: all tests okay


    //----------------------------------------------------------------------
    // test Enable(), Disable()

    irq->Enable ();
    // enabled = True

    irq->Disable ();
    // enabled = False

    // tested on 25.01.97, 14:45: all tests okay


    //----------------------------------------------------------------------
    // test Enabled()

    EBool result;

    result = irq->Enabled();  // return False
    irq->Enable ();
    result = irq->Enabled();  // return True

    // tested on 29.03.97, 11:10: all tests okay


    //----------------------------------------------------------------------
    // test Active()

    result = irq->Active ();  // return False
    ACProcess::SetActiveProcess (irq);
    // pretend that the process is active
    result = irq->Active ();  // return True

    // tested on 29.03.97, 11:10: all tests okay


    //----------------------------------------------------------------------
    // test RaiseAfter()

    ACProcess::SetActiveProcess (NULL);
    irq->RaiseAfter (10);  // (process not active)
    // state = Scheduled, assocTime = 10

    ACProcess::SetActiveProcess (irq);  // make the process "active"

    irq->RaiseAfter (10);  // (process is "active")
    // waitTime = 10

    // tested on 25.01.97, 14:45: all tests okay


    //----------------------------------------------------------------------
    // test CancelInterrupt()

    irq->CancelInterrupt ();  // process is still active
    // waitTime = NoAccount (-1), state = Scheduled

    irq->RaiseAfter (10);  // waitTime = 10
    ACProcess::SetActiveProcess (NULL);  // process not active
    irq->CancelInterrupt ();
    // waitTime = NoAccount (-1), state = Passive

    // tested on 29.03.97, 11:10: all tests okay


/*
    //----------------------------------------------------------------------
    // test Body()
    // note: test this part once, then comment it out

    irq->Enable();
    ACProcess::SetActiveProcess (NULL);
    irq->RaiseAfter (10);  // (process not active)
    ACProcess::SetActiveProcess (irq);
    irq->RaiseAfter (10);  // (process is "active")
    // waitTime = 10

    // trace to verify the following behaviour:
    // PreExecution(), InvokeISR() and PostExecution() is called, Hold (10)
    // is called, waitTime = -1, InvokeISR() called again, Cancel() called

    irq->Body();  // trace into the code!!!

    // tested on 29.03.97, 11:10: all tests okay
*/

    //----------------------------------------------------------------------

    
    return 0;
}


#include "inlines.hpp"


