//
// lantest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the lance class in lance.hpp
// functions CCrashingModule::Crash(), CCrashingModule::Recover() were
//   made public for this test
//
// created: 09.11.96
//
// last change: 04.02.99
//


#include <iostream.h>
#include <string.h>  // for memcpy()

#include "..\ci.hpp"
#include "..\clocksyn.hpp"
#include "..\ctrling.hpp"
#include "..\dmsgtemp.hpp"
#include "..\lance.hpp"
#include "..\network.hpp"
#include "..\msgform.hpp"
#include "..\sim_glob.hpp"
#include "..\structs.hpp"

#define Need_ACNetwork
#define Need_ACProcess
#define Need_CBitmask
#define Need_CBroadcastNetwork
#define Need_CClock
#define Need_CClockSync
#define Need_CControllingObject
#define Need_CSoftwareClock


// necessary globals
CNetworkList listOfNetworks;
CClockList listOfClocks;
INT32 t_mode (INT32, INT32, INT32*) { return 0; }

double g_currTime = 0;  // current clock time

extern INT32 ci_buf_size (INT16 size);
// only defined in ci.cpp

const addr broadcastAddress = BROADCAST_ADDRESS;
// need this to compare with the destination address for CLance::Send()

unsigned long g_streamOffset = 0;
// needed for distrib.cpp

// prototypes of static functions of lance.cpp
extern bufx *BufToBufx (char *message);
extern char *BufxToBuf (bufx *buf);
extern void ExtractMessage (bufx *buf, INT16 size, SINT32 nodeID,
    int if_num, STimeData *time, char **message, INT16 *msgsize);
extern int CI_GetBuffer (INT16 size, bufx **buf, int if_num, int area);
extern int CI_GetEnvelope (int mode, tenvx **tenv, int if_num, int area);
extern int CI_RetBuffer (bufx *buf);
extern int CI_RetEnvelope (tenvx *tenv);
extern int CI_Send (int mode, tenvx *tenv, bufx *buf, INT16 size,
                    int if_num, int area);

// prototypes of CI functions
extern int ci_init (int, ci_fn, ci_efn);
extern int ci_get_tbuf (INT16, bufx **, int, int);
extern int ci_ret_tbuf (bufx *);


int main()
{
    //----------------------------------------------------------------------
    // test the class CLance
    // correct behaviour of class CApplControlledObject, CCrashingModule
    // and the CI functions is assumed
    //----------------------------------------------------------------------

    CLance *lance;
    CControllingObject *ctrl = new CControllingObject ("QNAM", NULL);
    int if_num = 1;
    int area = 2;
    CBroadcastNetwork *network = new CBroadcastNetwork (ctrl, if_num);
    int mode = TS_PACKET;
    int retmode = TS_PACKET | RET_PKB_IMM;  // deallocate at once
    int result;
    tenvx *tenv;
    renvx *renv;
    bufx *buf;
    char *msg;  // user part of bufx (without msgID)
    INT32 size;  // size of msg
    addr testaddr;  // for testing message scrambling
    STimeData time;  // also for testing message scrambling
    INT32 msgID;
    int i;

    ci_init (if_num, (ci_fn)1, (ci_efn)1);  // init. the IFC

/*
    //----------------------------------------------------------------------
    // test BufxToBuf() and BufToBufx()
    // note: test of a local function, so it is commented out after the test

    char data[6];

    for (int i=0; i<6; i++) data[i] = i + 65;  // 'A' .. 'F'

    msg = BufxToBuf (NULL);  // return NULL
    msg = (char*) BufToBufx (NULL);  // return NULL

    msg = BufxToBuf ((bufx*)data);  // return pointer to 'E'
    msg = (char*) BufToBufx (msg);  // return pointer to 'A'

    // tested on 15.11.97, 15:50: all tests okay


    //----------------------------------------------------------------------
    // test ExtractMessage()
    //   correct behaviour of global function SetPosition() is assumed
    // note: test of a local function, so it is commented out after the test

    bufx *buf;
    STimeData nodePTime;
    SReplyData replyData;
    INT16 msgsize;

    size = 128;  // allocate enough memory to fit in all data

    ci_get_tbuf (size, &buf, if_num, 0);

    STimeData defData;
    defData.area = 3;
    defData.ms = 1;
    defData.ts = 25;
    defData.acc = 7;

    replyData.mode = 4;
    replyData.nodeID = 2;
    replyData.msReceive = 6;
    replyData.tsReceive = 10;
    replyData.thetaMinus = 0.4;
    replyData.thetaPlus = 0.3;

    memcpy (buf, &defData, sizeof (defData));
    memcpy ((char*)buf+sizeof (defData), &replyData, sizeof (replyData));
    // buf = {3, 1, 25, 7, 4, 2, 6, 10, 0.4, 0.3} (int 4 bytes, double 8 bytes)

    ExtractMessage (buf, sizeof (defData)+sizeof (replyData), 2, if_num,
        &nodePTime, &msg, &msgsize);
    // nodePTime: area=3, ms=1, ts=25, acc=7
    // msg = {4, 2, 6, 10, 0.4, 0.3}, msgsize=32

    delete[] msg;

    // tested on 15.11.97, 16:40: all tests okay so far


    SMsgPosition pos;  // make a different size/position entry

    pos.length[SMsgPosition::Area] = 3;
    pos.length[SMsgPosition::Macrostamp] = 0;
    pos.length[SMsgPosition::Timestamp] = 1;
    pos.length[SMsgPosition::Accuracy] = 2;
    pos.position[SMsgPosition::Area] = 9;
    pos.position[SMsgPosition::Macrostamp] = 8;
    pos.position[SMsgPosition::Timestamp] = 33;
    pos.position[SMsgPosition::Accuracy] = 3;
    SetPosition (pos, 5, if_num);  // store different data for node 5

    data[0]=1; data[1]=2;  // acc = 0x0201
    data[2]=1; data[3]=2; data[4]=3;  // area = 0x030201
    data[5]=27;  // ts = 27

    memcpy ((char*)buf+3, data, 2);  // copy acc
    memcpy ((char*)buf+9, data+2, 3);  // copy area
    memcpy ((char*)buf+33, data+5, 1);  // copy ts

    memcpy ((char*)buf, &replyData, 3);  // 0..2
    memcpy ((char*)buf+5, (char*)&replyData + 3, 4);  // 5..8
    memcpy ((char*)buf+12, (char*)&replyData + 7, 21);  // 12..32
    memcpy ((char*)buf+34, (char*)&replyData + 28, 4);  // 34..37

    ExtractMessage (buf, 38, 5, if_num, &nodePTime, &msg, &msgsize);
    // nodePTime: area=0x030201, ms=0, ts=27, acc=0x02000100
    // msg = {4, 2, 6, 10, 0.4, 0.3}, msgsize=32

    delete[] msg;
    ci_ret_tbuf (buf);

    // tested on 15.11.97, 17:00: all tests okay


    //----------------------------------------------------------------------
    // test the CI_Get...() and CI_Ret...() functions
    // note: test of a local function, so it is commented out after the test

    result = CI_GetEnvelope (mode, &tenv, if_num+1, 0);  // return -9
    result = CI_GetBuffer (size, &buf, if_num+1, 0);  // -9

    result = CI_GetEnvelope (mode, &tenv, if_num, 0);  // return 0
    result = CI_RetEnvelope (tenv);  // return 0

    result = CI_GetBuffer (size, &buf, if_num, 0);  // return 0
    result = CI_RetBuffer (buf);  // return 0

    result = CI_GetEnvelope (mode, &tenv, if_num, 0);  // return 0
    result = CI_GetBuffer (size, &buf, if_num, 0);  // return 0

    result = CI_Send (mode, tenv, buf, size, if_num+1, 0);// return -9
    result = CI_Send (mode, tenv, buf, size, if_num, 0);  // return 0

    // tested on 07.01.98, 13:50: all tests okay
*/


    //----------------------------------------------------------------------
    // test the constructor, Init() and Suspend() are tested as well

    cout << endl << "*** test of ctor ***" << endl;

    lance = new CLance (ctrl, 3);
    // suspended = True, clock=clockSync = NULL, area = 3, msgID = 0,
    // pSendScramble=pRecvScramble = 0, pSendDuplicate=pRecvDuplicate = 0,
    // msg_ = NULL, distDataSend=distDataRecv = Uniform,0,0,0,0,0
    // output: SetISR() called with ...,

    // tested on 04.02.99, 12:40: all tests okay


    //----------------------------------------------------------------------
    // test SetClockSync()

    CClockSync *cs = new CClockSync (ctrl, 0);

    lance->SetClockSync (cs);
    // clockSync = cs

    // tested on 18.11.96, 22:40: all tests okay


    //----------------------------------------------------------------------
    // test SetClock()

    cout << endl << "*** test of SetClock() ***" << endl;

    CClockInterface *clock = new CClockInterface (ctrl, 25e6, 0);

    lance->SetClock (clock);
    // clock_ = clock
    // output: EnableInterrupt() called with s=62

    lance->SetClock (clock);
    // clock_ = clock
    // output: DisableInterrupt() called with s=62
    //         DisableDutyTimer() called with s=62
    //         EnableInterrupt() called with s=62

    // tested on 13.01.98, 19:40: all tests okay


    //----------------------------------------------------------------------
    // test Insert()

    SSendDelayMcb *send = new SSendDelayMcb;
    SRecvDelayMcb *recv = new SRecvDelayMcb;

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer

    // store outgoing message
    send->send = True;
    send->deliveryTime = 7;
    send->mode = mode;
    send->tenv = tenv;
    send->msg = msg;
    send->msgsize = size;
    send->if_num = if_num;
    send->area = area;

    lance->Insert (send);
    // msg_ = send, listOfDelayedMessages is empty
    // output: InterruptAt (s=62, t=7) called

    // store incoming message
    recv->send = False;
    recv->deliveryTime = 5;
    recv->renv = renv;
    recv->buf = buf;
    recv->size = size;
    recv->if_num = if_num;

    lance->Insert (recv);
    // msg_ = recv, listOfDelayedMessages has one entry
    // output: InterruptAt (s=62, t=5) called

    recv = new SRecvDelayMcb;
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);

    recv->send = False;
    recv->deliveryTime = 6;
    recv->renv = renv;
    recv->buf = buf;
    recv->size = size;
    recv->if_num = if_num;

    lance->Insert (recv);
    // msg_ is not changed, listOfDelayedMessages has two entries

    // tested on 13.01.98, 19:30: all tests okay


    //----------------------------------------------------------------------
    // test DeleteDelayedMessages()

    lance->DeleteDelayedMessages();
    // msg_ =  NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    lance->DeleteDelayedMessages();
    // nothing should happen

    // tested on 28.07.98, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    SSingleParameter param;
    EBool res;

    param.type = CrashStatus;
    res = lance->GetParameter (param);  // return True
    // value = False (0)

    lance->Crash();
    res = lance->GetParameter (param);  // return True
    // value = True (1)

    lance->Recover();

    param.type = PCrash;
    param.value = 0.1;
    res = lance->SetParameter (param);  // return True
    // probCrash = 0.1

    param.type = PRecover;
    param.value = 0.2;
    res = lance->SetParameter (param);  // return True
    // probRecover = 0.2

    param.type = PCrash;
    res = lance->GetParameter (param);  // return True
    // value = 0.1

    param.type = PRecover;
    res = lance->GetParameter (param);  // return True
    // value = 0.2

    // tested on 11.01.98, 16:20: all tests okay so far


    SApplDistribution dist;
    dist.module = Lance;

    dist.type = DistributionSend;
    dist.data.dist = Exponential;
    dist.data.low = 1.3;
    dist.data.high = 2.1;
    dist.data.mean = 1.3;
    dist.data.freeparam = 2.2;
    dist.data.constant = 3.6;
    res = lance->SetParameter (dist);  // return True
    // distDataSend = ...

    dist.type = DistributionRecv;
    dist.data.dist = Weibull;
    dist.data.low = 2.9;
    dist.data.high = 3.1;
    dist.data.mean = 3.01;
    dist.data.freeparam = 4.6;
    dist.data.constant = 9.7;
    res = lance->SetParameter (dist);  // return True
    // distRecvSend = ...

    dist.type = DistributionSend;
    res = lance->GetParameter (dist);  // return True
    // dist.data = Exponential, 1.3, 2.1, 1.3, 2.2, 3.6

    dist.type = DistributionRecv;
    res = lance->GetParameter (dist);  // return True
    // dist.data = Weibull, 2.9, 3.1, 3.01, 4.6, 9.7

    // tested on 01.08.98, 10:30: all tests okay


    param.type = PScrambleSend;
    param.value = 0.7;
    res = lance->SetParameter (param);  // return True
    // pSendScramble = 0.7
    param.value = -0.1;
    res = lance->SetParameter (param);  // return False
    // pSendScramble unchanged
    param.value = 1.1;
    res = lance->SetParameter (param);  // return False
    // pSendScramble unchanged

    res = lance->GetParameter (param);  // return True
    // param.value = 0.7;

    param.type = PScrambleRecv;
    param.value = 0.9;
    res = lance->SetParameter (param);  // return True
    // pRecvScramble = 0.9
    param.value = -0.1;
    res = lance->SetParameter (param);  // return False
    // pRecvScramble unchanged
    param.value = 1.1;
    res = lance->SetParameter (param);  // return False
    // pRecvScramble unchanged

    res = lance->GetParameter (param);  // return True
    // param.value = 0.9;

    // tested on 04.02.99, 12:40: all tests okay so far


    param.type = PDuplicateSend;
    param.value = 0.8;
    res = lance->SetParameter (param);  // return True
    // pSendDuplicate = 0.8
    param.value = -0.1;
    res = lance->SetParameter (param);  // return False
    // pSendDuplicate unchanged
    param.value = 1.1;
    res = lance->SetParameter (param);  // return False
    // pSendDuplicate unchanged

    res = lance->GetParameter (param);  // return True
    // param.value = 0.8;

    param.type = PDuplicateRecv;
    param.value = 0.4;
    res = lance->SetParameter (param);  // return True
    // pRecvDuplicate = 0.4
    param.value = -0.1;
    res = lance->SetParameter (param);  // return False
    // pRecvDuplicate unchanged
    param.value = 1.1;
    res = lance->SetParameter (param);  // return False
    // pRecvDuplicate unchanged

    res = lance->GetParameter (param);  // return True
    // param.value = 0.4;

    // tested on 04.02.99, 12:40: all tests okay so far


    param.type = EventReport;
    res = lance->SetParameter (param);  // return False
    res = lance->GetParameter (param);  // return False

    // tested on 11.01.98, 16:20: all tests okay

    
    //----------------------------------------------------------------------
    // test ExecuteCommand()
    //   - calls Crash()
    //   - calls Reset(), which calls Dispose() and Init()
    //   so these fcts are tested as well

    cout << endl << "*** test of ExecuteCommand() ***" << endl;

    SCommand command;

    recv = new SRecvDelayMcb;
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);
    recv->send = False;
    recv->deliveryTime = 6;
    recv->renv = renv;
    recv->buf = buf;
    recv->size = size;
    recv->if_num = if_num;
    lance->Insert (recv);
    // msg_ = recv

    command.type = CrashModule;
    res = lance->ExecuteCommand (command);  // return True
    // crashed = True, msg_ = NULL, listOfDelayedMessages is empty

    command.type = RecoverModule;
    res = lance->ExecuteCommand (command);  // return True
    // crashed = False


    recv = new SRecvDelayMcb;
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);
    recv->send = False;
    recv->deliveryTime = 6;
    recv->renv = renv;
    recv->buf = buf;
    recv->size = size;
    recv->if_num = if_num;
    lance->Insert (recv);
    // msg_ = recv

    command.type = ResetModule;
    res = lance->ExecuteCommand (command);  // return True
    // maxSendDelay=maxRecvDelay = 0, distSendDelay=distRecvDelay = Uniform,
    // pSendScramble=pRecvScramble = 0, pSendDuplicate=pRecvDuplicate = 0,
    // probCrash=probRecover = 0, msg_ = NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    command.type = DeleteNetwork;
    res = lance->ExecuteCommand (command);  // return False

    // tested on 16.01.98, 12:30: all tests okay


    //----------------------------------------------------------------------
    // test GetBuffer(), GetEnvelope(), ReturnBuffer(), ReturnEnvelope(),
    // GetReceiveBuffer/Envelope(), RetReceiveBuffer/Envelope()

    msgID = 5;

    lance->GetEnvelope (&tenv);  // tenv is allocated
    lance->ReturnEnvelope (tenv);  // tenv is freed again

    size = 0;
    lance->GetBuffer (size, &msg, msgID);
    // msgID = 0

    size = 2;
    lance->GetBuffer (size, &msg, msgID);
    // msgID = 1, 'msg' to BufxToBuf(), copy msgID before
    // 'msg' [trace it!]
    lance->ReturnBuffer (msg);
    // should return BufToBufx() [trace it!]

    lance->GetBuffer (size, &msg, msgID);
    // msgID = 2
    lance->ReturnBuffer (msg);

    // tested on 07.01.98, 18:00: all tests okay so far


    lance->GetReceiveEnvelope (&renv);  // renv is allocated
    lance->RetReceiveEnvelope (renv);  // renv is freed again

    lance->GetReceiveBuffer (size, &buf);  // buf is allocated
    lance->RetReceiveBuffer (buf);  // buf is freed again

    // tested on 13.01.98, 13:00: all tests okay


    //----------------------------------------------------------------------
    // test ReportMessageEvent()

    lance->ReportMessageEvent (DiscardRecvMessage, 5, 3, 27);
    // should fill in type, sender, senderArea, msgID [trace it!]

    lance->ReportMessageEvent (DiscardSendMessage, 29);
    // should fill in type, msgID [trace it!]

    // tested on 11.01.98, 18:50: all tests okay


    //----------------------------------------------------------------------
    // test Scramble()

    msg = new char[15];
    for (i=0; i<15; i++) msg[i] = i+(int)'a';

    lance->Scramble (msg, 9);
    // first 9 chars should be scrambled (j-o okay)

    lance->Scramble (msg, 10);
    // first 10 chars should be scrambled (k-o okay)

    lance->Scramble (msg, 11);
    // first 11 chars should be scrambled (l-o okay)

    lance->Scramble (msg, 12);
    // first 12 chars should be scrambled (m-o okay)

    lance->Scramble (msg, 13);
    // first 12 chars should be scrambled (n-o okay)

    delete[] msg;

    // tested on 11.01.98, 18:30: all tests okay


    //----------------------------------------------------------------------
    // test ScrambleMessage()

    msg = new char[15];
    for (i=0; i<15; i++) msg[i] = i+(int)'a';
    for (i=0; i<ADDRCNT; i++) testaddr[i] = i+(int)'A';

    lance->ScrambleMessage (testaddr, msg, 9, 5);
    // nothing should happen (pSendScramble is 0)

    param.type = PScrambleSend;
    param.value = 1;
    lance->SetParameter (param);

    lance->ScrambleMessage (testaddr, msg, 9, 5);
    // first 9 chars should be scrambled (j-o okay), testaddr is scrambled,
    // time is scrambled

    // tested on 27.01.98, 19:00: all tests okay so far


    for (i=0; i<15; i++) msg[i] = i+(int)'a';
    time.ms = 1; time.ts = 2; time.acc = 3; time.area = 4;

    lance->ScrambleMessage (testaddr, &time, msg, 9, 12, 4, 5);
    // nothing should happen (pRecvScramble is 0)

    param.type = PScrambleRecv;
    param.value = 1;
    lance->SetParameter (param);

    time.ms = 1; time.ts = 2; time.acc = 3; time.area = 4;
    lance->ScrambleMessage (testaddr, &time, msg, 11, 12, 4, 5);
    // first 11 chars should be scrambled (l-o okay), testaddr is scrambled

    delete[] msg;

    // tested on 27.01.98, 20:20: all tests okay


    //----------------------------------------------------------------------
    // test SendNow()

    // note: ci_init() has not been called for the network, so CI won't
    //       accept the if_num+1

    lance->GetEnvelope (&tenv);  // request a dummy envelope
    lance->GetBuffer (size, &msg, msgID);  // request a dummy buffer
    for (i=0; i<size; i++) msg[i] = i;  // initialize it
    // msgID = 3, stands in the four bytes before msg


    lance->Crash ();
    // crashed = True

    result = lance->SendNow (mode, NULL, msg, 0, if_num+1, area);  // --> 0
    // suspended = False, extracted msgID = 3, tenv should not be returned
    // because it is NULL [trace it!], msg should be returned


    lance->Recover();
    // crashed = False

    lance->GetBuffer (size, &msg, msgID);  // request a dummy buffer
    for (i=0; i<size; i++) msg[i] = i;  // initialize it
    // msgID = 4

    result = lance->SendNow (mode, NULL, msg, 0, if_num+1, area);  // return -9
    // request of cibuf should fail [trace it!], tenv and msg are returned


    // note: ci_init() has been called for if_num, so that's no problem now

    lance->GetBuffer (size, &msg, msgID);  // request a dummy buffer
    result = lance->SendNow (mode, NULL, msg, size, if_num, area);  // 0
    // citenv and cibuf are requested, data is copied (da is BroadcastAddress),
    // citenv and cibuf are returned [trace it!], tenv and msg are returned

    tenv->da[0] = 16;
    lance->GetBuffer (size, &msg, msgID);  // request a dummy buffer
    result = lance->SendNow (mode, tenv, msg, size, if_num, area);  // 0
    // tenv->da is copied into citenv->da, citenv and cibuf are returned
    // [trace it!], tenv and msg should be returned


    delete network;
    // now a ci_send() will fail, even though buffer requests are possible

    lance->GetBuffer (size, &msg, msgID);  // request a dummy buffer
    for (i=0; i<size; i++) msg[i] = i;  // initialize it
    // msgID = 7, stands in the four bytes before msg

    result = lance->SendNow (retmode, NULL, msg, 0, if_num, area);  // -9
    // cibuf and citenv are requested, msg and tenv->da (Broadcast) are
    // scrambled, tenv and msg are freed [trace it!]

    // tested on 27.01.98, 19:50: all tests okay so far


    //----------------------------------------------------------------------
    // test Suspend()
    // note: it is assumed that after the test of SendNow(), suspended_
    //       is False

    lance->Suspend();
    // suspended = True

    // tested on 13.01.98, 12:40: all tests okay


    //----------------------------------------------------------------------
    // test AnnounceNow()

    size = 28;
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    for (i=0; i<size; i++) ((char*)buf)[i] = i;  // initialize it
    ((char*)buf)[17]=0; ((char*)buf)[18]=0; ((char*)buf)[19]=0;// msgID=16

    lance->SetClockSync (NULL);

    lance->AnnounceNow (renv, buf, size, if_num);
    // should return at once (no ClockSync)
    // should delete renv and buf [trace it!]


    lance->SetClockSync (cs);

    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    for (i=0; i<size; i++) ((char*)buf)[i] = i;  // initialize it
    ((char*)buf)[17]=0; ((char*)buf)[18]=0; ((char*)buf)[19]=0;// msgID=16

    lance->AnnounceNow (renv, buf, size, if_num);
    // should extract data and msgID, but return (suspended)
    // should delete message, renv and buf [trace it!]


    lance->GetBuffer (size, &msg, msgID);
    lance->SendNow (mode, NULL, msg, 0, if_num, area);
    // suspended = False
    lance->Crash();
    // crashed = True

    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    for (i=0; i<size; i++) ((char*)buf)[i] = i;  // initialize it
    ((char*)buf)[17]=0; ((char*)buf)[18]=0; ((char*)buf)[19]=0;// msgID=16

    lance->AnnounceNow (renv, buf, size, if_num);
    // should extract data and msgID, but return (crashed)
    // should delete message, renv and msg [trace it!]

    lance->Recover();
    // crashed = False


    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    for (i=0; i<size; i++) ((char*)buf)[i] = i;  // initialize it
    ((char*)buf)[17]=0; ((char*)buf)[18]=0; ((char*)buf)[19]=0;// msgID=16
    for (i=0; i<ADDRCNT; i++) (renv->sa)[i] = i;  // initialize address

    lance->AnnounceNow (renv, buf, size, if_num);
    // should extract data and msgID, scramble msg, sendTime and renv->sa,
    // then call clockSync->Announce() and at last delete message, renv and
    // buf [trace it!]

    // tested on 27.01.98, 20:20: all tests okay


    //----------------------------------------------------------------------
    // test DelayMessage()

    cout << endl << "*** test of DelayMessage() ***" << endl;

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);

    // set maximum delay to value > 0
    dist.type = DistributionSend;
    dist.data.dist = Uniform;
    dist.data.low = 0;
    dist.data.high = 12.5;
    dist.data.mean = 0;
    dist.data.freeparam = 0;
    dist.data.constant = 0;
    res = lance->SetParameter (dist);  // return True

    lance->DelayMessage (mode, tenv, msg, size, if_num, area);
    // msg_.deliveryTime = xx (0 < xx < 12.5)
    // output: InterruptAt (s=62, t=xx) called


    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);

    dist.type = DistributionSend;  // no delay
    dist.data.high = 0;
    res = lance->SetParameter (dist);  // return True

    lance->DelayMessage (mode, tenv, msg, size, if_num, area);
    // msg_.deliveryTime = 0
    // output: InterruptAt (s=62, t=0) called

    // tested on 21.09.98, 13:50: all tests okay so far


    lance->DeleteDelayedMessages();  // delete former messages

    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer

    dist.type = DistributionRecv;
    dist.data.high = 1;
    lance->SetParameter (dist);

    lance->DelayMessage (renv, buf, size, if_num);
    // msg_.deliveryTime = xx (0 < xx < 1)
    // output: InterruptAt (s=62, t=xx) called


    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer

    dist.type = DistributionRecv;  // no delay
    dist.data.high = 0;
    lance->SetParameter (dist);

    lance->DelayMessage (renv, buf, size, if_num);
    // msg_.deliveryTime = 0
    // output: InterruptAt (s=62, t=0) called


    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);

    lance->DelayMessage (mode, tenv, msg, size, if_num, area);
    // listOfDelayedMessages has two entries

    // tested on 21.09.98, 13:50: all tests okay


    //----------------------------------------------------------------------
    // test Send()

    cout << endl << "*** test of Send() ***" << endl;

    lance->DeleteDelayedMessages();  // delete former messages

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Suspend();

    dist.type = DistributionSend;
    dist.data.high = 1;
    lance->SetParameter (dist);

    lance->Send (mode, tenv, msg, size, if_num, area);
    // suspended = False, msg_.deliveryTime = xx (0 < xx < 1)
    // output: InterruptAt (s=62, t=xx) called


    dist.type = DistributionSend;  // no delay
    dist.data.high = 0;
    lance->SetParameter (dist);
    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Crash();

    lance->Send (mode, tenv, msg, size, if_num, area);
    // should delete the 'tenv' and 'msg', then return [trace it!]


    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Recover();

    lance->Send (mode, tenv, msg, size, if_num, area);
    // msg_.deliveryTime = 0
    // output: InterruptAt (s=62, t=0) called

    // tested on 21.09.98, 13:50: all tests okay so far


    lance->DeleteDelayedMessages();  // delete former messages

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);

    param.type = PDuplicateSend;
    param.value = 1;
    lance->SetParameter (param);

    lance->Send (mode, tenv, msg, size, if_num, area);
    // msg_.deliveryTime = 0, listOfDelayedMessages_ has 1 entry
    // output: InterruptAt (s=62, t=0) called

    param.type = PDuplicateSend;
    param.value = 0;
    lance->SetParameter (param);

    // tested on 04.02.99, 12:50: all tests okay


    //----------------------------------------------------------------------
    // test Announce()

    cout << endl << "*** test of Announce() ***" << endl;

    lance->DeleteDelayedMessages();  // delete former messages

    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    lance->Suspend();

    dist.type = DistributionRecv;
    dist.data.high = 1;
    lance->SetParameter (dist);

    lance->Announce (renv, buf, size, if_num);
    // should delete the 'renv' and 'buf', then return [trace it!]


    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    result = lance->SendNow (mode, tenv, msg, size, if_num, area);
    // suspended = False

    dist.type = DistributionRecv;
    dist.data.high = 0;
    lance->SetParameter (dist);
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    lance->Crash();

    lance->Announce (renv, buf, size, if_num);
    // should delete the 'renv' and 'buf', then return [trace it!]


    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer
    lance->Recover();

    lance->Announce (renv, buf, size, if_num);
    // msg_.deliveryTime = 0
    // output: InterruptAt (s=62, t=0) called

    // tested on 21.09.98, 13:50: all tests okay so far


    lance->DeleteDelayedMessages();  // delete former messages

    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);  // request a dummy buffer

    param.type = PDuplicateRecv;
    param.value = 1;
    lance->SetParameter (param);

    lance->Announce (renv, buf, size, if_num);
    // msg_.deliveryTime = 0, listOfDelayedMessages_ has 1 entry
    // output: InterruptAt (s=62, t=0) called

    param.type = PDuplicateRecv;
    param.value = 0;
    lance->SetParameter (param);


    // tested on 04.02.99, 12:50: all tests okay


    //----------------------------------------------------------------------
    // test isr()

    cout << endl << "*** test of isr() ***" << endl;

    lance->DeleteDelayedMessages();  // delete former messages

    // put some messages into the delay list
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);
    lance->Recover();
    lance->Announce (renv, buf, size, if_num);  // msg 1: receive, t=0
    // msg_.deliveryTime = 0
    // output: InterruptAt (s=62, t=0) called

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Send (mode, tenv, msg, size, if_num, area);  // msg 2: send, t=0

    dist.type = DistributionRecv;
    dist.data.high = 1;
    lance->SetParameter (dist);
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);
    lance->Announce (renv, buf, size, if_num);  // msg 3: receive, 0< t1 <1

    dist.type = DistributionSend;
    dist.data.high = 5;
    lance->SetParameter (dist);
    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Send (mode, tenv, msg, size, if_num, area);// msg 4: send, 0<t2<5

    lance->isr();
    // should extract first message, then call AnnounceNow()
    // should extract second message, then call SendNow()
    // output: InterruptAt (s=62, t=t1) called
    // should send event TimerEvent [trace it!]

    // tested on 21.09.98, 13:50: all tests okay so far


    g_currTime = 1;  // the isr() checks if the msg is due for delivery
    param.type = PCrash;
    param.value = 1;
    lance->SetParameter (param);
    // now the Lance will certainly crash in SendNow() and AnnounceNow()

    lance->isr();
    // should extract first message, then call AnnounceNow() (--> crash)
    // msg_ is NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    // tested on 16.01.98, 13:20: all tests okay so far


    g_currTime = 0;

    param.type = PCrash;
    param.value = 0;
    lance->SetParameter (param);  // disable crashing to delay some msges
    lance->Recover();  // recover the Lance

    dist.type = DistributionSend;
    dist.data.high = 1;
    lance->SetParameter (dist);
    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Send (mode, tenv, msg, size, if_num, area);// msg 1: send, 0<t1<1

    dist.type = DistributionRecv;
    dist.data.high = 50;
    lance->SetParameter (dist);
    lance->GetReceiveEnvelope (&renv);
    lance->GetReceiveBuffer (size, &buf);
    lance->Announce (renv, buf, size, if_num); // msg 2: receive, 0< t2 <50

    param.type = PCrash;
    param.value = 1;
    lance->SetParameter (param);  // enable crashing

    g_currTime = 1;

    lance->isr();
    // should extract first message, then call SendNow() (--> crash)
    // msg_ is NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    // tested on 21.09.98, 13:50: all tests okay so far


    param.type = PCrash;
    param.value = 0;
    lance->SetParameter (param);
    lance->Recover();  // recover the Lance

    g_currTime = 0;

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Send (mode, tenv, msg, size, if_num, area);// msg 1: send, 0<t1<1

    g_currTime = 1;

    lance->isr();
    // should extract first message, then call SendNow()
    // msg_ is NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    // tested on 26.01.98, 18:40: all tests okay so far


    g_currTime = 0;

    lance->GetBuffer (size, &msg, msgID);
    lance->GetEnvelope (&tenv);
    lance->Send (mode, tenv, msg, size, if_num, area);// msg 1: send, 0<t1<1

    g_currTime = 1;
    // DEBUGGER: set it to > t1-epsilon

    lance->isr();
    // should extract first message, then call SendNow()
    // msg_ is NULL, listOfDelayedMessages is empty
    // output: DisableDutyTimer (s=62) called

    // tested on 28.07.98, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test the destructor

    cout << endl << "*** test of dtor ***" << endl;

    delete lance;
    // output: DisableInterrupt() called with s=62
    //         DisableDutyTimer() called with s=62
    //         RemoveISR() called with ... (see data of ctor!)

    // tested on 13.01.98, 19:40: all tests okay


    delete ctrl;
    delete cs;
    delete clock;


    //----------------------------------------------------------------------

    return 0;
}



//--------------------------------------------------------------------------
// interrupt handling functions from interupt.hpp
//--------------------------------------------------------------------------

void SetISR (int vector, CNeedInterrupt *obj, CallFunc caller)
{
    cout << "SetISR() called with vector = " << vector;
    cout << ", @obj = " << (long)obj;
    cout << ", @callerfct = " << (long)caller << endl;
}


void RemoveISR (int vector, CNeedInterrupt *obj, CallFunc caller)
{
    cout << "RemoveISR() called with vector = " << vector;
    cout << ", @obj = " << (long)obj;
    cout << ", @callerfct = " << (long)caller << endl;
}


void InvokeISR (int vector)
{
    cout << "InvokeISR() called with vector = " << vector << endl;
}


void CallISR (CNeedInterrupt *obj)
{
    obj->isr();  // call the isr of the object
}


//--------------------------------------------------------------------------

CClockInterface::CClockInterface (CControllingObject *ctrl, double, int)
    : CApplControlledObject (ctrl) {}
CClockInterface::~CClockInterface () {}
void CClockInterface::DisableDutyTimer (EInterruptSource s)
  { cout << "CClockInterface::DisableDutyTimer() called with source ";
    cout << s << endl; }
CClock *CClockInterface::GetClock () { return NULL; }
EBool CClockInterface::SetParameter (const SParameter&) { return False; }
EBool CClockInterface::GetParameter (SParameter&) const { return False; }
EBool CClockInterface::ExecuteCommand (const SCommand&) { return False; }
EBool CClockInterface::SetAmortizationDuration (double) { return False; }
EBool CClockInterface::GpsConnected () { return False; }
void CClockInterface::AddGps (EGpsUnit) {}
void CClockInterface::RemoveGps (EGpsUnit) {}
double CClockInterface::GetRhoPlus() const {return 0;}
double CClockInterface::GetRhoMinus() const {return 0;}
double CClockInterface::GetGranularity() const {return 0;}
double CClockInterface::GetGranularityAcc() const {return 0;}
double CClockInterface::GetGranularityOsc() const {return 0;}
double CClockInterface::GetGranularityDT() const {return 0;}
INT32 CClockInterface::GetAccuracy() const {return 0;}
EBool CClockInterface::GetTimestamp (STimestamp& time)
  { SecondsToNtp (g_currTime, time); return False; }
void CClockInterface::SetTimestamp (const STimestamp&) {}
void CClockInterface::SetTimestamp (const SExtendedTimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const STimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const SExtendedTimestamp&) {}
void CClockInterface::EnableInterrupt (EInterruptSource source)
  { cout << "CClockInterface::EnableInterrupt() called with source ";
    cout << source << endl; }
void CClockInterface::DisableInterrupt (EInterruptSource source)
  { cout << "CClockInterface::DisableInterrupt() called with source ";
    cout << source << endl; }
EBool CClockInterface::InterruptPending (EInterruptSource) { return True; }
EBool CClockInterface::InterruptPending (EClockInterrupt) { return True;  }
void CClockInterface::ClearInterrupt (EInterruptSource) {}
void CClockInterface::ClearInterrupt (EClockInterrupt) {}
void CClockInterface::InterruptAt (EInterruptSource s, double t)
  { cout << "CClockInterface::InterruptAt() called with source ";
    cout << s << " and time " << t << endl; }
void CClockInterface::InterruptAfter (EInterruptSource s, double t)
  { cout << "CClockInterface::InterruptAfter() called with source ";
    cout << s << " and time " << t << endl; }
double CClockInterface::InterruptAtMult (EInterruptSource s, double t)
  { cout << "CClockInterface::InterruptAtMult() called with source ";
    cout << s << " and time " << t << endl;
    return 0; }
void CClockInterface::Init () {}
void CClockInterface::Dispose() {}
void CClockInterface::isr() {}
void CClockInterface::wobble_isr() {}
void CClockInterface::PreloadAmort (double, INT32, INT32) {}
void CClockInterface::Amortize (double, INT32, INT32) {}
int CClockInterface::ReportError (EReportType,char *,int,char *) { return 1; }
void CClockInterface::Crash () {}
void CClockInterface::Recover () {}
void CClockInterface::Reset () {}
void CClockInterface::ReportCrashChange () {}


#include "inlines.hpp"


