//
// lbastest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the abstract base list class in listbase.hpp
//
// created: 27.10.96
//
// last changed: 11.02.97
//


#include "..\listbase.hpp"


// this is a test object to check if the list handles its objects correctly
struct STest
{
    double val;

    STest (double v) { val = v; }
};



//**************************************************************************
// class CAccessList: this class is used to test the abstract base class;
//     its functions just cast the void pointer into a pointer to STest,
//     but have no other function
//**************************************************************************


struct SKey  // this structure contains the key(s) for sorting
{
    // all key elements are put here
    int key_;

    SKey (double key) {key_ = key;}
    // initialize the members
};


class CAccessList : public ACObjectList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CAccessList (INT32 aSize = ArrayAllocationSize) :
        ACObjectList (aSize) {};
    // allocate memory

    virtual ~CAccessList () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    STest* GetFirst () { return (STest*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    STest* GetLast () { return (STest*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    STest* GetNext () { return (STest*) ACObjectList::GetNext(); }
    // get the next object in the array or NULL if the end of the array
    // has been reached

    STest* GetPrev () { return (STest*) ACObjectList::GetPrev(); }
    // return the previous object in the array or NULL if the beginning
    // of the array has been reached

    STest* Find (SKey *key) { return (STest*) ACObjectList::Find (key); }
    // returns the object with this key; if the key does not exist,
    // NULL is returned


    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    STest* GetCurrentObject ()
        { return (STest*) ACObjectList::GetCurrentObject(); }
    // returns a pointer to the member that is currently being accessed or
    // NULL, if no element is being accessed

    SKey* GetCurrentObjectKey ()
        { return (SKey*) ACObjectList::GetCurrentObjectKey(); }


    //----------------------------------------------------------------------
    //--- remove objects
    //----------------------------------------------------------------------

    STest* Remove (SKey *key)
    {
        return (STest*) ACObjectList::Remove (key);
    }
    // remove an object from the array; if the key does not exist, NULL
    // is returned


    //----------------------------------------------------------------------
    //--- delete all member(s) of the array
    //----------------------------------------------------------------------

    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        STest* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll


protected:

    // this function is used to compare keys
    virtual int Compare (void* key1, void *key2) const
    {
        double k1, k2;
        k1 = ((SKey *)key1)->key_;
        k2 = ((SKey *)key2)->key_;

        if (k1 < k2) return -1;
        if (k1 > k2) return +1;
        return 0;
    }


    virtual void DeleteKey (void* key) { delete (SKey*) key; }
    // delete the key
};


 
//**************************************************************************
//**************************************************************************

int main()
{

    //----------------------------------------------------------------------
    // variables necessary for the test of objlist.hpp
    //----------------------------------------------------------------------

    CObjectElement *el1;

    STest *obj1;
    SKey *key1;
    STest *testobj;
    SKey *testkey;


    //----------------------------------------------------------------------
    // test the behaviour of the class CObjectElement
    //----------------------------------------------------------------------

    // test the constructor
    el1 = new CObjectElement;  // make a new array element
    // object=key = NULL

    obj1 = new STest (1.1);  // make a new object
    key1 = new SKey (1);  // make a new key

    // test SetObject
    el1->SetObject (obj1, key1);  // set the object with the key
    // object = obj1, key = key1
    
    testobj = (STest *)el1->GetObject();  // should return obj1
    testkey = (SKey *)el1->GetKey();  // should return key

    delete el1;  // delete el1, but not obj1
    delete obj1;
    delete key1;

    // tested on 28.10.96, 19:49: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of the class ACObjectList
    // (correct behaviour of class CObjectElement is assumed)
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // variables necessary for the test of objlist.hpp
    //----------------------------------------------------------------------

    CAccessList *list;
    STest *obj0, *obj2, *obj3, *obj4, *obj5;
    SKey  *key0, *key2, *key3, *key4, *key5;
    EBool test;

    // create six objects and their keys
    obj0 = new STest(0.0); key0 = new SKey (0);
    obj1 = new STest(1.1); key1 = new SKey (1);
    obj2 = new STest(2.2); key2 = new SKey (2);
    obj3 = new STest(3.3); key3 = new SKey (3);
    obj4 = new STest(4.4); key4 = new SKey (4);
    obj5 = new STest(5.5); key5 = new SKey (5);


    //----------------------------------------------------------------------
    // test the constructor and destructor

    // use default params
    list = new CAccessList;
    // _firstFree = 0, _current = 0, _array = *CObjectElement[_size],
    // _size=_allocSize=ArrayAllocationSize

    delete list;
    // this should call DeleteAll() and then the base constructor that
    // is responsible for deleting _array; trace into the functions to
    // verify that!


    // use another alloc size
    list = new CAccessList(2);
    // _firstFree = 0, _current = 0, _size=_allocSize=2,
    // _array = *CObjectElement[_size]

    // tested on 28.10.96, 20:24: all tests okay


    //----------------------------------------------------------------------
    // test the status functions in case of an empty list

    // test Empty on empty list
    test = list->Empty();  // should return True
    // test BeginOfList on empty list
    test = list->BeginOfList();  // should return False
    // test EndOfList on empty list
    test = list->EndOfList();  // should return False

    // tested on 28.10.96, 20:24: all tests okay


    //----------------------------------------------------------------------
    // test the access functions in case of an empty list

    // test GetFirst on empty list
    testobj = list->GetFirst();  // should return NULL
    // test GetLast on empty list
    testobj = list->GetLast();  // should return NULL
    // test GetNext on empty list
    testobj = list->GetNext();  // should return NULL
    // test GetPrev on empty list
    testobj = list->GetPrev();  // should return NULL

    // test Find on empty list
    testobj = list->Find(key1);  // should return NULL

    // test GetCurrentObject on empty list
    testobj = list->GetCurrentObject();  // should return NULL
    // test GetCurrentObjectKey on empty list
    testkey = list->GetCurrentObjectKey();  // should return NULL

    // tested on 28.10.96, 20:28: all tests okay


    //----------------------------------------------------------------------
    // test Insert() - Find() and InsertAtCurrent() are called
    
    // test Insert with NULL as the object
    test = list->Insert (NULL, key1);  // should return False

    // test Insert with NULL as the key
    test = list->Insert (obj1, NULL);  // should return False
    
    // test Insert with new object ID
    test = list->Insert (obj3, key3);  // should return True
    // current = 0, firstFree = 1

    // test Insert with duplicate
    test = list->Insert (obj3, key3);  // should return False
    // current = 0, firstFree = 1

    // test Insert with new object ID
    test = list->Insert (obj1, key1);  // should return True
    // current = 0, firstFree = 2, size=4, array points to a new location
    // InsertAtCurrent, AllocateMemory, UpdateArray tested completely

    // tested on 02.11.96, 18:36: all tests okay


    //----------------------------------------------------------------------
    // test the status functions in case of a non-empty list

    // test Empty
    test = list->Empty();  // should return False
    // Empty tested completely
    
    // test BeginOfList
    test = list->BeginOfList();  // should return True
    // test EndOfList
    test = list->EndOfList();  // should return False

    // tested on 28.10.96, 20:40: all tests okay


    //----------------------------------------------------------------------
    // test the access functions in case of a non-empty list

    // test GetFirst
    testobj = list->GetFirst();  // should return obj1
    // current=0
    // GetFirst tested completely
    
    // test GetLast
    testobj = list->GetLast();  // should return obj3
    // current=1
    // GetLast tested completely

    // test BeginOfList
    test = list->BeginOfList();  // should return False
    // BeginOfList tested completely

    // test EndOfList
    test = list->EndOfList();  // should return True
    // EndOfList tested completely

    // test GetNext
    testobj = list->GetNext();  // should return NULL
    // test GetPrev
    testobj = list->GetPrev();  // should return obj1
    // current=0
    testobj = list->GetPrev();  // should return NULL
    // GetPrev tested completely

    // test GetNext
    testobj = list->GetNext();  // should return obj3
    // current=1
    // GetNext tested completely

    // test GetCurrentObject
    testobj = list->GetCurrentObject();  // should return obj3
    // GetCurrentObject, CurrentIsValid, GetCurrent tested completely
    
    // test GetCurrentObjectKey
    testkey = list->GetCurrentObjectKey();  // should return key3
    // GetCurrentObjectKey tested completely

    // tested on 11.02.97, 19:10: all tests okay


    //----------------------------------------------------------------------
    // test Find()

    // list: (1, 3); current=1, firstFree=2, size=4


    testobj = list->Find (NULL);  // should return NULL

    testobj = list->Find (key1);  // should return obj1
    // current=0

    testobj = list->Find (key5);  // should return NULL
    // current=2

    // tested on 02.11.96, 18:41: all tests okay so far


    //----------------------------------------------------------------------
    // test Insert() - continued
    
    // list: (1, 3); current=2, firstFree=2, size=4
    test = list->Insert (obj5, key5);  // should return True
    // current=2, firstFree=3

    list->Insert (obj2, key2);
    // list: (1,2,3,5); current=1, firstFree=4, size=6
    
    list->GetFirst();
    // current=0
    
    test = list->Insert (obj4, key4);  // should return True
    // current=3, firstFree=5

    // tested on 02.11.96, 18:41: all tests okay so far


    //----------------------------------------------------------------------
    // test Find() - continued

    // list: (1,2,3,4,5); current=3, firstFree=5, size=6

    list->GetLast();
    // current=4

    testobj = list->Find (key1);  // should return obj1
    // current=0

    list->GetLast();
    list->GetPrev();
    // current=3

    testobj = list->Find (key0);  // should return NULL
    // current=0

    // Find tested completely --> Insert tested completely

    // tested on 02.11.96, 18:42: all tests okay


    //----------------------------------------------------------------------
    // test Remove() - Find() and RemoveAtCurrent() are called
    // correct behaviour of Find() is assumed

    // list: (1,2,3,4,5); current=0, firstFree=5, size=6

    testobj = list->Remove (key0);  // should return NULL
    // current = 0

    testobj = list->Remove (key5);  // should return obj5
    // list: (1,2,3,4); current = 3, firstFree=4

    testobj = list->Remove (key3);  // should return obj3
    // list: (1,2,4); current = 2, firstFree=3, size=6


    testobj = list->Remove (key4);
    testobj = list->Remove (key2);
    // list: (1); current=0, firstFree=1, size=4

    testobj = list->Remove (key1);
    // current=firstFree = 0, size = 2

    // Remove(), RemoveAtCurrent() and FreeMemory() tested completely

    // tested on 02.11.96, 19:00: all tests okay


    delete list;

    
    //----------------------------------------------------------------------

    
    return 0;
}
    


