//
// natest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the network accountant class in netacc.hpp
//
// note: the Set/GetDelta() etc. functions were made public for this test
//       (instead of protected), the Init/Dispose() functions too
//
// created: 05.10.96
//
// last changed: 28.09.98
//


#include "..\netacc.hpp"
#include "..\structs.hpp"


const int TestTries = 100;

int main()
{
    //----------------------------------------------------------------------
    // variables necessary for the test of netacc.hpp
    //----------------------------------------------------------------------

    CNetworkAccountant *netAcc;
    SSingleNetworkParameter singleParam;
    SNetworkData netParam;

    double delayInfo;
    EBool result;

    int i;
    double min, max, sum;  // for the test of GetArbitraryDelay()


    //----------------------------------------------------------------------
    // test the behaviour of the class CNetworkAccountant
    //    correct behaviour of the classes
    //        CUserAccountant, CTransmissionData, CRandom
    //    and the global function
    //        Random()
    //    is assumed
    //----------------------------------------------------------------------


    //----------------------------------------------------------------------
    // test the constructor - calls Init()

    netAcc = new CNetworkAccountant (2);
    // data = NULL, networkID=2, listOfDelays = "empty list",
    // initCalled = True

    // tested on 31.07.98, 09:30: all tests okay


    //----------------------------------------------------------------------
    // test Init(), Dispose()

    netAcc->Init();
    // should return at once (trace into the code to verify that!)

    netAcc->Dispose();
    // initCalled = False
    netAcc->Dispose();
    // should return at once (trace into the code to verify that!)

    netAcc->Init();
    // initCalled = True

    // tested on 04.01.97, 16:30: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetProbability()

    netParam.type = PCrash;
    netParam.sender = 1; netParam.receiver = 2; netParam.value = 0.1;
    result = netAcc->SetProbability (netParam);  // return False

    netParam.type = PEarly;
    netParam.value = -1;
    result = netAcc->SetProbability (netParam);  // return False
    netParam.value = 1.1;
    result = netAcc->SetProbability (netParam);  // return False
    netParam.value = 0.2;
    result = netAcc->SetProbability (netParam);  // return True

    netParam.type = PLate;
    netParam.value = 0.9;
    result = netAcc->SetProbability (netParam);  // return False
    netParam.value = -0.1;
    result = netAcc->SetProbability (netParam);  // return False
    netParam.value = 0.5;
    result = netAcc->SetProbability (netParam);  // return True

    netParam.type = PEarly;
    netParam.value = 0.6;
    result = netAcc->SetProbability (netParam);  // return False

    // tested on 25.08.97, 18:10: all tests okay so far

    delayInfo = netAcc->GetProbability (netParam);  // return 0.2
    netParam.type = PLate;
    delayInfo = netAcc->GetProbability (netParam);  // return 0.5
    netParam.receiver = 3;
    delayInfo = netAcc->GetProbability (netParam);  // return -1 (NoAcc)

    // tested on 25.08.97, 18:10: all tests okay


    //----------------------------------------------------------------------
    // test SetParameter() and GetParameter()

    singleParam.type = ClockDrift;
    result = netAcc->GetParameter (singleParam);  // return False
    result = netAcc->SetParameter (singleParam);  // return False

    // tested on 15.11.96, 19:18: all tests okay so far


    netParam.type = PEarly;
    netParam.sender = 1; netParam.receiver = 2; netParam.value = 0;
    result = netAcc->GetParameter (netParam);  // return True
    // value = 0.2

    netParam.type = PLate;
    result = netAcc->GetParameter (netParam);  // return True
    // value = 0.5

    netParam.type = PEarly;
    netParam.value = 0.3;
    result = netAcc->SetParameter (netParam);  // return True

    netParam.type = PLate;
    netParam.value = 0.4;
    result = netAcc->SetParameter (netParam);  // return True

    netParam.type = PEarly;
    result = netAcc->GetParameter (netParam);  // return True
    // value = 0.3

    netParam.type = PLate;
    result = netAcc->GetParameter (netParam);  // return True
    // value = 0.4

    // tested on 25.08.97, 18:20: all tests okay so far


    SNetDistributionP2P dist;
    dist.module = Network;
    dist.networkID = 1;  // ignored anyway?

    dist.type = DistributionTimely;
    dist.sender = 2;
    dist.receiver = 3;
    dist.data.dist = Gamma;
    dist.data.low = 3;
    dist.data.high = 4;
    dist.data.constant = 2;
    dist.data.mean = 3.5;
    dist.data.freeparam = 1;
    result = netAcc->SetParameter (dist);  // return True
    // 2+(3,4) with m=3.5 --> 5,6 with m=5.5
    // --> timely: dist,low,high,mean,freeparam,const = Gamma,3,4,3.5,1,2
    // --> fast: low,high,mean = 6,6,6
    // --> slow: high = 5

    dist.type = DistributionEarly;
    dist.data.dist = Exponential;
    dist.data.low = 2.1;  // greater than lower bound 
    dist.data.high = 2.2;
    dist.data.constant = 3;
    dist.data.mean = 2;
    dist.data.freeparam = 2;
    result = netAcc->SetParameter (dist);  // return False
    dist.data.low = 1;
    dist.data.high = 2.3;
    dist.data.mean = 2.1;
    result = netAcc->SetParameter (dist);  // return True
    // 3+(1,2.3) with m=2.1 --> 4,5 with m=4.5
    // (upper bound from timely, mean is set into the middle)
    // --> slow: dist,low,high,mean,freeparam,const = Exp,1,2,1.5,2,3
    dist.data.high = 1.2;
    dist.data.mean = 1;
    result = netAcc->SetParameter (dist);  // return True
    // 3+(1,1.2) with m=1 --> 4,5 with m=4 (upper bound from timely)
    // --> slow: dist,low,high,mean,freeparam,const = Exp,1,2,1,2,3

    dist.type = DistributionLate;
    dist.data.dist = Weibull;
    dist.data.low = 2.3;
    dist.data.high = 2.6;
    dist.data.constant = 3.1;
    dist.data.mean = 2.4;
    dist.data.freeparam = 2.5;
    result = netAcc->SetParameter (dist);  // return False
    dist.data.high = 4;  // 3.1+(2.3, 4)
    dist.data.mean = 2.4;
    result = netAcc->SetParameter (dist);  // return True
    // 3.1+(2.3,4) with m=2.4 --> 6,7.1 with m=6.55
    // (lower bound from timely, mean is set into the middle)
    // --> fast: dist,low,high,mean,freeparam,const = Weibull,2.9,4,3.45,2.5,3.1
    dist.data.mean = 3;
    result = netAcc->SetParameter (dist);  // return True
    // 3.1+(2.3,4) with m=3 --> 6,7.1 with m=6.1 (lower bound: timely)
    // --> fast: dist,low,high,mean,freeparam,const = Weibull,2.9,4,3,2.5,3.1

    // now set DistributionTimely again in a way that invalidates
    // the slow and fast intervals (that is, timely.low is below slow.low,
    // timely.high is above fast.high)
    dist.type = DistributionTimely;
    dist.data.dist = Gamma;
    dist.data.low = 3;
    dist.data.high = 8;
    dist.data.constant = 0;
    dist.data.mean = 3.5;
    dist.data.freeparam = 1;
    result = netAcc->SetParameter (dist);  // return True
    // 0+(3,8) with m=3.5 --> 3,8 with m=3.5
    // --> timely: dist,low,high,mean,freeparam,const = Gamma,3,8,3.5,1,0
    // --> fast: low,high,mean = 4.9,4.9,4.9 (constant = 3.1)
    // --> slow: low,high,mean = 0,0,0 (constant = 3)

    dist.data.low = 1;
    dist.data.high = 2;
    dist.data.mean = 1;
    dist.data.constant = 1;
    result = netAcc->SetParameter (dist);  // return True
    // --> timely: dist,low,high,mean,freeparam,const = Gamma,1,2,1,1,1
    // --> fast: low,high,mean = 3,3,3 (constant = 0!)
    // --> slow: low,high,mean = 2,2,2 (constant = 0!)

    // tested on 21.09.98, 16:10: all tests okay so far

    dist.type = DistributionEarly;
    dist.data.dist = Exponential;
    dist.data.low = 1;  // greater than lower bound 
    dist.data.high = 2;
    dist.data.constant = 0;
    dist.data.mean = 1.5;
    result = netAcc->SetParameter (dist);  // return True
    // 0+(1,2) with m=1.5 --> 1,2 with m=1.5
    // --> slow: dist,low,high,mean,freeparam,const = Exp,1,2,1.5,1,0

    dist.type = DistributionLate;
    dist.data.dist = Weibull;
    dist.data.low = 3;
    dist.data.high = 4;
    dist.data.constant = 0;
    dist.data.mean = 3.5;
    result = netAcc->SetParameter (dist);  // return True
    // 0+(3,4) with m=3.5 --> 3,4 with m=3.5
    // --> fast: dist,low,high,mean,freeparam,const = Weibull,3,4,3.5,1,0

    // now set DistributionTimely again in a way that invalidates the mean
    dist.type = DistributionTimely;
    dist.data.dist = Gamma;
    dist.data.low = 1.2;
    dist.data.high = 3.8;
    dist.data.constant = 0;
    dist.data.mean = 2;
    result = netAcc->SetParameter (dist);  // return True
    // 0+(1.2,3.7) with m=2 --> 1.2,3.8 with m=2
    // --> timely: dist,low,high,mean,freeparam,const = Gamma,1.2,3.8,2,1,0
    // --> fast: low = 3.8, high = 4, mean = 3.9 (constant = 0)
    // --> slow: low = 1, high = 1.2, mean = 1.1 (constant = 0)

    // SetDataParameter() tested completely

    // tested on 28.09.98, 14:50: all tests okay so far


    dist.type = DistributionEarly;
    result = netAcc->GetParameter (dist);  // return True
    // dist,low,high,mean,freeparam,const = Exp,2,2,2,2,0

    dist.type = DistributionTimely;
    result = netAcc->GetParameter (dist);  // return True
    // dist,low,high,mean,freeparam,const = Gamma,1,2,1,1,1

    dist.type = DistributionLate;
    result = netAcc->GetParameter (dist);  // return True
    // dist,low,high,mean,freeparam,const = Weibull,3,3,3,2.5,0

    dist.sender++;  // use another sender (no data stored)
    result = netAcc->GetParameter (dist);  // return False

    // GetDataParameter() tested completely

    // tested on 21.09.98, 16:10: all tests okay


    //----------------------------------------------------------------------
    // test the delay computation method GetArbitraryDelay()

    EArrival arrival = InTime;
    double myDmax = 12;
    double myDmin = 0.2;  // used in the code below to set the Dmax and Dmin

    // make a node to play around with
    dist.type = DistributionTimely;  // set Delta, EpsPlus, EpsMinus
    dist.sender = 2;
    dist.receiver = 3;
    dist.data.dist = Uniform;
    dist.data.low = 0.7;
    dist.data.high = 9.3;
    dist.data.constant = 0;
    dist.data.mean = 4.1;
    dist.data.freeparam = 0;
    result = netAcc->SetParameter (dist);  // return True

    dist.type = DistributionEarly;  // set Dmin = 0.2
    dist.data.low = myDmin;
    dist.data.high = 0.7;
    dist.data.mean = 0.5;
    result = netAcc->SetParameter (dist);  // return True

    dist.type = DistributionLate;  // set Dmax = 12
    dist.data.low = 9.3;
    dist.data.high = myDmax;
    dist.data.mean = 10;
    result = netAcc->SetParameter (dist);  // return True

    // so "early" is in (0.2, 0.7), "timely" in (0.7, 9.3) and "late" in
    // (9.3, 12)


    delayInfo = netAcc->GetArbitraryDelay (2, 4, arrival);  // NoAccount
    delayInfo = netAcc->GetArbitraryDelay (1, 1, arrival);  // DefaultOwnDelay (=0)

    // tested on 21.09.98, 16:30: all tests okay so far


    min = myDmax+1; max=0; sum=0;
    arrival = InTime;
    for (i=0; i<TestTries; i++)
    {
       delayInfo = netAcc->GetArbitraryDelay (2, 3, arrival);
       if (delayInfo < min) min=delayInfo;
       if (delayInfo > max) max=delayInfo;
       sum = sum+delayInfo;
    }
    sum = sum/i;
    // min should be approx. 0.7, max approx. 9.3, sum approx. 5


    min = myDmax+1; max=0; sum=0;
    arrival = Early;
    for (i=0; i<TestTries; i++)
    {
       delayInfo = netAcc->GetArbitraryDelay (2, 3, arrival);
       if (delayInfo < min) min=delayInfo;
       if (delayInfo > max) max=delayInfo;
       sum = sum+delayInfo;
    }
    sum = sum/i;
    // min should be approx. 0.2, max approx. 0.7, sum approx. 0.45


    min = myDmax+1; max=0; sum=0;
    arrival = Late;
    for (i=0; i<TestTries; i++)
    {
       delayInfo = netAcc->GetArbitraryDelay (2, 3, arrival);
       if (delayInfo < min) min=delayInfo;
       if (delayInfo > max) max=delayInfo;
       sum = sum+delayInfo;
    }
    sum = sum/i;
    // min should be approx. 9.3, max approx. 12, sum approx. 10.65

    // tested on 21.09.98, 16:30: all tests okay so far


    // force an interval that is a point
    dist.type = DistributionTimely;
    dist.data.low = 0.7;
    dist.data.high = 13;
    result = netAcc->SetParameter (dist);  // return True
    // --> fast: low=high=mean = 13

    min = 13+1; max=0; sum=0;
    arrival = Late;
    for (i=0; i<TestTries; i++)
    {
       delayInfo = netAcc->GetArbitraryDelay (2, 3, arrival);
       if (delayInfo < min) min=delayInfo;
       if (delayInfo > max) max=delayInfo;
       sum = sum+delayInfo;
    }
    sum = sum/i;
    // min=max=sum = 13

    // tested on 21.09.98, 17:10: all tests okay so far


    netParam.type = PEarly;
    netParam.sender = 2; netParam.receiver = 3; netParam.value = 0.2;
    netAcc->SetProbability (netParam);
    netParam.type = PLate;
    netParam.value = 0.5;
    netAcc->SetProbability (netParam);

    min=0; max=0;
    for (i=0; i<TestTries; i++)
    {
       arrival = EArrivalCount;  // let function choose
       netAcc->GetArbitraryDelay (2, 3, arrival);
       if (arrival==Early) min++;
       if (arrival==Late) max++;
    }
    min = min/i;
    max = max/i;
    // min=0.2, max=0.5 (approx.)

    // tested on 16.08.97, 17:30: all tests okay


    //----------------------------------------------------------------------
    // test IsConnected()

    result = netAcc->IsConnected (2,1);  // should return False

    result = netAcc->IsConnected (1,2);  // should return True
    // key1=1, key2=2, data = data(1,2)
    // IsConnected() tested completely

    // tested on 15.11.96, 21:26: all tests okay


    //----------------------------------------------------------------------
    // test DeleteNode()

    result = netAcc->DeleteNode (5);  // should return False

    // create some new nodes
    netParam.type = Delta;
    netParam.sender = 1; netParam.receiver = 4; netParam.value = 1.1;
    netAcc->SetParameter (netParam);
    netParam.receiver = 3;
    netAcc->SetParameter (netParam);
    netParam.sender = 2; netParam.receiver = 4;
    netAcc->SetParameter (netParam);
    netParam.sender = 4; netParam.receiver = 2;
    netAcc->SetParameter (netParam);
    // key1=4, key2=2, data(4,2) <- 1.1 (delta)
    // listOfDelays = (1,2),(1,3),(1,4); (2,3),(2,4); (4,2)

    result = netAcc->DeleteNode (4);  // should return True
    // key1=key2 = (-1), data = NULL
    // listOfDelays = (1,2),(1,3); (2,3)

    netParam.type = Delta;
    netParam.sender = 1; netParam.receiver = 2;
    netAcc->GetParameter (netParam);
    // key1=1, key2=2, data = data(1,2)

    result = netAcc->DeleteNode (3);  // should return True
    // listOfDelays = (1,2)
    // DeleteNode() tested completely

    // tested on 15.11.96, 21:28: all tests okay


    //----------------------------------------------------------------------
    // test Reset() - calls Dispose()

    netAcc->Reset();
    // data = NULL, listOfDelays = "empty list"

    // Reset() and Dispose() tested completely

    // tested on 21.09.98, 17:20: all tests okay


    delete netAcc;
    
    //----------------------------------------------------------------------
    
    return 0;
}
    

#include "inlines.hpp"

