//
// nettest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the network process classes in netproc.hpp and netbase.hpp
// test the network classes in network.hpp and netbase.hpp
// note: the Init/Dispose() functions were made public for this test
//
// created: 09.10.96
//
// last change: 21.09.98
//


#include <iostream.h>
#include <string.h>

#include "..\ci.hpp"
#include "..\clockifc.hpp"
#include "..\errbase.hpp"
#include "..\error.hpp"
#include "..\globals.hpp"
#include "..\lance.hpp"
#include "..\msgform.hpp"
#include "..\network.hpp"
#include "..\sim_gen.hpp"
#include "..\sim_glob.hpp"


#define Need_ACClockSyncAlgorithm
#define Need_CClassManager
#define Need_CControllingObject
#define Need_CFreeRun
#define Need_CFta
#define Need_CFtaWithFetzer
#define Need_templates


void GetPosition (SMsgPosition& pos, SINT32, SINT32)
{
    for (int i=0; i<SMsgPosition::EItemCount; i++)
    {
        pos.length[i]=4;
        pos.position[i]=4*i;
    }
}


const addr broadcastAddress = BROADCAST_ADDRESS;
// need this to set the destination address for ci_send()


int ci_init (int if_num, ci_fn ci_ann_addr, ci_efn ci_ann_err_addr);
int ci_get_tbuf (INT16 size, bufx **buf, int if_num, int area);
int ci_get_tenv (int mode, tenvx **tenv, int if_num, int area);
int ci_ret_tbuf (bufx *buf);
int ci_ret_tenv (tenvx *tenv);
int ci_send (int mode,tenvx *tenv,bufx *buf,INT16 size,int if_num,int area);


int main()
{
    CErrorHandler *handler = new CErrorHandler;
    InstallErrorHandler (handler);


    //----------------------------------------------------------------------
    // variables necessary for the test of netproc.hpp
    //----------------------------------------------------------------------

    CMessenger *mess;
    EBool result;
    SMsg *msg1 = new SMsg;
    SMsg *msg2 = new SMsg;
    SMsg *msg3 = new SMsg;


    //----------------------------------------------------------------------
    // test the behaviour of the base class ACNetworkProcess
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor

    mess = new CMessenger;
    // msg = NULL, initCalled = True

    // tested on 06.01.97, 21:35: all tests okay

/*
    //----------------------------------------------------------------------
    // test Init() and Dispose()

    mess->Init();
    // should return at once (trace into the code to verify that!)

    mess->Dispose();
    // initCalled = False
    mess->Dispose();
    // should return at once (trace into the code to verify that!)

    mess->Init();
    // initCalled = True

    // tested on 06.01.97, 21:35: all tests okay
*/

    //----------------------------------------------------------------------
    // test Active()

    result = mess->Active();  // return False

    ACProcess::SetActiveProcess (mess);  // make the process "active"
    result = mess->Active();  // return True

    // tested on 26.08.97, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test Insert()

    msg1->sender = 1;
    msg1->receiver = 2;
    msg2->sender = 1;
    msg2->receiver = 3;
    msg3->sender = 1;
    msg3->receiver = 4;

    mess->Insert (msg1, 10);
    // _msg->msg = msg1, _msg->time = 10
    // ACProcess::assocTime = 10 (inherited member!)

    mess->Insert (msg2, 15);
    // listOfMessages <- (msg2,15)

    mess->Insert (msg3, 5);
    // _msg->msg = msg3, _msg->time = 5, listOfMessages <- (msg1,10)
    // ACProcess::assocTime = 5

    // tested on 17.10.96, 19:51: all tests okay


    //----------------------------------------------------------------------
    // test GetFirst(), GetNext()

    SMsg *msg;

    msg = mess->GetFirst();  // return msg1
    msg = mess->GetNext();  // return msg2
    msg = mess->GetNext();  // return NULL

    // tested on 26.08.97, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    mess->Reset();
    // listOfMessages is empty, msg_ unchanged

    ACProcess::SetActiveProcess (NULL);

    mess->Reset();
    // msg_ = NULL, state=Passive

    // tested on 26.08.97, 15:40: all tests okay


    //----------------------------------------------------------------------
    // test Body()
/*
    msg1 = new SMsg;  // need to allocate anew, because they were deleted
    msg2 = new SMsg;  // during the reset
    msg3 = new SMsg;

    msg1->sender = 1; msg1->receiver = 2;
    msg2->sender = 1; msg2->receiver = 3;
    msg3->sender = 1; msg3->receiver = 4;

    mess->Insert (msg1, 10);  // insert everything as before
    mess->Insert (msg2, 15);
    mess->Insert (msg3, 5);
    // _msg->msg = msg3, _msg->time = 5, listOfMessages <- (msg1,10)
    // CProcess::assocTime = 5


    // for this test, set _currTime manually (in the debugger)
    // the code is always commented out after the test, because this
    // function is an endless loop
    
    // currTime = 0 (default), break at ACNetworkProcess::Body, at the
    // lines with Hold (...) and Cancel()
    // set _assocTime = 0

    mess->Body();
    // first run: nothing should have changed, execution breaks at
    // Hold() --> assocTime = 5

    // now set currTime = 10
    // _msg = (msg2,15), listOfMessages is empty (i.e. the first two
    // messages were delivered)

    // set currTime = 15
    // _msg = NULL, the function breaks on Cancel()
*/
    // tested on 17.10.96, 19:57: all tests okay
    // ACNetworkProcess tested completely


    delete mess;


    //----------------------------------------------------------------------
    // variables necessary for the test of netbase.hpp
    // note: the messenger was tested above, the delayer is not tested at
    //       all (the deliver method is very straightforward)
    //----------------------------------------------------------------------

    CBroadcastNetwork *network;
    CControllingObject *ctrl = new CControllingObject("QNAM", NULL);// dummy
    tenvx *tenv;
    bufx *buf;


    //----------------------------------------------------------------------
    // test the behaviour of the class ACNetwork (through a derived class)
    // correct behaviour of SMsg, CRandom and CNetworkAccountant is assumed
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor and Init()

    network = new CBroadcastNetwork (ctrl, 2);
    // networkID = 2, listOfNodes = {}, initCalled = True,
    // random=..., distLambda=distOmega = Uniform,0,0,0,0,0
    // listOfNetworks <- network(2)

    ci_init (2, (ci_fn)1, (ci_efn)1);

    // tested on 31.07.98, 10:20: all tests okay


    //----------------------------------------------------------------------
    // test Init() and Dispose()

    network->Init();
    // should return at once (trace into the code to verify that!)

    network->Dispose();
    // initCalled = False
    network->Dispose();
    // should return at once (trace into the code to verify that!)

    network->Init();
    // initCalled = True

    // tested on 06.01.97, 22:15: all tests okay


    //----------------------------------------------------------------------
    // test GetNetworkID()

    SINT32 networkID = 0;
    networkID = network->GetNetworkID();  // return 2

    // tested on 12.08.97, 21:40: all tests okay


    //----------------------------------------------------------------------
    // test the functions controlling the system parameters

    SSingleNetworkParameter param;
    SNetworkData arrprob;

    param.type = PCrash;
    param.value = 0.2;
    result = network->SetParameter (param);  // return True
    // probCrash = 0.2

    param.type = PRecover;
    param.value = 0.4;
    result = network->SetParameter (param);  // return True
    // probCrash = 0.4

    // tested on 25.08.97, 21:00: all tests okay so far

    arrprob.type = PEarly;
    arrprob.sender = 1;
    arrprob.receiver = 2;
    arrprob.value = 0.7;
    result = network->SetParameter (arrprob);  // return True

    arrprob.type = PLate;
    arrprob.value = 0.26;
    result = network->SetParameter (arrprob);  // return True

    // tested on 25.08.97, 21:00: all tests okay so far

    SNetDistribution dist;
    dist.module = Network;
    dist.networkID = 2;

    dist.type = DistributionLambda;
    dist.data.dist = Exponential;
    dist.data.low = 1.5;
    dist.data.high = 2.2;
    dist.data.mean = 1.6;
    dist.data.constant = 2.1;
    dist.data.freeparam = 3;
    result = network->SetParameter (dist);  // return True
    // distDataLambda = Exp, 1.5, 2.2, 1.6, 3, 2.1

    dist.type = DistributionOmega;
    dist.data.dist = Normal;
    dist.data.low = 0.2;
    dist.data.high = 0.7;
    dist.data.mean = 0.3;
    dist.data.constant = 1.4;
    dist.data.freeparam = 0.02;
    result = network->SetParameter (dist);  // return True
    // distDataLambda = Norm, 0.2, 0.7, 0.3, 0.02, 1.4

    // tested on 31.09.98, 10:20: all tests okay so far

    SNetDistributionP2P distp;
    distp.module = Network;
    distp.networkID = 2;
    distp.sender = 1;
    distp.receiver = 2;
    distp.type = DistributionTimely;
    distp.data.dist = Weibull;
    distp.data.low = 0.1;
    distp.data.high = 0.3;
    distp.data.mean = 0.2;
    distp.data.constant = 1.1;
    distp.data.freeparam = 1.2;
    result = network->SetParameter (distp);  // return True

    // tested on 31.09.98, 10:20: all tests okay so far

    param.type = ClockDrift;
    param.value = 0;
    result = network->SetParameter (param);  // return False

    // SetParameter() tested completely

    // tested on 15.02.97, 16:05: all tests okay


    param.type = PCrash;
    result = network->GetParameter (param);  // return True
    // value = 0.2

    param.type = PRecover;
    result = network->GetParameter (param);  // return True
    // value = 0.4

    // tested on 25.08.97, 21:00: all tests okay so far

    arrprob.type = PEarly;
    result = network->GetParameter (arrprob);  // return True
    // value = 0.7

    arrprob.type = PLate;
    result = network->GetParameter (arrprob);  // return True
    // value = 0.26;

    // tested on 25.08.97, 21:00: all tests okay so far

    dist.type = DistributionLambda;
    result = network->GetParameter (dist);  // return True
    // dist.data = Exp, 1.5, 2.2, 1.6, 3, 2.1

    dist.type = DistributionOmega;
    result = network->GetParameter (dist);  // return True
    // dist.data = Norm, 0.2, 0.7, 0.3, 0.02, 1.4

    // tested on 31.09.98, 10:20: all tests okay so far

    distp.type = DistributionTimely;
    distp.data.dist = Uniform;
    distp.data.low=distp.data.high=distp.data.mean=distp.data.constant=0;
    distp.data.freeparam = 0;
    result = network->GetParameter (distp);  // return True
    // distp.data: Wei, 0.1, 0.3, 0.2, 1.2, 1.1

    // tested on 31.09.98, 10:20: all tests okay so far


    param.type = CrashStatus;
    result = network->GetParameter (param);  // return True
    // value = False (0)

    param.type = ClockDrift;
    result = network->GetParameter (param);  // return False
    // GetParameter() tested completely

    // tested on 15.02.97, 16:05: all tests okay


    //----------------------------------------------------------------------
    // test Connect/Disconnect()

    network->Connect (2);
    // listOfNodes = {2}
    network->Connect (1);
    // listOfNodes = {1, 2}
    network->Connect (2);
    // listOfNodes = {1, 2}
    
    network->Disconnect (3);
    // listOfNodes = {1, 2}
    network->Disconnect (2);
    // listOfNodes = {1}

    // tested on 07.02.97, 19:05: all tests okay


    //----------------------------------------------------------------------
    // test the reset function (tests Dispose() as well)

    network->Reset();
    // distLambda=distOmega = Uniform,0,0,0,0,0 , listOfnodes = {},
    // crashed=False, probCrash=probRecover = 0

    // tested on 12.08.97, 21:40: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand()

    SCommand command;
    SNetworkConnectCommand conn;

    command.type = CrashModule;
    result = network->ExecuteCommand (command);  // return True
    // crashed = True

    command.type = RecoverModule;
    result = network->ExecuteCommand (command);  // return True
    // crashed = False

    // tested on 12.08.97, 21:40: all tests okay so far

    conn.type = ConnectNode;
    conn.nodeID = 2;
    result = network->ExecuteCommand (conn);  // return True
    // listOfnodes = {2}

    conn.nodeID = 3;
    result = network->ExecuteCommand (conn);  // return True
    // listOfnodes = {2, 3}

    conn.type = DisconnectNode;
    conn.nodeID = 2;
    result = network->ExecuteCommand (conn);  // return True
    // listOfnodes = {3}

    command.type = ResetModule;
    result = network->ExecuteCommand (command);  // return True
    // listOfNodes = {}

    command.type = SuspendSimulation;
    result = network->ExecuteCommand (command);  // return False

    // tested on 15.02.97, 16:05: all tests okay


    //----------------------------------------------------------------------
    // test SendMsg (tenvx *tenv,...)

    INT16 bufsize = 4;

    network->Connect (1);  // connect a node
    // listOfNodes = {1}

    mess = new CMessenger;  // need a process to get a current nodeID
    ACProcess::SetActiveProcess (mess);  // make the process "active"

    ci_get_tenv (0, &tenv, 2, 0);  // request a tenv
    ci_get_tbuf (bufsize, &buf, 2, 0);  // and a buffer

    INT32 msgID=2;  // the ID of the message is the first entry
    memcpy (buf, &msgID, 4);  // so copy it into buf

    ((ACNetwork *)network)->SendMsg (NULL, buf, bufsize, 0);
    // return at once (and report an error)

    memcpy (tenv->da, broadcastAddress, sizeof(addr));
    // broadcast message

    mess->SetNodeID (3);  // not connected
    ((ACNetwork *)network)->SendMsg (tenv, buf, bufsize, 0);
    // should return (and report that the node is not connected)

    mess->SetNodeID (1);  // this node is connected
    ((ACNetwork *)network)->SendMsg (tenv, buf, bufsize, 0);
    // msg->msgID=2, msg->broadcast = True [trace it!]
    // report that the msg is not timestamped

    for (int i=0; i<ADDRCNT; i++) tenv->da[i] = 0; // no broadcast message
    tenv->da[ADDRCNT-1] = 1;  // nodeID = 1

    tenv->length_field = (INT16)TS_PACKET;
    ((ACNetwork *)network)->SendMsg (tenv, buf, bufsize, 0);
    // msg->broadcast = False, msg->timestamped = True, msg->receiver = 1
    // [trace it!]

    ci_ret_tenv (tenv);  // return the tenv
    ci_ret_tbuf (buf);  // and the buffer

    // tested on 25.08.97, 21:20: all tests okay


    //----------------------------------------------------------------------
    // test Transmit()

    SNetworkData netData;

    network->Connect (2);  // connect a second node
    // listOfNodes = {1, 2}

    netData.type = Delta;  // store a transmission delay
    netData.sender = 1; netData.receiver = 2; netData.value = 4.5;
    network->SetParameter (netData);
    // networkAccountant <- (1,2)

    msg1 = new SMsg;  // the msg was deleted during the test of CMessenger
    msg1->sender = 1;
    msg1->receiver = 2;
    msg1->if_num = 2;
    msg1->area = 0;
    network->ACNetwork::Transmit (msg1);
    // should compute d=4.5, report event 'SendMessage', call the messenger
    // [trace it!]

    netData.type = PEarly;  // enforce early message
    netData.value = 1;
    network->SetParameter (netData);
    msg3 = new SMsg (*msg1);
    network->ACNetwork::Transmit (msg3);
    // should compute d<4.5, report event 'EarlyMessage', call the messenger
    // [trace it!]

    netData.type = PEarly; netData.value = 0;
    network->SetParameter (netData);
    netData.type = PLate;  // enforce late message
    netData.value = 1;
    network->SetParameter (netData);
    SMsg *msg4 = new SMsg (*msg1);
    network->ACNetwork::Transmit (msg4);
    // should compute d>4.5, report event 'LateMessage', call the messenger
    // [trace it!]

    msg2 = new SMsg;
    msg2->sender = 1;
    msg2->receiver = 3;
    msg2->if_num = 2;
    msg2->area = 4;
    msg2->msgID = 25;
    network->ACNetwork::Transmit (msg2);
    // report: computation of transmission delay has failed

    // tested on 25.08.97, 21:20: all tests okay


    //----------------------------------------------------------------------
    // test Crash()

    network->Crash();
    // reports event 'MessageLost' for "msg0" (from test of SendMsg),
    // msg1,3,4 [trace it!]

    network->Reset();

    // tested on 25.08.97, 21:20: all tests okay


    //----------------------------------------------------------------------
    // test ReportMessageEvent()

    network->ReportMessageEvent (EarlyMessage, msg2);
    // create event; set module=Network, type=EarlyMessage, networkID=2,
    // sender=1, area=4, receiver=3, msgID=25; report it
    // [trace it!]

    // tested on 25.08.97, 21:20: all tests okay


    //----------------------------------------------------------------------
    // test Deliver()

    CLance *lance1 = new CLance (ctrl, 0);
    CLance *lance2 = new CLance (ctrl, 1);
    listOfLances.Insert (lance1, 2, 0);
    listOfLances.Insert (lance2, 2, 1);
    
    msg2->receiver = 2;
    network->ACNetwork::Deliver (msg2);
    // 2x output: Lance::Announce() called

    // tested on 16.01.98, 13:50: all tests okay


    //----------------------------------------------------------------------
    // test the behaviour of the class CBroadcastNetwork
    // correct behaviour of the class ACNetwork is assumed
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test Transmit()

    CClockInterface *clock1, *clock11, *clock2;  // clocks

    msg1 = new SMsg;

    network->Transmit (msg1);  // nothing should happen (empty node list)


    // create two nodes 1,2 to play with
    network->Connect (1);
    network->Connect (2);

    // create three clocks to match the Lances
    clock1 = new CClockInterface (ctrl, 25e6, 0);
    listOfClocks.Insert (clock1, 1, 0);
    clock11 = new CClockInterface (ctrl, 25e6, 1);
    listOfClocks.Insert (clock11, 1, 1);
    clock2 = new CClockInterface (ctrl, 25e6, 2);
    listOfClocks.Insert (clock2, 2, 2);

    ACProcess::SetCurrentTime (10);


    msg1->sender = 1; msg1->area = 0;
    msg1->if_num = 2;
    msg1->broadcast = True;

    network->Crash();
    // crashed = True (messenger is reset; that is tested below!)

    network->Transmit (msg1);  // nothing should happen (network crashed)

    network->Recover();
    // crashed = False

    network->Transmit (msg1);  // msg should not be timestamped
    // messenger <- msg(1,0,1,True,2,buf=NULL,size=0),
    // msg(1,0,2,0...)

    msg1->timestamped = True;
    network->Transmit (msg1);
    // messenger <- msg(1,0,1,True,2,{"10" sec,"10" sec,0,0},{0, x,x,x,x
    // (=^= 10 sec),0,0,...},12),
    // msg(1,0,2,0...)

    network->Reset();
    network->Connect (1);
    network->Connect (2);

    msg1->broadcast = False; msg1->receiver = 1;
    network->Transmit (msg1);
    // messenger <- msg(1,0,1,False,0,{"10","10",0,0},{0,0,0,0; "10"; 0,0,
    // 0,0,...},32)

    // tested on 30.04.97, 17:20: all tests okay


    //----------------------------------------------------------------------
    // test Crash() - continued

    network->Crash();
    // crashed=True, messenger is empty, ReportCrashStatus() called
    // [trace: event = {Network, CrashStatusChanged, networkID, crashed}]

    // tested on 12.08.97, 22:10: all tests okay


    //----------------------------------------------------------------------
    // test SendMsg()

    network->Recover();

    network->SendMsg (msg1);
    // messenger <- msg(1,0,1,False,0,{"10","5",0,0},{0,0,0,0; "10"; 0,0,
    // 0,0,...},48)

    dist.type = DistributionLambda;
    dist.data.dist = Uniform;
    dist.data.low = 0;
    dist.data.high = 3.5;
    dist.data.mean = 0;
    dist.data.constant = 0;
    dist.data.freeparam = 0;
    result = network->SetParameter (dist);  // return True
    // lambda = 3.5

    network->SendMsg (msg1);  // calls the delayer
    // delayer <- (msg, delay)

    // tested on 21.09.98, 14:10: all tests okay


    delete network;
    delete ctrl;
    delete mess;

    DeinstallErrorHandler ();
    delete handler;

    //----------------------------------------------------------------------

    return 0;
}


CClockInterface::CClockInterface (CControllingObject *ctrl, double, int)
  : CApplControlledObject (ctrl) {}
CClockInterface::~CClockInterface () {}
EBool CClockInterface::ExecuteCommand (const SCommand&) { return False; }
EBool CClockInterface::SetParameter (const SParameter&) { return False; }
EBool CClockInterface::GetParameter (SParameter&) const { return False; }
EBool CClockInterface::SetAmortizationDuration (double) { return False; }
EBool CClockInterface::GpsConnected () { return False; }
void CClockInterface::AddGps (EGpsUnit) {}
void CClockInterface::RemoveGps (EGpsUnit) {}
double CClockInterface::GetGranularity() const {return 0;}
double CClockInterface::GetGranularityAcc() const {return 0;}
double CClockInterface::GetGranularityOsc() const {return 0;}
double CClockInterface::GetGranularityDT() const {return 0;}
double CClockInterface::GetRhoPlus() const {return 0;}
double CClockInterface::GetRhoMinus() const {return 0;}
INT32 CClockInterface::GetAccuracy() const {return 0;}
EBool CClockInterface::GetTimestamp (STimestamp& time)
{
    time.ms = 0;
    time.ts = (INT32)ACProcess::CurrentTime();
    time.acc = 0;
    return True;
}
void CClockInterface::SetTimestamp (const STimestamp&) {}
void CClockInterface::SetTimestamp (const SExtendedTimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const STimestamp&) {}
void CClockInterface::PreloadTimestamp (double, const SExtendedTimestamp&) {}
void CClockInterface::EnableInterrupt (EInterruptSource) {}
void CClockInterface::DisableInterrupt (EInterruptSource) {}
EBool CClockInterface::InterruptPending (EInterruptSource) { return False; }
EBool CClockInterface::InterruptPending (EClockInterrupt) { return False;  }
void CClockInterface::ClearInterrupt (EInterruptSource) {}
void CClockInterface::ClearInterrupt (EClockInterrupt) {}
void CClockInterface::InterruptAfter (EInterruptSource, double) {}
void CClockInterface::InterruptAt (EInterruptSource, double) {}
double CClockInterface::InterruptAtMult (EInterruptSource, double) { return 0; }
void CClockInterface::Init () {}
void CClockInterface::Dispose() {}
void CClockInterface::isr() {}
void CClockInterface::wobble_isr() {}
void CClockInterface::Amortize (double, INT32, INT32) {}
void CClockInterface::PreloadAmort (double, INT32, INT32) {}
void CClockInterface::DisableDutyTimer (EInterruptSource) {}
void CClockInterface::Crash () {}
void CClockInterface::Recover () {}
void CClockInterface::Reset () {}
void CClockInterface::ReportCrashChange () {}
int CClockInterface::ReportError (EReportType, char*, int, char*) { return 0; }


CLance::CLance (CControllingObject *ctrl, int) : CApplControlledObject (ctrl) {}
CLance::~CLance () {}
void CLance::Crash () {}
void CLance::isr () {}
void CLance::Reset () {}
EBool CLance::SetParameter (const SParameter&) {return False;}
EBool CLance::GetParameter (SParameter&) const {return False;}
EBool CLance::ExecuteCommand (const SCommand&) {return False;}
void CLance::Suspend() {}
void CLance::SetClockSync (CClockSync *) {}
void CLance::GetBuffer (INT16, char **, INT32&) {}
void CLance::GetEnvelope (tenvx **) {}
void CLance::ReturnBuffer (char *) {}
void CLance::ReturnEnvelope (tenvx *) {}
void CLance::GetReceiveBuffer (INT16 s, bufx **b) { *b = (bufx*)new char[s]; }
void CLance::GetReceiveEnvelope (renvx **r) { *r = new renvx; }
void CLance::Send (int, tenvx *, char *, INT16, int, int) {}
void CLance::Announce (renvx *, bufx *, INT16, int)
  { cout << "Lance::Announce() called" << endl; }
void CLance::Init () {}
void CLance::Dispose () {}
void CLance::ReportCrashChange () {}


#include "inlines.hpp"


