//
// proctest.cpp
//
// tests class ACProcess (only DOS_SIM, LINUX_SIM does not work)
//
// created: 09.10.98
//
// last change: 09.10.98
//


#include <iostream.h>

/*
// the following definition must be before the inclusion of process.hpp
#ifdef LINUX_SIM

class Process
{
public:
    void Hold (double time) { cout << "process holding " << time << " seconds" << endl;
};

#endif
*/

#include "..\process.hpp"


class CTestProcess : public ACProcess
{
public:
    CTestProcess (SINT32 nodeID = DefaultProcessID) : ACProcess (nodeID) {}
    void Body() {}
};


int main()
{
    //----------------------------------------------------------------------
    // test class ACProcess
    //  note: tests differ for LINUX_SIM and not LINUX_SIM
    //----------------------------------------------------------------------

    CTestProcess *process;
    const ACProcess *current;
    SINT32 nodeID;
    EBool result;
    double value;


#ifdef LINUX_SIM

    //----------------------------------------------------------------------
    // test the ctor

    process = new CTestProcess();
    // nodeID = DefaultNodeID (0)

    delete process;

    process = new CTestProcess (3);
    // nodeID = 3

    // tested on 09.10.98, 14:??: all tests okay


    //----------------------------------------------------------------------
    // test tm_wkafter()

    process->tm_wkafter (23);
    // output: process holding 0.23 seconds

    // tested on 09.10.98, 14:??: all tests okay

#else

    //----------------------------------------------------------------------
    // test the ctor

    process = new CTestProcess();
    // assocTime = 0, state = Passive, nodeID = DefaultNodeID (0),
    // currTime = 0, currentProcess = NULL

    delete process;

    process = new CTestProcess (3);
    // nodeID = 3, rest as above

    // tested on 09.10.98, 14:40: all tests okay


#ifdef DOS_SIM

    //----------------------------------------------------------------------
    // test simulation control (needed for debugging)

    process->SetCurrentTime (12.5);
    // currTime = 12.5

    process->SetActiveProcess (process);
    // currentProcess = process

    // tested on 09.10.98, 14:40: all tests okay

#endif

    //----------------------------------------------------------------------
    // test Suspend()

    process->Suspend();
    // state = Suspended

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test Cancel() and terminate()

    process->Cancel();
    // state = Passive

    process->terminate();
    // state = Terminated

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test process (re-)scheduling

    process->Activate ();
    // nothing should happen (process is not Passive)

    process->Cancel();
    process->Activate ();
    // assocTime = 12.5, state = Scheduled

    // tested on 09.10.98, 14:40: all tests okay so far


    process->SetCurrentTime (10);
    process->Suspend();
    process->ReActivate ();
    // nothing should happen (process is Suspended)

    process->terminate();
    process->ReActivate ();
    // nothing should happen (process is Terminated)

    process->Cancel();
    process->ReActivate ();
    // assocTime = 10, state = Scheduled

    process->SetCurrentTime (11.1);
    process->ReActivate ();
    // assocTime = 11.1, state = Scheduled

    // tested on 09.10.98, 14:40: all tests okay so far


    process->SetCurrentTime (1.2);
    process->Suspend();
    process->ReActivateAt (2.3);
    // nothing should happen (process is Suspended)

    process->terminate();
    process->ReActivateAt (2.3);
    // nothing should happen (process is Terminated)

    process->Cancel();
    process->ReActivateAt (2.3);
    // assocTime = 2.3, state = Scheduled

    process->ReActivateAt (4);
    // assocTime = 4, state = Scheduled

    process->ReActivateAt (0.3);  // 0.3 < current time
    // assocTime = 1.2, state = Scheduled

    // tested on 09.10.98, 14:40: all tests okay so far


    process->SetCurrentTime (1.2);
    process->Suspend();
    process->ReActivateDelay (4.5);
    // nothing should happen (process is Suspended)

    process->terminate();
    process->ReActivateDelay (4.5);
    // nothing should happen (process is Terminated)

    process->Cancel();
    process->ReActivateDelay (4.5);
    // assocTime = 5.7, state = Scheduled

    process->ReActivateDelay (4);
    // assocTime = 5.2, state = Scheduled

    process->ReActivateDelay (-1);
    // assocTime = 1.2, state = Scheduled

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test idle()

    result = process->idle ();  // should return False

    process->SetActiveProcess (process);
    process->Cancel();
    result = process->idle ();  // should return False

    process->SetActiveProcess (NULL);
    result = process->idle ();  // should return True

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test evtime()

    value = process->evtime();  // should return 1.2 (assocTime)

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test current()

    process->SetActiveProcess (NULL);
    current = ACProcess::current();  // should return NULL

    process->SetActiveProcess (process);
    current = ACProcess::current();  // should return process

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test CurrentTime()

    process->SetCurrentTime (1.5);
    value = process->CurrentTime ();  // should return 1.2

    process->SetCurrentTime (20.87);
    value = process->CurrentTime ();  // should return 20.87

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------
    // test Hold() and tm_wkafter()
    // assumption: assocTime = 1.2

    process->Hold (2.3);
    // assocTime = 3.5


#ifndef PSOS_SIM

    process->tm_wkafter (18);
    // assocTime = 3.68
    
#endif

    // tested on 09.10.98, 14:50: all tests okay

#endif  // not LINUX_SIM


    //----------------------------------------------------------------------
    // test SetNodeID(), GetNodeID()

    nodeID = process->GetNodeID ();  // return 3

    process->SetNodeID (5);
    // nodeID = 5

    nodeID = process->GetNodeID ();  // return 5

    // tested on 09.10.98, 14:40: all tests okay


    //----------------------------------------------------------------------

    return 0;
}
    

