//
// psostest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the queue classes in psos.cpp and the pSOS+ access functions in
// psos.h
//
// created: 21.9.96
//
// last changed: 7.12.96
//


#include "..\psos.hpp"


int main()
{

    //----------------------------------------------------------------------
    // variables necessary for the test of psos.cpp
    //----------------------------------------------------------------------

    // necessary for the test of CQueue
    EBool  test;
    INT32 testval;
    INT32 msg[4] = {'m','s','g','1'};
    INT32 testmsg[4] = {'0','0','0','0'};

    CQueue *queue;


    //----------------------------------------------------------------------
    // test the behaviour of the class CQueue
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor

    queue = new CQueue;  // make a new queue
    // first=last = NULL

    // tested on 21.9.96, 15:20: all tests okay


    //----------------------------------------------------------------------
    // test Send

    testval = queue->Send(msg);  // should return 0
    // first=last = msg1 (SLink)

    msg[3] = '2';
    testval = queue->Send(msg, True);  // should return 0
    // first = msg2 (SLink)

    msg[3] = '3';
    testval = queue->Send(msg);  // should return 0
    // last = msg3 (SLink)
    // order: msg2(=first), msg1, msg3(=last)

    // Send tested completely

    // tested on 21.9.96, 15:20: all tests okay

    
    //----------------------------------------------------------------------
    // test Empty()

    test = queue->Empty();  // should return False

    // tested on 21.9.96, 15:20: all tests okay so far


    //----------------------------------------------------------------------
    // test Receive()

    testval = queue->Receive(testmsg);  // should return 0
    // testmsg = msg2

    testval = queue->Receive(testmsg);  // should return 0
    // testmsg = msg1

    testval = queue->Receive(testmsg);  // should return 0
    // testmsg = msg3

    testval = queue->Receive(testmsg);  // should return ERR_NOMSG (55)
    // testmsg is still = msg3

    // Receive() tested completely

    // tested on 21.9.96, 15:20: all tests okay


    //----------------------------------------------------------------------
    // test Empty() - continued

    test = queue->Empty();  // should return True

    // Empty() tested completely

    // tested on 21.9.96, 15:20: all tests okay


    //----------------------------------------------------------------------

    delete queue;  // delete the queue


    //----------------------------------------------------------------------
    // test the behaviour of the class CQueueManager
    // (correct behaviour of class CQueue is assumed)
    //----------------------------------------------------------------------

    CQueueManager *manager;
    INT32 testqid;


    //----------------------------------------------------------------------
    // test constructor

    manager = new CQueueManager;
    // firstQueue = NULL

    // tested on 21.9.96, 16:25: all tests okay


    //----------------------------------------------------------------------
    // test Create()

    testval = manager->Create("AL", &testqid);  // return 0
    // testqid = 1; firstQueue = AL

    testval = manager->Create("CAPONE", &testqid);  // return 0
    // testqid = 2; firstQueue = CAPO
    // order: CAPO(qid=2), AL(qid=1)

    // Create() tested completely

    // tested on 21.9.96, 16:25: all tests okay


    //----------------------------------------------------------------------
    // test Ident()

    testval = manager->Ident ("NO", &testqid);  // return ERR_OBJNF (9)
    // testqid has not changed

    testval = manager->Ident ("AL", &testqid);  // return 0
    // testqid = 1

    testval = manager->Ident ("CAPONE", &testqid);  // return 0
    // testqid = 2

    // Ident() tested completely

    // tested on 21.9.96, 16:25: all tests okay


    //----------------------------------------------------------------------
    // test Send()

    msg[3] = '1';
    testval = manager->Send (3, msg);  // return ERR_OBJID (6)

    testval = manager->Send (0, msg);  // return ERR_OBJID (6)

    testval = manager->Send (1, msg);  // return 0

    msg[3] = '2';
    testval = manager->Send (1, msg, True);  // return 0

    // Send() tested completely, Find() tested completely

    // tested on 21.9.96, 16:25: all tests okay


    //----------------------------------------------------------------------
    // test Receive()

    testval = manager->Receive (3, testmsg);  // return ERR_OBJID (6)

    testval = manager->Receive (1, testmsg);  // return 0
    // testmsg = msg2

    // Receive() tested completely

    // tested on 21.9.96, 16:25: all tests okay


    //----------------------------------------------------------------------
    // test Delete()

    manager->Create("AL2", &testqid);
    manager->Create("AL3", &testqid);
    // testqid = 4; firstQueue = AL3

    // order of the queues: AL3(qid=4), AL2(3), CAPO(2), AL(1)


    testval = manager->Delete (4);  // return 0
    // firstQueue = AL2

    testval = manager->Delete (2);  // return 0
    testval = manager->Delete (2);  // return ERR_OBJID (6)

    testval = manager->Delete (1);  // return 0
    testval = manager->Delete (1);  // return ERR_OBJID (6)

    testval = manager->Delete (3);  // return 0
    testval = manager->Delete (3);  // return ERR_OBJID (6)

    // Delete() tested completely

    // tested on 7.12.96, 10:25: all tests okay


    //----------------------------------------------------------------------

    delete manager;


    //----------------------------------------------------------------------
    // test the behaviour of the pSOS+ queue functions
    // (correct behaviour of class CQueueManager is assumed)
    //----------------------------------------------------------------------

    testval = q_create ("AQ", 5, 7, &testqid);  // return 0
    // testqid = 1

    testqid = 0;
    testval = q_ident ("AQ", 15, &testqid);  // return 0
    // testqid = 1

    msg[3] = '1';
    testval = q_send (1, msg);  // return 0

    msg[3] = '2';
    testval = q_urgent (1, msg);  // return 0

    testval = q_receive (1, 3, 4, testmsg);  // return 0
    // testmsg = msg2

    testval = q_receive (1, 3, 4, testmsg);  // return 0
    // testmsg = msg1

    testval = q_receive (1, 3, 4, testmsg);  // return ERR_NOMSG (55)
    // testmsg = msg1

    // tested on 21.9.96, 16:34: all tests okay


    testval = q_delete (1);  // return 0

    // tested on 7.12.96, 10:25: all tests okay


    //----------------------------------------------------------------------

    
    return 0;
}
    



