//
// randtest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the random generator classes in random.hpp
//
// created: 30.09.96
//
// last changed: 07.12.97
//


#include "..\random.hpp"

const int NumberOfTrials = 100;
// number of draws per distribution function

enum ESeed { MSeed, LSeed, Distance };
// seeds for the random generators


int main()
{
    //----------------------------------------------------------------------
    // variables necessary for the test of rand.hpp
    //----------------------------------------------------------------------

    CRandom *rand1, *rand2;
    double result1;
    int i;
    double sum;
    int left, right;  // # elements to the left and right of the mean


    //----------------------------------------------------------------------
    // test the behaviour of the class CRandom
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the constructor

    rand1 = new CRandom;  // make a new random class
    rand2 = new CRandom (1);  // make a new random class, select stream 1

    // tested on 01.10.96, 19:03: all tests okay


    //----------------------------------------------------------------------
    // test CastDice()

    left = right = 0;
    for (i=0; i<NumberOfTrials; i++)
    {
       if (rand2->CastDice (0.7)) right++;
       else left++;
    }
    // left should be approx. 30%, right approx. 70% of NumberOfTrials

    // tested on 07.12.97, ??:30: all tests okay


    //----------------------------------------------------------------------
    // test Random()

    sum = 0;
    left = right = 0;
    for (i=0; i<NumberOfTrials; i++)
    {
       result1 = rand1->Random (Uniform, 3, 4);
       sum += result1;
       if (result1 < 3.5) left++;
       else right++;
    }
    sum = sum/NumberOfTrials;  // should be approx. 3.5
    // left and right should be approx. equal

/*
// Normal is not implemented (yet)

    sum = 0;
    left = right = 0;
    for (i=0; i<NumberOfTrials; i++)
    {
       result1 = rand1->Random (Normal, 1, 3.5);
       sum += result1;
       if (result1 < 2.25) left++;
       else right++;
    }
    sum = sum/NumberOfTrials;  // should be approx. 2.25
    // left and right should be approx. equal
*/


    sum = 0;
    left = right = 0;
    // find out the seeds
    for (i=0; i<NumberOfTrials; i++)
    {
       result1 = rand1->Random (Exponential, 1, 2);
       sum += result1;
       if (result1 < 1.5) left++;
       else right++;
    }
    sum = sum/NumberOfTrials;  // should be approx. 1.5
    // left should be greater than right

    // tested on 01.10.96, 19:03: all tests okay


    //----------------------------------------------------------------------
    // test Set/GetSeeds()

    long seeds[EDistributionCount][3];  // the seeds of the generators
    double before[10];  // draw 10 numbers
    double after[10];  // set the seeds, draw again, compare to before

    // check uniform distribution
    // note: check that NumberOfTrials + 1 < SizeOfSeries (the latter is
    // a constant in random.hpp)

    before[0] = rand1->Random (Uniform, 1, 2);  // draw one number
    // find out the seeds
    rand1->GetSeeds (Uniform, seeds[Uniform][MSeed], seeds[Uniform][LSeed],
                     seeds[Uniform][Distance]);
    // and draw the rest of the numbers into before
    for (i=1; i<10; i++) before[i] = rand1->Random (Uniform, 1, 2);

    // now reset the seeds
    rand1->SetSeeds (Uniform, seeds[Uniform][MSeed], seeds[Uniform][LSeed],
                     seeds[Uniform][Distance]);
    // and draw again, this time into after
    for (i=0; i<10; i++) after[i] = rand1->Random (Uniform, 1, 2);

    // now compare before with after: if before[i+1] = after[i], it's okay

    for (i=0; i<SizeOfSeries-NumberOfTrials; i++)
    // draw enough numbers to make the 'distance' member = SizeOfSeries
        rand1->Random (Uniform, 1, 2);

    // repeat the test
    before[0] = rand1->Random (Uniform, 1, 2);  // draw one number
    // find out the seeds
    rand1->GetSeeds (Uniform, seeds[Uniform][MSeed], seeds[Uniform][LSeed],
                     seeds[Uniform][Distance]);
    // and draw the rest of the numbers into before
    for (i=1; i<10; i++) before[i] = rand1->Random (Uniform, 1, 2);

    // now reset the seeds
    rand1->SetSeeds (Uniform, seeds[Uniform][MSeed], seeds[Uniform][LSeed],
                     seeds[Uniform][Distance]);
    // and draw again, this time into after
    for (i=0; i<10; i++) after[i] = rand1->Random (Uniform, 1, 2);

    // compare before with after: if before[i+1] = after[i], it's okay


    // check exponential distribution

    before[0] = rand1->Random (Exponential, 1, 2);  // draw one number
    // find out the seeds
    rand1->GetSeeds (Exponential, seeds[Exponential][MSeed],
                     seeds[Exponential][LSeed], seeds[Exponential][Distance]);
    // and draw the rest of the numbers into before
    for (i=1; i<10; i++) before[i] = rand1->Random (Exponential, 1, 2);

    // now reset the seeds
    rand1->SetSeeds (Exponential, seeds[Exponential][MSeed],
                     seeds[Exponential][LSeed], seeds[Exponential][Distance]);
    // and draw again into after
    for (i=0; i<10; i++) after[i] = rand1->Random (Exponential, 1, 2);

    // compare again: if before[i+1] = after[i], it's okay


    // tested on 25.01.97, 12:30: all tests okay


    delete rand1;
    delete rand2;


    //----------------------------------------------------------------------

    
    return 0;
}



