//
// socktest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test class CSocket [sockbase.?pp]
// note: all protected functions were made public for the test
// note: GetSocketMode() was made global for this test
// test class CStreamInterface [sockstre.?pp]
// test class CStreamServer [sockstre.?pp]
// test class CStreamClient [sockstre.?pp]
// test class CDatagramInterface [sockdgrm.?pp]
// test class CDatagramServer [sockdgrm.?pp]
// test class CDatagramClient [sockdgrm.?pp]
//
// created: 10.02.97
//
// last change: 29.07.98
//


#include "..\errbase.hpp"
#include "..\error.hpp"
#include "..\sockbase.hpp"
#include "..\sockdgrm.hpp"
#include "..\socket.hpp"
#include "..\sockstre.hpp"
#include "..\sockunix.hpp"
#include "..\structs.hpp"


//#define TEST_STATIC_FUNCS_SOCKBASE
//#define TEST_STATIC_FUNCS_SOCKSTRE
// uncomment it to test the static functions

extern ESocketMode GetSocketMode (double timeout);  // sockbase.cpp
extern INT32 Convert (INT32 src); // sockstre.cpp


int main()
{
    //----------------------------------------------------------------------
    // install an error handler (error reports will occur during the test)
    //----------------------------------------------------------------------

    CErrorHandler handler;
    InstallErrorHandler (&handler);


#ifdef TEST_STATIC_FUNCS_SOCKBASE
// commented out after the test
    //----------------------------------------------------------------------
    // test the global functions of sockbase.cpp
    //----------------------------------------------------------------------

    ESocketMode sockmode;
    sockmode = GetSocketMode (0);  // should return STimeout
    sockmode = GetSocketMode (-1);  // should return SNoTimeout

    // tested on 10.02.97, 18:30: all tests okay
#endif


    //----------------------------------------------------------------------
    // test the behaviour of class CSocket
    // correct behaviour of the socket IFC calls in socket.hpp is assumed
    //----------------------------------------------------------------------

    CSocket *sock;  // stream socket
    CSocket *dsock;  // datagram socket
    INT32 sid;
    CUnixAddress *addr = new CUnixAddress ("dummy");
    ACSocketAddress *sockaddr;
    EBool result;
    double timeout;
    SEvent *event;  // for sending data
    SEvent *revent;  // for receiving data
    INT32 size;  // size of the data


    //----------------------------------------------------------------------
    // test the ctor and Init()

    s_create (addr, SStream, &sid);  // make a socket
    dsock = new CSocket (addr, sid, False);
    // addr_ = addr, sid_ = sid, timeout = SocketTimeout (2), stream_=False

    delete dsock;
    // should have deleted the socket (check this by the following call
    // to s_create: if the same socket ID is returned, the socket was
    // deleted)

    s_create (addr, SStream, &sid);  // get a new socket
    dsock = new CSocket (addr, sid, False);  // create object anew

    s_create (addr, SStream, &sid);  // get a new socket

    sock = new CSocket (addr, sid, True);
    // stream_ = True, rest as above

    // tested on 06.04.97, 14:00: all tests okay


    //----------------------------------------------------------------------
    // test socket information functions

    sockaddr = sock->GetSocketAddress();  // should return addr
    sid = sock->GetSocketID();  // should return sid_
    result = sock->IsStream();  // should return True

    // tested on 06.04.97, 14:00: all tests okay


    //----------------------------------------------------------------------
    // test timeout functions

    timeout = sock->GetTimeout ();  // should return SockTimeout (2)

    sock->SetTimeout (5);
    // timeout = 5

    // tested on 10.02.97, 16:40: all tests okay


    //----------------------------------------------------------------------
    // test Reset()

    sock->Reset ();
    // timeout=SocketTimeout (2)

    // tested on 10.02.97, 18:50: all tests okay


    //----------------------------------------------------------------------
    // test the internal ctor and SetSocketID()

    delete sock;
    delete dsock;

    s_create (addr, SStream, &sid);  // get a new socket
    dsock = new CSocket (addr, False);  // create object anew
    // addr_ = addr, timeout = SocketTimeout (2), stream_=False
    dsock->SetSocketID (sid);
    // sid_ = sid

    s_create (addr, SStream, &sid);  // get a new socket
    sock = new CSocket (addr, sid, True);
    // addr_ = addr, timeout = SocketTimeout (2), stream_=True
    sock->SetSocketID (sid);
    // sid_ = sid

    // tested on 06.04.97, 14:00: all tests okay


    //----------------------------------------------------------------------
    // test Send()

    event = new SEvent;
    event->module = Network;
    event->type = MessageLost;
    event->time = 5;
    size = event->size;

    result = sock->Send (event, size);  // return True
    // s_send for stream sockets (no address parameter) should be called
    // [trace it!]

    result = dsock->Send (event, size);  // return True
    // s_send for datagram sockets (with address parameter) should be called
    // [trace it!]

    // tested on 06.04.97, 14:00: all tests okay


    //----------------------------------------------------------------------
    // test Receive (void*, INT32*) - calls Receive (void*, INT32*, double),
    // do that is tested as well

    revent = new SEvent;  // allocate memory
    size = event->size+1;  // let's hope that does not lead to problems

    result = sock->Receive (revent, &size);  // return True
    // size should be reduced by 1, revent=event

    result = sock->Receive (revent, &size);  // return False
    // size, revent have not changed

    // tested on 06.04.97, 14:00: all tests okay


    delete sock;
    delete dsock;
    delete event;
    delete revent;


#ifdef TEST_STATIC_FUNCS_SOCKSTRE
    //----------------------------------------------------------------------
    // test the global functions of sockstre.cpp
    //----------------------------------------------------------------------

    INT32 res = Convert (0x12345678);  // should return 0x78563412

    // tested on 29.07.98, 16:40: all tests okay
#endif


    //----------------------------------------------------------------------
    // test the behaviour of class CStreamInterface
    // correct behaviour of class CSocket is assumed
    //----------------------------------------------------------------------

    CStreamInterface *streamifc;
    SINT32 msgID;
    EMessageType type;


    //----------------------------------------------------------------------
    // test ctor and Init()

    s_create (addr, SStream, &sid);  // get a new socket
    streamifc = new CStreamInterface (addr, sid);
    // addr_ = addr, sid_ = sid, timeout = SocketTimeout (2), stream_=True,
    // sendMsgID=0, receiveMsgID=NoAccount (-1), byteOrderReversed=False

    // tested on 04.05.97, 18:50: all tests okay


    //----------------------------------------------------------------------
    // test message ID management

    msgID = streamifc->GetMessageID ();  // should return NoAccount (-1)

    // tested on 10.02.97, 16:40: all tests okay


    //----------------------------------------------------------------------
    // test buffer allocation functions

    event = (SEvent*)streamifc->GetMessageBuffer (sizeof(SEvent));
    streamifc->ReturnMessageBuffer (event);
    // should allocate/deallocate the memory (trace it!)

    // tested on 10.02.97, 16:40: all tests okay


    //----------------------------------------------------------------------
    // test SendMessage() - calls SendMessageHeader(), SendMessageID(),
    //     SendMessage (void*, INT32*), so these are tested as well

    event = new SEvent;
    event->module = Network;
    event->type = MessageLost;
    event->time = 5;

    msgID = streamifc->SendMessage (MsgEvent, event);  // return 0
    // sendMsgID = 1
    // the event should have been transmitted (see message reception below)
    // SendMessageHeader() tested completely

    // tested on 04.05.97, 19:00: all tests okay so far


    // now test again, but with reversed byte order
    // DEBUGGER: set streamifc->byteOrderReversed to True

    msgID = streamifc->SendMessage (MsgEvent, event);  // return 1
    // sendMsgID = 2, data size is converted event->size [trace it!]
    // the event should have been transmitted (see message reception below)
    // SendMessageID(), SendMessageID(INT32), SendMessage (void*, INT32*)
    // tested completely

    // DEBUGGER: set streamifc->byteOrderReversed to False

    // tested on 29.07.98, 17:10: all tests okay


    //----------------------------------------------------------------------
    // test ReceiveMessageHeader() and ReceiveMessage() - the latter
    //    calls ReceiveMessageID(), so that is tested as well

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgEvent

    result = streamifc->ReceiveMessage ((SBase **)&revent);  // True
    // revent = event
    // receiveMsgID = 0

    streamifc->ReturnMessageBuffer (revent);

    // tested on 04.05.97, 19:00: all tests okay so far


    // now test again, but with reversed byte order
    // DEBUGGER: set streamifc->byteOrderReversed to True

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgEvent

    result = streamifc->ReceiveMessage ((SBase **)&revent);  // True
    // revent = event (again, size is converted; we do not have to trace
    // to see it, it is enough to see that no Assert() fails)
    // receiveMsgID = 1

    streamifc->ReturnMessageBuffer (revent);

    // DEBUGGER: set streamifc->byteOrderReversed to False

    // tested on 29.07.98, 17:10: all tests okay so far


    result = streamifc->ReceiveMessageHeader (&type);  // return False
    result = streamifc->ReceiveMessage ((SBase **)&revent);  // False
    // (no message ID)

    // note: cannot test the other possible failures (no message size or
    // no message) because I can't create them without changing the code
    // of s_receive (there an Assert() checks the size of the data buffer
    // into which the data is being read)

    // tested on 04.05.97, 19:00: all tests okay so far


    // test messages that come in several packets

    sid = 1;  // DEBUGGER: set it to sid of streamifc!
    INT32 datasize = event->size-4;
    type = MsgEvent;
    s_send (sid, &type, 1);  // send the message header
    s_send (sid, &sid, 4);  // send the message ID (1 in this case)
    s_send (sid, &datasize, 4);  // send the data length
    s_send (sid, (char*)event+4, datasize-4);  // send first part of the data
    s_send (sid, (char*)event+4+datasize-4, 4);  // send the rest

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgEvent

    result = streamifc->ReceiveMessage ((SBase **)&revent);  // True
    // revent = event
    // receiveMsgID = 1

    streamifc->ReturnMessageBuffer (revent);

    // tested on 29.07.98, 17:40: all tests okay


    //----------------------------------------------------------------------
    // test special message functions

    result = streamifc->SendSynchronizationByte ();  // return True
    result = streamifc->Resynchronize ();  // return True

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgSynchronizationByte

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgResynchronize

    result = streamifc->SendByteOrder ();  // return True
    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgByteOrder

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = 0x34 (low byte)
    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = 0x12 (high byte)

    result = streamifc->SendByteOrder ();  // return True
    result = streamifc->ReceiveByteOrder ();  // return True
    // byteOrderReversed is still False

    // tested on 04.05.97, 18:50: all tests okay so far

    EModule module = Clock;

    result = streamifc->ReceiveModule (module);  // return False
    // module is not changed

    result = streamifc->SendModule (Supervisor);  // return True
    result = streamifc->SendModule (Controller);  // return True

    result = streamifc->ReceiveModule (module);  // return True
    // module = Supervisor
    result = streamifc->ReceiveModule (module);  // return True
    // module = Controller
    result = streamifc->ReceiveModule (module);  // return False
    // module is not changed

    // tested on 12.05.97, 16:00: all tests okay


    //----------------------------------------------------------------------
    // test ReplyToMessage() - calls SendMessageID (INT32)

    event->module = ClockSync;
    event->type = TimerEvent;
    event->time = 6;

    result = streamifc->ReplyToMessage (event);  // return True

    result = streamifc->ReceiveMessageHeader (&type);  // return True
    // type = MsgReply

    result = streamifc->ReceiveMessage ((SBase **)&revent);  // True
    // revent = event
    // receiveMsgID = 0

    streamifc->ReturnMessageBuffer (revent);

    // tested on 04.05.97, 19:20: all tests okay


    //----------------------------------------------------------------------
    // test Init()

    streamifc->Reset ();
    // sendMsgID=0, receiveMsgID=NoAccount (-1), timeout=SocketTimeout (2)

    // tested on 10.02.97, 18:50: all tests okay

    // note: CStreamInterface is tested except for the internal ctor,
    // which is tested through a derived class below

    delete streamifc;
    delete event;


    //----------------------------------------------------------------------
    // test the behaviour of class CStreamServer
    //     correct behaviour of CStreamInterface is assumed
    //----------------------------------------------------------------------

    CStreamServer *ssock;
    CStreamInterface *accsock;


    //----------------------------------------------------------------------
    // test the ctor - calls the protected ctor of CStreamInterface -->
    //     CStreamInterface is tested completely

    ssock = new CStreamServer (addr);
    // addr_ = addr, stream = True, sid_ = 1 (probably)
    // sendMsgID=0, receiveMsgID=NoAccount (-1), timeout=SocketTimeout (2),
    // byteOrderReversed=False

    // tested on 06.05.97, 13:20: all tests okay


#if (0)
// note: currently, there is no means to test Accept(), because it depends
// on a connection to the client, where the client sends some data during
// the call to Accept()
// 29.07.98

    //----------------------------------------------------------------------
    // test Accept()

    accsock = ssock->Accept ();
    // accsock: addr = ssock->addr, stream = True
    // sendMsgID=0, receiveMsgID=NoAccount (-1), timeout=SocketTimeout (2),
    // byteOrderReversed=False

    // tested on 10.02.97, 19:10: all tests okay

    delete accsock;
#endif

    delete ssock;


    //----------------------------------------------------------------------
    // test the behaviour of class CStreamClient
    //    correct behaviour of CStreamInterface is assumed
    //----------------------------------------------------------------------

    CStreamClient *csock;


#if (0)
// note: currently, there is no means to test Connect(), because it depends
// on a connection to the server, where the server sends some data during
// the call to Connect()
// 29.07.98

    //----------------------------------------------------------------------
    // test the ctor - calls Connect()

    csock = new CStreamClient (addr);
    // addr_ = addr, stream = True, sid_ = 1 (probably)
    // sendMsgID=0, receiveMsgID=NoAccount (-1), timeout=SocketTimeout (2),
    // byteOrderReversed=False

    delete csock;

    csock = new CStreamClient (addr, 7);
    // addr_ = addr, stream = True, sid_ = 1 (probably)
    // sendMsgID=0, receiveMsgID=NoAccount (-1), timeout=SocketTimeout (2),
    // byteOrderReversed=False
    // Connect() is called with 7 (trace it!)

    // tested on 10.02.97, 19:10: all tests okay


    delete csock;
#endif


    //----------------------------------------------------------------------
    // test the behaviour of class CDatagramInterface
    //    correct behaviour of CSocket is assumed
    //----------------------------------------------------------------------

    CDatagramInterface *datagramifc;


    //----------------------------------------------------------------------
    // test the ctor

    s_create (addr, SDatagram, &sid);  // make a socket
    datagramifc = new CDatagramInterface (addr, sid);
    // addr_ = addr, sid_ = sid, timeout = SocketTimeout (2), stream_=False

    // tested on 06.04.97, 18:10: all tests okay


    //----------------------------------------------------------------------
    // test SendMessage()

    event = new SEvent;
    event->module = Network;
    event->type = MessageLost;
    event->time = 5;
    size = event->size;

    result = datagramifc->Send (event, size);  // return True

    // tested on 06.04.97, 18:10: all tests okay


    //----------------------------------------------------------------------
    // test ReceiveMessage()

    revent = new SEvent;  // allocate memory
    size = event->size+1;  // let's hope that does not lead to problems

    result = datagramifc->Receive (revent, &size);  // return True
    // size should be reduced by 1, revent=event

    // tested on 06.04.97, 18:10: all tests okay


    // note: CDatagramInterface is tested except for the internal ctor,
    // which is tested through a derived class below

    delete datagramifc;
    delete event;
    delete revent;


    //----------------------------------------------------------------------
    // test the behaviour of class CDatagramServer
    //    correct behaviour of CDatagramInterface is assumed
    //----------------------------------------------------------------------

    //----------------------------------------------------------------------
    // test the behaviour of class CDatagramServer
    //     correct behaviour of CDatagramInterface is assumed
    //----------------------------------------------------------------------

    CDatagramServer *dssock;


    //----------------------------------------------------------------------
    // test the ctor - calls the protected ctor of CDatagramInterface -->
    //     CDatagramInterface is tested completely

    dssock = new CDatagramServer (addr);
    // addr_ = addr, stream = False, sid_ = 1 (probably)

    // tested on 06.04.97, 18:10: all tests okay


    delete dssock;


    //----------------------------------------------------------------------
    // test the behaviour of class CDatagramClient
    //    correct behaviour of CDatagramInterface is assumed
    //----------------------------------------------------------------------

    CDatagramClient *dcsock;


    //----------------------------------------------------------------------
    // test the ctor

    dcsock = new CDatagramClient (addr);
    // addr_ = addr, stream = False, sid_ = 1 (probably)

    // tested on 06.04.97, 18:10: all tests okay


    delete dcsock;


    //----------------------------------------------------------------------

    return 0;
}


