//
// suptest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the supervisor class in supervis.hpp
// note: DoPeriodicActivities() and ReportError() were made public for
// this test
//
// created: 17.01.97
//
// last changed: 29.07.98
//


#include "..\clockifc.hpp"
#include "..\error.hpp"
#include "..\globals.hpp"
#include "..\lance.hpp"
#include "..\sockunix.hpp"
#include "..\structs.hpp"
#include "..\supervis.hpp"


extern CClock *g_masterClock;


int main()
{
    //----------------------------------------------------------------------
    // test the behaviour of the class CSupervisor
    // correct behaviour of classes CControllingObject, CClockInterface,
    // CClockSync and CLance is assumed
    //----------------------------------------------------------------------

    CSupervisor *supervisor;
    CUnixAddress addr ("dummy");
    SSingleParameter param;  // for setting parameters
    SEventReport report;  // for setting event report parameters
    INT8 mask[1];  // event mask
    EBool result;
    CLance *lance;
    CClockInterface *clock;
    CClockSync *cs;

    CheckAssumptions();
    InstallAlgorithms();  // must do this before creating the supervisor


    //----------------------------------------------------------------------
    // test ctor and Init()

    supervisor = new CSupervisor ("QUEU", &addr, 1, 0, 25e6);
    // nodeID = 1, area = 0, snapshot = False, qid=...

    supervisor = NULL;
    supervisor = listOfSupervisors.Find (1,0);  // should return the sup.

    // tested on 29.07.98, 15:20: all tests okay


    //----------------------------------------------------------------------
    // test GetIdentification()

    SINT32 nodeID = 10;
    int area = 10;

    supervisor->GetIdentification (nodeID, area);
    // nodeID = 1, area = 0

    // tested on 14.05.97, 19:50: all tests okay


    //----------------------------------------------------------------------
    // test GetClock(), GetClockSync(), GetLance()

    clock = supervisor->GetClock();
    cs = supervisor->GetClockSync();
    lance = supervisor->GetLance();
    // check that in all modules the pointers to each other are correct

    // tested on 31.03.97, 12:00: all tests okay


    g_masterClock = clock->GetClock();
    // install the master clock (needed for NoteEvent())


    //----------------------------------------------------------------------
    // test Suspend/ResumeOperation()

    supervisor->ResumeOperation ();
    // clock->UTCINTEN2 (bit 29) is enabled (scheduled for next multiple
    // of the roundperiod)

    supervisor->SuspendOperation ();
    // clock->UTCINTEN2 (bit 29) disabled, lance and clocksync are suspended

    // tested on 29.07.97, 12:00: all tests okay
    // (note: can't really be tested except by tracing)


    //----------------------------------------------------------------------
    // test Set/GetParameter()

    // call the Lance (has no params, so it should always return False)
    param.module = Lance;
    result = supervisor->SetParameter (param);  // return False
    result = supervisor->GetParameter (param);  // return False

    // set the delay measurement cycle in ClockSync
    param.module = ClockSync;
    param.type = DelayMeasurementCycles;
    param.value = 5;
    result = supervisor->SetParameter (param);  // return True

    // set the clock rate
    param.module = Clock;
    param.type = ClockDrift;
    param.value = 1e-6;
    result = supervisor->SetParameter (param);  // return True

    report.module = Supervisor;
    // set the event logging things
    mask[0] = (INT8)(1 << SystemReport);  // allow normal system reports
    report.Set (EventClassReport, mask, 1);
    result = supervisor->SetParameter (report);  // return True

    param.module = Network;
    result = supervisor->SetParameter (param);  // return False


    // get delay measurement cycle
    param.module = ClockSync;
    param.type = DelayMeasurementCycles;
    result = supervisor->GetParameter (param);  // return True
    // param.value = 5

    // get the clock rate
    param.module = Clock;
    param.type = ClockDrift;
    result = supervisor->GetParameter (param);  // return True
    // param.value = 1e-6

    report.module = Supervisor;
    // set the event logging things
    mask[0] = (INT8)(1 << SystemReport);  // allow normal system reports
    report.Set (SystemReport, False);
    result = supervisor->GetParameter (report);  // return True
    // report.set = True

    param.module = Network;
    result = supervisor->GetParameter (param);  // return False

    // tested on 14.02.97, 18:20: all tests okay


    //----------------------------------------------------------------------
    // test ExecuteCommand ()

    SCommand command;
    command.type = EnableSnapshot;
    command.module = Supervisor;
    result = supervisor->ExecuteCommand (command);  // return True
    // snapshot = True

    command.type = DisableSnapshot;
    result = supervisor->ExecuteCommand (command);  // return True
    // snapshot = False

    command.type = EnableSnapshot;
    result = supervisor->ExecuteCommand (command);  // return True
    // snapshot = True

    command.type = ResetModule;
    command.module = ClockSync;
    result = supervisor->ExecuteCommand (command);  // return True

    command.module = Clock;
    result = supervisor->ExecuteCommand (command);  // return True

    command.module = Lance;
    result = supervisor->ExecuteCommand (command);  // return True

    command.module = Network;
    result = supervisor->ExecuteCommand (command);  // return False

    command.module = Supervisor;
    result = supervisor->ExecuteCommand (command);  // return True
    // snapshot = False

    // tested on 29.03.97, 10:40: all tests okay

/*
    //----------------------------------------------------------------------
    // test DoPeriodicActivities()

    supervisor->DoPeriodicActivities();
    // should return at once [trace it!]

    command.type = EnableSnapshot;
    command.module = Supervisor;
    supervisor->ExecuteCommand (command);
    // snapshot = True

    supervisor->DoPeriodicActivities();
    // should call clock->InitSnapshot() [trace it!]

    // tested on 29.03.97, 10:40: all tests okay


    //----------------------------------------------------------------------
    // test ReportError()

    supervisor->ReportError (Reporting, __FILE__, __LINE__, "test");
    // output on screen: module = Supervisor, nodeID = 1, area = 0

    // tested on 10.02.97, 22:30: all tests okay
*/

    //----------------------------------------------------------------------
    // test NoteEvent()

    report.module = Supervisor;
    // set the event logging things
    mask[0] = (INT8)(1 << SystemReport);  // allow event class
    report.Set (EventClassReport, mask, 1);
    supervisor->SetParameter (report);
    mask[0] = (INT8)(1 << DeliverMessage);  // allow that event report
    report.Set (EventReport, mask, 1);
    supervisor->SetParameter (report);

    SApplEvent *event = new SApplEvent;
    event->module = ClockSync;
    event->type = DeliverMessage;
    event->nodeID = 50;  // just dummies, are overwritten
    event->area = 50;
    event->time = 50;

    supervisor->NoteEvent (event);
    // --> event: nodeID = 1, area = 0, time = 0, mastertime = 0

    // tested on 07.12.97, 19:10: all tests okay


    //----------------------------------------------------------------------
    // test dtor

    delete supervisor;

    supervisor = listOfSupervisors.Find (1,0);  // should return NULL

    // tested on 10.02.97, 22:00: all tests okay


    //----------------------------------------------------------------------

    return 0;
}

