//
// temptest.cpp
//
// this is the file used to test the classes defined in the files as far
// as possible
// *) the tests were done using a debugger to verify the correct outcome
// *) all classes were compiled and debugged using Watcom 10.5 for Windows
//
// test the derived classes of ACObjectList in listbase.hpp
// in this case, ACNodeList is tested; in addition, those functions that
// do not need the key are tested (p.e. DeleteAll())
// so in future classes, only the Compare() method will have to be tested;
// all other functions are tested in this file
//
// created: 02.11.96
//
// last changed: 05.11.96
//


#include "..\nodetemp.hpp"


//**************************************************************************
// class CNodeTest: a test class to test ACNodeList; the objects are of
//      type int*
//**************************************************************************

class CNodeTest : public ACNodeList
{
public:

    //----------------------------------------------------------------------
    //--- constructor, destructor
    //----------------------------------------------------------------------

    CNodeTest (INT32 aSize = ArrayAllocationSize) :
        ACNodeList (aSize) {};
    // allocate memory

    virtual ~CNodeTest () { DeleteAll(); }
    // the destructor deletes all objects in the array


    //----------------------------------------------------------------------
    //--- move in the array
    //----------------------------------------------------------------------

    int* GetFirst () { return (int*) ACObjectList::GetFirst(); }
    // get the first object in the array, or NULL if the array is empty

    int* GetLast () { return (int*) ACObjectList::GetLast(); }
    // get the last object or NULL if the array is empty

    int* GetNext () { return (int*) ACObjectList::GetNext(); }
    // get the next object in the array or the last object if the end of
    // the array has been reached

    int* GetPrev () { return (int*) ACObjectList::GetPrev(); }
    // return the previous object in the array or the first if the beginning
    // of the array has been reached

    int* Find (addr* netAddress)
        { return (int*) ACNodeList::Find (netAddress); }


    //----------------------------------------------------------------------
    //--- remove objects
    //----------------------------------------------------------------------

    int* Remove (addr* netAddress)
        { return (int*) ACNodeList::Remove (netAddress); }



    //----------------------------------------------------------------------
    //--- get information about the current element
    //----------------------------------------------------------------------

    int* GetCurrentObject ()
        { return (int*) ACObjectList::GetCurrentObject(); }
    // returns a pointer to the member that is currently being accessed or
    // NULL, if no element is being accessed


    //----------------------------------------------------------------------
    //--- delete one or all member(s) of the array
    //----------------------------------------------------------------------

    // delete an object; if the index exists, the object with this index
    // is deleted and True is returned; otherwise False is returned
    // current points to the position where the element was deleted (or
    // should have be inserted), or to the last element if the deleted
    // object was (would have been) the last
    EBool Delete (addr* netAddress)
    {
        int* object;
        EBool found = False;

        object = (int*) Remove (netAddress);  // remove the object

        if (object != NULL)  // index does exist
        {
            delete object;  // delete the object

            found = True;
        }

        if (!CurrentIsValid()) object = GetLast();
        // set current to the last entry if it is not valid

        return found;
    } // Delete
    
    // delete all members of the array
    // if the array is empty, False is returned; else all objects are
    // deleted and True is returned
    EBool DeleteAll ()
    {
        int* object;
        
        if (Empty()) return False;
        else
        {
            while (!Empty())
            {
                object = GetLast();  // get the last item in the array
                RemoveAtCurrent();  // remove the array entry
                delete object;  // delete the object
            }

            return True;
        }
    } // DeleteAll
};

 
//**************************************************************************
//**************************************************************************

int main()
{
    //----------------------------------------------------------------------
    // variables necessary for the test of SKey
    //----------------------------------------------------------------------

    addr testAddress;
    SKey *key, *cmpKey;
    int cmpResult;

    for (int i=0; i<ADDRCNT; i++) testAddress[i] = i;

    key = new SKey (&testAddress);  // make a key with {0,1,2,3,4,5}

    testAddress[3] = 5;
    cmpKey = new SKey (&testAddress);  // make a key with {0,1,2,5,4,5}
    
    cmpResult = key->CompareTo (key);  // should return 0
    cmpResult = key->CompareTo (cmpKey);  // should return -1
    cmpResult = cmpKey->CompareTo (key);  // should return +1


    // tested on 05.11.96, 21:05: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of ACNodeList
    // correct behaviour of SKey is assumed
    //----------------------------------------------------------------------

    CNodeTest *list;
    int *obj1 = new int(1);
    int *obj2 = new int(2);
    addr key1, key2;
    key1[0] = 1;
    key2[0] = 2;
    int *testobj;
    addr testadr;
    EBool result;


    //----------------------------------------------------------------------
    // test the ctor (default parameter)

    list = new CNodeTest;
    // allocSize = ArrayAllocationSize (16)

    delete list;
    list = new CNodeTest(2);
    // allocSize = 2


    //----------------------------------------------------------------------
    // test Find()

    testobj = list->Find (&key1);  // should return NULL

    // Find() tested completely


    //----------------------------------------------------------------------
    // test GetCurrentObjectKey()

    result = list->GetCurrentObjectKey (&testadr);  // should return False


    //----------------------------------------------------------------------
    // test Insert()

    result = list->Insert (obj1, &key1);  // should return True
    // current=0, firstFree=1

    result = list->Insert (obj1, &key1);  // should return False
    // note: Compare with same value is tested as well

    // Insert() tested completely


    //----------------------------------------------------------------------
    // test GetCurrentObjectKey() - continued

    result = list->GetCurrentObjectKey (&testadr);  // should return True
    // testadr[0] = 1

    // GetCurrentObjectKey() tested completely


    //----------------------------------------------------------------------
    // test Remove()

    testobj = list->Remove (&key1);  // should return obj1


    //----------------------------------------------------------------------
    // test Compare()

    list->Insert (obj1, &key1);
    list->Insert (obj2, &key2);
    // note: Compare is tested with key1 < key2

    list->Remove (&key1);
    list->Remove (&key2);

    list->Insert (obj2, &key2);
    list->Insert (obj1, &key1);
    // note: Compare is tested with key2 > key1

    // Compare() tested completely


    // tested on 05.11.96, 21:14: all tests okay


    //----------------------------------------------------------------------
    // variables necessary for the test of CTemplate (the functions that
    // don't use the key)
    // correct behaviour of ACNodeList is assumed
    //----------------------------------------------------------------------


    //----------------------------------------------------------------------
    // test the move and information functions

    testobj = list->GetFirst();  // should return obj1
    testobj = list->GetLast();  // should return obj2
    testobj = list->GetPrev();  // should return obj1
    testobj = list->GetNext();  // should return obj2

    testobj = list->GetCurrentObject();  // should return obj2


    //----------------------------------------------------------------------
    // test the delete functions

    result = list->Delete (&key2);  // should return True
    // current=0, firstFree=1
    result = list->Delete (&key2);  // should return False

    result = list->DeleteAll ();  // should return True
    // current=firstFree = 0
    result = list->DeleteAll ();  // should return False


    // tested on 05.11.96, 21:15: all tests okay


    delete list;

    
    //----------------------------------------------------------------------

    
    return 0;
}
    


