//
// test.cpp
//
// for all kinds of tests that can't be done in another testfile
//
// *) test the handling of the bufx typedef
// *) test the necessity of virtual dtors
// *) test my own itoa() function for Linux
// *) test the handling of pointers to functions
// *) test the handling of class interrupt functions (using friends)
// *) test iostreams
// *) find out the error strings via strerror()
// *) check if sizeof() works correctly with classes
//
// created: 26.10.96
//
// last changed: 11.04.97
//


#include <malloc.h>  // for the heap functions
#include <iostream.h>
#include <strstream.h>  // for own streams
#include <string.h>  // for strerror()

#include "..\all.h"


//--------------------------------------------------------------------------
// needed to test the sizeof problem
class S1 { public: int i; virtual int size() { return sizeof(S1); }
};

struct S2 { public: int i; int size() { return sizeof(S2); }
};

class S3 { public:
    int i;
    virtual int size() { return (sizeof(*this) - sizeof(void*)); }
    // subtract the size of the pointer to the VMT
};

class S4 : public S3 { public:
    int j;
    virtual int size() { return (sizeof(*this) - sizeof(void*)); }
    // again subtract the size of the pointer to the VMT
};

class S5 { public:
    int i;
    virtual int Size() { return sizeof(*this); }
};

class S6 : public S5 { public:
    int j;
};

class S7 : public S5 { public:
    int j;
    virtual int Size() { return sizeof(*this); }
};

//--------------------------------------------------------------------------
// needed to test the virtual dtor problem (see below)
struct A { int i; };
struct B : public A { int j; int k; };


//--------------------------------------------------------------------------
// test itoa()
char *itoa (int value, char *buffer, int radix);


//--------------------------------------------------------------------------
// test handling of ptrs to functions
void isr() { cout << "isr was called" << endl; }


typedef void (*IsrPointer)();

IsrPointer isrTable[1];
// for storing the pointer

int SetISR (IsrPointer theISR)
{
    isrTable[0] = theISR;
    return 0;
}

IsrPointer GetISR (int vector)
{
    return isrTable[0];
}


//--------------------------------------------------------------------------
// test handling of isr as a friend

class CInterruptable
{
public:
    friend void ClassISR (CInterruptable *obj);

private:
    virtual void isr() =0;
};

void ClassISR (CInterruptable *obj)
{
    obj->isr();  // call the isr of the object
}

class A1 : public CInterruptable
{
public:
    A1 () { i = 0; }
private:
    virtual void isr() { i=3; }
    int i;
};

class A2 : public CInterruptable
{
public:
    A2 () { j = 0; }
private:
    virtual void isr() { j=5; }
    int j;
};


//--------------------------------------------------------------------------
// test iostream

class CMyStreamBuf : public streambuf
{
public:
    CMyStreamBuf (char *p, int l) : streambuf (p, l) {}

    virtual int overflow (int c = EOF)
    {
        cerr << "overflow was called" << endl;
        return EOF;
    }

    virtual int underflow ()
    {
        cerr << "underflow was called" << endl;
        return EOF;
    }
};


//--------------------------------------------------------------------------

int main()
{
    //----------------------------------------------------------------------
    // test sizeof()

    int size1 = sizeof (A);
    int size2 = sizeof (S1);
    int size3 = sizeof (S2);
    // should be equal, even though there should be a compiler-warning
    // in case of S1
    // okay, result: size1 and size3 are equal, size2 is 4 bytes greater
    // because it also contains the pointer to the VMT (I guess)

    // tested on 31.01.97, 09:30: all tests okay


    S3 *ob;
    S3 *ob3 = new S3; ob3->i = 3;
    S4 *ob4 = new S4; ob4->i = 4; ob4->j = 5;
    S4 *ob4a = new S4; ob4a->i = 0; ob4a->j = 0;

    ob = ob4;
    int s3 = ob3->size();  // should return 4
    int s4 = ob4->size();  // should return 8
    int s = ob->size();  // should return 8

    memcpy (ob4a, ob4, ob4->size());
    // ob4a: i=4, j=5
    // okay, result: i=4, j is untouched; the reason for this is that
    // the object contains the following variables: i, ptr to VMT, j,
    // ptr to VMT (watch->Compiler); so just i and the first ptr are
    // copied --> need a virtual function that copies all members of the
    // class into a buffer!

    // tested on 31.01.97, 10:00: all tests okay


    memcpy (ob4a, ob4, sizeof(*ob4));
    // ob4a: i=4, j=5

    // tested on 01.02.97, 09:10: all tests okay


    S5 *ob5 = new S5;
    S5 *ob5a =new S6;
    S5 *ob5b = new S7;

    size1 = ob5->Size();
    size2 = ob5a->Size();
    size3 = ob5b->Size();
    // hrmpf, result: size1=size2 = 8, size3 = 12; conclusion: every
    // derived class must overload the Size() function
    
    // tested on 01.02.97, 12:30: all tests okay


    size1 = sizeof(*ob5);
    size2 = sizeof(*ob5b);
    size3 = sizeof(*((S7*)ob5b));
    // hrmpf, result: size1=size2 = 8, size3 = 12; conclusion: I need the
    // Size() function in the classes, because 'sizeof' does not recognize
    // the real size of the variable

    // tested on 07.02.97, 17:20: all tests okay


    //----------------------------------------------------------------------
    // get a list of errors

    for (int i=0; i<50; i++)
        cout << i << ": " << strerror (i) << endl;

    // tested on 30.01.97, 19:40: errors are equal to those in the CLib Help


    //----------------------------------------------------------------------
    // want to test this bufx - typedef

    bufx b1;  // this is an array?? yes!

    b1[0] = 0;  // should be okay

    bufx *p = &b1;  // is that okay? well, it compiles with a warning...
    // ... but yes, it is okay

    (*p)[2] = 3;
    // p is a pointer to the array, so first you have to follow this
    // pointer to the array; then you can select a member using []


    // well, and now let's try to do something a bit more complicated...

    bufx *b2 = (bufx *)(new bufx);  // why is it necessary to cast?
    // because new always returns a pointer to the first element of the
    // array ("new bufx" allocates an array, not a single object of type
    // bufx)  (?)

    (*b2)[1] = 10;

    // tested on 26.10.96, --:--: all tests okay


    //----------------------------------------------------------------------
    // want to test if a virtual dtor is needed in the following situation

    A *a;
    B *b = new B;
    B *c = new B;
    a = b;  // use a to store a B*, delete a (see below)

    struct _heapinfo entry;  // watch that structure
    entry._pentry = NULL;
    int result;
    
    do  // go through the heap
    {
        result = _nheapwalk (&entry);
        // find the entry for ptr B, its size, and remember the next and
        // previous entries!
    } while (result != _HEAPEND);

    delete a;  // is the memory of B deallocated?
    // step into the assembler code, try to find out if all allocated
    // words are freed

    // okay, result: yes, everything is freed; obviously the size is stored
    // on the heap (there is a word before the structure which seems to
    // contain this information), and all memory for a pointer to this
    // address is freed (regardless of the type of the pointer); so the
    // moral is: virtual dtors in a class T are only needed if T::new was
    // overloaded to do something special, or if there are dynamic members
    // that have to be deallocated and it is important that the dtor of
    // a derived class is called if delete is called for a ptr to a base
    // class
    // virtual dtors are *not* needed in a case like the above, where there
    // are no class-specific actions to take on destruction


    // tested on 15.01.97, 16:30: all tests okay

    delete b;


    //----------------------------------------------------------------------
    // want to test my own itoa() function that isn't in the stdlib under
    // Linux

    char buffer[40];
    int base;
    char *retvalue;

    retvalue = itoa (0, buffer, 10);  // retvalue = "0"

    for (base = 2; base <= 40; base = base + 2)
    {
        retvalue = itoa (12765, buffer, base);
        cout << base << ": ";
        if (retvalue != NULL) cout << buffer;
        else cout << "not implemented!";
        cout << endl;
    }
    // should produce the following output:
    //       WATCOM atoi()      |   own implementation of atoi()
    //--------------------------|----------------------------------
    // 2: 11000111011101        |   same as WATCOM
    // 4: 3013131               |   same as WATCOM
    // 6: 135033                |   same as WATCOM
    // 8: 30735                 |   same as WATCOM
    // 10: 12765                |   same as WATCOM
    // 12: 7479                 |   same as WATCOM
    // 14: 491b                 |   same as WATCOM
    // 16: 31dd                 |   same as WATCOM
    // 18: 2373                 |   same as WATCOM
    // 20: 1bi5                 |   same as WATCOM
    // 22: 1485                 |   same as WATCOM
    // 24: m3l                  |   same as WATCOM
    // 26: imp                  |   same as WATCOM
    // 28: g7p                  |   same as WATCOM
    // 30: e5f                  |   same as WATCOM
    // 32: cet                  |   same as WATCOM
    // 34: b1f                  |   same as WATCOM
    // 36: 9ul                  |   same as WATCOM
    // 38: 8vz                  |   38: not implemented
    // 40: 7W5                  |   40: not implemented

    // tested on 11.04.97, 18:15: all tests okay


    //----------------------------------------------------------------------
    // test the handling of pointers to functions

    IsrPointer test;

    SetISR (isr);
    test = GetISR (0);
    (*test)();  // call test
    // expected output on screen: isr was called
    // ... and it *works*!

    // tested on 18.01.97, 22:20: all tests okay


    //----------------------------------------------------------------------
    // test handling of isr as a friend

    A1 obj1, obj2;
    A2 obj3;
    CInterruptable *vectable[3];
    vectable[0] = &obj1;
    vectable[1] = &obj2;
    vectable[2] = &obj3;

    ClassISR (vectable[0]);
    // obj1::i = 3

    ClassISR (vectable[1]);
    // obj2::i = 3

    ClassISR (vectable[2]);
    // obj3::j = 5

    // tested on 19.01.97, 13:40: all tests okay


/*
    //----------------------------------------------------------------------
    // test iostrstreams

    istrstream is (buffer, 40);
    ostrstream os (buffer, 40);  // hope that's correct?

    strcpy (buffer, "");  // clear the buffer

    os << "test" << endl;
    // buffer should contain "test"

    is >> b1;
    // b1 should contain "test", buffer should be empty(?)
    // well: b1 contains test, but buffer is *not* empty

    b1[0] = 0;
    is >> b1;
    // hm, that did not change b1

    buffer[0] = 'r';
    is >> b1;

    os << "okay" << endl;
    is >> b1;
    // okay, resume: after the first string has been read (\0 has been
    // encountered), is does nothing; so it can't be used

    // tested on 28.01.97, 11:30: all tests okay
*/


    //----------------------------------------------------------------------
    // test iostream

    CMyStreamBuf mybuf (buffer, 40);

    istream is (&mybuf);
    ostream os (&mybuf);  // hope that's correct?
    is.tie (&os);

    os << "test" << endl;

    is >> b1;
    // b1 should contain "test"

    // well: b1 does not contain anything, because overflow() and
    // underflow() were called; would have to implement some strategie
    // in those functions to make it work
    // --> forget it

    // tested on 28.01.97, 11:50: tests *not* okay


    //----------------------------------------------------------------------

    return 0;
}


// converts the value "value" in base "radix" to a string and writes the
// result into buffer; a pointer to that buffer is returned
// if radix < 2 or radix > 36, NULL is returned and nothing happens
char *itoa (int value, char *buffer, int radix)
{
    int index = 0;  // array index
    int mult = 1;  // for internal computations
    int res;  // result of division through mult

    if ((radix < 2) || (radix > 36)) return NULL;

    if ((radix == 10) && (value < 0))  // put a "-" as first item
    {
        buffer[index] = '-';
        index++;
    }
    if (value < 0) value = -value;  // make value a positive number

    if (value == 0)  // the routine below is not executed
    {
        buffer[index] = '0';
        index++;
    }

    // first find out the highest place
    while (value/mult > 0) { mult = mult*radix; }

    // okay, value is a multiple of mult/radix
    while ((mult = mult/radix) > 0)
    {
        res = value/mult;
        if (res < 10)
        {
            buffer[index] = (char)((int)'0' + res);
        }
        else
        {
            buffer[index] = (char)((int)'a' + (res-10));
        }
        index++;
        value = value - res*mult;
    }
    buffer[index] = '\0';
    return buffer;
}



