//
// classman.cpp
//
// provides facilities for dynamic installation of network and algorithm
// classes
//
// created: 02.07.97
//
// last change: 31.07.97
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 10:49:31 $
//


#include "classman.hpp"
#include "error.hpp"


//**************************************************************************
// class CClassManager: provides dynamic installation of classes derived
//      from a certain base class
//      you need to specify an identifier and a creation function which
//      should return an object of the class you want to install
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

// create an array with the given size
// assert: arraySize > 0
CClassManager::CClassManager (INT32 arraySize)
{
    Assert (arraySize > 0);

    size_ = arraySize;  // store the size
    createFunction_ = new INT32* [size_];  // create the array
    Assert (createFunction_ != NULL);

    for (INT8 i=0; i < size_; i++) createFunction_[i] = NULL;
    // set all entries in the array to NULL
}

// delete the array
CClassManager::~CClassManager ()
{
    delete[] createFunction_;  // delete the array
}


//--------------------------------------------------------------------------
//--- insert/return function pointers
//--------------------------------------------------------------------------

// insert a function with the identifier 'id'
// assert: id < size
void CClassManager::SetFunction (INT32 id, INT32 *functionPointer)
{
    Assert (id < size_);
    createFunction_[id] = functionPointer;
}

// return the function that was stored for the identifier 'id', or
// NULL if no function was stored
INT32 *CClassManager::GetFunction (INT32 id)
{
    if (id >= size_) return NULL;
    return createFunction_[id];
}


