//
// clockbas.cpp
//
// only needed for setting the clock template
//
// created: 19.02.97
//
// last change: 14.08.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:45:47 $
//


#include <stdlib.h>

#include "clockbas.hpp"
#include "constant.hpp"
#include "structs.hpp"
#include "error.hpp"


CClockTemplate *clockTemplate = new CClockTemplate;
// set the template; in order to use it, one has to implement its NewClock()
// function



//**************************************************************************
// class CClock: interface to the clock (both hardware and software);
//      provides functions for accessing all registers of the clock; could
//      be used as a hardware clock, but a derived class is used for that
//**************************************************************************

static const double TwoPow56 = 4294967296.*16777216.;
static const double TwoPow16 = 65536.;


//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

CClock::CClock (double f, SUtcsuRead *read, SUtcsuWrite *write)
{
    Assert (read != NULL);
    Assert (write != NULL);
    Assert (f > 0);
    utcsuRead_ = read;
    utcsuWrite_ = write;
    utcinten1_ = 0;  // assume that no interrupts are enabled
    utcinten2_ = 0;
    oscillatorFrequency_ = f;
    granularity_ = DefaultClockGranularity;
    granularityAcc_ = DefaultAccGranularity;
    granularityDT_ = 1/TwoPow16;  // granularity of the duty timer
    granularityOsc_ = 1/f;
    granularityS_ = 1/TwoPow56;  // internal clock setting granularity
}


//--------------------------------------------------------------------------
//--- configuration of system parameters
//--------------------------------------------------------------------------

// sets the given parameter to its new value; if the parameter does
// not exist or if the value is not valid, False is returned
// known params: ClockGranularity, AccuracyTransmissionLoss
EBool CClock::SetParameter (const SParameter& param)
{
    switch (param.type)
    {
        case ClockGranularity:
            if (((SSingleParameter*)&param)->value <= 0) return False;
            granularity_ = ((SSingleParameter*)&param)->value;
            break;
        case AccuracyTransmissionLoss:
            if (((SSingleParameter*)&param)->value <= 0) return False;
            granularityAcc_ = ((SSingleParameter*)&param)->value;
            break;

        default:  // unknown parameter
            return False;
    } // switch

    return True;
}

// returns the value of the given parameter in the appropriate field of
// SParameter; if the parameter does not exist, False is returned
EBool CClock::GetParameter (SParameter& param) const
{
    switch (param.type)
    {
        case ClockGranularity:
            ((SSingleParameter*)&param)->value = granularity_;
            break;
        case AccuracyTransmissionLoss:
            ((SSingleParameter*)&param)->value = granularityAcc_;
            break;

        default:  // unknown parameter
            return False;
    } // switch

    return True;
}


//--------------------------------------------------------------------------
//--- enable/disable single interrupt bits
//--------------------------------------------------------------------------

// enables all set bits of mask1 in UTCINTEN1, and all bits of mask2
// in UTCINTEN2; the bits that are not set in the mask keep their old
// state; the function always returns True
EBool CClock::EnableInterrupt (int, INT32 mask1, INT32 mask2)
{
    utcinten1_ |= mask1;  // add new interrupts to the masks
    utcinten2_ |= mask2;

    SetUTCINTEN1 (utcinten1_);  // enable the interrupts
    SetUTCINTEN2 (utcinten2_);

    return True;
}

// disables all set bits of mask1 in UTCINTEN1, and all bits of mask2
// in UTCINTEN2; the bits that are not set in the mask keep their old
// state; the function always returns True
EBool CClock::DisableInterrupt (int, INT32 mask1, INT32 mask2)
{
    utcinten1_ &= ~mask1;  // remove interrupts from the masks
    utcinten2_ &= ~mask2;

    SetUTCINTEN1 (utcinten1_);  // disable the interrupts
    SetUTCINTEN2 (utcinten2_);

    return True;
}



