//
// err2eval.cpp
//
// implementation of the error report class used for storing and processing
// error reports
// note: this implementation sends the error messages to a datagram socket
//
// created: 31.03.97
//
// last change: 11.03.98
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 10:49:31 $
//


#ifdef PSOS_SIM
  #include <psos.h>  // for k_fatal()
  #define ABORTFUNC k_fatal
  #define ABORTPARAMS 0x10000, K_LOCAL
  // first param is the error code, second the flags (only local node is
  // shutdown
#else
  #include <stdlib.h>  // for abort()
  #define ABORTFUNC abort
  #define ABORTPARAMS
  // abort does not have any parameters
#endif
#include <string.h>  // for strlen()

#include "constant.hpp"
#include "err2eval.hpp"
#include "sockdgrm.hpp"

#define SocketCreateFailed \
    "CErrorEval::CErrorEval(): creation of the socket has failed"


//**************************************************************************
// class CErrorEval: stores strings (error messages) in a queue and
//      periodically sends them to the evaluation
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

// create a datagram (client) socket
CErrorEval::CErrorEval (char qname[4], ACSocketAddress *addr)
    : CErrorTask (qname)
{
    socket_ = new CDatagramClient (addr);
    if (socket_ == NULL)
    {
        EmergencyProcessMessage (SocketCreateFailed);
        ABORTFUNC (ABORTPARAMS);  // stop program execution
        // note: if the socket is ==NULL, the message can't be sent, but at
        // least program execution is aborted (and maybe output to the
        // screen is possible)
    }
}

// delete the socket, emergency flush the queue
CErrorEval::~CErrorEval()
{
    Flush();  // try to empty the queue
    EmergencyFlush();  // messages still in queue are processed differently
    // note: since the error reports may come from a faulty socket,
    // redirect the emergency flush to somewhere else
    delete socket_;
}


//--------------------------------------------------------------------------
//--- internal message output
//--------------------------------------------------------------------------

// send the message to the evaluation
void CErrorEval::ProcessMessage (char *string)
{
    int i=0;

    // transmit the message
    while ((i<MaxTrials) && !socket_->SendMessage (string, strlen (string)+1))
        i++;
    // note: the order of the conditions is important!

    if (i>=MaxTrials) EmergencyProcessMessage (string);
    // could not send the message -> at least print it to the screen, if
    // possible
}

// send the message to the screen (using the ProcessMessage() function of
// CErrorTask)
void CErrorEval::EmergencyProcessMessage (char *string)
{
    CErrorTask::ProcessMessage (string);
    // print the message to the screen
}


