//
// ftoa.cpp
//
// implementation of ftoa() using gcvt()
//
// note: a comparison of gcvt() and intended ftoa() output
//       (for width 14):
//           ftoa =   1.350000e+01, gcvt = 13.5
//           ftoa =   0.000000e+00, gcvt = 0
//           ftoa =  -1.000000e+00, gcvt = -1
//           ftoa =  -2.900000e+00, gcvt = -2.9
//           ftoa =  -2.960000e+01, gcvt = -29.6
//           ftoa =  -1.000000e-57, gcvt = -1.e-57
//           ftoa =   1.000000e+36, gcvt = 1.e+36
//           ftoa =  -2.000000e+01, gcvt = -20
//           ftoa =   5.300000e+01, gcvt = 53
//           ftoa =   0.000000e+00, gcvt = 0
//
// created: 31.08.98
//
// last change: 07.10.98
//
// RCS version control information:
//     $Revision: 1.3 $$State: Exp $$Date: 1999/01/28 17:52:19 $
//


#include <stdio.h>
#include <stdlib.h>  // for gcvt()
#include <string.h>  // for strcat() ...

const int Precision = 8;
// number of digits after comma


// returns 'f' as a string with at least 'width' characters in %e format; 
// stores the string in 'string', which must be big enough to hold the 
// converted string; if 'string' is NULL, NULL is returned
char *ftoa (double f, int width, char *string)
{
    if (string == NULL) return NULL;
    // let the user decide what to do in this case

    char buf[50];  // temporary string buffer
    char *p;  // pointer in the string buffer
    int prec=0;  // numbers behind the comma; must be signed!
    int exponent=0;  // adder to exponent
    int sign=0;  // 0 for positive, 1 for negative values
    int origlen=0;  // original string length

    gcvt (f, width, buf);  // convert the float to a string
    strcpy (string, buf);  // copy the whole string

    // check if there is a sign
    if (string[0] == '-') sign = 1;  // store the sign
    else sign = 0;

    origlen = strlen (string);  // store original string length

    // now check the result of gcvt() and modify it as required
    if ((p=strchr (string, '.')) == NULL)  // integer value
    // the string looks like "-3nn\0" or like "-3\0"
    {
        exponent = origlen-1-sign;  // store the exponent
        // (the exponent is the #digits after the comma)

        if (origlen > 1+sign)  // we have to shift data to insert '.'
        // the string looks like "-3nn\0", not like "-3\0"
        {
            memmove (string+2+sign, string+1+sign, origlen-sign);
            // this moves the '\0' as well
        }

        string[1+sign] = '.';  // insert the comma
        // now it looks like "-3.nn\0" or like "-3."

        if (origlen < 2+sign) string[2+sign] = '\0';  // terminate string
        // now it looks like "-3.nn\0" or like "-3.\0"

        // now pad with zeros to get a precision of at least 'Precision'
        for (prec=strlen(string)-2-sign; prec<Precision; prec++)
             string[2+sign+prec] = '0';
        // now it looks like "-3.nn0000" or like "-3.000000"

        // overwrite everything after the 'Precision'-th digit after comma
        // note: the floating point value is always rounded down!
        string[Precision+2+sign] = 'e';
        string[Precision+3+sign] = '+';
        string[Precision+4+sign] = '0'+(exponent/10)%10;
        // the % is in case exponent>99
        string[Precision+5+sign] = '0'+exponent%10;
        string[Precision+6+sign] = '\0';
        // note: if exponent >= 100, then it is cut off!
        // now it looks like "-3.nn0000e+00" or like "-3.000000e+00"
    } // if
    else  // it has at least a comma
    // the string looks like "-3n.nn\0" or like "-3.e-2\0" or like "-.e-2\0"
    {
        exponent = 0;  // initialize exponent

        if (p==string+sign)  // nothing before comma?
        {
            // move the comma after first non-zero digit
            while (p[exponent+1] == '0') exponent++;
            // note: this assumes that no string looks like ".0" or like ".e3"!

            *p = p[exponent+1];  // copy first non-zero digit
            p[1] = '.';  // store the comma
            memmove (p+2, p+exponent+2, strlen(p+exponent+2)+1);
            // copy rest of string behind comma
            p++;  // p is again on the comma
            exponent = -exponent-1;  // we need a negative value and we have
            // to include the first non-zero digit in the exponent
        } // if

        if (p > string+sign+1)  // more than one digit before the comma?
        {
            // move the comma
            exponent = p-(string+sign+1);  // store # new digits behind comma
            memmove (string+sign+2, string+sign+1, exponent);
            string[sign+1] = '.';
        } // if

        // now the number looks like "-3.nnn\0" or "-3.e-2\0" or "-0.e-2\0"
        if ((p = strchr (string, 'e')) != NULL)  // there is an exponent
        // the number looks like "-3.e-2\0" or "-0.e-2\0"
        {
            if (p[1] == '-') prec = -1; // exponent is negative!
            else prec = 1;
            // 'prec' is used to extract the exponent in the loop below

            // get the additional exponent
            for (p=string+strlen(string)-1;
                 (*p!='e') && (*p!='-') && (*p!='+');  // until we find 'e,-,+'
                 p--, prec=prec*10)
                exponent += (int)(*p - '0') * prec;

            // remove the 'e', we do not need it anymore
            if (*p != 'e') p--;  // skip the '-' or '+'
            *p = '\0';  // terminate the string on the 'e'
        }  // if


        // the number looks like "-3.nnn\0"
        // now pad with zeros to get a precision of at least 'Precision'
        for (prec=strlen(string)-2-sign; prec<Precision; prec++)
             string[2+sign+prec] = '0';

        // overwrite everything after the Precision-th digit after comma
        // note: the floating point value is always rounded down!
        string[Precision+2+sign] = 'e';
        if (exponent < 0)
        {
            string[Precision+3+sign] = '-';
            exponent = -exponent;
        }
        else string[Precision+3+sign] = '+';
        string[Precision+4+sign] = '0'+(exponent/10)%10;
        // the % is in case exponent>99
        string[Precision+5+sign] = '0'+exponent%10;
        string[Precision+6+sign] = '\0';
        // note: if exponent >= 100, then it is cut off!
        // now it looks like "-3.nnn000e+01"
    } // else

    if (strlen (string) < width)  // we have to add spaces in front
    {
        prec = width-strlen(string);
        memmove (string+prec, string, strlen(string)+1);
        for (; prec>0; prec--) string[prec-1] = ' ';
    }

    return string;
}


