//
// general.cpp
//
// definitions of the classes and functions declared in general.hpp
//
// created: 09.07.96
//
// last change: 02.09.98
//
// RCS version control information:
//     $Revision: 1.9 $$State: Exp $$Date: 1998/09/02 14:08:47 $
//


#include <stdlib.h>  // for NULL

#include "psos.h"

#include "distrib.hpp"
#include "error.hpp"
#include "general.hpp"
#include "process.hpp"


//**************************************************************************
// global functions for dis-/enabling interrupts
//**************************************************************************

static const INT32 InterruptMask = 0x0700;
// for modification of the interrupt level using t_mode()

// disables all interrupts, returns old value of the task mode
INT32 DisableInterrupts ()
{
    INT32 oldmode;

//    t_mode (InterruptMask, (INT32)GetClockIntLevel()<<8, &oldmode);
/**/// note: replace this with 0x0700 if all interrupts should be masked out

    t_mode (InterruptMask, InterruptMask, &oldmode);  // mask out all INTs

    return oldmode;
}

// set interrupt mask to the 'tmode' value
void EnableInterrupts (INT32 tmode)
{
    INT32 oldmode;
    t_mode (InterruptMask, tmode, &oldmode);
    // set the (interrupt bits of the) mask to the given value
}


//**************************************************************************
// global random functions
//**************************************************************************

// returns True with the probability prob, False otherwise
// assert: the probability must be in [0, 1]
EBool CastDice (double prob)
{
    static CUniform *stream = new CUniform (0, 1);
    Assert ((prob >= 0) && (prob <= 1));  // prob in [0,1]
    if ((*stream)() <= prob) return True;
    else return False;
}


//**************************************************************************
// convert integer to string
//**************************************************************************

#ifndef DOS_SIM
// itoa() is not ANSI, so it is only available when using Watcom (DOS)

// converts the value "value" in base "radix" to a string and writes the
// result into buffer; a pointer to that buffer is returned
// if radix < 2 or radix > 36, NULL is returned and nothing happens
char *itoa (int value, char *buffer, int radix)
{
    int index = 0;  // array index
    int mult = 1;  // for internal computations
    int res;  // result of division through mult

    if ((radix < 2) || (radix > 36)) return NULL;

    if ((radix == 10) && (value < 0))  // put a "-" as first item
    {
        buffer[index] = '-';
        index++;
    }
    if (value < 0) value = -value;  // make value a positive number

    if (value == 0)  // is not covered by the algorithm below
    {
        buffer[index] = '0';
        index++;
    }

    // first find out the highest place
    while (value/mult > 0) { mult = mult*radix; }

    // okay, value is a multiple of mult/radix
    while ((mult = mult/radix) > 0)
    {
        res = value/mult;
        if (res < 10)
        {
            buffer[index] = (char)((int)'0' + res);
        }
        else
        {
            buffer[index] = (char)((int)'a' + (res-10));
        }
        index++;
        value = value - res*mult;
    }
    buffer[index] = '\0';
    return buffer;
}

#endif


