//
// globals.cpp
//
// definition of global items that are needed both in pure and
// hardware-based simulation
//
// note: for adding new algorithm types, see the comments to function
//       InstallAlgorithms() below
//
// note: if operating in pure simulation, see file sim_glob.cpp for the
//       installation of networks
//
// created: 26.10.96
//
// last change: 19.01.99
//
// RCS version control information:
//     $Revision: 1.3 $$State: Exp $$Date: 1999/01/19 14:41:17 $
//


#include "algfetz.hpp"
#include "algfree.hpp"
#include "algfta.hpp"
#include "algop.hpp"
#include "algmarz.hpp"
#include "algrate.hpp"
#include "algfti.hpp"
#include "classman.hpp"
#include "clockbas.hpp"
#include "globals.hpp"
#include "gpsbase.hpp"
#include "template.hpp"


//**************************************************************************
// global variables
//**************************************************************************

CAlgorithmManager algorithmManager;
// the algorithm manager that returns the creation function of a certain
// clock synchronization algorithm

CClock *g_masterClock = NULL;
// the master clock
// when porting to hardware, the controller should set it to a value
// different from NULL to use function GetGlobalTime()

int g_masterArea = NoAccount;
// the area of the master clock
// when porting to hardware, the controller should set it to a value
// within [0, NumberOfAreas-1]



//**************************************************************************
// global object lists
//**************************************************************************

CSupervisorList listOfSupervisors;
// list of all supervisors in the system



//**************************************************************************
// initialization functions
//**************************************************************************

// installs all clock synchronization algorithm types known in the system
// note: if you wish to add a new algorithm type, you must insert its creation
// function into the algorithm manager; just copy the two lines that are
// commented out below, and replace 'YourAlgorithmType' with the enumeration
// constant of your algorithm (you must add one in enum EAlgorithmType in
// file general.hpp), 'YourClassName' with the name of your algorithm
// class (you must have derived one from class ACClockSyncAlgorithm in file
// algbase.hpp) and 'CreateObject' with the name of your create function
// (you must supply your class with such a static function)
void InstallAlgorithms()
{
    algorithmManager.SetFunction (FreeRun, CFreeRun::CreateFreeRun);
    algorithmManager.SetFunction (FaultTolerantAverage, CFta::CreateFta);
    algorithmManager.SetFunction (Midpoint, CMidpoint::CreateMidpoint);
    algorithmManager.SetFunction (Fetzer, CFtaWithFetzer::CreateFtaWithFetzer);
    algorithmManager.SetFunction (OP, COP::CreateOP);
    algorithmManager.SetFunction (Marzullo, CMarzullo::CreateMarzullo);
    algorithmManager.SetFunction (Rate, CRate::CreateRate);
    algorithmManager.SetFunction (FaultTolerantIntersection, CFti::CreateFti);

//    algorithmManager.SetFunction (YourAlgorithmType,
//                                  YourClassName::CreateObject);
}


// checks all (or most) of the assumptions that the simulation makes about
// its global items
void CheckAssumptions()
{
    Assert (clockTemplate != NULL);
    Assert (gpsTemplate != NULL);
}



//**************************************************************************
// tidy up functions
//**************************************************************************

// deletes the templates, sets them to NULL; this function must be called
// once at the end of program execution, otherwise the memory will never
// be freed
void DeleteTemplates()
{
    delete clockTemplate;
    clockTemplate = NULL;

    delete gpsTemplate;
    gpsTemplate = NULL;
}

