//
// netgen.cpp
//
// some global functions that are needed when dealing with node identifiers
// and network addresses
// note: put the functions into sim_gen.cpp
//
// created: 18.11.96
//
// last change: 25.12.96
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 10:49:31 $
//


#include <stdlib.h>  // for the NULL macro

#include "netgen.hpp"
#include "error.hpp"


//**************************************************************************
// conversion nodeID <--> network address
//**************************************************************************

// computes the network address for a node with the user identifier nodeID
// assert: networkAddress != NULL
// assert: nodeID >= 0
void NodeToNetwork (SINT32 nodeID, addr* networkAddress)
{
    INT32 i;

    Assert (networkAddress != NULL);
    Assert (nodeID >= 0);  // only valid node IDs are considered

    // the network address must be an internet address, so the nodeID
    // is split into bytes; the lo-byte is stored in sa[ADDRCNT-1]
    // note: since Watcom appears to shift around, only sizeof(INT32)
    // bytes are taken from nodeID, the rest of networkAddress is filled
    // with zeros
    for (i=0; (i<ADDRCNT) && (i<sizeof(INT32)/sizeof(INT8)); i++)
    {
        (*networkAddress)[ADDRCNT-1-i] = (INT8)(nodeID >> (8*i));
    }
    for (i=sizeof(INT32)/sizeof(INT8); i<ADDRCNT; i++)
    {
        (*networkAddress)[ADDRCNT-1-i] = 0;
    }
}

// computes the user node ID for a network address
// assert: networkAddress != NULL
void NetworkToNode (const addr* networkAddress, SINT32& nodeID)
{
    Assert (networkAddress != NULL);
    nodeID = 0;

    // just reverse the above process
    for (INT32 i=0; (i<ADDRCNT) && (i<sizeof(INT32)/sizeof(INT8)); i++)
    {
        nodeID += ((SINT32)(*networkAddress)[ADDRCNT-1-i]) << (8*i);
    }
}


