//
// sockdgrm.cpp
//
// implementation of the basic datagram socket interface class, and of the
// access classes for client and server datagram sockets
//
// created: 07.02.97
//
// last change: 06.04.97
//
// RCS version control information:
//     $Revision: 1.2 $$State: Stable $$Date: 1998/07/03 12:45:47 $
//


#include "constant.hpp"
#include "error.hpp"
#include "sockdgrm.hpp"
#include "socket.hpp"



//**************************************************************************
// class CDatagramInterface: used as an interface to a datagram socket;
//      can send/receive datagram messages
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor
//--------------------------------------------------------------------------

// assume that a datagram socket with the ID 'sid' exists and is open
// assert: addr != NULL
CDatagramInterface::CDatagramInterface (ACSocketAddress* addr, INT32 sid)
    : CSocket (addr, sid, False)
{
}


//--------------------------------------------------------------------------
//--- message transmission
//--------------------------------------------------------------------------

// transmits the message over the socket; returns True if the operation
// was successful, else False
// assert: data != NULL
EBool CDatagramInterface::SendMessage (void *data, INT32 size)
{
    return CSocket::Send (data, size);
}


//--------------------------------------------------------------------------
//--- message reception
//--------------------------------------------------------------------------

// reads the data from the socket (assumes that 'data' points to a
// buffer of size '*size'); blocks according to the internal timeout;
// modifies '*size' to reflect the number of bytes read; returns True
// if the receive operation was successful, else False
// assert: data, size != NULL
EBool CDatagramInterface::ReceiveMessage (void *data, INT32 *size)
{
    return CSocket::Receive (data, size);
}


//--------------------------------------------------------------------------
//--- internal constructor
//--------------------------------------------------------------------------

// store the address of the socket
// assert: addr != NULL
CDatagramInterface::CDatagramInterface (ACSocketAddress* addr)
    : CSocket (addr, False)
{
}



//**************************************************************************
// class CDatagramServer: creates a datagram socket
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

// create the socket
// assert: addr != NULL
CDatagramServer::CDatagramServer (ACSocketAddress* addr)
    : CDatagramInterface (addr)
{
    INT32 sid;
    Assert (s_create (addr, SDatagram, &sid) == SOkay);  // create socket
    SetSocketID (sid);  // store the ID
}

// close the socket (unlink it if necessary)
CDatagramServer::~CDatagramServer ()
{
    s_delete (GetSocketID(), GetSocketAddress());
    // delete the socket, maybe unlink it
}



//**************************************************************************
// class CDatagramClient: identifies a datagram socket
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor
//--------------------------------------------------------------------------

// identify the socket
// assert: addr != NULL
CDatagramClient::CDatagramClient (ACSocketAddress* addr)
    : CDatagramInterface (addr)
{
    INT32 sid;
    Assert (s_ident (addr, SDatagram, &sid) == SOkay);  // identify socket
    SetSocketID (sid);  // store the ID
}


