//
// sockunix.cpp
//
// implementation of the unix domain socket address
//
// created: 06.02.97
//
// last change: 11.03.97
//
// RCS version control information:
//     $Revision: 1.1 $$State: Stable $$Date: 1998/06/16 10:49:31 $
//


#include <stdlib.h>

#include "error.hpp"
#include "sockunix.hpp"


#ifndef DOS_SIM

#include <sys/socket.h>
#include <sys/un.h>
#include <string.h>


//**************************************************************************
// class CUnixAddress: unix domain address
//**************************************************************************

//--------------------------------------------------------------------------
//--- constructor, destructor
//--------------------------------------------------------------------------

CUnixAddress::CUnixAddress (const char* name)
{
    Assert (name != NULL);
    addr_ = new sockaddr_un;  // make a new structure
    Assert (addr_ != NULL);
    addr_->sun_family = AF_UNIX;  // family is Unix
    strcpy (addr_->sun_path, name);  // store the name
}


CUnixAddress::CUnixAddress (const CUnixAddress& un)
{
    addr_ = new sockaddr_un;
    Assert (addr_ != NULL);
    addr_->sun_family = AF_UNIX;
    strcpy (addr_->sun_path, un.addr_->sun_path);
}


CUnixAddress::~CUnixAddress ()
{
    delete addr_;
}


//--------------------------------------------------------------------------
//--- necessary functions
//--------------------------------------------------------------------------

// returns the size of the address structure
int CUnixAddress::Size () const
{
    return sizeof (sockaddr_un);
}

// returns the address family
int CUnixAddress::Family () const
{
    return addr_->sun_family;
}

// returns the address structure itself
sockaddr* CUnixAddress::Addr() const
{
    return (sockaddr*)addr_;
}

#else

// just some dummy functions for DOS
CUnixAddress::CUnixAddress (const char* name)
{
    Assert (name != NULL);
}

CUnixAddress::CUnixAddress (const CUnixAddress&) {}

CUnixAddress::~CUnixAddress () {}


//--------------------------------------------------------------------------
//--- necessary functions
//--------------------------------------------------------------------------

int CUnixAddress::Size () const { return 0; }
int CUnixAddress::Family () const { return 0; }
sockaddr* CUnixAddress::Addr() const { return 0; }

#endif


